package com.alpha.lagouapk.manager

import com.ApiService
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.commonlibs.utils.ListUtil
import com.lagou.commonlibs.utils.LogUtils
import com.lagou.commonlibs.utils.StringUtil
import com.lagou.lagouee.bean.jsonbean.position.PositionMarkResponse
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse

/**
 * 职位角标管理类
 */
object PositionLabelManager {
    /**
     * 职位Id+场景->职位标签图标Url
     */
    private var positionLabelMap = HashMap<String, String?>()
    private const val STATIC_SPLIT = "_&_"

    /**
     * 首页职位
     */
    const val POSITION_SCENE_HOME = "APR"

    /**
     * 搜索职位
     */
    const val POSITION_SCENE_SEARCH = "APQ"

    /**
     * 职位详情
     */
    const val POSITION_SCENE_DETAIL = "APD_NEW"

    @JvmOverloads
    fun fetchPositionsLabel(
        positionIds: List<Long>?,
        scene: String? = null,
        callback: ((Boolean) -> Unit)?) {
        if (ListUtil.isEmpty(positionIds)) {
            return
        }
        fetchPositionsLabel(positionIds?.joinToString(","),scene,callback)
    }

    /**
     * @param positionIds 职位id，多个职位id用","分割
     */
    @JvmOverloads
    fun fetchPositionsLabel(positionIds:String?,scene:String? = null,callback: ((Boolean) -> Unit)?){
        ApiService.api.requestPositionMark(positionIds, scene).enqueue(object : BaseCallback<PositionMarkResponse>(false) {
            override fun onSuccess(
                request: HttpRequest<*>?,
                response: HttpResponse<PositionMarkResponse>?) {
                val positionLabels = response?.content?.positionMarks
                if (!positionLabels.isNullOrEmpty()) {
                    positionLabels.filter { it.positionId > 0 }.forEach {
                        positionLabelMap[packageKey(it.positionId.toLong(),scene)] = it.positionLabelIcon
                    }
                    callback?.invoke(true)
                }
            }

            override fun onFailure(request: HttpRequest<*>?,
                                   response: HttpResponse<PositionMarkResponse>?) {
                super.onFailure(request, response)
                callback?.invoke(false)
            }
        })
    }

    @JvmOverloads
    fun fetchPositionLabel(
        positionId: Long,
        scene: String? = null,
        callback: ((Boolean) -> Unit)?) {
        fetchPositionsLabel(arrayListOf(positionId), scene, callback)
    }

    @JvmOverloads
    private fun packageKey(positionId:Long,scene: String? = null):String = positionId.toString() + STATIC_SPLIT + (scene ?: "")

    /**
     * 获取缓存的职位标签url
     */
    @JvmOverloads
    fun getLabelUrl(positionId:Long,scene:String? = null):String? {
        return positionLabelMap[packageKey(positionId,scene)]
    }
}