package com.alpha.lagouapk.manager

import com.ApiService
import com.alpha.lagouapk.bean.SimpleAttachResumeSyncBean
import com.alpha.lagouapk.bean.SimpleProjectExpBean
import com.alpha.lagouapk.bean.SimpleWorkExpBean
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse

class SimpleResumeSyncManager private constructor(){
    /**
     * 需要同步的工作经历
     * key：公司名称
     * value：工作经历对象
     */
    private var workExpMap:MutableMap<String,SimpleWorkExpBean> = mutableMapOf()

    /**
     * 需要同步的项目经理
     * key:项目名称
     * value:项目经历对象
     */
    private var projectExpMap:MutableMap<String,SimpleProjectExpBean> = mutableMapOf()
    companion object{
        @JvmStatic
        val instance:SimpleResumeSyncManager by lazy(mode = LazyThreadSafetyMode.SYNCHRONIZED){
            SimpleResumeSyncManager()
        }
    }

    /**
     * 获取需要同步的简历数据
     */
    fun fetchSyncResumeData(){
        reset()
        ApiService.api.syncSimpleResume().enqueue(object: BaseCallback<SimpleAttachResumeSyncBean>(false){
            override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<SimpleAttachResumeSyncBean>?) {
                val simpleAttachResumeSyncBean = response?.content
                simpleAttachResumeSyncBean?.let {
                    it.workExperienceVos?.asSequence()?.filter { workExpBean ->
                        workExpBean != null && workExpBean.companyName.isNullOrEmpty().not()
                    }?.map { workExpBean ->
                        workExpBean?.workContent = workExpBean?.workContent?.replace("\n", "<br />")
                        workExpBean
                    }?.forEach { workExpBean ->
                        workExpMap[workExpBean?.companyName!!] = workExpBean
                    }

                    it.projectExperienceVos?.asSequence()?.filter { projectExpBean ->
                        projectExpBean != null && projectExpBean.projectName.isNullOrEmpty().not()
                    }?.map { projectExpBean ->
                        projectExpBean?.projectRemark = projectExpBean?.projectRemark?.replace("\n", "<br />")
                        projectExpBean
                    }?.forEach { projectExpBean ->
                        projectExpMap[projectExpBean?.projectName!!] = projectExpBean
                    }
                }
            }

        })
    }

    fun getWorkCompanyNames():List<String>{
        return workExpMap.map { it.key }
    }

    fun getProjectNames():List<String>{
        return projectExpMap.map { it.key }
    }

    fun getWorkExpByCompanyName(companyName:String): SimpleWorkExpBean? {
        return workExpMap[companyName]
    }

    fun getProjectExpByProjectName(projectName:String):SimpleProjectExpBean?{
        return projectExpMap[projectName]
    }

    private fun reset(){
        workExpMap.clear()
        projectExpMap.clear()
    }
}