package com.alpha.lagouapk.tools

import android.app.Activity
import android.os.Bundle
import android.view.View
import android.view.animation.AlphaAnimation
import android.widget.ViewFlipper
import androidx.fragment.app.Fragment
import com.alpha.lagouapk.R
import com.common.manager.db.CommonDbManager
import com.geetest.onelogin.activity.OneLoginActivity

/**
 * @date： Crate by wangjunwei on 2023/6/7
 * @description：
 * 类似ActivityLifecycleCallbacks的代理类，目前功能控制一键登录页面动态背景展示
 **/
class PageLifeCycleHelper {

    companion object {

        var viewFlipper: ViewFlipper? = null
        var isNewLoginStyle: Boolean? = null

        /***
         * 挂在到特定Activity上，根据需要后续可扩展到其它逻辑
         */
        @JvmStatic
        fun onActivityPostCreated(activity: Activity, savedInstanceState: Bundle?) {
            if (activity !is OneLoginActivity) {
                return
            }

            if (!isNewLoginStyle()) {
                return
            }

            val viewFlipper = activity.findViewById<ViewFlipper>(R.id.viewFlipper)
            val vmSmegma = activity.findViewById<View>(R.id.vm_smegma)

            if (viewFlipper == null || vmSmegma == null){
                return
            }

            viewFlipper.visibility = View.VISIBLE
            vmSmegma.visibility = View.VISIBLE
            handleNewLoginAnim(viewFlipper, activity)
        }


        @JvmStatic
        fun onActivityResumed(activity: Activity) {
            viewFlipper?.apply {
                if (!isFlipping){
                    startFlipping()
                }
            }
        }
        /***
         * 页面不可见暂停动画，降低开销
         */
        @JvmStatic
        fun onActivityStopped(activity: Activity) {
            viewFlipper?.apply {
                if (isFlipping){
                    stopFlipping()
                }
            }
        }
        @JvmStatic
        fun onActivityDestroyed(activity: Activity) {
            viewFlipper?.apply {
                if (isFlipping){
                    stopFlipping()
                }
                viewFlipper = null
            }
        }


        @JvmStatic
        fun onFragmentResumed(fragment: Fragment) {
            viewFlipper?.apply {
                if (!isFlipping){
                    startFlipping()
                }
            }
        }
        /***
         * 页面不可见暂停动画，降低开销
         */
        @JvmStatic
        fun onFragmentStopped(fragment: Fragment) {
            viewFlipper?.apply {
                if (isFlipping){
                    stopFlipping()
                }
            }
        }
        @JvmStatic
        fun onFragmentDestroyView(fragment: Fragment) {
            viewFlipper?.apply {
                if (isFlipping){
                    stopFlipping()
                }
                viewFlipper = null
            }
        }

        /**
         * 登录页面动态背景
         */
        @JvmStatic
        fun handleNewLoginAnim(viewFlipper: ViewFlipper?, activity: Activity) {
            if (viewFlipper == null || !isNewLoginStyle()) {
                return
            }
            viewFlipper.addView(activity.layoutInflater.inflate(R.layout.image_login_new_one, null))
            viewFlipper.addView(activity.layoutInflater.inflate(R.layout.image_login_new_two, null))
            viewFlipper.addView(activity.layoutInflater.inflate(R.layout.image_login_new_three, null))
            viewFlipper.addView(activity.layoutInflater.inflate(R.layout.image_login_new_four, null))
            viewFlipper.addView(activity.layoutInflater.inflate(R.layout.image_login_new_five, null))
            // 创建淡入和淡出动画
            val fadeIn = AlphaAnimation(0f, 1f)
            fadeIn.duration = 1250
            val fadeOut = AlphaAnimation(1f, 0f)
            fadeOut.duration = 1250
            viewFlipper.inAnimation = fadeIn
            viewFlipper.outAnimation = fadeOut
            viewFlipper.isAutoStart = true
            viewFlipper.flipInterval = 3000
            viewFlipper.startFlipping()
        }

        /***
         * 为了保持两次调用isNewLoginStyle返回的结果一致，特定义isNewLoginStyle字段，只取第一次初始化的值，
         * 如果第一次初始化时服务器结果还没有回来则取上一次的默认值
         */
        @JvmStatic
        fun isNewLoginStyle(): Boolean {
            if (isNewLoginStyle == null){
                val commonConfig = CommonDbManager.getInstance().commonConfig
                isNewLoginStyle = commonConfig != null && commonConfig.isNewLoginStyle
            }

            if (isNewLoginStyle == null){
                isNewLoginStyle = false
            }

            isNewLoginStyle?.let {
                return it
            }

            return false
        }
    }
}