package com.alpha.lagouapk.ui.adapter

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.AggregateMessageBean
import com.alpha.lagouapk.widget.viewholder.RecommendPositionTipsView
import com.alpha.lagouapk.widget.viewholder.RecommendPositionView
import com.common.chat.item.LoadingItem
import com.common.chat.item.UnknownItem
import com.common.widget.WifiView
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.bean.json.ShowLogInfo
import com.lagou.commonlibs.utils.ListUtil

/**
 * @author 作者：ryan.lei
 * @date 创建时间：2022/2/22 15:01
 * @Description 描述：求职助手Lucy 职位推荐列表页Adapter
 **/
class OfflineAssistantLucyAdapter(
    val mContext: Context,
    val mData: List<AggregateMessageBean.Message>
) : RecyclerView.Adapter<OfflineAssistantLucyAdapter.RecommendPositionViewHolder>(),
    LoadingItem.Callback {

    companion object {
        /**
         * loadding
         */
        const val LOADDING = 1

        /**
         * 空页面
         */
        const val EMPTY = 2

        /**
         * 网络异常
         */
        const val NET_ERROR = 3

        /**
         * 推荐文案样式
         */
        const val RECOMMEND_POSITION_TIPS = 4

        /**
         * 推荐职位样式
         */
        const val RECOMMEND_POSITION = 5
    }

    private var mHasMore: Boolean = false
    private var mEmpty: Boolean = false
    private var mNetError: Boolean = false
    private var mOnLoadMoreListener: OnLoadMoreListener? = null
    private val mLoadingItem: LoadingItem = LoadingItem(mContext)

    override fun onCreateViewHolder(
        parent: ViewGroup,
        viewType: Int
    ): RecommendPositionViewHolder {
        val itemView = when (viewType) {
            LOADDING -> {
                mLoadingItem.layoutParams = RecyclerView.LayoutParams(
                    RecyclerView.LayoutParams.MATCH_PARENT,
                    RecyclerView.LayoutParams.WRAP_CONTENT
                )
                mLoadingItem.setCallback(this)
                //最后return出mLoadingItem
                mLoadingItem
            }
            EMPTY ->
                LayoutInflater.from(mContext).inflate(R.layout.layout_gogo_empty, parent, false)
            NET_ERROR -> {
                val wifiView = WifiView(mContext)
                wifiView.layoutParams = RecyclerView.LayoutParams(
                    RecyclerView.LayoutParams.MATCH_PARENT,
                    RecyclerView.LayoutParams.MATCH_PARENT
                )
                //最后return出wifiView
                wifiView
            }
            RECOMMEND_POSITION ->
                RecommendPositionView(mContext)
            RECOMMEND_POSITION_TIPS ->
                RecommendPositionTipsView(mContext)
            else ->
                UnknownItem(mContext)
        }
        return RecommendPositionViewHolder((itemView))
    }

    override fun onBindViewHolder(holder: RecommendPositionViewHolder, position: Int) {
        val bean = mData[position]
        when (holder.itemViewType) {
            LOADDING ->
                (holder.itemView as LoadingItem).startLoad()
            NET_ERROR ->
                (holder.itemView as WifiView).setReLoadListener {
                    mOnLoadMoreListener?.onReload()
                }
            RECOMMEND_POSITION -> {
                (holder.itemView as RecommendPositionView).setData(bean)
                //埋点
                val info = ShowLogInfo()
                    .setPage_Id("j1tix")
                    .setAddress_Id("k1tiy")
                    .setContentId(bean.positionId.toLong())
                AnalyticHelper.getInstance().sendShowInfo(info)
            }
            RECOMMEND_POSITION_TIPS ->
                (holder.itemView as RecommendPositionTipsView).setData(bean)
        }
    }

    override fun getItemCount(): Int {
        val count =
            if (ListUtil.isEmpty(mData))
                0
            else
                mData.size
        if (mEmpty || mNetError) {
            //显示empty和网络异常
            return 1
        }
        return if (mHasMore)
            count + 1
        else
            count
    }

    override fun getItemViewType(position: Int): Int {
        if (mNetError) {
            return NET_ERROR
        }
        if (mEmpty) {
            return EMPTY
        }
        if (mHasMore && position == itemCount - 1) {
            return LOADDING
        }

        val message = mData[position]
        val isTip = message.isTip
        return if (isTip)
            RECOMMEND_POSITION_TIPS
        else
            RECOMMEND_POSITION
    }

    class RecommendPositionViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView)

    fun setOnLoadMoreListener(listener: OnLoadMoreListener) {
        mOnLoadMoreListener = listener
    }

    override fun onLoadingStart() {
        mOnLoadMoreListener?.onLoadMore(mLoadingItem)
    }

    interface OnLoadMoreListener {
        fun onLoadMore(loadingItem: LoadingItem?)

        fun onReload()
    }
}