package com.alpha.lagouapk.ui.company.list

import android.annotation.SuppressLint
import android.app.Activity
import android.content.Context
import android.os.Bundle
import android.text.TextUtils
import android.util.SparseArray
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentManager
import androidx.fragment.app.FragmentPagerAdapter
import androidx.viewpager.widget.ViewPager
import com.alpha.lagouapk.LagouMainPage
import com.alpha.lagouapk.R
import com.alpha.lagouapk.observer.EventTags
import com.alpha.lagouapk.tools.AnalyticUtilsC
import com.alpha.lagouapk.tools.IntentUtils
import com.alpha.lagouapk.tools.SPUtils
import com.alpha.lagouapk.ui.company.CompanyRankingListFragment
import com.alpha.lagouapk.ui.company.video.CompanyCategoryVideoFragment
import com.lagou.baselib.BaseKotlinFragment
import com.lagou.commonlibs.event.Event
import com.lagou.commonlibs.event.OkBus
import com.lagou.commonlibs.utils.LogUtils
import com.lagou.commonlibs.utils.VisibilityUtil
import com.lagou.commonlibs.web.router.RouteUtils
import kotlinx.android.synthetic.main.fragment_company_container.*


class CompanyContainerFragment : BaseKotlinFragment() {
    override fun fillUi() {
        mAddCompanyListTab = SPUtils.isExpectBeijingCity()
        mAddVideoTab = SPUtils.getShowCompanyVideoTab()
        mSchemeTabName  = arguments?.getString("scheme_tab_name")
        if (mAddCompanyListTab || mAddVideoTab) {
            // 多个Fragment
            showViews(fl_title_container, vp_content_company)
            hideViews(fl_company_container)
            addMultiFragment()
        } else {
            // 单一Fragment
            hideViews(fl_title_container, vp_content_company, v_tab_shadow)
            showViews(fl_company_container)
            addSingleFragment()
        }

        iv_search.setOnClickListener {
            IntentUtils.startSearchPositionActivity(requireActivity(), "")
        }

    }

    companion object{
        @JvmStatic
        fun newInstance(isShowTitle:Boolean = false,mSchemeTabName:String?):CompanyContainerFragment{
            val companyContainerFragment = CompanyContainerFragment()
            val bundle = Bundle()
            bundle.putBoolean("show_title",isShowTitle)
            bundle.putString("scheme_tab_name",mSchemeTabName)
            companyContainerFragment.arguments = bundle
            return companyContainerFragment
        }
    }


    @SuppressLint("StaticFieldLeak")
    private var mFragment: Fragment? = null
    /**
     * 是否是多个Fragment
     */
    private var mAddCompanyListTab: Boolean = false
    /**
     * 是否增加公司视频tab
     */
    private var mAddVideoTab: Boolean = false
    /**
     * scheme跳转过来的时候tab的value
     */
    var mSchemeTabName:String?= ""

    private val VALUE_TAB_NAME_VIDEO = "公司探秘"
    private val VALUE_TAB_NAME_GREAT_COMPANY = "榜单"
    private val VALUE_TAB_NAME_COMPANY_DEPOT = "公司库"

    private var companyVideoReddotEvent: Event? = null

    private var mSchemeTypeEvent: Event? = null

    @SuppressLint("StaticFieldLeak")
    private var mActivity:Activity? = null

    private var currentTab:Int = -1

    private var mAdapter: CompanyPagerAdapter? = null

    override fun getLayoutId(): Int {
        return R.layout.fragment_company_container
    }

    override fun onAttach(context: Context) {
        mActivity = if (context is Activity) context else null
        super.onAttach(context)
    }

    override fun onDetach() {
        mActivity = null
        super.onDetach()
    }


    /**
     * 添加单一Fragment
     */
    private fun addSingleFragment() {
        val fragmentManager = childFragmentManager
        mFragment = fragmentManager.findFragmentByTag(CompanyListFragment.TAG) as? CompanyListFragment
        if (mFragment == null) {
            mFragment = CompanyListFragment.newInstance(true)
        }
        try {
            fragmentManager.beginTransaction()
                    .replace(R.id.fl_company_container, mFragment!!, CompanyListFragment.TAG)
                    .commit()
        } catch (e: Exception) {
            e.printStackTrace()
        }
        //取消红点
        SPUtils.setCompanyVideoRedDot(LagouMainPage.SP_VALUE_COMPANY_NO_REDDOT)
        tl_navigation_company.hideMsg(0)
        if (requireActivity() is CompanyRedPointCallback){
            val mCall = requireActivity() as CompanyRedPointCallback
            mCall.showOrHideRedPoint(false)
        }
    }

    fun showViews(vararg vs: View){
        VisibilityUtil.setVisible(*vs)

    }

    fun hideViews(vararg vs: View) {
        VisibilityUtil.setGone(*vs)
    }


    /**
     * 添加多个Fragment
     */
    private fun addMultiFragment() {
        val fragments = ArrayList<Fragment>()
        if (mAddVideoTab) {
            // 探秘视频
            fragments.add(CompanyCategoryVideoFragment.newInstance())
        }
        if (mAddCompanyListTab) {
            // 好公司
            fragments.add(CompanyRankingListFragment.newInstance())
        }
        // 公司列表
        fragments.add(CompanyListFragment.newInstance(false))

        val tabTitles: Array<String>
        tabTitles = if (fragments.size == 3) {
            arrayOf(VALUE_TAB_NAME_VIDEO, VALUE_TAB_NAME_GREAT_COMPANY, VALUE_TAB_NAME_COMPANY_DEPOT)
        } else {
            if (mAddCompanyListTab) {
                arrayOf(VALUE_TAB_NAME_GREAT_COMPANY, VALUE_TAB_NAME_COMPANY_DEPOT)
            } else {
                arrayOf(VALUE_TAB_NAME_VIDEO, VALUE_TAB_NAME_COMPANY_DEPOT)
            }
        }
        mAdapter = CompanyPagerAdapter(childFragmentManager, fragments)
        vp_content_company.addOnPageChangeListener(object : ViewPager.SimpleOnPageChangeListener() {
            override fun onPageSelected(position: Int) {
                AnalyticUtilsC.companyTabClick(position)
                if (fragments[position] is CompanyListFragment) {
                    val fragment = getCompanyListFragment()
                    fragment?.refreshShadow()
                }
                if (fragments[position] is CompanyCategoryVideoFragment) {
                    //取消红点
                    SPUtils.setCompanyVideoRedDot(LagouMainPage.SP_VALUE_COMPANY_NO_REDDOT)
                    tl_navigation_company.hideMsg(position)
                    if (requireActivity() is CompanyRedPointCallback){
                        val mCall = requireActivity() as CompanyRedPointCallback
                        mCall.showOrHideRedPoint(false)
                    }
                }
            }
        })
        vp_content_company.offscreenPageLimit = fragments.size - 1
        vp_content_company.adapter = mAdapter
        vp_content_company.currentItem = getRedDotState(tabTitles)

        tl_navigation_company.setTabSpace(18)
        tl_navigation_company.setViewPager(vp_content_company, tabTitles)

        tl_navigation_company.currentTab = getRedDotState(tabTitles)
        // 手动触发onPageSelected，设置TabLayout选中首个Tab
        tl_navigation_company.onPageSelected(getRedDotState(tabTitles))

        companyVideoReddotEvent = Event {
            showCompanyVideoReddot(tabTitles)
        }

        OkBus.getInstance().register(EventTags.REDDOT_COMPANY_VIDEO, companyVideoReddotEvent)

        mSchemeTypeEvent = Event {
            if (it != null && it.obj is String) {
                mSchemeTabName = RouteUtils.getQueryParameter(it.obj as String, "tab", String::class.java)
                tl_navigation_company.currentTab = getRedDotState(tabTitles)
                vp_content_company.currentItem = getRedDotState(tabTitles)
            }
        }
        OkBus.getInstance().register(EventTags.COMPANY_DISCOVER_TAG, mSchemeTypeEvent)

        if (requireActivity().parent is LagouMainPage) {
            if (SPUtils.getCompanyVideoRedDot() == LagouMainPage.SP_VALUE_COMPANY_HAVE_REDDOT) {
                showCompanyVideoReddot(tabTitles)
            }
        }
    }

    override fun onVisibleToUser(isVisibleToUser: Boolean) {
        super.onVisibleToUser(isVisibleToUser)
        if(currentTab >= 0 && isVisibleToUser){
            if (isVisibleToUser && mAdapter?.getItem(currentTab) is CompanyCategoryVideoFragment) {
                //取消红点
                SPUtils.setCompanyVideoRedDot(LagouMainPage.SP_VALUE_COMPANY_NO_REDDOT)
                tl_navigation_company.hideMsg(0)
                if (requireActivity() is CompanyRedPointCallback){
                    val mCall = requireActivity() as CompanyRedPointCallback
                    mCall.showOrHideRedPoint(false)
                }
            }
        }
    }

    /**
     * 初始化探秘视频的红点
     * 当'探秘视频'在第一个tab或者没有'探秘视频'的tab时,红点不显示
     *
     * @param tabTitles
     */
    private fun showCompanyVideoReddot(tabTitles: Array<String>?) {
        if (tabTitles == null) {
            return
        }
        for (i in 0 until  tabTitles.size) {
            if (VALUE_TAB_NAME_VIDEO == tabTitles[i]) {
                if (i >= 0) {
                    tl_navigation_company.showDot(i)
                    break
                }
            }
        }
    }

    private fun getRedDotState(tabTitles: Array<String>?):Int{
        val tabPosition = getCurrentTab(tabTitles)
        LogUtils.d("Anneducation  tab $tabPosition  current item ${mAdapter?.getItem(tabPosition)}")
        if (isVisibleToUser && mAdapter?.getItem(tabPosition) is CompanyCategoryVideoFragment) {
            //取消红点
            SPUtils.setCompanyVideoRedDot(LagouMainPage.SP_VALUE_COMPANY_NO_REDDOT)
            LogUtils.d("Anneducation  hide msg")
            tl_navigation_company.hideMsg(0)
            if (requireActivity() is CompanyRedPointCallback){
                LogUtils.d("Anneducation  hide dot")
                val mCall = requireActivity() as CompanyRedPointCallback
                mCall.showOrHideRedPoint(false)
            }
        }
        return tabPosition
    }

    private fun getCurrentTab(tabTitles: Array<String>?): Int {
        if (tabTitles == null) {
            currentTab = 0
            return currentTab
        }
        if (TextUtils.isEmpty(mSchemeTabName)) {
            currentTab = 0
            return currentTab
        }

        for (i in tabTitles.indices) {
            if (VALUE_TAB_NAME_VIDEO == tabTitles[i] && "tmsp" == mSchemeTabName) {
                currentTab = i
                break
            }
            if (VALUE_TAB_NAME_GREAT_COMPANY == tabTitles[i] && "hgs" == mSchemeTabName) {
                currentTab = i
                break
            }
            if (VALUE_TAB_NAME_COMPANY_DEPOT == tabTitles[i] && "gsk" == mSchemeTabName) {
                currentTab = i
                break
            }
        }
        return currentTab
    }


    fun getCompanyListFragment(): CompanyListFragment? {
        var fragment: CompanyListFragment? = null
        if (mAddCompanyListTab) {
            if (mAdapter != null) {
                fragment = mAdapter!!.companyListFragment
            }
        } else {
            fragment = mFragment as CompanyListFragment?
        }
        return fragment
    }

    private fun checkShadowHeight(height: Int) {
        val params = v_tab_shadow.getLayoutParams()
        if (params.height != height) {
            params.height = height
            v_tab_shadow.setLayoutParams(params)
        }
    }

     override fun onDestroy() {
        super.onDestroy()
        OkBus.getInstance().unRegister(companyVideoReddotEvent)
        OkBus.getInstance().unRegister(mSchemeTypeEvent)
    }

    class CompanyPagerAdapter(fm: FragmentManager, private val mList: List<Fragment>) : FragmentPagerAdapter(fm) {
        private val mFragments: SparseArray<Any> = SparseArray()

        /**
         * 取出CompanyListFragment实例
         *
         *
         * 适配器内部对Fragment做了复用处理，此方法可得到真正的Fragment对象
         * CompanyListFragment是mList的最后一个元素
         *
         *
         */
        val companyListFragment: CompanyListFragment?
            get() {
                if (mList.size > 1) {
                    val o = mFragments.get(1)
                    return if (o is CompanyListFragment) o else null
                }
                return null

            }

        override fun instantiateItem(container: ViewGroup, position: Int): Any {
            val fragment = super.instantiateItem(container, position)
            mFragments.put(position, fragment)
            return fragment
        }

        override fun destroyItem(container: ViewGroup, position: Int, `object`: Any) {
            mFragments.remove(position)
            super.destroyItem(container, position, `object`)
        }

        override fun getItem(position: Int): Fragment {
            return mList[position]
        }

        override fun getCount(): Int {
            return mList.size
        }
    }

}

