package com.alpha.lagouapk.ui.course

import android.content.Intent
import android.view.View
import androidx.fragment.app.FragmentActivity
import com.alpha.lagouapk.Interface.logout.LogoutObserver
import com.alpha.lagouapk.Interface.logout.LogoutServer
import com.alpha.lagouapk.LagouApplication
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.course.DownloadEduRes
import com.alpha.lagouapk.tools.IntentUtils
import com.common.manager.net.CommonNetManager
import com.lagou.annotation.apt.RouterRule
import com.lagou.audiolib.dialog.AudioeBarDialog
import com.lagou.audiolib.event.CourseEventTags
import com.lagou.audiolib.manager.AudioPlayerManager
import com.lagou.audiolib.service.MediaPlayerService
import com.lagou.audiolib.utils.AudioIntentUtils
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListenerImpl
import com.lagou.baselib.util.BarUtil
import com.lagou.commonlibs.event.Event
import com.lagou.commonlibs.event.OkBus
import com.lagou.commonlibs.utils.ToastUtil
import com.lagou.course.adapter.PurchasedListAdapter
import com.lagou.course.bean.BigCourseRecord
import com.lagou.course.bean.CourseRecord
import com.lagou.course.bean.MemberAdsBarBean
import com.lagou.course.bean.PurchasedNewBean
import com.lagou.course.inter.PurchasedListAllItemClickedCallback
import com.lagou.course.manager.HomeVipDialogManager
import com.lagou.course.manager.PurchasedPageClick
import com.lagou.course.manager.PurchasedPageClickManager
import com.lagou.course.purchased.PurchasedListFragment
import com.lagou.course.utils.CourseIntentUtils
import com.lagou.course.utils.CourseSPUtils
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.android.synthetic.main.activity_course_main.fl_container
import kotlinx.android.synthetic.main.activity_purchase_main.*

/**
 * @date 创建时间：2020-02-24
 * @author 作者：codeman
 * @Description  描述：已购页面
 * @version
 */
@RouterRule("lagou://lagou.com/purchased")
class PurchasedMainActivity : BaseActivity(), LogoutObserver, PurchasedListAllItemClickedCallback{

    private var downloadEduRes: DownloadEduRes? = null

    override fun getContentLayoutId() = R.layout.activity_purchase_main

    override fun init() {
        BarUtil.statusColorBarInit3(this, R.id.titleBar)
        supportFragmentManager.beginTransaction().add(R.id.frame_container,
                PurchasedListFragment.newInstance()).commit()
        val click  = PurchasedPageClick()
        click.setOnPurchasedClickListener(this)
        PurchasedPageClickManager.getInstance().registerCallback(PurchasedPageClickManager.PURCHASED_LIST, click)
        //音频
        OkBus.getInstance().register(CourseEventTags.START_AUDIO, audioEvent)
        LogoutServer.getInstance().registerObserver(this)
        if (AudioPlayerManager.getInstance(mContext).isRunning) {
            startAudio(false)
        } else {
            onClosePlayBarDialog()
        }
        requestDownloadEdu()
        titleBar.setLeftImgListener { finish()}
        titleBar.setTvRightDrawable(R.drawable.navigation_ic_arrow_s, R.dimen.dp_4)
        titleBar.setRightTvListener { IntentUtils.startLagouEduApp(this, downloadEduRes)}
    }

    private fun requestDownloadEdu() {
        CommonNetManager.requestDownEduApp(object : OnAdapterCommonListenerImpl<DownloadEduRes, HttpResponse<DownloadEduRes>>(baseActivity) {
            override fun onSuccess(t: DownloadEduRes?) {
                super.onSuccess(t)
                downloadEduRes = t
                downloadEduRes?.apply {
                    if (isCanShow) {
                        titleBar.tvRight.visibility = View.VISIBLE
                    }
                    titleBar.setRightTxt("拉勾教育")
                }
            }
            override fun isShowToast() = false
        })
    }

    override fun isEnableCommonBarSetting(): Boolean {
        return false
    }

    override fun onDestroy() {
        super.onDestroy()
        PurchasedPageClickManager.getInstance().removeCallBack(PurchasedPageClickManager.PURCHASED_LIST)
        OkBus.getInstance().unRegister(audioEvent)
        LogoutServer.getInstance().removeObserver(this)
        dialog?.close()
    }

    var dialog: AudioeBarDialog? = null
    private val audioEvent = Event { startAudio(true) }

    private fun startAudio(isPlay: Boolean) {
        if (dialog == null) {
            dialog = AudioeBarDialog(this)
        }
        dialog!!.setData()
        if (isPlay) {
            dialog!!.startPlay()
            // 开启服务
            val context = LagouApplication.getLagouApplication()
            AudioIntentUtils.startServiceNoException(context, Intent(context, MediaPlayerService::class.java))
        }
        if (!dialog!!.isShowing()) {
            dialog!!.show(fl_container)
        }
    }

    override fun onClosePlayBarDialog() {
        if (dialog != null) {
            dialog!!.close()
            dialog = null
        }
    }
    /**
     * 非大课点击
     */
    override fun onPurchasedCourseClick(courseRecord: CourseRecord, itemType: Int, identityCode: Int, isClickGroup : Boolean) {
        if (courseRecord.vipFreeCourse &&
                (identityCode == MemberAdsBarBean.VIP_STATUS_FREE_OUT_OF_DATA || identityCode == MemberAdsBarBean.VIP_STATUS_YEAR_OUT_OF_DATA)) {
            //曾经是会员的时候领取的课程,现在不是会员了,就别想着看了
            if (mContext is FragmentActivity) {
                HomeVipDialogManager.getInstance().showVipOutOfDateDialog(mContext as FragmentActivity)
            }
        } else {
            //默认情况
            if (courseRecord.openWebToStudy.isNullOrEmpty()) {
                val type = if (itemType == PurchasedListAdapter.TYPE_VIDEO_COURSE) PurchasedListAdapter.TYPE_VIDEO_COURSE else PurchasedListAdapter.TYPE_COURSE
                CourseIntentUtils.startStudyRecordActivity(mContext, courseRecord.id, type, isClickGroup)
            }else{
                //存在需要跳转H5页面地址
                IntentUtils.startLagouWebViewActivity(mContext,courseRecord.openWebToStudy)
            }
        }
    }
    /**
     * 大课点击
     */
    override fun onPurchasedBigCourseClick(bigCourseRecord: BigCourseRecord, itemType: Int, isClickGroup: Boolean) {
        when (bigCourseRecord.prepayProgress) {
            PurchasedNewBean.BIG_COURSE_STATUS_PAY_DEPOSIT -> ToastUtil.show(mContext, "请联系课程顾问支付尾款")
            PurchasedNewBean.BIG_COURSE_STATUS_PAY_TAIL -> ToastUtil.show(mContext, "请联系课程顾问加入班级")
            else -> {
                val isShowH5: Boolean = CourseSPUtils.getBigCourseShowH5()
                if (isShowH5) {
                    IntentUtils.startAudioWebViewActivity(mContext, bigCourseRecord.h5Url)
                } else {
                    if (!bigCourseRecord.minorCourse) {
                        if (bigCourseRecord.openWebToStudy.isNullOrEmpty()) {
                            CourseIntentUtils.startBigCourseActivity(mContext, bigCourseRecord.id, isClickGroup)
                        } else{
                            IntentUtils.startLagouWebViewActivity(mContext,bigCourseRecord.openWebToStudy)
                        }
                    } else  {
                        if (bigCourseRecord.openWebToStudy.isNullOrEmpty()) {
                            CourseIntentUtils.startSmallCourseActivity(mContext, bigCourseRecord.id, isClickGroup)
                        } else {
                            IntentUtils.startLagouWebViewActivity(mContext,bigCourseRecord.openWebToStudy)
                        }
                    }
                }
            }
        }
    }
}

