package com.alpha.lagouapk.ui.dialog

import android.os.Bundle
import android.text.TextUtils
import androidx.fragment.app.FragmentManager
import com.alpha.lagouapk.LagouApplication
import com.alpha.lagouapk.R
import com.alpha.lagouapk.tools.IntentUtils
import com.alpha.lagouapk.tools.NotificationUtils
import com.alpha.lagouapk.tools.SPUtils
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.commonlibs.ui.dialog.BaseDialogFragment
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.utils.StringUtil
import kotlinx.android.synthetic.main.dialog_open_push_other_home.image_notification_close
import kotlinx.android.synthetic.main.dialog_open_push_other_home.tv_example_txt
import kotlinx.android.synthetic.main.dialog_open_push_other_home.tv_open_notification_click

/**
 * @date： Crate by wangjunwei on 2023/10/16
 * @description：提示开启push弹窗，除了首页，涉及：JD、公司主页、HR主页
 **/
class NoticeOpenOtherDialog : BaseDialogFragment() {

    private var companyName: String? = null
    private var positionName: String? = null
    private var scene: String? = null

    companion object {
        const val KEY_SCENE_TYPE = "scene_type"

        const val POSITION_CO_MESSAGE_OPEN = "position_co_message_open"//收藏职位
        const val COMPANY_CO_MESSAGE_OPEN = "company_co_message_open"//关注公司

        const val KEY_COMPANY_NAME = "company_name"
        const val KEY_POSITION_NAME = "position_name"

        @JvmStatic
        fun getInstance(scene: String, companyName: String?, positionName: String?): NoticeOpenOtherDialog{
            val bundle = Bundle()
            bundle.putString(KEY_SCENE_TYPE, scene)
            bundle.putString(KEY_COMPANY_NAME, companyName)
            bundle.putString(KEY_POSITION_NAME, positionName)
            val noticeOpenOtherDialog = NoticeOpenOtherDialog()
            noticeOpenOtherDialog.arguments = bundle
            return noticeOpenOtherDialog
        }
    }

    override fun getLayoutId(): Int {
      return R.layout.dialog_open_push_other_home
    }

    override fun initConfigAtCreate() {
        super.initConfigAtCreate()
        scene = arguments?.getString(KEY_SCENE_TYPE)
        companyName = arguments?.getString(KEY_COMPANY_NAME)
        positionName = arguments?.getString(KEY_POSITION_NAME)
        AnalyticHelper.getInstance().sendExploreInfo(if (isJDScene()) "p1uee" else "q1uef")
    }

    override fun fillUI() {
        super.fillUI()
        //1 设置示例文案
        //1.1 公司名称、职位名称都为空
        var showMajor = "你收藏的公司，新发布了你的期望职位，此刻HR在线！3秒前查看了你的信息，现在投递抢占先机！"
        //1.2 公司名称不为空，职位名称不为空
        if (!TextUtils.isEmpty(companyName) && !TextUtils.isEmpty(positionName)) {
            //公司主页的场景
            showMajor = ResourceUtils.getString(
                R.string.open_push_collect_complete,
                StringUtil.getBeyondDot(companyName, 6),
                StringUtil.getBeyondDot(positionName, 8)
            )
        }else if (!TextUtils.isEmpty(positionName) && TextUtils.isEmpty(companyName)){
            //JD页的场景
            showMajor = ResourceUtils.getString(
                R.string.open_push_collect_position, positionName)
        }
        tv_example_txt.text = showMajor
    }

    override fun initListener() {
        super.initListener()
        //1 关闭弹窗
        image_notification_close.setOnClickListener {
            dismiss()
        }
        //2 去开启逻辑
        tv_open_notification_click.setOnClickListener {
            if (!NotificationUtils.isNotificationEnabled(LagouApplication.getLagouApplication())){
                //系统设置关闭&&APP开启--跳转系统设置
                IntentUtils.startPushSettingPage(activity)
            }else{
                //非设置页面的弹窗都需要先跳转至设置页面
                IntentUtils.startPushSet(activity, scene)
            }
            dismiss()
            AnalyticHelper.getInstance().sendClickInfo(if (isJDScene()) "p1uee" else "q1uef",
                if (isJDScene()) "t1uei" else "u1uej")
        }
    }

    private fun isJDScene():Boolean {
        return TextUtils.equals(scene, POSITION_CO_MESSAGE_OPEN)
    }

    private fun isCompanyScene():Boolean {
        return TextUtils.equals(scene, COMPANY_CO_MESSAGE_OPEN)
    }

    override fun getWidth(): Int {
        return (0.8 * screenWidth).toInt()
    }
}