package com.alpha.lagouapk.ui.dialog
import android.app.Activity
import android.content.Context
import android.os.Bundle
import android.text.TextUtils
import android.view.Gravity
import android.view.View
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import com.alpha.lagouapk.R
import com.lagou.commonlibs.ui.dialog.BaseBottomSheetDialogFragment
import com.lagou.commonlibs.utils.ResourceUtils

/**
 * 关闭职位弹窗
 */
class PositionFeedbackDialog:BaseBottomSheetDialogFragment(),View.OnClickListener {
    private val dp_50 = ResourceUtils.getDimenSize(R.dimen.dp_50)
    private val dp_15 = ResourceUtils.getDimenSize(R.dimen.dp_15)
    private val dp_05 = ResourceUtils.getDimenSize(R.dimen.dp_05)
    private var positionName:String? = null
    private var reasonArray = arrayOf("行业不匹配","年限不匹配","学历不匹配","薪资低","沟通过/面试过/入职过","不想看这个公司/品牌","其它原因")
    private var reasonTypeArray = arrayOf("INDUSTRY_NOT_MATCH","WORK_YEAR_NOT_MATCH","EDUCATION_NOT_MATCH","SALARY_TOO_LOW","EVER_CONTACT","NOT_INTEREST_COMPANY","OTHER")
    private var mOptionContainer:LinearLayout? = null
    private var ivClose:ImageView? = null
    private var onItemClick:((String)->Unit)? = null
    private var mScene:Int = SCENE_HOME
    companion object {
        const val KEY_POSITION_NAME = "key_position_name"
        const val KEY_SCENE = "key_scene"
        const val SCENE_SEARCH = 1
        const val SCENE_HOME = 2
        @JvmStatic
        fun getInstance(scene:Int,positionName:String,onItemClick:((String)->Unit)?): PositionFeedbackDialog {
            val bundle = Bundle()
            bundle.putString(KEY_POSITION_NAME,positionName)
            bundle.putInt(KEY_SCENE,scene)
            val dialog = PositionFeedbackDialog()
            dialog.setOnItemClick(onItemClick)
            dialog.arguments = bundle
            return dialog
        }
    }

    private fun setOnItemClick(onItemClick: ((String) -> Unit)?) {
        this.onItemClick = onItemClick
    }

    override fun initView(view: View?) {
        super.initView(view)
        mOptionContainer = view?.findViewById(R.id.llOptionContainer)
        ivClose = view?.findViewById(R.id.ivFeedbackClose)
        ivClose?.setOnClickListener(this)
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        positionName = arguments?.getString(KEY_POSITION_NAME)
        mScene = arguments?.getInt(KEY_SCENE) ?: SCENE_HOME
    }

    override fun getLayoutId(): Int = R.layout.dialog_position_feedback

    override fun fillUI() {
        super.fillUI()
        addOption()
    }


    override fun onClick(v: View?) {
        super.onClick(v)
        when(v?.id){
            R.id.ivFeedbackClose -> dismiss()
        }
    }

    private fun addOption() {
        mOptionContainer?.let {
            var firstOption:String? = null
            if (mScene == SCENE_HOME){
                firstOption = if (positionName.isNullOrEmpty()) null else "不想找".plus("“").plus(positionName).plus("”")
            }else if (mScene == SCENE_SEARCH){
                firstOption = if (positionName.isNullOrEmpty()) null else "与".plus("“").plus(positionName).plus("”").plus("不相关")
            }
            if (firstOption.isNullOrEmpty().not()){
                val itemView = generateOptionView(firstOption)
                mOptionContainer!!.addView(itemView)
                mOptionContainer!!.addView(generateDividerView())
                itemView.setOnClickListener {
                    onItemClick?.invoke(if (mScene == SCENE_HOME) "NOT_INTEREST_JOG" else "KEYWORD_NOT_MATCH")
                    dismiss()
                }
            }
            val arrayLength = reasonArray.size
            reasonArray.forEachIndexed { index:Int, option:String ->
                val itemView = generateOptionView(option)
                mOptionContainer!!.addView(itemView)
                if (index < arrayLength - 1){
                    mOptionContainer!!.addView(generateDividerView())
                }
                itemView.setOnClickListener {
                    onItemClick?.invoke(reasonTypeArray[index])
                    dismiss()
                }
            }
        }

    }

    private fun generateOptionView(reason:String?):TextView{
        val textView = TextView(context).apply {
            val layoutParams = LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT,dp_50)
            layoutParams.leftMargin = dp_15
            layoutParams.rightMargin = dp_15
            this.layoutParams = layoutParams
            gravity = Gravity.CENTER_VERTICAL
            textSize = 14f
            setTextColor(ResourceUtils.getColor(R.color.color_666666))
            maxLines = 1
            ellipsize = TextUtils.TruncateAt.MIDDLE
            text = reason
        }
        return textView
    }

    private fun generateDividerView():View{
        return View(context).apply {
            this.layoutParams = LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT,dp_05)
            setBackgroundColor(ResourceUtils.getColor(R.color.color_e8e9eb))
        }
    }

    override fun isDraggable() = false
}