package com.alpha.lagouapk.ui.home

import android.content.Context
import android.graphics.drawable.GradientDrawable
import android.text.TextUtils
import android.util.AttributeSet
import android.util.TypedValue
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.widget.FrameLayout
import android.widget.LinearLayout
import android.widget.TextView
import com.alpha.lagouapk.LagouApplication
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.HomeWorkStatusBean
import com.alpha.lagouapk.ui.home.v2.JobStatusDialogPresenter
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.bean.json.ClickInfo
import com.lagou.commonlibs.utils.ListUtil
import com.lagou.commonlibs.utils.ResourceUtils
import kotlinx.android.synthetic.main.item_home_work_status_view.view.*

/**
 * @description ：首页选择当前求职状态的卡片
 * Created by @author YongliangWang on 2021/10/19 14:26
 */
class HomeWorkStatusView(context: Context, attrs: AttributeSet?) : FrameLayout(context, attrs) {
    init {
        LayoutInflater.from(context).inflate(R.layout.item_home_work_status_view, this)
    }

    fun setData(bean: HomeWorkStatusBean?) {
        bean?.let { it ->
            if ((!it.isShowCard) || ListUtil.isEmpty(it.statusList)) {
                visibility = View.GONE
                return
            }
            visibility = View.VISIBLE
            tv_status_title.text = it.title
            tv_status_sub_title.text = it.subtitle

            ll_status_container.apply {
                removeAllViews()
                it.statusList.apply {
                    filterNotNull()
                    //添加View
                    for ((index, e) in withIndex()) {
                        generateItemView(e, index == size - 1)?.apply { addView(this) }
                    }
                }
            }
        }
    }

    private fun generateItemView(bean: HomeWorkStatusBean.StatusListBean?, isLast: Boolean): TextView? {
        bean?.let {
            val textView = TextView(context)
            with(textView) {
                text = it.btnText
                setTextSize(TypedValue.COMPLEX_UNIT_SP, 12f)
                setTextColor(ResourceUtils.getColor(R.color.color_333333))
                setPadding(33.toDx(), 6.toDx(), 33.toDx(), 6.toDx())
                maxLines = 1
                gravity = Gravity.CENTER
                ellipsize = TextUtils.TruncateAt.END

                background = GradientDrawable().apply {
                    shape = GradientDrawable.RECTANGLE
                    cornerRadius = 2.toDx().toFloat()
                    setColor(ResourceUtils.getColor(R.color.color_ffffff))
                }

                layoutParams = LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, LinearLayout.LayoutParams.WRAP_CONTENT).apply {
                    marginEnd = if (isLast) 0 else 6.toDx()
                }

                setOnClickListener {
                    //只修改求职状态 ，不修改 到岗时间
                    JobStatusDialogPresenter.getInstance().updateStatusArrivalHome(bean.btnText, JobStatusDialogPresenter.onUpdateStatusArrivalTimeCallBack {
                        this@HomeWorkStatusView.visibility = View.GONE

                        //埋点上报
                        val info = ClickInfo().setAddress_id("y1t95").setPage_id("ea00")
                        bean.params?.forEach { info.addClick_props(it.key, it.value) }
                        AnalyticHelper.getInstance().sendClickInfo(info)
                    })
                }
            }
            return textView
        }
        return null
    }
}

fun Int.toDx(): Int = this.apply {
    val scale = LagouApplication.getLagouApplication().resources.displayMetrics.density
    return ((this * scale + 0.5f).toInt())
}

fun Float.toDx(): Int {
    val scale = LagouApplication.getLagouApplication().resources.displayMetrics.density
    return (this * scale + 0.5f).toInt()
}
