package com.alpha.lagouapk.ui.home.choice

import android.app.Activity
import android.content.Intent
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import android.text.TextUtils
import android.view.View
import com.ApiService
import com.alpha.lagouapk.Interface.SearchFilterListener
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.*
import com.alpha.lagouapk.manager.ResumeManager
import com.alpha.lagouapk.tools.IntentUtils
import com.alpha.lagouapk.ui.home.choice.SchoolRecruitmentActivity.Companion.ROUTER_SCHOOL_RECRUITMENT
import com.alpha.lagouapk.ui.home.choice.SchoolRecruitmentActivity.Companion.ROUTER_SHIXI_RECRUITMENT
import com.alpha.lagouapk.ui.position.SearchNetPase
import com.alpha.lagouapk.ui.position.filter.ExposurePositionFilterActivity
import com.alpha.lagouapk.ui.position.filter.SchoolRecruitmentAdapter
import com.alpha.lagouapk.ui.search.SearchPositionFragment
import com.alpha.lagouapk.widget.searchview.FilterWindow
import com.lagou.annotation.apt.RouterRule
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.bean.json.ClickInfo
import com.lagou.baselib.analytic.definition.c.AID
import com.lagou.baselib.analytic.definition.c.PageId
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.baselib.util.BarUtil
import com.lagou.commonlibs.utils.DateUtils
import com.lagou.commonlibs.web.router.RouteUtils
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.android.synthetic.main.activity_net_error_layout.*
import kotlinx.android.synthetic.main.activity_school_recruitment.*

/**
 * 校招
 *
 * @author :  Alfred
 * @date   : 2019-07-22 15:04
 *
 */
@RouterRule(ROUTER_SCHOOL_RECRUITMENT, ROUTER_SHIXI_RECRUITMENT)
class SchoolRecruitmentActivity : BaseActivity(), SearchFilterListener {
    private var mAdapter: SchoolRecruitmentAdapter? = null
    private val mPositionList = mutableListOf<PositionInfo>()
    private val mSearchBean = SearchBean()
    private var mShowId: String? = null
    private var mPageNo = 1
    private lateinit var mLinearLayoutManager: LinearLayoutManager

    private var isCitySelected: Boolean = false

    override fun getContentLayoutId(): Int {
        return R.layout.activity_school_recruitment
    }

    override fun getStatusBarColor(): Int {
        return R.color.color_f8f9fa
    }

    private var mTitleStr = ""
    private var mSubtitle = ""
    var jobNature = ArrayList<String>()
    var isShool: Boolean = true // 是否是校招职位

    override fun init() {
        val routeUrl = RouteUtils.getRouteUrl(intent)
        if (ROUTER_SCHOOL_RECRUITMENT.equals(routeUrl)) {
            // 校招
            mTitleStr = PositionRecType.SCHOOL_RECRUITMENT.title
            mSubtitle = PositionRecType.SCHOOL_RECRUITMENT.describe
            jobNature.add("全职")
            jobNature.add("兼职")
            jobNature.add("实习")
            mSearchBean.isSchoolJob = true
        } else if (ROUTER_SHIXI_RECRUITMENT.equals(routeUrl)) {
            // 实习
            isShool = false
            mTitleStr = PositionRecType.SHIXI_POSITION.title
            mSubtitle = PositionRecType.SHIXI_POSITION.describe

            // 需求：实习职位，筛选条件中工作性质 只有 "兼职"、"实习"选项
            jobNature.add("兼职")
            jobNature.add("实习")

            // 默认选中的数据
            var defaultJobNature = ArrayList<String>()
            defaultJobNature.add("实习")
            mSearchBean.jobNature = defaultJobNature
        }


        mSearchBean.isShieldCompany = false
        mSearchBean.searchType = "position_school"

        swipeLayout.setColorSchemeResources(R.color.theme_text_color)
        swipeLayout.setDistanceToTriggerSync(400)

        recyclerView.layoutManager = LinearLayoutManager(mContext)
        mLinearLayoutManager = recyclerView.layoutManager as LinearLayoutManager

        titleBar.setCenterTvTxt(mTitleStr)
        titleBar.setLeftImgListener { finish() }
        ivBack.setOnClickListener { finish() }

        swipeLayout.setOnRefreshListener {
            resetSearch()
            requestData(false)
        }
        recyclerView.setOnLoadMoreListener {
            mPageNo++
            requestData(false)
        }
        recyclerView.addOnScrollListener(object : RecyclerView.OnScrollListener() {

            override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
                super.onScrolled(recyclerView, dx, dy)
                when (mLinearLayoutManager.findFirstVisibleItemPosition()) {
                    0 -> {
                        BarUtil.statusBarColor(this@SchoolRecruitmentActivity, R.color.color_f8f9fa)
                        titleBar.visibility = View.GONE
                        ivBack.visibility = View.VISIBLE
                    }
                    else -> {
                        BarUtil.statusBarColor(this@SchoolRecruitmentActivity, R.color.white)
                        titleBar.visibility = View.VISIBLE
                        ivBack.visibility = View.GONE
                    }
                }
                if (dy < 0 && mLinearLayoutManager.findFirstVisibleItemPosition() == 0) {
                    //下滑
                    searchLayout.visibility = View.GONE
                } else if (dy > 0 && mLinearLayoutManager.findFirstVisibleItemPosition() > 0) {
                    //上滑
                    searchLayout.visibility = View.VISIBLE
                }
            }

            override fun onScrollStateChanged(recyclerView: RecyclerView, newState: Int) {
                super.onScrollStateChanged(recyclerView, newState)
                if (mShouldScroll) {
                    mShouldScroll = false
                    smoothMoveToPosition(recyclerView, 1)
                }
            }
        })

        weak_network_btn.setOnClickListener { initData() }

        initData()
    }

    private fun initData() {
        swipeLayout.isRefreshing = true

        if (ResumeManager.getInstance().resumeBean.expectJob == null) {
            requestExpectInfo()
        } else {
            val expectJob = ResumeManager.getInstance().resumeBean.expectJob
            initExpectInfo(expectJob.expectPosition, expectJob.expectCity)

            requestData(false)
        }
    }

    private fun requestExpectInfo() {
        ApiService.api.expectJobInfo.enqueue(object : BaseCallback<ExpectJobBean>() {

            override fun onSuccess(request: HttpRequest<*>, response: HttpResponse<ExpectJobBean>) {
                if (response.content != null) {
                    networkErrorStatus(false)
                    val expectJobBean = response.content
                    initExpectInfo(expectJobBean.expectPosition, expectJobBean.expectCity)
                    requestData(false)
                } else {
                    networkErrorStatus(true)
                }
            }

            override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<ExpectJobBean>?) {
                super.onFailure(request, response)
                swipeLayout.isRefreshing = false
                networkErrorStatus(true)
            }
        })
    }

    private fun initExpectInfo(expectPosition: String, expectCity: String) {
        mSearchBean.keyword = expectPosition
        val addressBean = SearchBean.Address()
        val cityBean = SearchBean.City()
        addressBean.city = cityBean
        mSearchBean.setmSelectedAddress(addressBean)
        cityBean.cityName = expectCity

        initView()
    }

    private fun initView() {
        searchLayout.setCityTabName(mSearchBean.getmSelectedAddress()?.city?.cityName)
        searchLayout.positionText.text = "职位"
        // 初始化显示职位筛选卡片中，工作性质标签内容
        searchLayout.getmFilterWindow().positionWindow.setJobTypeValue(jobNature)

        var type = ExposurePositionFilterActivity.TYPE_SCHOOL_RECRUITMENT
        if (!isShool) {
            type = ExposurePositionFilterActivity.TYPE_SHIXI_RECRUITMENT
        }
        mAdapter = SchoolRecruitmentAdapter(this, mPositionList, type, mSearchBean)
        mAdapter?.setHeaderContent(mTitleStr, mSubtitle)
        recyclerView.adapter = mAdapter
        mAdapter?.setOnSearchLayoutItemClickListener {
            smoothMoveToPosition(recyclerView, 1)
            when (it) {
                0 -> searchLayout.cityWrapper.performClick()
                1 -> searchLayout.positionWrapper.performClick()
                2 -> searchLayout.companyWrapper.performClick()
                3 -> searchLayout.sortWrapper.performClick()
            }
        }
        mAdapter?.setOnPositionClickAnalytic {
            val clickInfo = ClickInfo().setAddress_id(if(isShool) AID._1myr else "1p1g").setShowId(mShowId).setPage_id(pageId).setContent_id(it.toString())
            AnalyticHelper.getInstance().sendClickInfo(clickInfo)
        }

//        searchLayout.setIsCityFilter(true)
        searchLayout.setListener(this)
        searchLayout.getmFilterWindow().companyWindow.setShowShiledCompany(true)
        searchLayout.setData(mSearchBean)
        searchLayout.cityWrapper.setOnClickListener {
            IntentUtils.startCityListActivity(mContext, true,false)
        }
    }

    private fun requestData(isShowLoading: Boolean) {
        searchLayout.closeFilerWindow()
        if (isShowLoading) {
            showLoadingView()
        }
        ApiService.api.simpleSearchPosition(SearchNetPase.parseSearchReqBean(mPageNo, SearchPositionFragment.SEARCH_POSITION_PAGE_SIZE, mShowId,false, mSearchBean)).enqueue(object : BaseCallback<SearchPositionBean>() {
            override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<SearchPositionBean>) {
                hideLoadingView()
                swipeLayout.isRefreshing = false

                val bean = response.content
                if (bean != null && bean.positionCardVos != null) {
                    networkErrorStatus(false)
                    mShowId = bean.showId
                    mAdapter?.setShowId(mShowId)
                    setData(convertPositionInfo(bean.positionCardVos))
                } else {
                    networkErrorStatus(true)
                }
            }

            override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<SearchPositionBean>?) {
                super.onFailure(request, response)
                hideLoadingView()
                swipeLayout.isRefreshing = false
                networkErrorStatus(true)

            }
        })
    }

    private fun setData(list: MutableList<PositionInfo>) {
        if (mPageNo == 1) {
            mPositionList.clear()
        }
        setBottomView(list.size)
        mPositionList.addAll(list)

        // 初始化首次进入时，列表中的筛选bar选中状态
        mAdapter?.setTabItemTextSelected(isCitySelected, searchLayout.positionText.isSelected, searchLayout.companyText.isSelected, searchLayout.sortText.isSelected)

        mAdapter?.notifyDataSetChanged()
    }

    private fun networkErrorStatus(isShow: Boolean) {
        if ((isShow && weak_network_layout.visibility == View.VISIBLE) || (!isShow && weak_network_layout.visibility != View.VISIBLE)) {
            return
        }
        if (isShow) {
            weak_network_layout.visibility = View.VISIBLE
            contentView.visibility = View.GONE
            titleBar.visibility = View.VISIBLE
            ivBack.visibility = View.GONE
        } else {
            weak_network_layout.visibility = View.GONE
            contentView.visibility = View.VISIBLE
            titleBar.visibility = View.GONE
            ivBack.visibility = View.VISIBLE
        }
    }

    private fun resetSearch() {
        mPageNo = 1
        mShowId = null
        recyclerView.reset()
        recyclerView.scrollToPosition(0)
        searchLayout.visibility = View.GONE
    }

    /**
     * 设置底部下拉加载更多footView
     *
     * @param size
     */
    private fun setBottomView(size: Int) {
        recyclerView.completeLoadMore()
        if (size < SearchPositionFragment.SEARCH_POSITION_PAGE_SIZE) {
            recyclerView.setNoMore()
            if (mPageNo == 1) {
                recyclerView.completeNoBottom()
            }

        }
    }


    /** 目标项是否在最后一个可见项之后 */
    private var mShouldScroll: Boolean = false
    /** 记录目标项位置 */
    private var mToPosition: Int = 0

    /**
     * 滑动到指定位置
     * @param mRecyclerView
     * @param position
     */
    private fun smoothMoveToPosition(mRecyclerView: RecyclerView, position: Int) {
        // 第一个可见位置
        val firstItem = mRecyclerView.getChildLayoutPosition(mRecyclerView.getChildAt(0))
        // 最后一个可见位置
        val lastItem = mRecyclerView.getChildLayoutPosition(mRecyclerView.getChildAt(mRecyclerView.childCount - 1))

        if (position < firstItem) {
            // 如果跳转位置在第一个可见位置之前，就smoothScrollToPosition可以直接跳转
            mRecyclerView.smoothScrollToPosition(position)
        } else if (position <= lastItem) {
            // 跳转位置在第一个可见项之后，最后一个可见项之前
            // smoothScrollToPosition根本不会动，此时调用smoothScrollBy来滑动到指定位置
            val movePosition = position - firstItem
            if (movePosition >= 0 && movePosition < mRecyclerView.childCount) {
                val top = mRecyclerView.getChildAt(movePosition).top
                mRecyclerView.smoothScrollBy(0, top)
            }
        } else {
            // 如果要跳转的位置在最后可见项之后，则先调用smoothScrollToPosition将要跳转的位置滚动到可见位置
            // 再通过onScrollStateChanged控制再次调用smoothMoveToPosition，执行上一个判断中的方法
            mRecyclerView.smoothScrollToPosition(position)
            mToPosition = position
            mShouldScroll = true
        }
    }


    private fun convertPositionInfo(list: List<PositionInfoBean>): MutableList<PositionInfo> {
        val positionInfoList = mutableListOf<PositionInfo>()
        for (positionInfoBean in list) {
            val positionInfo = PositionInfo()
            positionInfo.pd_businessZone = positionInfoBean.businessZone
            positionInfo.pd_city = positionInfoBean.city
            positionInfo.pd_companyId = positionInfoBean.companyId
            positionInfo.pd_companyLogo_uri = positionInfoBean.companyLogo
            positionInfo.pd_companyName = positionInfoBean.companyName
            positionInfo.pd_companySize = positionInfoBean.companySize

            try {
                positionInfo.pd_createTime = DateUtils.dateTimeProcess(positionInfoBean.createTime.toLong())
            } catch (e: Exception) {
                e.printStackTrace()
            }
            positionInfo.pd_district = positionInfoBean.district
            positionInfo.pd_education = positionInfoBean.education
            positionInfo.pd_financeStage = positionInfoBean.financeStage
            positionInfo.pd_hasDeliverResume = if (positionInfoBean.isHasDeliverResume) 1 else -1
            positionInfo.pd_industryField = positionInfoBean.industryField
            positionInfo.pd_positionId = positionInfoBean.positionId
            positionInfo.pd_positionLabels = positionInfoBean.positionLabels
            positionInfo.pd_positionName = positionInfoBean.positionName
            positionInfo.pd_salary = positionInfoBean.salary
            positionInfo.pd_salaryMonth = positionInfoBean.salaryMonth
            positionInfo.pd_workYear = positionInfoBean.workYear
            positionInfo.expectJobId=positionInfoBean.expectJobId
            positionInfoList.add(positionInfo)
        }
        return positionInfoList
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == IntentUtils.REQUEST_CODE_CITIES && resultCode == Activity.RESULT_OK && data != null) {
            val city = data.getStringExtra("city")
            if (!TextUtils.isEmpty(city) && !TextUtils.equals(city, searchLayout.cityTabName.text.toString())) {
                mSearchBean.getmSelectedAddress().city.cityName = city
                searchLayout.cityTabName.setTextColor(ContextCompat.getColor(this, R.color.theme_text_color))
                searchLayout.visibility = View.GONE
                searchLayout.setCityTabName(city)
                isCitySelected = true
                searchLayout.cityTabName.isSelected = isCitySelected
                //修改了城市，重新请求该城市的置顶结果
                onFilterLayoutToSearch(null)
            }
        }
    }

    override fun onFilterLayoutToSearch(type:FilterWindow.Type?) {
        mAdapter?.setTabItemTextSelected(isCitySelected, searchLayout.positionText.isSelected, searchLayout.companyText.isSelected, searchLayout.sortText.isSelected)
        resetSearch()
        requestData(false)
    }

    override fun onCancel() {
    }

    override fun onBackPressed() {
        if (searchLayout.isShowFiler) {
            searchLayout.closeFilerWindow()
        } else {
            super.onBackPressed()
        }
    }

    override fun getPageId(): String {
        if (isShool) {
            // 校招
            return PageId._1myq
        } else {
            // 实习
            return "1p1f"
        }
    }

    companion object {
        /**
         * 校招
         */
        const val ROUTER_SCHOOL_RECRUITMENT = "lagou://lagou.com/positionlist/schoolRecruit"
        /**
         * 实习
         */
        const val ROUTER_SHIXI_RECRUITMENT = "lagou://lagou.com/positionlist/practiceRecruit"
    }

}