package com.alpha.lagouapk.ui.home.chushai

import android.animation.Animator
import android.animation.AnimatorSet
import android.animation.ObjectAnimator
import android.content.Context
import android.content.DialogInterface
import android.os.Bundle
import android.view.View
import android.view.WindowManager
import androidx.fragment.app.DialogFragment
import com.ApiService
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.ChuShaiPositionCardBean
import com.alpha.lagouapk.bean.ChuShaiPositionRes
import com.alpha.lagouapk.ui.home.dialog.CHomeDialogManager
import com.common.utils.ViewUtils
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.bean.json.ClickInfo
import com.lagou.baselib.analytic.bean.json.ShowLogInfo
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.commonlibs.utils.ListUtil
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.component.ui.dialog.LgBaseDialog
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.android.synthetic.main.dialog_chushai_main.*

/**
 * 初筛宝弹窗
 */
class ChuShaiDialog :LgBaseDialog(), View.OnClickListener{
    private var mCurrentShowIndex:Int = 0
    private var mPositionList = mutableListOf<ChuShaiPositionCardBean>()
    private var mPageNo:Int = 1
    private var mShowId:String? = null
    private var mHasNextPage:Boolean = true
    private var mInitCardNum:Int = 0
    companion object{
        //感兴趣的职位列表
        const val KEY_INTEREST_POSITION_LIST = "key_interest_position_list"
        const val KEY_INTEREST_POSITION_SHOW_ID = "key_interest_position_show_id"
        const val KEY_INTEREST_INIT_PAGE_NO = "key_interest_init_page_no"
        const val PAGE_SIZE = 10
        const val ANIM_DURATION = 300L
        var TRANSLATE_X = ResourceUtils.getDimenSize(R.dimen.dp_600)
        var TRANSLATE_Y = ResourceUtils.getDimenSize(R.dimen.dp_310)
        @JvmStatic
        fun getInstance(positionList:ArrayList<ChuShaiPositionCardBean>?,showId:String?,pageNo:Int): ChuShaiDialog {
            val bundle = Bundle()
            bundle.putParcelableArrayList(KEY_INTEREST_POSITION_LIST,positionList)
            bundle.putString(KEY_INTEREST_POSITION_SHOW_ID,showId)
            bundle.putInt(KEY_INTEREST_INIT_PAGE_NO,pageNo)
            val dialog = ChuShaiDialog()
            dialog.arguments = bundle
            return dialog
        }
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        mShowId = arguments?.getString(KEY_INTEREST_POSITION_SHOW_ID)
        mPositionList = arguments?.getParcelableArrayList<ChuShaiPositionCardBean>(KEY_INTEREST_POSITION_LIST) ?: mutableListOf<ChuShaiPositionCardBean>()
        mPageNo = arguments?.getInt(KEY_INTEREST_INIT_PAGE_NO,1) ?: 1
        mInitCardNum = mPositionList.size
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setStyle(DialogFragment.STYLE_NORMAL, R.style.dialog_full_screen)
    }

    override fun getLayoutId() = R.layout.dialog_chushai_main
    override fun initView(view: View) {
        super.initView(view)
        ViewUtils.setViewsClick(this,mTvNoInterest,mLlInterest,mIvClose)
        dialog?.window?.decorView?.systemUiVisibility = View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN or View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION or View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR
    }

    override fun fillUI() {
        context?.let {
            frontCard.setData(mPositionList[mCurrentShowIndex])
            backCard.setData(nextPosition())
            analyticCardShow(mPositionList[mCurrentShowIndex].positionId.toLong())
        }

    }

    override fun isCanceledOnTouchOutside() = false

    override fun getWidth() = WindowManager.LayoutParams.MATCH_PARENT
    override fun getHeight(): Int {
        return WindowManager.LayoutParams.MATCH_PARENT
    }

    /**
     * 检查是否是最后一个卡片
     */
    private fun checkIsLastCard() = mCurrentShowIndex == mPositionList.size - 1

    private fun nextPosition():ChuShaiPositionCardBean?{
        val nextIndex = mCurrentShowIndex + 1
        if (nextIndex >= mPositionList.size){
            return null
        }
        return mPositionList[nextIndex]
    }

    /**
     * 点击感兴趣or不感兴趣
     */
    private fun afterClickInterestOrNot(isInterest:Boolean){
        if(checkIsLastCard()) {
            dismiss()
        }else{
            //检查是否需要进行下一批的请求
            checkAndRequestNextPage()
            //执行感兴趣或不感兴趣动画
            startAnim(isInterest)
        }
    }

    /**
     * 执行感兴趣or不感兴趣动画
     */
    private fun startAnim(isInterest: Boolean) {
        //动画执行期间不响应点击事件
        setViewEnable(false)

        val frontCard:ChuShaiCard? = flChuShaiCardContainer.getChildAt(1) as ChuShaiCard
        val translateX = ObjectAnimator.ofFloat(frontCard,"translationX",0f,if (isInterest) TRANSLATE_X.toFloat() else -TRANSLATE_X.toFloat())
        translateX.duration = ANIM_DURATION
        val translateY = ObjectAnimator.ofFloat(frontCard,"translationY",0f,-TRANSLATE_Y.toFloat())
        translateY.duration = ANIM_DURATION
        val rotation = ObjectAnimator.ofFloat(frontCard,"rotation",0f,if (isInterest) 45f else -45f)
        rotation.duration = ANIM_DURATION
        val fadeOut = ObjectAnimator.ofFloat(frontCard,"alpha",1f,0f)
        fadeOut.duration = ANIM_DURATION

        val animSet = AnimatorSet()
        animSet.playTogether(translateX,translateY,rotation,fadeOut)
        animSet.start()
        animSet.addListener(object:Animator.AnimatorListener{
            override fun onAnimationStart(animation: Animator?) {
            }

            override fun onAnimationEnd(animation: Animator?) {
                if (flChuShaiCardContainer == null){
                    return
                }
                setViewEnable(true)
                resetAnim(frontCard)
                mCurrentShowIndex++
                flChuShaiCardContainer.removeView(frontCard)
                val nextPosition = nextPosition()
                frontCard?.setData(nextPosition)
                flChuShaiCardContainer.addView(frontCard,0)
                val frontCard = flChuShaiCardContainer.getChildAt(1) as? ChuShaiCard
                analyticCardShow(frontCard?.getPositionId()?.toLong() ?: 0L)
            }

            override fun onAnimationCancel(animation: Animator?) {
                setViewEnable(true)
            }

            override fun onAnimationRepeat(animation: Animator?) {
            }

        })
    }

    private fun resetAnim(view:View?){
        view?.translationX = 0f
        view?.translationY = 0f
        view?.alpha = 1f
        view?.rotation = 0f
    }

    private fun setViewEnable(isEnable: Boolean){
        mLlInterest?.isEnabled = isEnable
        mTvNoInterest?.isEnabled = isEnable
    }

    private fun checkAndRequestNextPage(){
        if (mPositionList.size - (mCurrentShowIndex + 1) == 3 && mHasNextPage){
            //还剩三个卡片时，开始请求下一页数据
            ApiService.api.fetchInterestPosition(++mPageNo, PAGE_SIZE,mShowId).enqueue(object: BaseCallback<ChuShaiPositionRes?>(false){
                override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<ChuShaiPositionRes?>?) {
                    val positions = response?.content?.positionList
                    mHasNextPage = ListUtil.size(positions) >= PAGE_SIZE
                    response?.content?.let {
                        mPageNo = it.pageNo
                    }
                    positions?.let {
                        mPositionList.addAll(it)
                    }
                }
            })
        }
    }

    override fun onClick(v: View?) {
        val positionId = mPositionList.getOrNull(mCurrentShowIndex)?.positionId ?: 0
        when(v?.id){
            R.id.mTvNoInterest ->{
                ApiService.api.markPositionNoInterest(positionId).enqueue(object:BaseCallback<Any?>(){
                    override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<Any?>?) {}
                })
                afterClickInterestOrNot(false)
                AnalyticHelper.getInstance().sendClickInfo(ClickInfo().setPage_id("ea00").setAddress_id("t1ucc").setContent_id(positionId))
            }
            R.id.mLlInterest ->{
                ApiService.api.markPositionInterest(mPositionList.getOrNull(mCurrentShowIndex)?.positionId ?: 0).enqueue(object:BaseCallback<Any?>(){
                    override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<Any?>?) {}

                })
                afterClickInterestOrNot(true)
                AnalyticHelper.getInstance().sendClickInfo(ClickInfo().setPage_id("ea00").setAddress_id("u1ucd").setContent_id(positionId))
            }
            R.id.mIvClose ->{
                AnalyticHelper.getInstance().sendClickInfo(ClickInfo().setPage_id("ea00").setAddress_id("v1uce").setContent_id(positionId))
                dismiss()
            }
        }
    }

    override fun onDismiss(dialog: DialogInterface) {
        super.onDismiss(dialog)
        CHomeDialogManager.markShowState(CHomeDialogManager.FIRST_PASS_PRECIOUS,false)
    }

    private fun analyticCardShow(positionId:Long){
        val showLogInfo = ShowLogInfo().apply {
            setPage_Id("ea00")
            setAddress_Id("s1ucb")
            setContentId(positionId)
            addShowProp("show_num",mInitCardNum)
        }
        AnalyticHelper.getInstance().sendShowInfo(showLogInfo)
    }

}