package com.alpha.lagouapk.ui.home.promotion

import android.content.Context
import android.graphics.LinearGradient
import android.graphics.Shader
import android.graphics.drawable.GradientDrawable
import android.text.TextUtils
import android.util.AttributeSet
import android.util.TypedValue
import android.view.Gravity
import android.view.LayoutInflater
import android.view.ViewGroup
import android.widget.RelativeLayout
import android.widget.TextView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.PositionInfo
import com.lagou.commonlibs.imageload.ImageLoadUtils
import com.lagou.commonlibs.utils.DeviceUtils
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.lagouee.utils.DataServer
import kotlinx.android.synthetic.main.layout_promotion_position_hot_view.view.*

/**
 * @author 作者：ryan.lei
 * @date 创建时间：2022/1/7 20:26
 * @Description 描述：
 **/
class PromotionPositionHotView : RelativeLayout {
    private var viewWidth : Int = 0
    constructor(context: Context) : this(context, null)
    constructor(context: Context, attrs: AttributeSet?) : this(context, attrs, 0)
    constructor(context: Context, attrs: AttributeSet?, defStyleAttr: Int) : super(
        context,
        attrs,
        defStyleAttr
    ) {
        LayoutInflater.from(context).inflate(R.layout.layout_promotion_position_hot_view, this)
        viewWidth = DeviceUtils.getScreenWidth() / 2 - ResourceUtils.getDimenSize(R.dimen.dp_7)
    }

    fun setData(bean: PositionInfo?, isFirstHotPosition: Boolean) {
        if (bean == null) {
            return
        }
        context?.let {
            ivHotCardCompanyTab.visibility = if (isFirstHotPosition) VISIBLE else GONE
            ImageLoadUtils.with(it)
                .load(bean.pd_companyLogo_uri)
                .placeholder(R.drawable.occupy_logo)
                .error(R.drawable.default_bg_gray)
                .into(ivHotCardCompanyLogo)
            tvHotCardCompanyName.text = bean.pd_companyName
            tvHotCardCompanyFinance.text = bean.pd_companySize
            tvHotCardPositionName.text = bean.pd_positionName
            tvHotCardPositionSalary.text =
                DataServer.getSalaryText(bean.pd_salary, bean.pd_salaryMonth)
            if (!TextUtils.isEmpty(tvHotCardPositionSalary.text)) {
                val linearGradient = LinearGradient(
                    0f,
                    0f,
                    tvHotCardPositionSalary.paint.textSize * tvHotCardPositionSalary.text.length,
                    0f,
                    ResourceUtils.getColor(R.color.color_c78e64),
                    ResourceUtils.getColor(R.color.color_ad6c31),
                    Shader.TileMode.CLAMP
                )
                tvHotCardPositionSalary.paint.shader = linearGradient
                tvHotCardPositionSalary.invalidate()
            }
            flHotCardPositionLabel.labels = bean.pd_positionLabels
            if (bean.is51Position){
                flHotCardPositionLabel.addView(generateTagView(),0)
            }
            if (TextUtils.isEmpty(bean.pd_positionLocation)) {
                llHotCardLocation.visibility = GONE
            } else {
                llHotCardLocation.visibility = VISIBLE
                tvHotCardLocationDesc.text = bean.pd_positionLocation
            }
        }
    }
    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        super.onMeasure(MeasureSpec.makeMeasureSpec(viewWidth,MeasureSpec.EXACTLY),heightMeasureSpec)
    }

    private fun generateTagView():TextView{
        val textView = TextView(context).apply {
            layoutParams = ViewGroup.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT,ResourceUtils.getDimenSize(R.dimen.dp_16))
            setPadding(ResourceUtils.getDimenSize(R.dimen.dp_5),0,ResourceUtils.getDimenSize(R.dimen.dp_5),0)
            gravity = Gravity.CENTER
            text = ResourceUtils.getString(R.string._51)
            setTextSize(TypedValue.COMPLEX_UNIT_SP,10f)
            setTextColor(ResourceUtils.getColor(R.color.color_f28e45))

            val drawable = GradientDrawable()
            drawable.shape = GradientDrawable.RECTANGLE
            drawable.cornerRadius = ResourceUtils.getDimenSize(R.dimen.dp_2).toFloat()
            drawable.setStroke(ResourceUtils.getDimenSize(R.dimen.dp_05), ResourceUtils.getColor(R.color.color_f28e45))
            background = drawable
        }
        return textView
    }
}