package com.alpha.lagouapk.ui.interview.view

import android.app.Activity
import android.view.View
import com.ApiService
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.deliver.DeliverGetBean
import com.alpha.lagouapk.bean.interview.CCommentDetail
import com.alpha.lagouapk.observer.EventTags
import com.alpha.lagouapk.tools.DialogUtils
import com.alpha.lagouapk.tools.UserHeadUtils
import com.lagou.annotation.apt.RouterRule
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListenerImpl
import com.lagou.commonlibs.event.OkBus
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.utils.SafeTypeConvert
import com.lagou.lagouee.manager.net.InterviewNetManager
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.android.synthetic.main.activity_comment_detail.*

/**
 * @date 创建时间：2020/8/3
 * @author 作者：codeman
 * @Description  描述：面试评价详情
 * @version
 */
@RouterRule("lagou://lagou.com/commentDetail?orderId={orderId}")
class CCommentDetailActivity : BaseActivity() {

    private var orderId: Long? = null
    private var cCommentDetail: CCommentDetail? = null

    private val onCommentDetailListener =
        object : OnAdapterCommonListenerImpl<CCommentDetail, HttpResponse<CCommentDetail>>(this) {
            override fun onSuccess(commentDetail: CCommentDetail?) {
                cCommentDetail = commentDetail
                wifi_view.visibility = View.GONE
                nest_content.visibility = View.VISIBLE
                fillUi(commentDetail)
            }

            override fun onNoInternet() {
                super.onNoInternet()
                wifi_view.visibility = View.VISIBLE
                nest_content.visibility = View.GONE
            }
        }


    companion object {
        const val KEY_ORDER_ID = "key_order_id"
    }

    override fun getContentLayoutId(): Int {
        return R.layout.activity_comment_detail
    }


    override fun init() {
        orderId = intent.getLongExtra(KEY_ORDER_ID, -1)
        initListener()
        getCCommentDetail()
    }

    private fun initListener() {
        title_bar.setLeftImgListener {
            AnalyticHelper.getInstance().sendClickInfo("1qs1")
            finish()
        }
        title_bar.setRightImgListener {
            AnalyticHelper.getInstance().sendClickInfo("1qs2")
            cCommentDetail?.apply {
                DialogUtils.getDelInterviewCommentDialog(this@CCommentDetailActivity) { _, resultCode ->
                    if (resultCode == Activity.RESULT_OK) {
                        ApiService.api.deleteInterviewComment(interviewId.toLong())
                            .enqueue(object : BaseCallback<Any?>() {
                                override fun onSuccess(
                                    request: HttpRequest<*>?,
                                    response: HttpResponse<Any?>?
                                ) {
                                    OkBus.getInstance().onEvent(EventTags.UPDATE_INTERVIEW_DETAIL)
                                    OkBus.getInstance().onEvent(
                                        EventTags.DELETE_INTERVIEW_COMMENT,
                                        DeliverGetBean.STATE_DEL_COMMENT
                                    )
                                    finish()
                                }
                            })
                    }
                }.show((this@CCommentDetailActivity).supportFragmentManager, "")
            }
        }
        wifi_view.setReLoadListener {
            getCCommentDetail()
        }
    }

    private fun getCCommentDetail() {
        if (orderId == null || orderId == -1L) {
            return
        }
        InterviewNetManager.getCCommentDetail(orderId, onCommentDetailListener)
    }

    private fun fillUi(commentDetail: CCommentDetail?) {
        commentDetail?.apply {
            iv_interview_icon?.apply {
                UserHeadUtils.setUserHeader(this, portrait)
            }
            iv_interview_sex?.apply {
                when (sex) {
                    "男" -> setImageResource(R.drawable.icon_male_flag)
                    "女" -> setImageResource(R.drawable.icon_female_flag)
                    else -> setImageDrawable(null)
                }
            }
            notify_bar_comment_status_notice.visibility =
                if (interviewType == 4) View.VISIBLE else View.GONE
            notify_bar_comment_status_notice.setIsShowClose(false)
            tv_name.text = name
            tv_time.text = showTime
            tv_comment_txt.text = content
            tv_interview_position.text =
                ResourceUtils.getString(R.string.interview_position_tag, positionName)

            if (isClicked) {
                tv_useful.visibility = View.VISIBLE
                tv_useful.text = usefulCount.toString()
            } else {
                tv_useful.visibility = View.GONE
            }

            //2 综合评分
            tv_fraction.text = comprehensiveScore
            rate_bar_desc.rating = SafeTypeConvert.toInt(describeScore)
            rate_bar_interviewer.rating = SafeTypeConvert.toInt(interviewScore)
            rate_bar_other.rating = SafeTypeConvert.toInt(companyScore)

            //3 标签
            fl_interview_tag.fillLabelView(tags)
        }
    }

    override fun getPageId() = "1qrz"
}