package com.alpha.lagouapk.ui.login.guidePerfect

import android.content.Intent
import android.net.Uri
import android.text.TextUtils
import android.view.View
import com.ApiService
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.CUserTypeAndVipStateBean
import com.alpha.lagouapk.bean.GradeUserAdBean
import com.alpha.lagouapk.bean.ResumeBaseInfo
import com.alpha.lagouapk.manager.AdManager
import com.alpha.lagouapk.manager.AttachGuideManager
import com.alpha.lagouapk.manager.ResumeManager
import com.alpha.lagouapk.tools.BaseUtils
import com.alpha.lagouapk.tools.IntentUtils
import com.alpha.lagouapk.ui.dialog.DefaultHeadDialog
import com.alpha.lagouapk.ui.home.v2.JobStatusDialogPresenter
import com.alpha.lagouapk.ui.home.v2.JobStatusDialogPresenter.OnStatusArrivalTimeCallBack
import com.alpha.lagouapk.ui.login.perfect.EditInfoCode
import com.alpha.lagouapk.ui.login.perfect.EditInfoDialog
import com.alpha.lagouapk.ui.resume.JobStateWheelstrategy
import com.common.SensitiveLogic
import com.common.SensitiveLogic.CheckSensitiveCallback
import com.common.SensitiveLogic.checkExistSensitiveKey
import com.common.integrate.picture.UploadFileNetManager
import com.common.inter.PictureSelectListener
import com.common.manager.PictureSelectTakeManager
import com.common.mvp.listener.OnUploadPictureListener
import com.common.utils.GlideHelper.Companion.cacheSignWithIn1Day
import com.common.utils.SpUserClearUtils
import com.common.utils.SpUserNotClearUtils
import com.lagou.annotation.aspect.SingleClick
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.baselib.privacy.KEY_CAMERA_SCENE_RESUME_AVATAR
import com.lagou.baselib.privacy.KEY_STORE_SCENE_RESUME_AVATAR
import com.lagou.baselib.privacy.SceneRightsCollect
import com.lagou.commonlibs.engine.ThreadUtil
import com.lagou.commonlibs.imageload.ImageLoadUtils
import com.lagou.commonlibs.utils.*
import com.lagou.commonlibs.view.WheelDialog
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse
import com.lxj.xpopup.XPopup
import kotlinx.android.synthetic.main.activity_perfect_points.*
import kotlinx.android.synthetic.main.activity_register_postcard.*
import java.util.*

/**
 * 完善加分项
 */
class PerfectPointsActivity : BaseActivity(), View.OnClickListener, PictureSelectListener,
    PictureSelectTakeManager.ShowDefaultHeadDialogListener {
    private val mPictureManager: PictureSelectTakeManager = PictureSelectTakeManager()
    private lateinit var resumeBaseInfo: ResumeBaseInfo
    private val mDefaultHeadDialog: DefaultHeadDialog = DefaultHeadDialog.getInstance()
    private var mArrivalTimeList: List<String>? = null
    private var mStatusList: List<String>? = null

    companion object {
        const val EXPECT_STATUS_SPLIT = "-"
    }

    override fun getContentLayoutId(): Int = R.layout.activity_perfect_points

    override fun init() {
        val resumeBean = ResumeManager.getInstance().resumeBean
        AttachGuideManager.getInstance().intoTarget(resumeBean)
            .setFromPage(AttachGuideManager.PERSONAL_INFORMATION).syncDescResume()
        resumeBaseInfo = resumeBean.baseInfo
        this.initListener()
        this.fetchExpectJobStatus()
        this.initView()
    }

    private fun initView() {
        if (!resumeBaseInfo.portrait.isNullOrEmpty()) {
            ImageLoadUtils.with(this).load(resumeBaseInfo.portrait).dontAnimate()
                .placeholder(R.drawable.icon_login_head).error(R.drawable.icon_login_head)
                .into(ivUserHead);
        }
        if (resumeBaseInfo.isFemale or resumeBaseInfo.isMale) {
            tvMale.isSelected = resumeBaseInfo.isMale
            tvFemale.isSelected = resumeBaseInfo.isFemale
        }
        val resumeBean = ResumeManager.getInstance().resumeBean
        resumeBean.expectJob?.let {
            mEditJobStatusItem.content = StringUtil.jointString(
                EXPECT_STATUS_SPLIT,
                resumeBean.expectJob.expectStatus,
                resumeBean.expectJob.expectArrivalTime
            )
            resumeBaseInfo.status = resumeBean.expectJob.expectStatus
            resumeBaseInfo.arrivalTime = resumeBean.expectJob.expectArrivalTime
        }
        if (resumeBaseInfo.email.isNullOrEmpty().not()) {
            mEditEmailItem.content = resumeBaseInfo.email
        }
        if (resumeBaseInfo.selfDescription.isNullOrEmpty().not()) {
            mEditAdvantageItem.content = BaseUtils.getHtmlStr(resumeBaseInfo.selfDescription)
        }
    }

    private fun initListener() {
        mPerfectPointsTitle.setLeftImgListener { finish() }
        mPerfectPointsTitle.setRightTvListener {
            enterMainActivity(this)
        }
        setClickViews(ivUserHead, tvMale, tvFemale, mEditJobStatusItem, mEditEmailItem, mEditAdvantageItem, tvEnterNext)
        mPictureManager.setIsNeedZoom(true)
        mDefaultHeadDialog.setListener { url, _ ->
            ImageLoadUtils.with(this).load(url).dontAnimate()
                .placeholder(R.drawable.icon_login_head).error(R.drawable.icon_login_head)
                .into(ivUserHead);
            resumeBaseInfo.portrait = url
        }
    }

    @SingleClick
    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.ivUserHead -> mPictureManager.showCompression(this, true, this, SceneRightsCollect(KEY_CAMERA_SCENE_RESUME_AVATAR, KEY_STORE_SCENE_RESUME_AVATAR))
            R.id.tvMale -> {
                tvMale.isSelected = true
                tvFemale.isSelected = false
                resumeBaseInfo.sex = tvMale.text.toString()
            }
            R.id.tvFemale -> {
                tvMale.isSelected = false
                tvFemale.isSelected = true
                resumeBaseInfo.sex = tvFemale.text.toString()
            }
            R.id.mEditJobStatusItem -> jobStatusItemClick()
            R.id.mEditEmailItem -> emailItemClick()
            R.id.mEditAdvantageItem -> {
                val content =
                    if (resumeBaseInfo.selfDescription.isNullOrEmpty()) mEditAdvantageItem.content else resumeBaseInfo.selfDescription
                IntentUtils.startGuideEditExample(this, EditInfoCode.TYPE_SELFIND, content)
            }
            R.id.tvEnterNext -> {

                val map: MutableMap<String, String> = HashMap()
                val emailNameKey = SensitiveLogic.SensitiveKey.RESUME_EMAIL_WARN.name
                val advantageNameKey = SensitiveLogic.SensitiveKey.RESUME_ADVANTAGE_WARN.name
                map[emailNameKey] = resumeBaseInfo?.email
                map[advantageNameKey] = resumeBaseInfo?.selfDescription
                checkExistSensitiveKey(map, object : CheckSensitiveCallback {
                    override fun checkFinish(keyList: ArrayList<String>?,keyAndErrorMsgMap:HashMap<String,String>?) {
                        if (!keyAndErrorMsgMap.isNullOrEmpty()){
                            ToastUtil.showCenter(this@PerfectPointsActivity, ResourceUtils.getString(R.string.content_warn))
                            sensitiveWordsShow(keyAndErrorMsgMap)
                            return
                        }
                        resetSensitiveError()
                        commit()
                    }
                })
            }
        }
    }

    private fun sensitiveWordsShow(keyAndErrorMsgMap:HashMap<String,String>){
        val emailNameKey = SensitiveLogic.SensitiveKey.RESUME_EMAIL_WARN.name
        val advantageNameKey = SensitiveLogic.SensitiveKey.RESUME_ADVANTAGE_WARN.name

        val isExistEmailKey = keyAndErrorMsgMap.containsKey(emailNameKey)
        val isExistAdvantageKey = keyAndErrorMsgMap.containsKey(advantageNameKey)
        mEditEmailItem.setLineBottomRed(isExistEmailKey).setErrorDesc(if (isExistEmailKey) keyAndErrorMsgMap[emailNameKey]  else null)
        mEditAdvantageItem.setLineBottomRed(isExistAdvantageKey).setErrorDesc(if (isExistAdvantageKey) keyAndErrorMsgMap[advantageNameKey] else null)

    }

    private fun resetSensitiveError(){
        mEditEmailItem.setLineBottomRed(false).setErrorDesc(null)
        mEditAdvantageItem.setLineBottomRed(false).setErrorDesc(null)
    }

    /**
     * 获取用户是否符合展示首投激励模块的条件
     *
     */
    private fun checkCUserTypeAndVipState() {
        ApiService.api.fetchCUserTypeAndVipState()
            .enqueue(object : BaseCallback<CUserTypeAndVipStateBean?>(false) {
                override fun onSuccess(
                    request: HttpRequest<*>?,
                    response: HttpResponse<CUserTypeAndVipStateBean?>?
                ) {
                    response?.content?.let {
                        SpUserClearUtils.getInstance().cUserType = it.getcUserType()
                        SpUserClearUtils.getInstance()
                            .setShowHomeRecommendPositionGroup(!it.isReceiveVip)
                    }
                    jumpToNextPage()
                }

                override fun onFailure(
                    request: HttpRequest<*>?,
                    response: HttpResponse<CUserTypeAndVipStateBean?>?
                ) {
                    super.onFailure(request, response)
                    jumpToNextPage()
                }
            })
    }

    /**
     * 跳转至下一个页面, 根据请求接口返回的结果来判断是跳转到GuideSplashActivity, 还是GuideAddWeChatActivity
     */
    private fun jumpToNextPage() {
        SpUserClearUtils.getInstance().markFirstDeliverPrizeByPostcard(true)
        val adKey =
            SpUserNotClearUtils.getInstance().getGradeUserAdKey(AdManager.AD_APP_LOGIN_WELCOME)
        ApiService.api.getGradeUserAd(AdManager.AD_APP_LOGIN_WELCOME, adKey)
            .enqueue(object : BaseCallback<GradeUserAdBean?>(false) {
                override fun onSuccess(
                    request: HttpRequest<*>?,
                    response: HttpResponse<GradeUserAdBean?>?
                ) {
                    if (response == null || response.content == null) {
                        IntentUtils.startGuideSplashActivity(mContext)
                        return
                    }
                    val bean = response.content!!
                    if (!TextUtils.equals(adKey, bean.key)) {
                        ThreadUtil.io {
                            SpUserNotClearUtils.getInstance()
                                .saveGradeUserAdKey(AdManager.AD_APP_LOGIN_WELCOME, bean.key)
                        }
                    }
                    IntentUtils.startGuideAddWeChatActivity(mContext, bean)
                }

                override fun onFailure(
                    request: HttpRequest<*>?,
                    response: HttpResponse<GradeUserAdBean?>
                ) {
                    super.onFailure(request, response)
                    IntentUtils.startGuideSplashActivity(mContext)
                }
            })
    }


    /**
     * 保存进入下一步
     */
    private fun commit() {
        showLoading()
        ApiService.api.savePersonAdditionalData(resumeBaseInfo)
            .enqueue(object : BaseCallback<Any>() {
                override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<Any>?) {
                    if (isActivityDestroyed) {
                        return
                    }
                    hideLoading()
                    val resumeBean = ResumeManager.getInstance().resumeBean
                    resumeBean.baseInfo = resumeBaseInfo
                    ResumeManager.getInstance().resumeBean = resumeBean
                    checkCUserTypeAndVipState()
                }

                override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<Any>?) {
                    super.onFailure(request, response)
                    if (isActivityDestroyed) {
                        return
                    }
                    hideLoading()
                }
            })
    }

    /**
     * 请求期望
     */
    private fun fetchExpectJobStatus() {
        JobStatusDialogPresenter.getInstance()
            .fetchExpectJobStatusStaticData(OnStatusArrivalTimeCallBack { bean ->
                if (bean == null) {
                    return@OnStatusArrivalTimeCallBack
                }
                mStatusList = bean.statusList
                mArrivalTimeList = bean.arrivalTimeList
            })
    }

    /**
     * 当前状态点击事件
     */
    private fun jobStatusItemClick() {
        val wheelData = mutableListOf<List<String>>()
        if (!mStatusList.isNullOrEmpty()) {
            wheelData.add(mStatusList!!)
        }
        if (!mArrivalTimeList.isNullOrEmpty()) {
            wheelData.add(mArrivalTimeList!!)
        }
        val columnSize = wheelData.size
        val levelCurr = arrayListOf<String>()
        val expectStatus = mEditJobStatusItem.content?.substringBefore(
            EXPECT_STATUS_SPLIT,
            JobStateWheelstrategy.STATE_OFF_DUTY
        ) ?: JobStateWheelstrategy.STATE_OFF_DUTY
        val expectArrival = mEditJobStatusItem.content?.substringAfter(
            EXPECT_STATUS_SPLIT,
            JobStateWheelstrategy.TIME_now
        ) ?: JobStateWheelstrategy.TIME_now
        levelCurr.add(expectStatus)
        levelCurr.add(expectArrival)
        WheelDialog(this).apply {
            mWheelStrategy = JobStateWheelstrategy()
            builder(
                ResourceUtils.getString(R.string.job_hunting_status_label),
                R.id.mEditJobStatusItem,
                wheelData,
                levelCurr,
                emptyList(),
                WheelDialog.MAX
            )
            setOnDataPicListener { _, data ->
                if (data == null || data.size < columnSize || columnSize == 0) {
                    return@setOnDataPicListener
                }
                var status = if (columnSize == 1) "" else data[0]
                val arrivalTime = data[1 % columnSize]
                mEditJobStatusItem.content =
                    StringUtil.jointString(EXPECT_STATUS_SPLIT, status, arrivalTime)
                resumeBaseInfo.status = status
                resumeBaseInfo.arrivalTime = arrivalTime
            }
        }.show()
    }

    /**
     * 邮箱点击事件
     */
    private fun emailItemClick() {
        val editEmailDialog = EditInfoDialog(this, EditInfoCode.TYPE_EMAIL, mEditEmailItem.content)
        editEmailDialog.setOnResultListener { result ->
            mEditEmailItem.setContent(result).setErrorDesc(null)
            resumeBaseInfo.email = result
        }
        XPopup.Builder(this).moveUpToKeyboard(false).asCustom(editEmailDialog).show();
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (resultCode != RESULT_OK) {
            return
        }
        mPictureManager.onActivityResult(requestCode, resultCode, data)
        val editContent = data?.extras?.getString(IntentUtils.INTENT_KEY_EDITINFO_CONTENT)
        if (requestCode == EditInfoCode.TYPE_SELFIND) {
            mEditAdvantageItem.setContent(BaseUtils.getHtmlStr(editContent)).setErrorDesc(null)
            resumeBaseInfo.selfDescription = editContent
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        mPictureManager.onDestroy()
    }

    override fun uploadPhoto(photoUri: Uri?) {
        if (isActivityDestroyed || photoUri == null) {
            return
        }
        UploadFileNetManager.uploadSmallPicture(
            photoUri,
            this,
            object : OnUploadPictureListener<String?> {
                override fun uploadSuccess(picUrl: String?) {
                    resumeBaseInfo.portrait = picUrl
                }

                override fun uploadFail(response: HttpResponse<*>?) {
                    ToastUtil.show(
                        baseActivity,
                        ResourceUtils.getString(R.string.head_pic_upload_fail_error)
                    )
                }

                override fun isActive(): Boolean {
                    return baseActivity.isActive
                }
            })
        val defaultAvatar =
            if (resumeBaseInfo.isMale) R.drawable.default_boy_headpic1 else R.drawable.default_girl_headpic1
        ImageLoadUtils.with(mContext).load(photoUri).dontAnimate().placeholder(defaultAvatar)
            .error(defaultAvatar).signature(cacheSignWithIn1Day()).into(ivUserHead)
    }

    override fun showDefaultHeadDialog() {
        if (resumeBaseInfo.isMale) {
            mDefaultHeadDialog.setAvatarShowType(1)
        } else if (resumeBaseInfo.isFemale) {
            mDefaultHeadDialog.setAvatarShowType(2)
        } else {
            mDefaultHeadDialog.setAvatarShowType(0)
        }
        mDefaultHeadDialog.show(supportFragmentManager)
    }

    override fun getPageId() = "1q33"
    override fun getExploreParams(): MutableMap<String, Int> {
        val map = mutableMapOf<String, Int>()
        map["identity"] = 1
        return map
    }
}