package com.alpha.lagouapk.ui.login.guidePerfect

import android.content.Intent
import android.os.Bundle
import android.text.InputFilter
import android.text.TextUtils
import android.view.View
import android.view.ViewGroup
import android.view.inputmethod.EditorInfo
import androidx.core.widget.NestedScrollView
import com.ApiService
import com.alpha.lagouapk.R
import com.alpha.lagouapk.UserRoleManager
import com.alpha.lagouapk.bean.EducationExperienceBean
import com.alpha.lagouapk.bean.ResumeBaseInfo
import com.alpha.lagouapk.bean.position.PositionLabelGroup
import com.alpha.lagouapk.bean.resume.ResumeAiConfig
import com.alpha.lagouapk.tools.ComUtil
import com.alpha.lagouapk.tools.CommonDef
import com.alpha.lagouapk.tools.IntentUtils
import com.alpha.lagouapk.ui.dialog.ResumeAiEnterDialog
import com.alpha.lagouapk.ui.login.bean.ExpectInfo
import com.alpha.lagouapk.ui.login.bean.PositionTypeAndTagsBean
import com.alpha.lagouapk.ui.login.bean.PostcardInfoBean
import com.alpha.lagouapk.ui.login.perfect.EditInfoCode
import com.alpha.lagouapk.ui.login.perfect.EditInfoDialog
import com.alpha.lagouapk.ui.resume.ai.ResumeAiGenActivity.Companion.DEFAULT_MAX_SEND_COUNT
import com.alpha.lagouapk.ui.resume.edit.v2.ResumePositionTagActivity
import com.common.SensitiveLogic
import com.common.utils.CommonIntentUtils.Companion.startTouristWebActivity
import com.lagou.annotation.aspect.SingleClick
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.baselib.util.BarUtil
import com.lagou.commonlibs.utils.*
import com.lagou.commonlibs.utils.wheelstrategy.BirthdayWheelStrategy
import com.lagou.commonlibs.utils.wheelstrategy.DataWheelStrategy
import com.lagou.commonlibs.utils.wheelstrategy.EducationLevelStrategy
import com.lagou.commonlibs.utils.wheelstrategy.SalaryWheelStrategy
import com.lagou.commonlibs.view.EditItemView
import com.lagou.commonlibs.view.MaxInputWatcher
import com.lagou.commonlibs.view.WheelDialog
import com.lagou.commonlibs.view.filter.EmojiFilter
import com.lagou.component.ui.dialog.listener.LgDialogListener
import com.lagou.lagouee.utils.DataServer
import com.lagou.lagouee.utils.DialogUtils
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse
import com.lxj.xpopup.XPopup
import kotlinx.android.synthetic.main.activity_register_postcard.*
import java.util.*

/**
 * 注册流程中的名片页
 */
class PostcardActivity:BaseActivity(),View.OnClickListener {
    companion object{
        const val DEFAULT_SALARY_HIGH = "100K以上"
        const val PLS_PERFECT_INFO = "请完善相关信息"
        const val SOURCE_REGISTER = 1
        const val SOURCE_POSITION_DETAIL = 2;
    }
    private var isNeedCheckInfoByName = true
    override fun getContentLayoutId(): Int = R.layout.activity_register_postcard
    //职位三级类code
    private var mPositionCodes = arrayOfNulls<String>(3)
    //职位标签
    private var mPositionLabels : ArrayList<PositionLabelGroup.Label?>? = null
    //页面来源
    private var mSource = SOURCE_REGISTER

    /***
     * 选择产生的职位名称
     */
    private var positionName: String? = ""

    override fun init() {
        mSource = intent?.extras?.getInt(IntentUtils.INTENT_KEY_PAGE_SOURCE) ?: SOURCE_REGISTER
        this.initViewBySource()
        this.initStatusBar()
        this.initListener()
        this.requestPostcardInfo()
    }

    private fun initViewBySource(){
        mTitleBar.tvRight.visibility = if (mSource == SOURCE_REGISTER) View.VISIBLE else View.GONE
        mTvNextStep.text = if (mSource == SOURCE_REGISTER) ResourceUtils.getString(R.string.next_step) else ResourceUtils.getString(R.string.complete)
    }

    private fun initStatusBar() {
        BarUtil.transparentStatusBar(this)
        mStatusBar.layoutParams = mStatusBar.layoutParams.apply { height = BarUtil.getStatusBarHeight(this@PostcardActivity) }
    }

    private fun initListener() {
        mTitleBar.setLeftImgListener { onBackPressed() }
        mTitleBar.setRightTvListener { jump() }
        initNameListener()
        mScrollView.setOnScrollChangeListener(NestedScrollView.OnScrollChangeListener { _, _, scrollY, _, _ ->
            val bgColor = if (scrollY == 0) ResourceUtils.getColor(R.color.transparent) else ResourceUtils.getColor(R.color.color_00b38a)
            mStatusBar.setBackgroundColor(bgColor)
            mTitleBar.setBackgroundColor(bgColor)
        })
        setClickViews(mEditUserCityItem,mEditSchoolNameItem,mEditEducationItem,mEditWorkYearItem,
            mEditCompanyItem,mEditPositionNameItem,mEditExpectSalaryItem,mEditPositionLabelItem,mTvNextStep,mEditBirthdayItem)
    }

    /**
     * 跳过，需要先判断是否达到了名片
     */
    @SingleClick
    private fun jump(){
        AnalyticHelper.getInstance().sendClickInfo("q1u1c","v1u1h")
        checkIsPostcardPerfect(object:CheckIsPostcardPerfectCallback{
            override fun onPostcardPerfect(isPerfect: Boolean,isCanDeliver:Boolean) {
                if (isActivityDestroyed){
                    return
                }
                if (isPerfect){
                    //已经完善,进入首页
                    enterMainActivity(this@PostcardActivity)
                    return
                }
                //未完善，检测是否弹出AI弹窗
                checkShowResumeAiDialog(false)
            }
        })
    }

    /**
     * 姓名初始化设置
     */
    private fun initNameListener(){
        val inputWatcher = MaxInputWatcher(10).apply {
            setIsCalTextLength(true)
            setInputType(MaxInputWatcher.INPUT_TYPE_DOUBLE)
            setMaxInputListener(object:MaxInputWatcher.OnMaxInputListener{
                override fun equalMaxLength(isEqual: Boolean) {
                    if (isEqual){
                        ToastUtil.showCenter(mContext, resources.getString(R.string.name_input_limit))
                    }
                }

                override fun onTextChanged(s: String?) {
                    isNeedCheckInfoByName = if (s.isNullOrEmpty()){
                        checkInfoIsPerfect()
                        true
                    }else{
                        if (isNeedCheckInfoByName){
                            checkInfoIsPerfect()
                        }
                        false
                    }
                }

                override fun showLengthTip(length: Int, text: String?) {}

            })
        }
        val onFocusChangeListener = { _:View, hasFocus:Boolean ->
            mEditUserNameItem.setLineBottomGreen(hasFocus)
            if (hasFocus){
                mEditUserNameItem.setErrorDesc(null)
            }
        }
       mEditUserNameItem.editText.apply {
           imeOptions = EditorInfo.IME_ACTION_DONE
           filters = arrayOf<InputFilter>(EmojiFilter())
           addTextChangedListener(inputWatcher)
           setOnFocusChangeListener(onFocusChangeListener)
           setOnClickListener{
               PostcardViewHelper.setLineState(
                   if (hasFocus()) PostcardViewHelper.LineState.GREEN
                   else PostcardViewHelper.LineState.NORMAL,mEditUserNameItem.lineBottom)
           }
        }
        setEditTextHideByTouchOther(true, null, mEditUserNameItem.editText)
    }

    override fun onClick(v: View?) {
        when(v?.id){
            //所在城市
            R.id.mEditUserCityItem -> IntentUtils.startCityListActivity(mContext)
            //学校名称
            R.id.mEditSchoolNameItem -> inputDialogListener(EditInfoCode.TYPE_SCHOOL, mEditSchoolNameItem.content)
            //学历
            R.id.mEditEducationItem -> educationItemClick()
            //首次参加工作时间
            R.id.mEditWorkYearItem -> workDateItemClick()
            //最近就职的公司
            R.id.mEditCompanyItem -> inputDialogListener(EditInfoCode.TYPE_COMPANY, mEditCompanyItem.content)
            //职位名称
            R.id.mEditPositionNameItem -> {
                if (hasWorkExperience()){
                    IntentUtils.startSelectPositionAndTagsActivityOther(this,
                        positionName, IntentUtils.REQUEST_CODE_POSITION_NAME)
                    return
                }
                IntentUtils.startSelectPositionAndTagsActivityExcept(this,
                    positionName, IntentUtils.REQUEST_CODE_POSITION_NAME)
            }
            //职位类别
            //R.id.mEditPositionTypeItem -> IntentUtils.startExpectPositionActivity(mContext)
            //职位标签
            R.id.mEditPositionLabelItem -> requestPositionLabels()
            //期望薪资
            R.id.mEditExpectSalaryItem -> salaryItemClick()
            //出生日期点击
            R.id.mEditBirthdayItem -> birthdayClick()
            //下一步
            R.id.mTvNextStep -> commit()
        }
    }

    /**
     * 出生日期点击
     */
    private fun birthdayClick() {
        val curBirthday = mEditBirthdayItem.content
        val defaultBirth = (DateUtils.getCurrentYear() - 22).toString() + ".06.15"
        val birthday = if (curBirthday.isNullOrEmpty()) defaultBirth else curBirthday
        var arrSize = birthday.split(".").size
        val dateFormat = if (arrSize == 3)  "yyyy.MM.dd" else "yyyy.MM"
        val strategy = BirthdayWheelStrategy(DateUtils.dateToUnixTimestamp(birthday, dateFormat))
        val options = strategy.data
        WheelDialog(this).apply {
            mWheelStrategy = strategy
            builder(ResourceUtils.getString(R.string.birthday), mEditBirthdayItem.id,options, strategy.curSelectData,strategy.getLimit(options), WheelDialog.MIN)
            setOnDataPicListener { _, data ->
                if (data == null || data.size < 3){
                    return@setOnDataPicListener
                }
                val date = StringUtil.jointString(".",data.asList())
                mEditBirthdayItem.content = date
                checkInfoIsPerfect()
            }
        }.show()
    }

    /**
     * 期望薪资点击
     */
    private fun salaryItemClick(){
        val curSalary = mEditExpectSalaryItem.content
        val curSalaryList = if (curSalary.isNullOrEmpty() || curSalary.split("-").size != 2){
            DataServer.getDefaultSalary()
        } else {
            if (curSalary.toUpperCase(Locale.ROOT) == DEFAULT_SALARY_HIGH){
                arrayListOf(GuideExpectJobActivity.DEFAULT_SALARY_HIGH,"")
            }else{
                arrayListOf(curSalary.substringBefore("-"),curSalary.substringAfter("-"))
            }
        }
        WheelDialog(this).apply {
            mWheelStrategy = SalaryWheelStrategy()
            builder(DataServer.getSalaryArrC(),curSalaryList)
            setOnDataPicListener { _, data ->
                if (data.size < 2){
                    return@setOnDataPicListener
                }
                mEditExpectSalaryItem.content = if (data[0] == DEFAULT_SALARY_HIGH) data[0] else "${data[0]}-${data[1]}"
                checkInfoIsPerfect()
            }
            show()
        }
    }

    /**
     * 首次参加工作时间L
     */
    private fun workDateItemClick(){
        val options = ComUtil.getTwoCulumnOptions(4,CommonDef.EDIT_TYPE_BASE)
        val curList = ComUtil.getJoinWorkTime(mEditWorkYearItem.content)
        val resumeBaseInfo = ResumeBaseInfo()
        resumeBaseInfo.birthday = mEditBirthdayItem.content
        val limitList = ComUtil.getLimit(4,CommonDef.EDIT_TYPE_BASE,resumeBaseInfo,null,null,null)
        WheelDialog(this).apply{
            mWheelStrategy = DataWheelStrategy()
            builder(resources.getString(R.string.first_job_time),R.id.mEditWorkYearItem,options,curList,limitList,WheelDialog.MAX).setOnDataPicListener(object : WheelDialog.onDataPicListener{
                override fun onDataPic(key: Int, data: Array<out String>?) {
                    if (data == null || data.size < 2){
                        return
                    }
                    var workDate = ""
                    val noWorkText = resources.getString(R.string.no_work_exp)
                    if (data[0] == noWorkText){
                        workDate = noWorkText
                        mEditPositionNameItem.titleName = ResourceUtils.getString(R.string.expect_position)
                        if (workDate != mEditWorkYearItem.content){
                            resetViewByWorkDateType()
                            hideViews(mEditCompanyItem,mEditPositionLabelItem).showViews(mEditPositionNameItem)
                        }
                    }else{
                        workDate = StringUtil.jointString(".",data[0].trim(),data[1].trim())
                        mEditPositionNameItem.titleName = ResourceUtils.getString(R.string.position_name)
                        //具体的工作时间
                        if (mEditWorkYearItem.content.isNullOrEmpty() || mEditWorkYearItem.content == noWorkText){
                            //如果之前选的是暂无工作经历，那么需要丢弃之前选的最近就职公司、职位名称、职位类型、职位标签
                            resetViewByWorkDateType()
                            hideViews(mEditPositionLabelItem).showViews(mEditCompanyItem,mEditPositionNameItem)
                        }
                    }
                    mEditWorkYearItem.content = workDate
                    checkInfoIsPerfect()
                }
            }).show()
        }
    }

    /**
     * 具体时间->暂无工作经历 or 暂无工作经历 -> 具体时间
     * 清空职位类别，细分标签、职位名称、最近就职的公司
     */
    private fun resetViewByWorkDateType(){
        positionName = ""
        mEditPositionLabelItem.content = null
        mEditPositionNameItem.content = null
        mEditCompanyItem.content = null

        mPositionLabels = null
        mPositionCodes[0] = null
        mPositionCodes[1] = null
        mPositionCodes[2] = null

    }
    /**
     * 学历点击事件
     */
    private fun educationItemClick(){
       val options =  ComUtil.getTwoCulumnOptions(2, CommonDef.PerfectType.PERFECT_EDUCATION_LEVEL)
        val curList = if (mEditEducationItem.content.isNullOrEmpty() or !mEditEducationItem.content.contains(EducationExperienceBean.BEN_KE_SEPARATOR)){
            arrayListOf<String>(EducationExperienceBean.BEN_KE,EducationExperienceBean.TONG_ZHAO)
        } else{
            mEditEducationItem.content.split(EducationExperienceBean.BEN_KE_SEPARATOR)
        }
        WheelDialog(this).apply {
            mWheelStrategy = EducationLevelStrategy()
            builder(resources.getString(R.string.edu_education),R.id.li_education_level, options, curList, arrayListOf(), WheelDialog.MAX).setOnDataPicListener(object:WheelDialog.onDataPicListener{
                override fun onDataPic(key: Int, data: Array<out String>?) {
                   if (data == null || data.isEmpty()){
                       return
                   }
                    mEditEducationItem.content = if (data.size == 2) StringUtil.jointString(
                        EducationExperienceBean.BEN_KE_SEPARATOR,
                        data[0],data[1]
                    ) else data[0]
                    checkInfoIsPerfect()
                }

            }).show()
        }
    }

   private fun inputDialogListener(editCode:Int,editContent:String?){
        val editInfoDialog = EditInfoDialog(this, editCode, editContent)
        XPopup.Builder(this)
            .moveUpToKeyboard(false)
            .asCustom(editInfoDialog)
            .show()
        editInfoDialog.setOnResultListener { result ->
            when(editCode){
                EditInfoCode.TYPE_SCHOOL -> mEditSchoolNameItem.setContent(result).setErrorDesc(null)
                EditInfoCode.TYPE_COMPANY -> mEditCompanyItem.setContent(result).setErrorDesc(null)
            }
            checkInfoIsPerfect()
        }
    }


    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (resultCode != RESULT_OK) {
            return
        }
       data?.let{ it ->
           it.extras?.let {bundle: Bundle ->
               when(requestCode){
                   //城市
                   IntentUtils.REQUEST_CODE_CITIES -> {
                       val city = bundle.getString("city")
                       mEditUserCityItem.content = city
                   }
                   //职位标签
                   IntentUtils.REQUEST_CODE_POSITION_LABEL -> {
                       val selectLabels: ArrayList<PositionLabelGroup.Label?>? =  bundle.getSerializable(ResumePositionTagActivity.KEY_SELECTED_TAGS) as? ArrayList<PositionLabelGroup.Label?>
                       mPositionLabels = selectLabels
                       mEditPositionLabelItem.content =
                           selectLabels?.filterNotNull()?.joinToString("、") { it.name }
                   }
                   //职位名称
                   IntentUtils.REQUEST_CODE_POSITION_NAME -> {
                        val positionNameTypeAndTags: PositionTypeAndTagsBean? = bundle.getSerializable(IntentUtils.INTENT_KEY_DATA) as? PositionTypeAndTagsBean
                       mEditPositionNameItem.visibility = View.VISIBLE
                       mPositionCodes[0] = positionNameTypeAndTags?.expectJobInfo?.newPositionCode1
                       mPositionCodes[1] = positionNameTypeAndTags?.expectJobInfo?.newPositionCode2
                       mPositionCodes[2] = positionNameTypeAndTags?.expectJobInfo?.newPositionCode3

                       mPositionLabels = positionNameTypeAndTags?.expectJobInfo?.tagCodes?.filterIndexed { index, _ -> index < 3 } as? ArrayList<PositionLabelGroup.Label?>
                       //以下无论哪种情况传值统统传递三级类
                       // 1 无工作经历，作为职位类别使用，显示三级职位类别，同时作为positionName的值
                       if (!hasWorkExperience()){
                           positionName = positionNameTypeAndTags?.expectJobInfo?.newPositionName3
                           mEditPositionNameItem.content = positionNameTypeAndTags?.expectJobInfo?.newPositionName3
                       }else {
                           //2 有工作经历有职位名称传值职位名称，无职位名称传值职位三级类（显示有职位名称显示职位名称，无显示三级类）
                           positionName = positionNameTypeAndTags?.getPositionNameShow()
                           mEditPositionNameItem.content = positionNameTypeAndTags?.getPositionNameShow()
                       }
                       if (!ListUtil.isEmpty(mPositionLabels)){
                           mEditPositionLabelItem.content = mPositionLabels?.filterNotNull()?.joinToString("、"){it.name}
                           mEditPositionLabelItem.visibility = View.VISIBLE
                       }else {
                           requestPositionLabels()
                       }
                   }
               }
               checkInfoIsPerfect()
           }
       }
    }

    /**
     * 请求职位标签接口
     */
    private fun requestPositionLabels() {
        showLoading()
        ApiService.api.getTagRelation(mPositionCodes[1],mPositionCodes[2]).enqueue(object:
            BaseCallback<ArrayList<PositionLabelGroup>>() {
            override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<ArrayList<PositionLabelGroup>>?) {
                if (isActivityDestroyed){
                    return
                }
                hideLoading()
                mEditPositionLabelItem.visibility = if (response?.content.isNullOrEmpty()) View.GONE else View.VISIBLE
                if (mEditPositionLabelItem.visibility == View.VISIBLE){
                    IntentUtils.startPositionLabelNoCustomActivity(this@PostcardActivity,response?.content,mPositionLabels,ResumePositionTagActivity.VALUE_FROM_GUIDE_EXPECT_JOB)
                }
            }

            override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<ArrayList<PositionLabelGroup>>?) {
                if (isActivityDestroyed){
                    return
                }
                hideLoading()
                mEditPositionLabelItem.visibility = View.GONE
            }
        })
    }

    /**
     * 请求名片信息
     */
    private fun requestPostcardInfo(){
        showLoading()
        ApiService.api.fetchPostcardInfo().enqueue(object:BaseCallback<PostcardInfoBean>(false){
            override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<PostcardInfoBean>?) {
                if (isActivityDestroyed) {
                    return
                }
                hideLoading()
                val postcardInfoBean: PostcardInfoBean? = response?.content
                mPositionCodes[0] = postcardInfoBean?.expectJobInfo?.newPositionCode1
                mPositionCodes[1] = postcardInfoBean?.expectJobInfo?.newPositionCode2
                mPositionCodes[2] = postcardInfoBean?.expectJobInfo?.newPositionCode3
                mPositionLabels = postcardInfoBean?.expectJobInfo?.tagCodes
                postcardInfoBean?.let {
                    fillView(postcardInfoBean)
                }
            }

            override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<PostcardInfoBean>?) {
               if (isActivityDestroyed){
                   return
               }
                hideLoading()
            }

        })
    }

    /**
     * 填充View
     */
    private fun fillView(bean: PostcardInfoBean) {
        mEditUserNameItem.content = bean.userName
        mEditUserCityItem.content = bean.city
        mEditSchoolNameItem.content = bean.schoolName
        mEditEducationItem.content = bean.getEducationWithSuffix()
        mEditWorkYearItem.content = bean.joinWorkTime
        mEditCompanyItem.content = bean.companyName
        //mEditPositionTypeItem.content = bean.expectJobInfo?.newPositionName3
        mEditPositionLabelItem.content = mPositionLabels?.filterNotNull()?.filterIndexed { index, _ -> index < 3 }?.joinToString("、"){ it.name }
        mEditExpectSalaryItem.content = bean.expectJobInfo?.expectSalary
        mEditBirthdayItem.content = bean.birthday
        if (bean.positionName.isNullOrEmpty().not()){
            //如果职位名称不为空，需要展示出来
            showViews(mEditPositionNameItem)
            mEditPositionLabelItem.visibility = if (mEditPositionLabelItem.content.isNullOrEmpty()) View.GONE else View.VISIBLE
        }
        if (bean.joinWorkTime.isNullOrEmpty().not()){
            if (bean.joinWorkTime == ResourceUtils.getString(R.string.no_work_exp)){
                //暂无工作经历
                mEditPositionNameItem.titleName = ResourceUtils.getString(R.string.expect_position)
                positionName = bean?.expectJobInfo?.newPositionName3
                mEditPositionNameItem.content = bean?.expectJobInfo?.newPositionName3
                resetViewByWorkDateType()
                hideViews(mEditCompanyItem,mEditPositionLabelItem).showViews(mEditPositionNameItem)
            }else{
                //有具体的工作时间
                mEditPositionNameItem.titleName = ResourceUtils.getString(R.string.position_name)
                positionName = bean.getPositionNameShow()
                mEditPositionNameItem.content = bean.getPositionNameShow()
                showViews(mEditCompanyItem,mEditPositionNameItem)
            }
        }
        checkInfoIsPerfect()
    }

    /**
     * 提交保存，下一步
     */
    private fun commit() {
        clearFocus()
       val isNotPerfect = checkInfoIsPerfect(isShowRedLine = true, isCommit = true)
        if (isNotPerfect) {
            return
        }
       val userName = SensitiveLogic.SensitiveKey.RESUME_USER_NAME_WARN.name
       val schoolName = SensitiveLogic.SensitiveKey.RESUME_EDU_SCHOOL_NAME_WARN.name
       val companyName = SensitiveLogic.SensitiveKey.RESUME_WORK_COMPANY_NAME_WARN.name
       val positionName = SensitiveLogic.SensitiveKey.RESUME_WORK_POSITION_NAME_WARN.name
       val map = mapOf(
           userName to mEditUserNameItem.content,
           schoolName to mEditSchoolNameItem.content,
           companyName to mEditCompanyItem.content,
           positionName to mEditPositionNameItem.content
       )
       SensitiveLogic.checkExistSensitiveKey(map, object:SensitiveLogic.CheckSensitiveCallback{
               override fun checkFinish(keyList: ArrayList<String>?,keyAndErrorMsgMap:HashMap<String,String>?) {
                   if (!keyAndErrorMsgMap.isNullOrEmpty()){
                       ToastUtil.showCenter(this@PostcardActivity, ResourceUtils.getString(R.string.content_warn))
                       sensitiveWordsShow(keyAndErrorMsgMap)
                       return
                   }
                   resetSensitiveError()
                   //敏感词校验完成，进行下一步保存名片信息
                   savePostcardInfo()
               }
           })
    }

    private fun sensitiveWordsShow(keyAndErrorMsgMap:HashMap<String,String>){
        val userName = SensitiveLogic.SensitiveKey.RESUME_USER_NAME_WARN.name
        val schoolName = SensitiveLogic.SensitiveKey.RESUME_EDU_SCHOOL_NAME_WARN.name
        val companyName = SensitiveLogic.SensitiveKey.RESUME_WORK_COMPANY_NAME_WARN.name
        val positionName = SensitiveLogic.SensitiveKey.RESUME_WORK_POSITION_NAME_WARN.name

        val isExistNameKey = keyAndErrorMsgMap.containsKey(userName)
        val isExistSchoolKey = keyAndErrorMsgMap.containsKey(schoolName)
        val isExistCompanyKey = keyAndErrorMsgMap.contains(companyName)
        val isExistPositionNameKey = keyAndErrorMsgMap.contains(positionName)

        mEditUserNameItem.setLineBottomRed(isExistNameKey).setErrorDesc(if (isExistNameKey) keyAndErrorMsgMap[userName] else null)
        mEditSchoolNameItem.setLineBottomRed(isExistSchoolKey).setErrorDesc(if (isExistSchoolKey) keyAndErrorMsgMap[schoolName] else null)
        mEditCompanyItem.setLineBottomRed(isExistCompanyKey).setErrorDesc(if (isExistCompanyKey) keyAndErrorMsgMap[companyName] else null)
        mEditPositionNameItem.setLineBottomRed(isExistPositionNameKey).setErrorDesc(if (isExistPositionNameKey) keyAndErrorMsgMap[positionName] else null)
    }
    private fun resetSensitiveError(){
        mEditUserNameItem.setLineBottomRed(false).setErrorDesc(null)
        mEditSchoolNameItem.setLineBottomRed(false).setErrorDesc(null)
        mEditCompanyItem.setLineBottomRed(false).setErrorDesc(null)
        mEditPositionNameItem.setLineBottomRed(false).setErrorDesc(null)
    }

    private fun clearFocus() {
        mEditUserNameItem.editText.clearFocus()
    }

    /**
     * 保存用户名片
     */
    private fun savePostcardInfo(){
        //用户身份，1-学生身份，2-职场人身份
        val postcardInfoBean = PostcardInfoBean()
        postcardInfoBean.userIdentity = ResumeBaseInfo.WORKER_INT
        postcardInfoBean.userName = mEditUserNameItem.content
        postcardInfoBean.city = mEditUserCityItem.content
        postcardInfoBean.schoolName = mEditSchoolNameItem.content
        postcardInfoBean.education = mEditEducationItem.content.substringBefore(EducationExperienceBean.BEN_KE_SEPARATOR,mEditEducationItem.content)
        postcardInfoBean.joinWorkTime = mEditWorkYearItem.content
        postcardInfoBean.companyName = mEditCompanyItem.content
        //mEditPositionNameItem.content 移动端工程师(android开发) 只用来显示用，提交还是作为两个字段
        if (hasWorkExperience()){
            postcardInfoBean.positionName = positionName
        }
        postcardInfoBean.setIsUnifiedEntrance(mEditEducationItem.content.substringAfter(EducationExperienceBean.BEN_KE_SEPARATOR,""))
        postcardInfoBean.expectJobInfo = ExpectInfo(
            newPositionCode1 = mPositionCodes[0],
            newPositionCode2 = mPositionCodes[1],
            newPositionCode3 = mPositionCodes[2],
            expectSalary = mEditExpectSalaryItem.content,
            tagCodes = mPositionLabels
        )
        postcardInfoBean.birthday = mEditBirthdayItem.content
        showLoading()
        ApiService.api.savePostcardInfo(postcardInfoBean).enqueue(object:BaseCallback<Any>(){
            override fun onSuccess(p0: HttpRequest<*>?, p1: HttpResponse<Any>?) {
                UserRoleManager.getInstance().isChanging = false
                if (isActivityDestroyed){
                    return
                }
                hideLoading()
                if (mSource == SOURCE_REGISTER){
                    //为了复用完善教育经历、工作经历的页面，需要拉取一下最新的简历信息，用来展示页面信息
                    syncResumeAndEnterGuideEducation(this@PostcardActivity)
                }else{
                    finish()
                }
            }

            override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<Any>?) {
                super.onFailure(request, response)
                if (isActivityDestroyed){
                    return
                }
                hideLoading()
            }
        })
    }

    /**
     * 校验信息是否完善
     * @param isShowRedLine 是否展示itemView底部的红线
     */
    fun checkInfoIsPerfect(isShowRedLine:Boolean = false,isCommit:Boolean = false):Boolean{
        //存储的每个item是否完善
        val editItemFields = hashMapOf<EditItemView,Boolean>()
        getNotPerfectFields(mScrollContentParent,editItemFields)
        val isNotPerfect = editItemFields.any { !it.value }
        mTvNextStep.alpha = if (isNotPerfect) 0.3f else 1.0f
        if (isCommit && isNotPerfect){
            val notPerfectFields = editItemFields.filterValues { !it }.map { it.key }
            val notPerfectTip = if (notPerfectFields.size == 1) "请完善${notPerfectFields[0].titleName}" else PLS_PERFECT_INFO
            ToastUtil.showCenter(this,notPerfectTip)
        }
        if (isShowRedLine){
            editItemFields.forEach {
                it.key.setLineBottomRed(!it.value)
            }
        }
        return isNotPerfect
    }
    private fun getNotPerfectFields(parent:ViewGroup, editItemFields:MutableMap<EditItemView,Boolean>){
        val childCount = parent.childCount
        for (index in 0 until childCount){
            val childView = parent.getChildAt(index)
            if (childView.visibility != View.VISIBLE){
                continue
            }
            when(childView){
                is EditItemView -> {
                    if (childView.id == R.id.mEditEducationItem){
                        //学历，本科若没选择统招or非统招，也算作不完善
                        editItemFields[childView] = !(childView.isEmpty
                                || (childView.content.contains(EducationExperienceBean.BEN_KE) && !childView.content.contains(EducationExperienceBean.TONG_ZHAO)))
                    }else{
                        editItemFields[childView] = !childView.isEmpty
                    }
                }
                is ViewGroup -> getNotPerfectFields(childView,editItemFields)
            }
        }
    }

    override fun isEnableCommonBarSetting() = false

    @SingleClick
    override fun onBackPressed() {
        checkIsPostcardPerfect(object:CheckIsPostcardPerfectCallback{
            override fun onPostcardPerfect(isPerfect: Boolean,isCanDeliver:Boolean) {
                if (isActivityDestroyed){
                    return
                }
                if (isPerfect){
                    defaultBackAction()
                    return
                }
                //未完善，检测是否弹出AI弹窗
                checkShowResumeAiDialog(true)
            }
        })
    }

    /**
     * 检测是否展示简历Ai弹窗
     * @param isBack 点击的是否是返回，如果是，弹出挽留弹出；如果点击的是跳过，则进入游客模式
     */
    fun checkShowResumeAiDialog(isBack:Boolean){
        ApiService.api.fetchResumeAiConfig().enqueue(object:BaseCallback<ResumeAiConfig>(false){
            override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<ResumeAiConfig>?) {
                if (isActivityDestroyed){
                    return
                }
                val enable = response?.content?.enable == true
                if (enable){
                    ResumeAiEnterDialog.getInstance(response?.content?.limitTimes ?: DEFAULT_MAX_SEND_COUNT) {
                        titleBarActionBy(isBack)
                    }.show(supportFragmentManager,"ResumeAiEnterDialog")
                }else{
                    titleBarActionBy(isBack)
                }
            }

            override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<ResumeAiConfig>?) {
                titleBarActionBy(isBack)
            }
        })
    }

    /**
     * 默认返回
     */
    private fun defaultBackAction() {
        DialogUtils.showExitPostcardDialog(supportFragmentManager, object : LgDialogListener() {
            override fun onLeft() {
                finish()
            }
        })
    }

    /**
     * 点击返回或者跳过的行为
     */
    private fun titleBarActionBy(isBack:Boolean){
        if (isBack) defaultBackAction() else startTouristWebActivity(this@PostcardActivity)
    }
    
    

    override fun getPageId() = "q1u1c"

    /**
     * 是否有工作经历，依据标题内容
     * 职位名称 -  有工作经历
     * 期望职位 - 无工作经历
     */
    fun hasWorkExperience() : Boolean {
        return TextUtils.equals(ResourceUtils.getString(R.string.position_name), mEditPositionNameItem.titleName)
    }
}