@file:JvmName("RegisterProcessHelper")
package com.alpha.lagouapk.ui.login.guidePerfect

import android.content.Context
import android.text.TextUtils
import com.ApiService
import com.alpha.lagouapk.LagouMainPage
import com.alpha.lagouapk.bean.ResumeBaseInfo
import com.alpha.lagouapk.bean.ResumeBean
import com.alpha.lagouapk.manager.CUserManager
import com.alpha.lagouapk.manager.ResumeManager
import com.alpha.lagouapk.tools.IntentUtils
import com.alpha.lagouapk.tools.IntentUtils.startLagouMainActivity
import com.alpha.lagouapk.tools.IntentUtils.startSelectRole
import com.alpha.lagouapk.ui.login.SelectRoleActivity
import com.common.manager.JanusManager
import com.common.utils.SpUserClearUtils
import com.common.webview.LagouWebviewActivity
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.commonlibs.utils.ActivityStackHelper
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse

/**
 * 总开关关闭：走之前老流程
 * 学生身份：走之前的老流程
 */
fun registerProcess(context: Context, listener: RegisterProcessListener?, schemaUrl: String?){
    var isStudentIdentity = CUserManager.getInstance().cUserIdentity == ResumeBaseInfo.STUDENT_INT
    if (isStudentIdentity){
        listener?.onRegisterOldProcess()
    }else{
        //判断是否已达到名片标准，若达到，则直接进入首页，否则进入名片编辑页
        ApiService.api.resumeBasic.enqueue(object : BaseCallback<ResumeBaseInfo>(false){
            override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<ResumeBaseInfo>?) {
//                if (context is BaseActivity && context.isActivityDestroyed){
//                   return
//                }
                val resumeBaseInfo : ResumeBaseInfo? =  response?.content
                if (resumeBaseInfo == null){
                    listener?.onRegisterOldProcess()
                }else{
                    if (resumeBaseInfo.isPostcardPerfect){
                        //已达到名片标准，进入首页
                        enterMainActivity(context,resumeBaseInfo.isCanDeliver, schemaUrl)
                    }else{
                        //未达到名片标准，进入身份选择页
                        startSelectRole(SelectRoleActivity.TYPE_ACCOUNT_LOGIN, context)
                    }
                }
            }
            override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<ResumeBaseInfo>?) {
                super.onFailure(request, response)
                if (context is BaseActivity && context.isActivityDestroyed){
                    return
                }
                startSelectRole(SelectRoleActivity.TYPE_ACCOUNT_LOGIN, context)
            }
        })
    }
}

fun enterMainActivity(context:Context,showFirstDeliverPride:Boolean = false){
    enterMainActivity(context, showFirstDeliverPride, null)
}

fun enterMainActivity(context:Context,showFirstDeliverPride:Boolean = false, schemaUrl: String?){
    SpUserClearUtils.getInstance().markFirstDeliverPrizeByPostcard(showFirstDeliverPride)
    startLagouMainActivity(context)
    if (TextUtils.isEmpty(schemaUrl)){
        ActivityStackHelper.clearAndFinishOtherActivity(LagouMainPage::class.java)
        return
    }
    IntentUtils.startLagouWebViewActivity(context, schemaUrl)
    ActivityStackHelper.clearAndFinishOtherActivity(LagouMainPage::class.java, LagouWebviewActivity::class.java)
}

/**
 * 同步简历，并进入教育经历编辑页
 */
fun syncResumeAndEnterGuideEducation(context: Context){
    if (context is BaseActivity && context.isActivityEnable){
        context.showLoading()
    }
    ApiService.api.resumeInfo.enqueue(object:BaseCallback<ResumeBean>(false){
        override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<ResumeBean>?) {
            if (context is BaseActivity && context.isActivityEnable){
                context.hideLoading()
            }
            response?.content?.let {
                ResumeManager.getInstance().resumeBean = it
            }
            IntentUtils.startGuideEducationActivity(context,GuideEducationActivity.FROM_POSTCARD)
        }
        override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<ResumeBean>) {
            if (context is BaseActivity && context.isActivityEnable){
                context.hideLoading()
            }
            IntentUtils.startGuideEducationActivity(context,GuideEducationActivity.FROM_POSTCARD)
        }
    })
}


/**
 * 检测用户名片是否完善
 */
fun checkIsPostcardPerfect(callback:CheckIsPostcardPerfectCallback){
    ApiService.api.resumeBasic.enqueue(object : BaseCallback<ResumeBaseInfo>(false){
        override fun onSuccess(p0: HttpRequest<*>?, response: HttpResponse<ResumeBaseInfo>?) {
            callback.onPostcardPerfect(response?.content?.isPostcardPerfect ?: false,response?.content?.isCanDeliver ?: false)
        }

        override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<ResumeBaseInfo>?) {
            callback.onPostcardPerfect(isPerfect = false, isCanDeliver = false)
        }
    })
}

interface RegisterProcessListener{
    //老流程回调
    fun onRegisterOldProcess()
}
interface CheckIsPostcardPerfectCallback{
    fun onPostcardPerfect(isPerfect:Boolean,isCanDeliver:Boolean)
}