package com.alpha.lagouapk.ui.login.guidePerfect

import android.text.TextUtils
import android.view.View
import androidx.recyclerview.widget.LinearLayoutManager
import com.ApiService
import com.alpha.lagouapk.R
import com.alpha.lagouapk.tools.IntentUtils
import com.alpha.lagouapk.ui.login.bean.PositionTypeAndTagsBean
import com.alpha.lagouapk.ui.position.PositionTypeDefaultFragment
import com.alpha.lagouapk.widget.decoration.DividerItemDecoration
import com.common.afterTextChanged
import com.common.inter.OnItemClickListener
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.commonlibs.utils.FragmentUtils
import com.lagou.commonlibs.utils.ListUtil
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.android.synthetic.main.activity_select_position_tags.*

/**
 * 选择职位三级类和标签
 * 展示样式如下
 * 职位名称
 * 一级类-二级类-三级类
 * 标签
 */
class SelectPositionTypeAndTagsActivity : BaseActivity() {
    private var data: ArrayList<PositionTypeAndTagsBean>? = null
    private lateinit var adapter: SelectPositionAndTagsAdapter
    private var positionTypeDefaultFragment: PositionTypeDefaultFragment? = null

    /***
     * 场景区分
     * 求职期望: "EXPECT_JOB"
     * 工作经历："WORK_EXPERIENCE"
     */
    private var fromSource: String?  = PositionTypeAndTagsBean.WORK_EXPERIENCE

    companion object {
        //来源场景
        const val KEY_SOURCE = "key_source"
    }

    override fun getContentLayoutId() = R.layout.activity_select_position_tags
    override fun init() {
        fromSource = intent?.getStringExtra(KEY_SOURCE)
        if (fromSource == null) fromSource = PositionTypeAndTagsBean.WORK_EXPERIENCE

        mTitleBar.setLeftImgListener { finish() }
        mRecyclerView.layoutManager = LinearLayoutManager(this)
        mRecyclerView.addItemDecoration(
            DividerItemDecoration(
                this,
                DividerItemDecoration.HORIZONTAL_LIST,
                ResourceUtils.getDimenSize(R.dimen.dp_05),
                ResourceUtils.getColor(R.color.color_e7e9eb)
            )
        )
        adapter = SelectPositionAndTagsAdapter(this)
        adapter.setOnItemClickListener(object : OnItemClickListener<PositionTypeAndTagsBean> {
            override fun onItemClickListener(
                itemView: View,
                position: Int,
                bean: PositionTypeAndTagsBean?
            ) {
                val data = intent.putExtra(IntentUtils.INTENT_KEY_DATA, bean)
                setResult(RESULT_OK, data)
                finish()
            }
        })
        mRecyclerView.adapter = adapter
        mInputPosition.apply {
            afterTextChanged { s ->
                requestData(s?.toString())
            }

            var positionType = intent.getStringExtra(IntentUtils.INTENT_KEY_POSITION_TYPE)
            setText(positionType)
            if (TextUtils.isEmpty(positionType)){
                handleSuggestPage(positionType)
            }
            requestFocus()
        }

        tvPositionName.text = if (isExceptSource()) "选择你的职位类型" else "填写职位名称"
        mInputPosition.hint = if (isExceptSource()) "搜索职位类型" else "填写职位名称"
    }

    /**
     * 根据输入词请求数据
     */
    private fun requestData(keyword: String?) {
        //1 联想词为空直接使用本地默认的选项，同时置空上次联想词数据
        if (TextUtils.isEmpty(keyword)){
            data = null
            handleSuggestPage(keyword)
            return
        }
        ApiService.api.fetchPositionTypesAndTags(keyword, fromSource).enqueue(object :
            BaseCallback<List<PositionTypeAndTagsBean>>(false) {
            override fun onSuccess(
                request: HttpRequest<*>?,
                response: HttpResponse<List<PositionTypeAndTagsBean>>?) {
                data = response?.content as? ArrayList<PositionTypeAndTagsBean>?
                handleSuggestPage(keyword)
            }
        })
    }

    private fun handleSuggestPage(keyword: String?) {
        //1 能够推荐出来suggest选项，直接显示出来
        if (!TextUtils.isEmpty(keyword) && !ListUtil.isEmpty(data)) {
            mRecyclerView.visibility = View.VISIBLE
            frameDefault.visibility = View.GONE
            adapter.setData(keyword, data)
            return
        }

        //2 推荐不出来联想词，直接使用现有的所有职类列表，提供用户选择
        data = null
        mRecyclerView.visibility = View.GONE
        frameDefault.visibility = View.VISIBLE
        if (positionTypeDefaultFragment == null) {
            positionTypeDefaultFragment = PositionTypeDefaultFragment.getInstance()
            FragmentUtils.addFragmentAtOnce(
                supportFragmentManager,
                positionTypeDefaultFragment,
                frameDefault.id
            )
        }
    }

    override fun onBackPressed() {
        //super.onBackPressed()
        finish()
    }

    /***
     * 是否是：求职期望场景 fromSource = EXPECT_JOB
     */
    fun isExceptSource(): Boolean {
        return TextUtils.equals(PositionTypeAndTagsBean.EXPECT_JOB, fromSource)
    }
}