package com.alpha.lagouapk.ui.position

import android.view.View
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.lifecycle.lifecycleScope
import com.ApiService
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.AnswerQuestionInfoBean
import com.alpha.lagouapk.bean.PositionDetailBean
import com.alpha.lagouapk.bean.ResumeSummaryBean
import com.alpha.lagouapk.bean.deliver.DeliverBean
import com.alpha.lagouapk.bean.deliver.DeliverReqBean
import com.alpha.lagouapk.tools.DialogUtils
import com.alpha.lagouapk.tools.IntentUtils
import com.alpha.lagouapk.ui.dialog.AnswerQuestionDialog
import com.common.FROM_POSITION_DETAIL_SCAN
import com.common.config.StatusCode
import com.lagou.annotation.aspect.SingleClick
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.baselib.privacy.KEY_CAMERA_SCENE_SCAN
import com.lagou.commonlibs.ui.dialog.CommonDialog
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.async
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

/**
 * 职位详情页 答题闯关活动
 */
class PositionDetailAnswerQuestionLogic(val fragment: PositionDetailFragment):View.OnClickListener{
    private  var mLLBottomButtonNormal:LinearLayout = fragment.findViewById(R.id.jd_chat_post_normal)
    private  var mLLBottomButtonActivity:LinearLayout = fragment.findViewById(R.id.jd_chat_post_answer_questions)
    private var mTvPostNormal:TextView = fragment.mBtnPost
    private var mTvPostActivity:TextView = fragment.findViewById(R.id.tv_answer_question_post)
    private var mTvPostAndAnswer:TextView = fragment.findViewById(R.id.tv_answer_question)
    private var mIvLabel:ImageView = fragment.findViewById(R.id.iv_jd_go_through_label)
    private var mPositionDetailData:PositionDetailBean? = null
    private var mState:Int = DELIVER_NOT_ANSWER_NOT
    private var mAnswerState:Int = -1
    private var mPostState:Int = -1
    companion object{
        //答完题
        const val STATE_ANSWER_FINISH = 2
        //答题中
        const val STATE_ANSWER_PART = 1
        //未答题
        const val STATE_ANSWER_NOT = 0

        //已投递，已答题
        const val DELIVER_FINISH_ANSWER_FINISH = 3
        //已投递，未答题
        const val DELIVER_FINISH_ANSWER_NOT = 4
        //已投递，答题中
        const val DELIVER_FINISH_ANSWER_ING = 5
        //未投递,已答题
        const val DELIVER_NOT_ANSWER_FINISH = 6
        //未投递,未答题
        const val DELIVER_NOT_ANSWER_NOT = 7
    }

    init {
        mTvPostActivity.setOnClickListener(this)
        mTvPostAndAnswer.setOnClickListener(this)
    }

    fun setData(bean: PositionDetailBean,postState:Int){
        bean.isOverlord = bean.isOverlord
        if (bean.isOverlord.not()){
            //不是霸王面
            mLLBottomButtonNormal.visibility = View.VISIBLE
            mLLBottomButtonActivity.visibility = View.GONE
            mIvLabel.visibility = View.GONE
            return
        }
        this.mPositionDetailData = bean
        mAnswerState = bean.answerStatus
        mPostState = postState
        mLLBottomButtonNormal.visibility = View.GONE
        mLLBottomButtonActivity.visibility = View.VISIBLE
        mIvLabel.visibility = View.VISIBLE
        mState = getState(mAnswerState)
        setBottomButtonByState()
        updatePostState()
    }

    /**
     * 设置底部按钮
     */
    private fun setBottomButtonByState() {
        //设置左侧按钮
        updateLeftButton()
        //设置右侧按钮
       updateRightButton()
    }

    private fun updateRightButton(){
        when(mState){
            DELIVER_FINISH_ANSWER_FINISH -> {
                mTvPostActivity.isEnabled = false
                mTvPostAndAnswer.visibility = View.GONE
                mTvPostActivity.text = ResourceUtils.getString(R.string.deliver_finish_answer_finish)
            }
            DELIVER_FINISH_ANSWER_NOT ->  mTvPostAndAnswer.text = ResourceUtils.getString(R.string.go_through_answer_question)
            DELIVER_FINISH_ANSWER_ING -> mTvPostAndAnswer.text = ResourceUtils.getString(R.string.go_through_answer_question_continue)
            DELIVER_NOT_ANSWER_FINISH,DELIVER_NOT_ANSWER_NOT -> mTvPostAndAnswer.text = ResourceUtils.getString(R.string.deliver_and_answer_question_go_through)
        }
    }

    private fun updateLeftButton(){
        when(mPostState){
            PositionDetailFragment.BUTTON_POST_STATE_DELIVER_FINISH -> {
                //已投递
                mTvPostActivity.text = ResourceUtils.getString(R.string.deliver_successed)
                mTvPostActivity.isEnabled = false
            }
            PositionDetailFragment.BUTTON_POST_STATE_DELIVER_INIT -> {
                //未投递
                mTvPostActivity.text = ResourceUtils.getString(R.string.send_resume)
                mTvPostActivity.isEnabled = true
            }
            PositionDetailFragment.BUTTON_POST_STATE_DELIVER_ING -> {
                //投递中
                mTvPostActivity.isEnabled = false
                mTvPostActivity.text = ResourceUtils.getString(R.string.deliver_loading)
            }
            PositionDetailFragment.BUTTON_POST_STATE_OFFLINE -> {
                //已下线
                mTvPostActivity.text = ResourceUtils.getString(R.string.lagou_position_noenable_txt)
                mTvPostActivity.isEnabled = false
                mTvPostAndAnswer.visibility = View.GONE
            }

        }
    }

    private fun getState(answerState:Int):Int {
        var state:Int = -1
        when(mPostState){
            PositionDetailFragment.BUTTON_POST_STATE_DELIVER_FINISH -> {
                //已投递
                when(answerState){
                    STATE_ANSWER_NOT -> state = DELIVER_FINISH_ANSWER_NOT
                    STATE_ANSWER_PART -> state =  DELIVER_FINISH_ANSWER_ING
                    STATE_ANSWER_FINISH -> state = DELIVER_FINISH_ANSWER_FINISH
                }
            }
            PositionDetailFragment.BUTTON_POST_STATE_DELIVER_INIT ->{
                //未投递
                when(answerState){
                    STATE_ANSWER_NOT -> state = DELIVER_NOT_ANSWER_NOT
                    STATE_ANSWER_FINISH -> state = DELIVER_NOT_ANSWER_FINISH
                }
            }
        }
        return state
    }
    @SingleClick
    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.tv_answer_question_post -> {//立即投递
                mTvPostNormal.performClick()
            }
            R.id.tv_answer_question -> {//答题
                when(mState){
                    //已投递，答题中，进入到扫码页

                    DELIVER_FINISH_ANSWER_ING ->  IntentUtils.startCaptureBarcodeActivity(fragment.detailActivity, "","前往PC端输入网址：e.lagou.com，将二维码放入框内，即可进入答题页",FROM_POSITION_DETAIL_SCAN, KEY_CAMERA_SCENE_SCAN,mPositionDetailData?.positionId ?: 0)
                    DELIVER_NOT_ANSWER_FINISH, DELIVER_NOT_ANSWER_NOT -> {
                        //未投递，先校验是否可投递
                        mTvPostAndAnswer.isEnabled = false
                        checkAndShowDialog()
                    }
                    DELIVER_FINISH_ANSWER_NOT ->{
                        //已投递，获取答题信息
                        fragment.lifecycleScope.launch(Dispatchers.Main) {
                           val answerInfo = getAnswerInfo(false) ?: return@launch
                            showDialog(null,answerInfo!!)
                        }
                    }
                }
            }
        }
    }

    /**
     * 请求简历与答题信息，展示弹窗
     */
    private fun prepareShowDialog(){
        fragment.lifecycleScope.launch(Dispatchers.Main) {
            val resumeJob =  async { getResumeList() }
            val answerJob =  async { getAnswerInfo(true) }
            val resumeList =  resumeJob.await()
            val answerInfo = answerJob.await()
            mTvPostAndAnswer.isEnabled = true
            if (resumeList == null || answerInfo == null){
                return@launch
            }
            showDialog(resumeList.filterNotNull(),answerInfo!!)
        }
    }

    /**
     * 先检测是否可投递，然后再投递
     */
    private fun checkAndShowDialog() {
        val reqBean = DeliverReqBean()
        reqBean.positionId = mPositionDetailData?.positionId?.toString() ?: "0"
        ApiService.api.deliverCheck(reqBean).protect().enqueue(object : BaseCallback<DeliverBean?>(false){
            override fun onSuccess(p0: HttpRequest<*>?, p1: HttpResponse<DeliverBean?>?) {
                if (fragment.isActivityEnable.not()){
                    return
                }
                prepareShowDialog()
            }

            override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<DeliverBean?>?) {
                super.onFailure(request, response)
                mTvPostAndAnswer.isEnabled = true
                if (fragment.isActivityEnable.not() || response == null){
                    return
                }
                val state = response.state
                val message = response.message
                var dialogFragment = when(state){
                    //职位已下线、职位不存在
                    StatusCode.POSITION_OUT_DATE,StatusCode.POSITION_NOT_EXIST -> DialogUtils.getNoPositionDialog(fragment.detailActivity)
                    // 用户信息与投递职位不匹配 // 提示: 后端会依据不同情况构造内容。 (注：对应TCP状态码 8007, 8009~8014)
                    StatusCode.RESUME_NOT_MATCH -> DialogUtils.getResumeNotMatch(message,object: CommonDialog.DismissListener {
                        override fun onDialogDismiss(requestCode: Int, resultCode: Int) {
                           if (resultCode == CommonDialog.DISMISS_RESULT_OK){
                               prepareShowDialog()
                           }
                        }
                    })
                    else -> DialogUtils.getDeliveryFailDialog(fragment.detailActivity)
                }
                if (fragment.isActivityEnable) {
                    dialogFragment?.show(fragment.detailActivity.supportFragmentManager, "PositionErrorDialog")
                }
            }
        })

    }

    /**
     * 获取简历信息
     */
    private suspend fun getResumeList():MutableList<ResumeSummaryBean?>?{
       return withContext(Dispatchers.IO){
           ApiService.api.fetchResumeList()
               .callback(object:BaseCallback<MutableList<ResumeSummaryBean?>>(){ override fun onSuccess(p0: HttpRequest<*>?, p1: HttpResponse<MutableList<ResumeSummaryBean?>>?) {} })
               .execute()?.content
        }
    }

    /**
     * 答题信息
     * @param isDeliverAndAnswer 默认false,true表示选择了投递并答题闯关
     */
    private suspend fun getAnswerInfo(isDeliverAndAnswer:Boolean):AnswerQuestionInfoBean?{
        if (mPositionDetailData == null){
            return null
        }
        return withContext(Dispatchers.IO) {
            ApiService.api.fetchAnswerQuestionInfo(mPositionDetailData!!.positionId,isDeliverAndAnswer)
                .callback(object : BaseCallback<AnswerQuestionInfoBean>() {
                    override fun onSuccess(p0: HttpRequest<*>?, p1: HttpResponse<AnswerQuestionInfoBean>?) {} })
                .execute()?.content
        }
    }

    /**
     * 展示答题弹窗
     */
    private fun showDialog(resumeList:List<ResumeSummaryBean>?,answerQuestionInfo:AnswerQuestionInfoBean){
        val answerState = if (answerQuestionInfo.hasAnswer) STATE_ANSWER_FINISH else STATE_ANSWER_NOT
        val state = getState(answerState)
        val positionId = mPositionDetailData?.positionId ?:0
        val publisherId = mPositionDetailData?.publisherId ?: 0
        val dialog =  AnswerQuestionDialog.getInstance(state,resumeList,answerQuestionInfo,positionId,publisherId,fragment.mExpectJobId,fragment.setDeliverFrom())
        dialog.setOnDeliverFinishListener(object: AnswerQuestionDialog.OnActionListener {
            override fun onDeliverFinish() {
                //投递完成，更新按钮
                updatePostState(PositionDetailFragment.BUTTON_POST_STATE_DELIVER_FINISH)
            }

            override fun onScan() {
                val desText = "前往PC端输入网址：e.lagou.com，将二维码放入框内，即可进入答题页"
                IntentUtils.startCaptureBarcodeActivity(fragment.detailActivity, "",desText, FROM_POSITION_DETAIL_SCAN,KEY_CAMERA_SCENE_SCAN,mPositionDetailData?.positionId ?: 0)
            }

        })
        //弹出选择简历对话框
        if (!dialog.isAdded && dialog.dialog?.isShowing != true) {
            dialog.show(fragment.baseActivity.supportFragmentManager,"")
        }
    }

    //投递按钮状态
    fun updatePostState(postState: Int = mPostState) {
        if (mPositionDetailData?.isOverlord != true){
            return
        }
        mPostState = postState
        if (postState == PositionDetailFragment.BUTTON_POST_STATE_DELIVER_FINISH){
            //左侧按钮投递导致了状态发生变化
            mState = getState(mAnswerState)
            setBottomButtonByState()
        }else{
            updateLeftButton()
        }

    }
}