package com.alpha.lagouapk.ui.position

import android.content.Context
import android.graphics.Color
import android.text.Html
import android.text.TextUtils
import android.util.AttributeSet
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.widget.FrameLayout
import android.widget.RelativeLayout
import android.widget.TextView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.PositionDetailBean
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.commonlibs.imageload.ImageLoadUtils
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.utils.ShowUtils
import com.lagou.commonlibs.utils.StringUtil
import com.lagou.commonlibs.view.FlowLayout
import kotlinx.android.synthetic.main.layout_positiondetail_company_card.view.flow_com_labels
import kotlinx.android.synthetic.main.layout_positiondetail_company_card.view.iv_more
import kotlinx.android.synthetic.main.layout_positiondetail_company_card.view.jd_company_logo
import kotlinx.android.synthetic.main.layout_positiondetail_company_card.view.jd_company_name
import kotlinx.android.synthetic.main.layout_positiondetail_company_card.view.ll_interview_comment
import kotlinx.android.synthetic.main.layout_positiondetail_company_card.view.rb_interview_score
import kotlinx.android.synthetic.main.layout_positiondetail_company_card.view.tv_company_info
import kotlinx.android.synthetic.main.layout_positiondetail_company_card.view.tv_interview_comment_num
import java.util.Locale


/**
 * 职位详情页公司卡片
 */
class PositionDetailCompanyCard @JvmOverloads constructor(context: Context, attrs:AttributeSet? = null, defStyle:Int = 0):FrameLayout(context,attrs,defStyle) {
    private var isSingleLine = false
    private val DP_7 = ResourceUtils.getDimenSize(R.dimen.dp_7)
    private val DP_2_5 = ResourceUtils.getDimenSize(R.dimen.dp_2_05)
    init {
        LayoutInflater.from(context).inflate(R.layout.layout_positiondetail_company_card,this)
    }
    fun setData(bean:PositionDetailBean?,onCompanyClick:((Int) -> Unit)?){
        if (bean == null){
            return
        }
        if (bean.isDelegateCompanyHide || bean.isDelegateCompanyShowWithNotMatch) {
            visibility = View.GONE
            return
        }
        ImageLoadUtils.with(context).load(bean.logo).error(R.drawable.occupy_logo).placeholder(R.drawable.occupy_logo).into(jd_company_logo)
        jd_company_name.text = if (bean.isDelegatePosition) bean.showName else bean.companyName
        if (bean.isAuthenticated) {
            ShowUtils.setTextViewDrawableImg(jd_company_name, R.drawable.company_ic_bluev_light_small, Gravity.END)
        }
        setInterviewComment(bean)
        setCompanyCityAndFinanceStage(bean)
        setExceedListener(flow_com_labels, bean.companyLabel)
        this.setOnClickListener{
            onCompanyClick?.invoke(if (bean.isDelegateCompanyShowWithMatch) bean.lgCompanyId else bean.companyId)
        }
    }

    /**
     * 设置公司融资情况
     */
    private fun setCompanyCityAndFinanceStage(bean:PositionDetailBean) {
        val isInterviewCommentVisible = ll_interview_comment.visibility != View.GONE
        val layoutParams = tv_company_info.layoutParams as RelativeLayout.LayoutParams
        layoutParams.addRule(RelativeLayout.BELOW, if (isInterviewCommentVisible) R.id.ll_interview_comment else R.id.jd_company_name)
        tv_company_info.layoutParams = layoutParams
        val str: String = StringUtil.jointString("    ", bean.financeStage, bean.companySize, bean.industryField)
        if (!TextUtils.isEmpty(str)) {
            tv_company_info.visibility = View.VISIBLE
            tv_company_info.text = str
        } else {
            tv_company_info.visibility = View.GONE
        }
    }

    /**
     * 设置面试评价
     */
    private fun setInterviewComment(bean: PositionDetailBean) {
        if (0f == bean.companyScore * bean.interviewExperienceCountByCompany) {
            ll_interview_comment.visibility = GONE
            return
        }
        ll_interview_comment.visibility = VISIBLE
        rb_interview_score.rating = (bean.companyScore + 0.5f).toInt()
        val num = String.format(Locale.getDefault(), "共<font color= \"#FF6B47\">%d</font>条评价", bean.interviewExperienceCountByCompany)
        tv_interview_comment_num.text = Html.fromHtml(num)
        rb_interview_score.isEditable = false
    }
    private fun setExceedListener(layout: FlowLayout, texts: List<String>?) {
        isSingleLine = true
        addLabelView(layout, texts)
        layout.setIExceedMaxNum { isExceed ->
            if (isSingleLine && isExceed) {
                if (iv_more.visibility != VISIBLE) {
                    AnalyticHelper.getInstance().sendShowInfo("ep00", "a1skd")
                }
                iv_more.visibility = VISIBLE
            }
        }
        iv_more.setOnClickListener { v ->
            isSingleLine = false
            layout.setMaxLines(3)
            layout.requestLayout()
            iv_more.visibility = GONE
            AnalyticHelper.getInstance().sendClickInfo("ep00", "a1skd")
        }
    }
    private fun addLabelView(layout: FlowLayout, texts: List<String>?) {
        if (texts.isNullOrEmpty()) {
            layout.visibility = GONE
            return
        }
        layout.setMaxLines(1)
        layout.visibility = VISIBLE
        layout.removeAllViews()
        var i = 0
        val size = texts.size
        while (i < size) {
            val s = texts[i]
            val labelView: TextView = generateNewLabelView()
            labelView.visibility = VISIBLE
            labelView.text = s
            layout.addView(labelView)
            i++
        }
    }
    private fun generateNewLabelView(): TextView {
        val labelView = TextView(context)
        labelView.setTextColor(Color.parseColor("#999999"))
        labelView.textSize = 12f
        labelView.setBackgroundResource(R.drawable.bg_label_e8e9eb)
        labelView.maxLines = 1
        labelView.setPadding(DP_7, DP_2_5, DP_7, DP_2_5)
        return labelView
    }
}