package com.alpha.lagouapk.ui.position.filter

import android.content.Context
import androidx.recyclerview.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.PositionInfo
import com.alpha.lagouapk.bean.SearchBean
import com.alpha.lagouapk.ui.home.choice.PositionRecType
import com.lagou.commonlibs.utils.DensityUtil
import com.lagou.commonlibs.utils.DeviceUtils
import com.lagou.commonlibs.utils.ShowUtils
import kotlinx.android.synthetic.main.card_position_rec_header.view.*
import kotlinx.android.synthetic.main.position_search_filter.view.*

/**
 * @author :  Alfred
 * @date   : 2019-07-23 17:22
 * 校园招聘适配器
 *
 * 写的很丑陋,用这种方案之前没有考虑到点击筛选项 自动滑动到顶部吸顶 当数据很少不满一屏的时候 是不能滑动的 导致就没法实现那种交互  所以底部加白色的footer 解决方案不够优雅
 * 由于Searchlayout 是一个布局 顶部的筛选框 跟下面的布局是一个view 没有分离 没有用PopupWindow 所以用约束布局那一套 就没法用 不是浮在上面的 当展开筛选项的时候 下面的内容就被顶下去了
 * 之后改造Searchlayout
 */
class SchoolRecruitmentAdapter(val context: Context, positionList: MutableList<PositionInfo>,val type: Int, searchBean: SearchBean) : PositionFilterAdapter(context, positionList, type, searchBean) {

    private var inflater: LayoutInflater = LayoutInflater.from(context)
    private var isCitySelected: Boolean = false
    private var isPositionSelected: Boolean = false
    private var isCompanySelected: Boolean = false
    private var isSortSelected: Boolean = false

    private var title: String = ""
    private var subtitle: String = ""

    /**
     * 设置header里的标题和副标题
     */
    fun setHeaderContent(title:String, subtitle:String) {
        this.title = title
        this.subtitle = subtitle
    }

    override fun getItemCount(): Int {
        return when {
            //列表为空 header + 筛选项 + 空图片
            mPositionList.isEmpty() -> 3
            // 列表个数小于5 不满一屏 不能滑动 底部加footer  header + 筛选项 + footer + 列表
            mPositionList.size < 5 -> super.getItemCount() + 3
            // 内容满一屏 可以滑动 header + 筛选项 + 列表
            else -> super.getItemCount() + 2
        }
    }

    override fun getItemViewType(position: Int): Int {
        return when (position) {
            0 -> VIEW_TYPE_HEADER
            1 -> VIEW_TYPE_FILTER
            2 -> if (mPositionList.isEmpty()) VIEW_TYPE_EMPTY else VIEW_TYPE_CONTENT
            mPositionList.size + 2 -> VIEW_TYPE_FOOTER
            else -> VIEW_TYPE_CONTENT
        }
    }


    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        return when (viewType) {
            VIEW_TYPE_HEADER -> HeaderViewHolder(inflater.inflate(R.layout.card_position_rec_header, parent, false))
            VIEW_TYPE_FILTER -> FilterViewHolder(inflater.inflate(R.layout.position_search_filter, parent, false))
            VIEW_TYPE_EMPTY -> EmptyViewHolder(inflater.inflate(R.layout.item_empty_position, parent, false))
            VIEW_TYPE_FOOTER -> {
                val view = View(context)
                view.layoutParams = ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT,
                    DeviceUtils.getScreenHeight() - DensityUtil.dip2px(context, 200.0f))
                FooterViewHolder(view)
            }
            else -> super.onCreateViewHolder(parent, viewType)

        }
    }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        when (position) {
            0 -> (holder as HeaderViewHolder).bind()
            1 -> (holder as FilterViewHolder).bind()
            else -> super.onBindViewHolder(holder, position - 2)

        }
    }

    fun setTabItemTextSelected(a: Boolean, b: Boolean, c: Boolean, d: Boolean) {
        isCitySelected = a
        isPositionSelected = b
        isCompanySelected = c
        isSortSelected = d

        notifyItemChanged(1)
    }

    companion object {
        const val VIEW_TYPE_HEADER = 0
        const val VIEW_TYPE_FILTER = 1
        const val VIEW_TYPE_EMPTY = 2
        const val VIEW_TYPE_CONTENT = 3
        const val VIEW_TYPE_FOOTER = 4
    }

    inner class HeaderViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {

        fun bind() {
            itemView.iv_icon.setImageResource(R.drawable.campus_ic_campus_nor)
            itemView.tv_title.text = ShowUtils.setTextViewBoldColorSize(context, false, title, 0, 2, R.color.color_ffba0d, -1)
            itemView.tv_second_title.text = subtitle
        }
    }

    inner class FilterViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {

        fun bind() {
            itemView.search_filter_position.text = "职位"
            if (mSearchBean.getmSelectedAddress().city.cityName != null) {
                itemView.search_filter_city.text = mSearchBean.getmSelectedAddress().city.cityName
            }
            itemView.search_filter_city_warpper.setOnClickListener {
                mCallback?.invoke(0)
            }
            itemView.search_filter_position_wrapper.setOnClickListener {
                mCallback?.invoke(1)
            }
            itemView.search_filter_company_wrapper.setOnClickListener {
                mCallback?.invoke(2)
            }
            itemView.search_filter_sort_wrapper.setOnClickListener {
                mCallback?.invoke(3)
            }
            itemView.search_filter_city.isSelected = isCitySelected
            itemView.search_filter_position.isSelected = isPositionSelected
            itemView.search_filter_company.isSelected = isCompanySelected
            itemView.search_filter_sort.isSelected = isSortSelected

        }
    }

    inner class EmptyViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView)

    inner class FooterViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView)

    fun setOnSearchLayoutItemClickListener(callback: ((position: Int) -> Unit)) {
        mCallback = callback
    }

    var mCallback: ((position: Int) -> Unit)? = null
}