package com.alpha.lagouapk.ui.position.manager

import android.util.ArrayMap
import com.ApiService
import com.alpha.lagouapk.LagouApplication
import com.alpha.lagouapk.bean.RequestRecordBean
import com.common.KEY_POSITION_FEEDBACK_CONFIG
import com.common.bean.json.CommonEntryConfig.PositionFeedbackConfig
import com.lagou.baselib.util.mmkv.BaseMKVUtils
import com.lagou.baselib.util.mmkv.MKVUtils
import com.lagou.commonlibs.engine.ThreadUtil
import com.lagou.commonlibs.utils.DateUtils
import com.tencent.mmkv.MMKV

/**
 * 职位反馈记录管理类
 */
object FeedbackPositionRecordManager {

    private var mPositionMap = ArrayMap<String,Long>()
    private var config:PositionFeedbackConfig? = null
    const val SCENE_HOT_JOB = "HOT_JOB"
    const val SCENE_NEARBY_JOB = "NEARBY_JOB"
    const val SCENE_NEWEST_JOB = "NEWEST_JOB"
    const val SCENE_SEARCH = "SEARCH"
    private var mRequestRecordMap:ArrayMap<String,RequestRecordBean> = ArrayMap()
    fun init(){
        mPositionMap.clear()
        if (getSwitchState() == State.CLOSE){
            FeedbackPositionMKVUtils.getInstance().clearAll()
            return
        }
        //同步文件中的内容到内存中
        ThreadUtil.io {
            val allKeys = FeedbackPositionMKVUtils.getInstance().allKeys()
            allKeys?.forEach {
                mPositionMap[it] = FeedbackPositionMKVUtils.getInstance().decodeLong(it,0)
            }
        }
    }
    fun recordClosedPosition(positionId:Long,uniqueKey:String?,scene:String?,optionType:String?){
        report(positionId,uniqueKey,scene,optionType)
        if (getSwitchState() != State.OPEN){
            return
        }
        val closeTime = System.currentTimeMillis()
        mPositionMap[positionId.toString()] = closeTime
        FeedbackPositionMKVUtils.getInstance().encodeLong(positionId.toString(),closeTime)
    }
    private fun report(positionId:Long,uniqueKey: String?,scene:String?,optionType:String?){
        val recordBean = mRequestRecordMap[uniqueKey]
        ApiService.api.feedbackPosition(scene,positionId,optionType,recordBean?.requestUrl,recordBean?.requestParams).enqueue()
    }

    fun recordRequestInfo(uniqueKey:String,bean:RequestRecordBean){
        try {
            mRequestRecordMap[uniqueKey] = bean
        }catch (e: Exception){
            e.printStackTrace()
        }
    }

    fun checkPositionIsOverTime(positionId:Long):Boolean{
        if (isSwitchClose()){
            //开关关闭
            return true
        }
        try {
            val time:Long = mPositionMap.getOrDefault(positionId.toString(),0)
            if (DateUtils.diffDay(time, System.currentTimeMillis()) > (config?.value ?: 0)){
                return true
            }
        }catch (e: NoSuchMethodError){
            e.printStackTrace()
        }
        return false
    }

    fun isSwitchClose() = getSwitchState() == State.CLOSE;

    /**
     * 开关控制是否打开
     */
    private fun getSwitchState():State{
        if (config == null) {
            config = MKVUtils.getInstance().decodeParcelable(KEY_POSITION_FEEDBACK_CONFIG, PositionFeedbackConfig::class.java)
        }
        if (config == null){
            return State.INIT
        }
        return if(config!!.isOpen) State.OPEN else State.CLOSE
    }

    class FeedbackPositionMKVUtils private constructor():BaseMKVUtils(){
        companion object Instance{
            init {
                val mmkvFileName = "/lagou_mmkv_position_del_time"
                val application = LagouApplication.getLagouApplication()
                MMKV.initialize(application,"${application.filesDir}$mmkvFileName" )
            }
            private var mInstance: FeedbackPositionMKVUtils? = null

            @JvmStatic
            fun getInstance(): FeedbackPositionMKVUtils {
                if (mInstance == null) {
                    synchronized(FeedbackPositionMKVUtils::class.java) {
                        if (mInstance == null) {
                            mInstance = FeedbackPositionMKVUtils()
                        }
                    }
                }
                return mInstance!!
            }
        }

        override fun getMMapId() = "mmapId_record_position"
    }
    enum class State{
        INIT,
        OPEN,
        CLOSE
    }
    enum class FeedbackScene{
        HOT_JOB,//推荐/热门
        NEARBY_JOB,//附近
        NEWEST_JOB,//最新
        SEARCH//搜索
    }
}