package com.alpha.lagouapk.ui.rebuild_login

import com.lagou.socketchannel.http.HttpResponse

/**
 * @date： Crate by wangjunwei on 2023/4/17
 * @description：登录相关的状态码
 **/
class LoginStateStatus {
    companion object {
        /***
         * 需要注册
         */
        const val NEED_REGISTER = 300003

        /***
         * 用户输入手机号和sim不匹配
         */
        const val PHONE_NOT_MATCH_SIM = 300001

        /***
         * 账号或者密码不存在
         */
        const val ACCOUNT_PASSWORD_ERROR = 203002


        /***
         * 黑卡或者其它风险用户情况，不使用一键登录，还回归老的登录方式
         */
        const val ACCOUNT_BLACK_ONE = 203019
        const val ACCOUNT_RISK_TWO = 203020


        /***
         * 是否需要注册
         */
        fun isNeedRegister(state: Int): Boolean{
            return NEED_REGISTER == state
        }

        /**
         * 账号不匹配
         */
        fun isPhoneNotMatch(state: Int): Boolean {
            return PHONE_NOT_MATCH_SIM == state
        }

        /***
         * 账号或者密码错误
         */
        fun isAccountPasswordError(state: Int):Boolean {
            return ACCOUNT_PASSWORD_ERROR == state
        }

        /**
         * 黑卡或者风险用户，关闭一键登录授权页面，回归现有的登录流程
         */
        fun isOneLoginBlackRisk(state: Int): Boolean{
            return ACCOUNT_BLACK_ONE == state || ACCOUNT_RISK_TWO == state
        }

        /**
         * 是否被封禁
         */
        fun isForbiddenState(state: Int): Boolean {
            return HttpResponse.State.FORBIDDEN == state
        }

        fun isCommitClose(state: Int): Boolean {
            return HttpResponse.State.COMMIT_CLOSE == state;
        }

        /***
         * 是否变更绑定
         */
        fun isChangeBind(state: Int): Boolean {
            return HttpResponse.State.CHANGE_BIND == state
        }

    }
}