package com.alpha.lagouapk.ui.resume.ai

import android.text.TextUtils
import com.alpha.lagouapk.bean.resume.ResumeAiResponse
import com.lagou.commonlibs.engine.ThreadUtil
import com.lagou.commonlibs.utils.StringUtil
import java.util.LinkedList
import java.util.Timer
import java.util.TimerTask

/**
 * 通过操作下标偏移的方式，实现打字机效果，更适合流式逐步给出内容
 * 不确定：接口请求流式出文案，速度不定
 * 匀速：本地控制显示匀速出
 * 注意：流式给出第一条文案就要及时显示到列表，后续有更新及时更新本地匀速出的数据源
 * 保障：timer 定时器不关闭，除非本地匀速出文案结束（currentIndex >= resumeAiResponse.content.length）&接口出文案也结束的情况下（finishResponse不为空）
 */
class DelayedTextDisplayOffset(var resumeAiResponse:ResumeAiResponse,val updateUi:((ResumeAiResponse?,Boolean,Boolean)-> Unit)?) {
    /**
     * 是否有手动滑动意图，有的话中断自动滑动（smoothToNewestContent），等到生成结束的时候在滑动到最后
     */
    private var isHandSlide = false
    private var finishResponse: ResumeAiResponse? = null
    private var timer: Timer? = null
    /**
     * timer定时器管理类
     */
    private val timerList = mutableListOf<Timer?>()
    /**
     * 打字效果的当前下标
     */
    private var currentIndex: Int = 0

    /***
     * 当前偏移量，注意判断不要越界
     */
    private var offsetCount = 0

    init {
        //处理在一开始接口就返回错误，否则无法在错误信息下显示"换个问题"
        finishResponse = if (resumeAiResponse.isFinishMoe()) resumeAiResponse else null
    }
    /**
     * 流式过程中不停地更新文本，避免重复开启任务流，判断只有当当前执行流程结束时才重新激活
     */
    fun updateGoalContent(aigcRes: ResumeAiResponse) {
        if (aigcRes.isGeneratingMode()){
            resumeAiResponse = aigcRes
        }else {
            //正常结束先缓存起来等到打字效果偏移量走完了在更新
            finishResponse = aigcRes
        }
    }

    fun startDelayedTextDisplay() {
        isHandSlide = false
        displayTextWithDelay()
    }

    /**
     * 判断当次的提问回答是否结束：元数据为空或者打字过程结束
     */
    fun isDisplayTextEnd(): Boolean{
        if (TextUtils.isEmpty(resumeAiResponse.content)){
            return true
        }
        return currentIndex >= resumeAiResponse.content!!.length
    }

    private fun displayTextWithDelay() {
        timer = Timer()
        timerList.add(timer)
        timer?.scheduleAtFixedRate(object : TimerTask() {
            override fun run() {
                if (TextUtils.isEmpty(resumeAiResponse.content)) {
                    return
                }
                try {
                    //fix Attempt to invoke virtual method 'int java.lang.String.length()' on a null object reference
                    if (currentIndex < resumeAiResponse.content!!.length) {
                        // 在此处处理显示文本的逻辑，例如将 text 设置到 TextView 中显示
                        offsetCount = StringUtil.generateRandomNumber(1, 3)
                        if (currentIndex + offsetCount > resumeAiResponse.content!!.length) {
                            offsetCount = resumeAiResponse.content!!.length - currentIndex
                        }
                        val resultTxt = resumeAiResponse.content!!.substring(currentIndex, currentIndex + offsetCount)
                        val resumeAiResponse = ResumeAiHelper.createOffsetText(resultTxt, resumeAiResponse)
                        ThreadUtil.ui {
                            updateUi?.let { it(resumeAiResponse,!isHandSlide,false) }
                            currentIndex += offsetCount
                        }
                        return
                    }
                    finishResponse?.apply {
                        resumeAiResponse.contentType = this.contentType
                        resumeAiResponse.code = this.code
                        resumeAiResponse.contextId = this.contextId
                        resumeAiResponse.id = this.id
                        resumeAiResponse.filterContent = this.filterContent
                        finishResponse = null
                        ThreadUtil.ui {
                            updateUi?.let { it(resumeAiResponse,true,true) }
                            isHandSlide = false
                            timer?.cancel()
                            timerList.remove(timer)
                        }
                    }

                }catch (e:Exception){
                    e.printStackTrace()
                }
            }
        }, 0, 25)
    }

    fun removeTimers() {
        try {
            timerList.forEach { it?.cancel()}
        } catch (e: Exception) {
        }
    }

    fun setHandSlide(isHandSlide: Boolean) {
        this.isHandSlide = isHandSlide
    }

}