package com.alpha.lagouapk.ui.resume.ai

import android.content.Context
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.resume.ResumeAiResponse
import com.lagou.commonlibs.utils.ListUtil
import com.lagou.commonlibs.utils.StringUtil
import com.lagou.socketchannel.http.event.HttpEventResponse

/**
 * 简历AI生成文本适配器
 * by leon 2023.11.1
 */
class ResumeAiAdapter(private val mContext: Context,private var onImportResume:
    ((ResumeAiResponse)-> Unit)? = null) : RecyclerView.Adapter<BaseResumeAiVH>() {
    val mData:MutableList<ResumeAiResponse> = mutableListOf()
    companion object{
        const val ITEM_TYPE_EXAMPLE = 1
        const val ITEM_TYPE_REQUEST = 2
        const val ITEM_TYPE_RESPONSE = 3
        const val ITEM_TYPE_LOADING = 4
        const val ITEM_TYPE_EXCEPTION = 5
        const val ITEM_TYPE_CHANGE_AI = 6
    }

    /***
     * AIGC生成内容显示新增和更新（招呼语、用户提、正常生成、异常文案等）
     */
    fun handleAiContent(aigcRes: ResumeAiResponse?){
        //新的生成内容
        aigcRes?.apply {
            val index = mData.indexOf(this)
            //1 不包含aigcReleaseRes，直接新增
            if (index == -1){
                mData.add(aigcRes)
                notifyItemInserted(mData.size - 1)
                return
            }
            //2 包含aigcReleaseRes，根据isAppend的状态来决定是追加还是其它操作
            val currentAIGCReleaseRes = mData[index]
            ResumeAiHelper.updateContent(aigcRes,currentAIGCReleaseRes)
            notifyItemChanged(index, 0)
        }
    }

    /**
     * 获取要所有生成文本的上下文
     */
    fun getContextIds(): String? {
        if (ListUtil.isEmpty(mData)){
            return ""
        }
        val contextList = mutableListOf<String>()
        for(index in mData.indices.reversed()){
            val bean = mData[index]
            if (bean.isChangedAI){
                break
            }
            if (!TextUtils.isEmpty(bean.contextId)){
                contextList.add(0, bean.contextId!!)
            }
        }
        return StringUtil.jointString(",", contextList)
    }


    override fun getItemViewType(position: Int): Int {
        val resumeAiBean = mData[position]
        if (resumeAiBean.contentType == ResumeAiResponse.INPUT_TYPE){
            return ITEM_TYPE_REQUEST
        }
        if (resumeAiBean.contentType == ResumeAiResponse.GENERATE_TYPE){
            return ITEM_TYPE_RESPONSE
        }
        if (resumeAiBean.contentType == ResumeAiResponse.LOADING_TYPE){
            return ITEM_TYPE_LOADING
        }
        if (resumeAiBean.contentType == ResumeAiResponse.EXCEPTION_TYPE){
            return ITEM_TYPE_EXCEPTION
        }
        if (resumeAiBean.contentType == ResumeAiResponse.CHANGE_AI_TYPE){
            return ITEM_TYPE_CHANGE_AI
        }
        return ITEM_TYPE_EXAMPLE
    }

    override fun getItemCount(): Int  = mData.size

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseResumeAiVH {
        if (viewType == ITEM_TYPE_REQUEST) {
            return ResumeAiRequestVH(LayoutInflater.from(mContext).inflate(R.layout.item_resume_ai_request, parent, false))
        }
        if (viewType == ITEM_TYPE_RESPONSE){
            return ResumeAiResponseVH(LayoutInflater.from(mContext).inflate(R.layout.item_resume_ai_response, parent, false),onImportResume)
        }
        if(viewType == ITEM_TYPE_LOADING){
            return ResumeAiLoadingVH(LayoutInflater.from(mContext).inflate(R.layout.item_resume_ai_loading, parent, false))
        }
        if(viewType == ITEM_TYPE_EXCEPTION){
            return ResumeAiExceptionVH(LayoutInflater.from(mContext).inflate(R.layout.item_resume_ai_exception, parent, false))
        }
        if (viewType == ITEM_TYPE_CHANGE_AI){
            return ResumeAiChangeVH(LayoutInflater.from(mContext).inflate(R.layout.item_resume_ai_chagne,parent,false))
        }
        return ResumeAiExampleVH(LayoutInflater.from(mContext).inflate(R.layout.item_resume_ai_example,parent,false))
    }

    override fun onBindViewHolder(holder: BaseResumeAiVH, position: Int) {
        holder.onBindViewHolder(mData[position])
    }

    fun notifyItemFinish(id: Long) {
        mData.forEachIndexed { index, bean ->
            if (bean.id == id) {
                bean.code = HttpEventResponse.State.FINISH
                notifyItemChanged(index)
                return
            }
        }

    }
    /**
     * 发送文案之后，立马移除换个问题
     */
    fun removeAiChangeQuestionLast(){
        if (ListUtil.isEmpty(mData)){
            return
        }
        val lastIndex = mData.size - 1
        val bean = mData[lastIndex]
        if (bean.contentType == ResumeAiResponse.CHANGE_AI_TYPE && bean.isChangedAI.not()){
            mData.removeAt(lastIndex)
            notifyItemRemoved(lastIndex)
        }
    }
}



