package com.alpha.lagouapk.ui.resume.ai

import android.annotation.SuppressLint
import android.text.TextUtils
import android.view.MotionEvent
import android.view.View
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.recyclerview.widget.LinearLayoutManager
import com.ApiService
import com.alpha.lagouapk.LagouMainPage
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.resume.ResumeAiResponse
import com.alpha.lagouapk.tools.IntentUtils
import com.alpha.lagouapk.ui.resume.edit.v2.ResumeEditActivity
import com.blankj.utilcode.util.MapUtils
import com.common.SensitiveLogic
import com.common.afterTextChanged
import com.common.utils.KeyboardUtils
import com.common.utils.ViewUtils
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.baselib.net.httpnetback.BaseEventCallback
import com.lagou.baselib.util.BarUtil
import com.lagou.commonlibs.engine.ThreadUtil
import com.lagou.commonlibs.utils.ActivityStackHelper
import com.lagou.commonlibs.utils.LogUtils
import com.lagou.commonlibs.utils.NetUtils
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.utils.SafeTypeConvert
import com.lagou.commonlibs.utils.StringUtil
import com.lagou.commonlibs.utils.ToastUtil
import com.lagou.commonlibs.view.SpaceItemDecoration
import com.lagou.lagouee.bean.jsonbean.aigc.AIGCReleaseRes
import com.lagou.lagouee.manager.net.PositionNetManager
import com.lagou.lagouee.ui.dialog.AIGCReleaseDialog
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse
import com.lagou.socketchannel.http.event.HttpEventHelper
import com.lagou.socketchannel.http.event.HttpEventResponse
import kotlinx.android.synthetic.main.activity_resume_ai_main.*
import kotlinx.android.synthetic.main.dialog_aigc_release.recycle_ai_content
import kotlinx.android.synthetic.main.dialog_aigc_release.tv_ai_send
import java.util.ArrayList
import java.util.HashMap

/**
 * Ai简历生成页面
 */
class ResumeAiGenActivity:BaseActivity() {
    private var maxSendCount = DEFAULT_MAX_SEND_COUNT
    private lateinit var resumeAiAdapter: ResumeAiAdapter
    /**
     * 打字效果处理器
     */
    private var displayTextDisplayOffset: DelayedTextDisplayOffset? = null
    /**
     * 可以发送的次数
     */
    private var canSendCount = DEFAULT_MAX_SEND_COUNT
    /**
     * AI是否正在生成中
     */
    private var isAiGenerating = false
    private var httpRequest: HttpRequest<String>? = null
    companion object{
        const val KEY_MAX_SEND_COUNT = "max_send_count"
        const val DEFAULT_MAX_SEND_COUNT = 3
        const val EN_BOUND = 100
        const val AI_BUSY = "主人请不要着急, 先等我把话说完, 再提出新的问题"
        const val example = "Hi~，我是拉勾简历助手，很高兴为你服务。\n请简要告诉我你的基本履历，我来现场帮你写一份简历。\n\n例如: 我叫拉勾勾，毕业于北京中关村社区大学软件工程专业，曾就职于北京拉勾网络技术有限公司，担任Java开发工程师。"
    }
    override fun getContentLayoutId(): Int = R.layout.activity_resume_ai_main
    override fun isEnableCommonBarSetting() = false
    override fun init() {
        BarUtil.headerImageBarInit(this, R.id.resumeAiTitleBar)
        maxSendCount = intent?.extras?.getInt(KEY_MAX_SEND_COUNT) ?: DEFAULT_MAX_SEND_COUNT
        canSendCount = maxSendCount
        initView()
        initListener()
    }

    @SuppressLint("SetTextI18n")
    private fun initView() {
        tvMaxSendNum.text = "X $maxSendCount"
        resumeAiAdapter = ResumeAiAdapter(this,onImportResume = {
            //一键导入
            AnalyticHelper.getInstance().sendClickInfo("b1ues","d1ueu")
            exportAiToResume(it)
        })
        resumeAiRecyclerView.apply {
            layoutManager = LinearLayoutManager(this@ResumeAiGenActivity)
            addItemDecoration(SpaceItemDecoration(ResourceUtils.getDimenSize(R.dimen.dp_10),SpaceItemDecoration.SpaceOrientation.VERTICAL))
            adapter = resumeAiAdapter
        }
        //1 样例灌入
        showExample()
    }

    /**
     * 导出Ai简历到在线简历
     */
    private fun exportAiToResume(bean:ResumeAiResponse) {
        showLoading()
        ApiService.api.importResumeAi(bean.content).enqueue(object: BaseCallback<Any?>(){
            override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<Any?>?) {
                hideLoading()
                if (isActivityDestroyed) {
                    return
                }
                IntentUtils.startLagouMainActivity(this@ResumeAiGenActivity)
                IntentUtils.startResumeEditActivityFrom(this@ResumeAiGenActivity,ResumeEditActivity.FROM_RESUME_AI)
                finish()
            }

            override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<Any?>?) {
                super.onFailure(request, response)
                hideLoading()
            }
        })
    }

    private fun initListener() {
        val editLayoutParams = editResumeAiInput.layoutParams as ConstraintLayout.LayoutParams
        KeyboardUtils.registerSoftInputChangedListener(this) { height: Int ->
            val defaultBottomMargin = ResourceUtils.getDimenSize(R.dimen.dp_14)
            if (height == 0) {
                //键盘隐藏
                editLayoutParams.bottomMargin = defaultBottomMargin
            } else {
                //键盘弹起
                editLayoutParams.bottomMargin = height + defaultBottomMargin
            }
            editResumeAiInput.layoutParams = editLayoutParams
        }
        ivResumeAiBeta.setOnClickListener {
            val isVisible = ivResumeAiLogo.visibility == View.VISIBLE
            if (isVisible) ViewUtils.setViewGone(ivResumeAiLogo,tvResumeAiDesc) else ViewUtils.setViewVisibility(ivResumeAiLogo,tvResumeAiDesc)
            ivResumeAiBetaArrow.rotation = if (isVisible) 90f else 180f
        }
        flResumeAiSend.setOnClickListener {
            AnalyticHelper.getInstance().sendClickInfo("b1ues","c1uet")
            if (!NetUtils.isNetworkAvailable(this@ResumeAiGenActivity)){
                ToastUtil.show(this@ResumeAiGenActivity, ResourceUtils.getString(R.string.no_network))
                return@setOnClickListener
            }
            if (canSendCount <= 0) {
                ToastUtil.show(this@ResumeAiGenActivity, "AI服务次数已达上限")
                return@setOnClickListener
            }
            //0 Ai是否正在生成中
            if (isAiGenerating) {
                ToastUtil.show(this@ResumeAiGenActivity, AI_BUSY)
                return@setOnClickListener
            }
            if  (displayTextDisplayOffset != null && displayTextDisplayOffset?.isDisplayTextEnd() == false) {
                ToastUtil.show(this@ResumeAiGenActivity, AI_BUSY)
                return@setOnClickListener
            }
            //1 空校验
            var text = editResumeAiInput.text?.toString()?.trim() ?: ""
            if (TextUtils.isEmpty(text)) {
                ToastUtil.show(this@ResumeAiGenActivity, "请输入内容")
                return@setOnClickListener
            }
            //2 超限文本截取前100个发送：先校验敏感词，通过之后继续后边的流程
            val endIndex = StringUtil.countCnIndex(text, EN_BOUND)
            val clipText = StringUtil.trimText(text, endIndex)
            isAiGenerating = true
            checkSensitiveLogic(clipText, text)
        }
        //recycleview列表滚动监听，人一旦触碰到屏幕终止列表自动滑动
        resumeAiRecyclerView.setOnTouchListener {_, event ->
            val action = event.action
            if (action == MotionEvent.ACTION_DOWN || action == MotionEvent.ACTION_MOVE) {
                // 手指按下或者滑动，人手触碰滑动开始
                displayTextDisplayOffset?.setHandSlide(true)
            }
            false
        }
        resumeAiTitleBar.setLeftImgListener{ finish() }
        editResumeAiInput.afterTextChanged {
            tvResumeAiWarn.visibility = View.GONE
        }
    }

    /***
     * 样例灌入，模拟打字的效果
     */
    private fun showExample() {
        val exampleList = ResumeAiHelper.uponTextCreate(example, ResumeAiResponse.EXAMPLE_TYPE, System.currentTimeMillis())
        if (exampleList.isEmpty()){
            return
        }
        displayTextWithDelay(exampleList.iterator())
    }

    /**
     * 展示输入与响应消息
     */
    private fun showInputAndResponse(inputTxt:String,clipText: String) {
        //1 移除最后一行是换个问题，始终保持只有一个换个问题在最底部
        resumeAiAdapter.removeAiChangeQuestionLast()
        //2.用户发的问题插入聊天列表中
        val inputMsg = ResumeAiHelper.createInputMsg(inputTxt)
        resumeAiAdapter.handleAiContent(inputMsg)
        KeyboardUtils.hideSoftInput(editResumeAiInput)
        editResumeAiInput?.setText("")
        smoothToBottom()
        //4.显示服务端响应的答案
        httpRequest = requestInput(clipText)
        httpRequest?.apply {
            //3.显示loading中状态
            resumeAiAdapter.handleAiContent(ResumeAiHelper.createLoadingMsg(tag))
        }
    }
    /***
     * Ai 响应结果更新页面
     */
    private fun requestInput(trimText: String?): HttpRequest<String>? {
        displayTextDisplayOffset = null
        return ResumeAiHelper.fetchResumeAi(trimText,resumeAiAdapter.getContextIds(),ResumeAiResponse.COMPLETE, object : BaseEventCallback<String>() {
                override fun onSuccess(httpRequest: HttpRequest<*>, httpResponse: HttpEventResponse<String>) {
                    if (httpResponse.isFinish) {
                        isAiGenerating = false
                        canSendCount--
                    }
                    if (isActivityDestroyed) {
                        return
                    }
                    onRequestCall(httpRequest, httpResponse)
                }

                override fun onFailure(httpRequest: HttpRequest<*>, httpEventResponse: HttpEventResponse<String>) {
                    //101的情况不出toast，直接在列表中以会话的形式出来
                    if (httpEventResponse.code != HttpEventResponse.State.FAIL){
                        super.onFailure(httpRequest, httpEventResponse)
                    }
                    isAiGenerating = false
                    if (isActivityDestroyed) {
                        return
                    }
                    onRequestCall(httpRequest, httpEventResponse)

                }

            override fun onClose(httpRequest: HttpRequest<*>?, httpEventResponse: HttpEventResponse<String>?) {
                super.onClose(httpRequest, httpEventResponse)
                isAiGenerating = false
            }

                private fun onRequestCall(httpRequest: HttpRequest<*>, httpResponse: HttpEventResponse<String>) {
                    tvMaxSendNum.text = "X $canSendCount"
                    flResumeAiSend.setBackgroundResource(if (canSendCount <= 0) R.drawable.shape_cccccc_r2 else R.drawable.shape_00b38a_r2)
                    val resumeAiResponse = ResumeAiHelper.createByResponse(httpResponse,httpRequest.tag,canSendCount,maxSendCount)

                    //如果是101情况，且content为空则取message，因为这个异常的文案要出现在会话列表中
                    if (resumeAiResponse.isFailMode() && TextUtils.isEmpty(resumeAiResponse.content)){
                        resumeAiResponse.content = httpResponse.message
                    }
                    resumeAiResponse.contentType = resumeAiResponse.uponCodeType()
                    if (displayTextDisplayOffset == null) {
                        displayTextDisplayOffset = DelayedTextDisplayOffset(resumeAiResponse){bean, isScrollToBottom,isFinish ->
                            resumeAiAdapter.handleAiContent(bean)
                            //如果是有手动滑动的意图，则中断当前生成内容会话的自动滑动，直到当前内容生成结束才重置isHandSlide状态
                            if (isScrollToBottom){
                                smoothToBottom()
                            }
                            //处理下是否显示换个问题，在adapter内实现该方法，前提：换个问题显示在每次AIGC生成结束之后
                            //1 未点击过换个问题时，只在AIGC生成结束之后追加一个换个问题的item
                            //2 点击过换个问题，换个问题卡片保留，在最后一行追加更换问题卡片
                            //3 最后一个AIGCS生成文案之后不展示"换个问题"
                            if (isFinish && canSendCount > 0){
                                val changeAiMsg = ResumeAiHelper.createChangeAiMsg()
                                resumeAiAdapter.handleAiContent(changeAiMsg)
                            }
                        }
                        displayTextDisplayOffset?.startDelayedTextDisplay()
                        return
                    }
                    //有新的流式文本更新
                    displayTextDisplayOffset?.updateGoalContent(resumeAiResponse)
                }
            })
    }


    private fun displayTextWithDelay(iterator: MutableIterator<ResumeAiResponse>) {
        if (iterator.hasNext().not()){
            return
        }
        ThreadUtil.uiDelay({ // 在此处处理显示文本的逻辑，例如将 text 设置到 TextView 中显示
            val aigcReleaseRes = iterator.next()
            resumeAiAdapter.handleAiContent(aigcReleaseRes)
            smoothToBottom()
            iterator.remove()
            if (iterator.hasNext().not()){
                resumeAiAdapter.notifyItemFinish(aigcReleaseRes.id)
            }
            displayTextWithDelay(iterator)
        }, 25) // 延迟时间为25毫秒
    }

    /***
     * 自动平滑到底部，方便用户看到最新的输出
     */
    private fun smoothToBottom() {
        var lastItemIndex = resumeAiAdapter.itemCount
        resumeAiRecyclerView.smoothScrollToPosition(lastItemIndex)
    }
    /***
     * 敏感词校验
     */
    private fun checkSensitiveLogic(clipText: String, originalContent: String) {
        val map = mapOf(SensitiveLogic.SensitiveKey.RESUME_WORK_POSITION_NAME_WARN.name to clipText)
        SensitiveLogic.checkExistSensitiveKey(map, object : SensitiveLogic.CheckSensitiveCallback {
            override fun checkFinish(keyList: ArrayList<String>?, keyAndErrorMsgMap: HashMap<String, String>?) {
                if (MapUtils.isNotEmpty(keyAndErrorMsgMap)) {
                    isAiGenerating = false
                    tvResumeAiWarn.visibility = View.VISIBLE
                    tvResumeAiWarn.text = StringUtil.jointString(",", keyAndErrorMsgMap?.values?.toList())
                    return
                }
                tvResumeAiWarn.visibility = View.GONE
                //敏感词校验通过，开始执行Ai生成流程
                showInputAndResponse(originalContent,clipText)
            }
        })
    }

    override fun onDestroy() {
        super.onDestroy()
        KeyboardUtils.unregisterKeyboard(this)
        HttpEventHelper.getInstance().cancel(httpRequest)
        displayTextDisplayOffset?.removeTimers()
    }

    override fun getPageId(): String {
        return "b1ues"
    }

}