package com.alpha.lagouapk.ui.resume.ai

import android.text.TextUtils
import com.ApiService
import com.alpha.lagouapk.bean.resume.ResumeAiResponse
import com.lagou.baselib.net.httpnetback.BaseEventCallback
import com.lagou.commonlibs.utils.SafeTypeConvert
import com.lagou.commonlibs.utils.StringUtil
import com.lagou.lagouee.bean.jsonbean.aigc.AIGCReleaseRes
import com.lagou.lagouee.manager.net.PositionNetManager
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.event.HttpEventHelper
import com.lagou.socketchannel.http.event.HttpEventResponse

class ResumeAiHelper {
    companion object {
        /**
         * 机器人输入文本一定的规则随机打散文本，生成打字效果依赖的数据源
         * @param example
         * @param contentType
         * @param id
         * @return
         */
        fun uponTextCreate(example: String?, contentType: Int, id: Long): MutableList<ResumeAiResponse> {
            val shuffleList = StringUtil.shuffleText(example, 1, 3)
            val list: MutableList<ResumeAiResponse> = mutableListOf()
            val size = shuffleList.size
            for (index in 0 until size) {
                val aigcReleaseRes = ResumeAiResponse()
                aigcReleaseRes.id = id
                aigcReleaseRes.content = shuffleList[index]
                aigcReleaseRes.contextId = ""
                aigcReleaseRes.contentType = contentType
                aigcReleaseRes.isAppend = true
                aigcReleaseRes.code = HttpEventResponse.State.SUCCESS
                list.add(aigcReleaseRes)
            }
            return list
        }
        fun updateContent(source: ResumeAiResponse,target:ResumeAiResponse) {
            if (target.content == null) {
                target.content = ""
            }
            target.contextId = source.contextId
            if (!source.isAppend) {
                target.content = source.content
            } else {
                target.content += source.content
            }
            target.filterContent = source.filterContent
            target.code = source.code
            target.contentType = source.contentType
            target.isLoading = source.isLoading
            target.leftCount = source.leftCount
            target.maxCount = source.maxCount
        }

        fun createInputMsg(inputTxt:String?): ResumeAiResponse {
            val inputMsg = ResumeAiResponse()
            inputMsg.id = System.currentTimeMillis()
            inputMsg.content = inputTxt
            inputMsg.contextId = ""
            inputMsg.contentType = ResumeAiResponse.INPUT_TYPE
            inputMsg.code = HttpEventResponse.State.SUCCESS
            return inputMsg
        }

        /***
         * 根据文本一以及类型构建新的ResumeAiResponse实体
         * @param txt 文本
         * aigcRes.contentType 内容类型
         * aigcRes.contextId 上下文id
         * aigcRes.id 单次请求的业务id，用于区分是否是同一组会话
         * @return
         */
        fun createOffsetText(txt: String?, aigcRes:ResumeAiResponse?): ResumeAiResponse? {
            if (TextUtils.isEmpty(txt) || aigcRes == null) {
                return null
            }
            val resumeAiResponse = ResumeAiResponse()
            resumeAiResponse.id = aigcRes.id
            resumeAiResponse.content = txt
            resumeAiResponse.contextId = aigcRes.contextId
            resumeAiResponse.contentType = aigcRes.contentType
            resumeAiResponse.isAppend = true
            resumeAiResponse.code = aigcRes.code
            resumeAiResponse.leftCount = aigcRes.leftCount
            resumeAiResponse.maxCount = aigcRes.maxCount
            return resumeAiResponse
        }

        /***
         * AICC 生成JD流式调用-全量方式，不带上下文
         * @param jobBrief JD简单描述
         * @param callback
         */
        fun fetchResumeAi(jobBrief: String?, contextIds: String?,mode:Int, callback: BaseEventCallback<*>): HttpRequest<String>? {
            var request: HttpRequest<String>? = null
            request = if (TextUtils.isEmpty(contextIds)) {
                ApiService.api.fetchResumeAi(jobBrief, mode)
            } else {
                ApiService.api.fetchResumeAi(jobBrief, mode, contextIds)
            }
            //String url = "http://172.16.123.219:12770/chat/openJd";
            // tag 赋值务必要提前，tag标识一次独立的流式请求，一次请求多次响应，用户端看到的loading、生成职位内容都是共用一套ItemView，判断是否是一个ItemView用的就是这个tag
            //String url = "http://172.16.123.219:12770/chat/openJd";
            // tag 赋值务必要提前，tag标识一次独立的流式请求，一次请求多次响应，用户端看到的loading、生成职位内容都是共用一套ItemView，判断是否是一个ItemView用的就是这个tag
            return request.tag(HttpEventHelper.getInstance().clientTag).enqueueEvent(callback)
        }

        fun createByResponse(httpResponse: HttpEventResponse<String>,tag:Any?,leftCount: Int,maxCount: Int): ResumeAiResponse {
            val resumeAiResponse = ResumeAiResponse()
            resumeAiResponse.contextId = httpResponse.contextId
            resumeAiResponse.id = SafeTypeConvert.toLong(tag)
            resumeAiResponse.code = httpResponse.code
            resumeAiResponse.content = httpResponse.content
            resumeAiResponse.filterContent = httpResponse.filterContent
            resumeAiResponse.leftCount = leftCount
            resumeAiResponse.maxCount = maxCount
            return resumeAiResponse;
        }

        /**
         * 创建loading中的消息
         */
        fun createLoadingMsg(tag: Any?):ResumeAiResponse {
            val resumeAiResponse = ResumeAiResponse()
            resumeAiResponse.id = SafeTypeConvert.toLong(tag)
            resumeAiResponse.isLoading = true
            resumeAiResponse.contentType = ResumeAiResponse.LOADING_TYPE
            return resumeAiResponse
        }

        fun createChangeAiMsg(): ResumeAiResponse {
            val changeAiMsg = ResumeAiResponse()
            changeAiMsg.id = System.currentTimeMillis()
            changeAiMsg.contentType = ResumeAiResponse.CHANGE_AI_TYPE
            changeAiMsg.isChangedAI = false
            return changeAiMsg
        }
    }
}
