package com.alpha.lagouapk.ui.resume.card

import android.animation.Animator
import android.animation.Animator.AnimatorListener
import android.animation.AnimatorSet
import android.animation.ObjectAnimator
import android.content.Context
import android.graphics.Bitmap
import android.text.TextUtils
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.ViewTreeObserver
import android.widget.RelativeLayout
import androidx.constraintlayout.widget.ConstraintLayout
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.ResumeBean
import com.alpha.lagouapk.manager.CUserInfoManager
import com.alpha.lagouapk.manager.ResumeManager
import com.alpha.lagouapk.share.ShareResumeTempView
import com.alpha.lagouapk.tools.IntentUtils
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.bean.json.ClickInfo
import com.lagou.baselib.analytic.definition.c.AID
import com.lagou.baselib.privacy.KEY_CAMERA_SCENE_SCAN
import com.lagou.commonlibs.engine.ThreadUtil
import com.lagou.commonlibs.utils.*
import kotlinx.android.synthetic.main.card_resume_edit_operate.view.*
import java.io.File


/**
 * @date： Crate by wangjunwei on 2023/7/25
 * @description：简历编辑页聚合操作，生成图片、生成附件、预览、PC编辑
 **/
class ResumeEditOperateCard : ConstraintLayout {

    init {
        LayoutInflater.from(context).inflate(R.layout.card_resume_edit_operate, this, true)
        setBackgroundResource(R.drawable.shape_f3fffc_r24_05)
    }

    constructor(context: Context) : this(context, null)
    constructor(context: Context, attrs: AttributeSet?) : this(context, attrs, 0)
    constructor(context: Context, attrs: AttributeSet?, defStyleAttr: Int) : super(
        context,
        attrs,
        defStyleAttr
    ) {
        initListener(context)
        initAnimation()
    }


    private var resumeBean: ResumeBean? = null

    /**
     * 动画垂直方向偏移的距离
     */
    private var dy = 0f
    private var mShowAnimatorSet: AnimatorSet? = null
    private var showTranslationAnimator: ObjectAnimator? = null
    private var showAlphaAnimator: ObjectAnimator? = null
    private var mHideAnimatorSet: AnimatorSet? = null
    private var hideTranslationAnimator: ObjectAnimator? = null
    private var hideAlphaAnimator: ObjectAnimator? = null
    private val DURATION = 600L

    private val DP_33_05 = ResourceUtils.getDimenSize(R.dimen.dp_3305)


    fun fillUi(resumeBean: ResumeBean?) {
        this.resumeBean = resumeBean
    }

    private fun initListener(context: Context) {
        lin_generate_image.setOnClickListener {
            //生成图片
            resumeBean?.let {
                val cardContent = ResumeManager.getInstance().parseCardTypeToArray(it)
                val tempView = ShareResumeTempView(context, it.cardType)
                tempView.setData(it, cardContent[0], cardContent[1])
                val bitmap: Bitmap =
                    ViewShotUtils.convertViewToBitmap(tempView, DeviceUtils.getScreenWidth())
                val temp: String = BitmapUtils.saveBitmapPng(
                    SDCardUtils.getImagePath(context)
                            + File.separator + CUserInfoManager.getInstance().resumeId + System.currentTimeMillis(),
                    bitmap
                )
                if (TextUtils.isEmpty(temp)) {
                    ToastUtil.show(context, ResourceUtils.getString(R.string.generate_image_fail))
                } else {
                    IntentUtils.startShareBitmapViewActivity(context, temp, 3)
                }
            }
        }
        lin_generate_attachment.setOnClickListener {
            //生成附件(智能模版选择页--模版预览页)
            IntentUtils.startResumeTemplateActivity(context)
            AnalyticHelper.getInstance().sendClickInfo("1jgm", "q1ua3")
        }
        lin_preview.setOnClickListener {
            //预览
            sendClickInfo("1jhh")
            IntentUtils.startResumePreviewActivity(context)
        }
        lin_pc_edit.setOnClickListener {
            //电脑端填写
            sendClickInfo("1jhg")
            IntentUtils.startCaptureBarcodeActivity(
                context as? BaseActivity,
                ResourceUtils.getString(R.string.scancode_show_host), KEY_CAMERA_SCENE_SCAN
            )
        }
    }

    private fun initAnimation(){
        viewTreeObserver.addOnPreDrawListener(object :
            ViewTreeObserver.OnPreDrawListener {
            override fun onPreDraw(): Boolean {
                val height = height
                if (height > 0){
                    viewTreeObserver.removeOnPreDrawListener(this)
                    val layoutParams =layoutParams as RelativeLayout.LayoutParams
                    dy = (layoutParams.bottomMargin + height).toFloat()
                    hideTranslationAnimator = ObjectAnimator.ofFloat(this@ResumeEditOperateCard, "translationY", 0f, dy)
                    mHideAnimatorSet = AnimatorSet()
                    mHideAnimatorSet?.duration = DURATION
                    mHideAnimatorSet?.playTogether(hideTranslationAnimator, hideAlphaAnimator)
                    mHideAnimatorSet?.addListener(object : Animator.AnimatorListener {
                        override fun onAnimationStart(animation: Animator?) {

                        }
                        override fun onAnimationEnd(animation: Animator?) {
                            visibility = GONE
                        }

                        override fun onAnimationCancel(animation: Animator?) {
                        }

                        override fun onAnimationRepeat(animation: Animator?) {
                        }
                    })

                    showTranslationAnimator = ObjectAnimator.ofFloat(this@ResumeEditOperateCard, "translationY", dy, 0f)
                    mShowAnimatorSet = AnimatorSet()
                    mShowAnimatorSet?.duration = DURATION
                    mShowAnimatorSet?.playTogether(showTranslationAnimator, showAlphaAnimator)
                    mShowAnimatorSet?.addListener(object : AnimatorListener {
                        override fun onAnimationStart(animation: Animator?) {
                            visibility = VISIBLE
                        }

                        override fun onAnimationEnd(animation: Animator?) {
                        }

                        override fun onAnimationCancel(animation: Animator?) {
                        }

                        override fun onAnimationRepeat(animation: Animator?) {
                        }

                    })
                }
                return true
            }
        })

        showAlphaAnimator = ObjectAnimator.ofFloat(this, "alpha", 0f, 1f)
        hideAlphaAnimator = ObjectAnimator.ofFloat(this, "alpha", 1f, 0f)
    }


    fun showAnimationDelay(){
        //之所以延迟，防止滑动到停止切换太快，隐藏动画还未执行完，又执行展示动画导致页面切换问题
        ThreadUtil.uiDelay({
            showAnimation()
        },DURATION)
    }


    fun showAnimation(){
        if (visibility == VISIBLE){
            return
        }
        if (mShowAnimatorSet?.isRunning == true){
            return
        }
        if (mHideAnimatorSet?.isRunning == true){
            //mHideAnimatorSet?.cancel()
            return
        }
        mShowAnimatorSet?.start()
    }

    fun hideAnimation(){
        if (visibility == GONE){
            return
        }
        if (mShowAnimatorSet?.isRunning == true){
            return
        }
        if (mHideAnimatorSet?.isRunning == true){
            //mShowAnimatorSet?.cancel()
            return
        }
        mHideAnimatorSet?.start()
    }

   private fun sendClickInfo(aid: String?) {
        val clickInfo = ClickInfo()
        clickInfo.addClick_props(AID.key_complete_new, AID.value_complete_new_mutong)
        clickInfo.address_id = aid
        AnalyticHelper.getInstance().sendClickInfo(clickInfo)
    }
}
