package com.alpha.lagouapk.ui.resume.edit.template

import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.ExpectJobBean
import com.alpha.lagouapk.bean.resume.ResumeTemplateBean
import com.alpha.lagouapk.manager.net.CResumeNetManager
import com.alpha.lagouapk.observer.EventTags
import com.common.integrate.pay.BusinessPayProxy
import com.common.pdfview.pdfviewer.listener.OnErrorListener
import com.common.pdfview.pdfviewer.listener.OnLoadCompleteListener
import com.common.utils.PdfUtils
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.download.DownloadManager
import com.lagou.baselib.download.GateLagouDownloadTask
import com.lagou.baselib.download.SimpleDownloadListener
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListenerImpl
import com.lagou.commonlibs.event.Event
import com.lagou.commonlibs.event.OkBus
import com.lagou.commonlibs.inter.OnSingleMethodListener
import com.lagou.commonlibs.utils.ListUtil
import com.lagou.commonlibs.utils.LogUtils
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.socketchannel.http.HttpParam
import com.lagou.socketchannel.http.HttpResponse
import com.lagou.socketchannel.http.Utils
import com.lagou.socketchannel.http.crypto.SecretHttpController
import kotlinx.android.synthetic.main.activity_resume_template_core.*
import java.io.File

/**
 * @date： Crate by wangjunwei on 2023/7/27
 * @description：跳转到简历模版核心页面：承揽模版效果预览、购买、导出等功能
 */
class ResumeTemplateCoreActivity : BaseActivity() {

    private var resumeTemplateSelectBean: ResumeTemplateBean? = null
    private var resumeTemplateList: ArrayList<ResumeTemplateBean>? = null
    private var expectJobBean: ExpectJobBean? = null
    private var expectJobList: List<ExpectJobBean>? = null

    private val paySuccessEvent = Event { vm_select_view.fillUIPaySuccess() }

    /**
     * 简历加载是否正在进行中
     */
    private var isLoading = false

    /***
     * 支付组件代理类
     */
    private var businessPayProxy: BusinessPayProxy? = null

    /**
     * 从后台回到前台是否要请求接口
     * 既要：保证当前页面的状态能够及时更新
     * 又要尽可能避免刷新过于频繁
     */
    private var isFlushRequest = true

    companion object {

        /**
         * 选择的模版
         */
        const val KEY_TEMPLATE_SELECT = "template_select"

        /**
         * 简历可选模版key，来自上个页面
         */
        const val KEY_TEMPLATE_LIST = "template_list"
    }

    override fun getContentLayoutId(): Int {
        return R.layout.activity_resume_template_core
    }

    override fun init() {
        //window?.decorView?.systemUiVisibility = View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION

        resumeTemplateSelectBean = intent?.getParcelableExtra(KEY_TEMPLATE_SELECT)
        resumeTemplateList = intent?.getParcelableArrayListExtra(KEY_TEMPLATE_LIST)
        vm_select_view.fillUi(resumeTemplateList, resumeTemplateSelectBean)
        initListener()
    }

    override fun onResume() {
        super.onResume()
        //存在编辑简历、期望的等场景（场景太多，不太好很精准），所以把请求页面数据放到onResume
        if (isFlushRequest){
            isFlushRequest = false
            fetchExpectList()
        }
    }

    fun setFlushRequest(){
        isFlushRequest = true
    }

    private fun initListener() {
        OkBus.getInstance()
            .register(EventTags.PAY_SUCCESS_RESUME_TEMPLATE_AUTOMATIC, paySuccessEvent)
        getPayProxy()?.onCreate()
        ltab_title.setLeftImgListener {
            finish()
        }
    }

    /**
     * 获取求职期望列表
     */
    private fun fetchExpectList() {
        CResumeNetManager.fetchExpectList(object :
            OnAdapterCommonListenerImpl<ArrayList<ExpectJobBean>, HttpResponse<ArrayList<ExpectJobBean>>>(this@ResumeTemplateCoreActivity) {
            override fun onSuccess(expectJobList: ArrayList<ExpectJobBean>?) {
                super.onSuccess(expectJobList)
                expectJobBean = ListUtil.first(expectJobList)
                this@ResumeTemplateCoreActivity.expectJobList = expectJobList
                //多余一个期望的情况下，才显示更换期望
                if (ListUtil.size(expectJobList) > 1){
                    ltab_title.setRightTxt(ResourceUtils.getString(R.string.change_except))
                    ltab_title.setRightTvListener {
                        //弹出更换期望弹窗
                        ResumeTemplateChangeExpectDialog.getInstance(expectJobList,
                            OnSingleMethodListener<ExpectJobBean> { t ->
                                expectJobBean = t
                                previewResume(resumeTemplateSelectBean)
                                vm_select_view.updateOtherParamsExpectId(t?.id)
                            }).show(supportFragmentManager)
                        AnalyticHelper.getInstance().sendClickInfo("t1ua6", "m1ubf")
                    }
                }
                previewResume(resumeTemplateSelectBean)
            }
        })
    }

    fun previewResume(resumeTemplateSelectBean: ResumeTemplateBean?) {
        if (isLoading){
            return
        }

        isLoading = true
        this.resumeTemplateSelectBean = resumeTemplateSelectBean
        resumeTemplateSelectBean?.let {
            showLoading()

            var url = "https://gate.lagou.com/v1/entry/onlineResume/templates/preview"
            val queries = mutableListOf<HttpParam>()
            queries.add(HttpParam("templateId", it.templateId))
            expectJobBean?.id.apply {
                queries.add(HttpParam("expectId", this))
            }
            url = Utils.addParamsToUrl(url, queries)
            downloadResume(url)
        }
    }

    fun downloadResume(url: String?) {
        DownloadManager
            .newTask(GateLagouDownloadTask(url))
            .dest(PdfUtils.getResumeTemplateCacheDir())
            .fileName(PdfUtils.getLocalPdfResumeFile(ResumeTemplateBean.getLocalResumeName(expectJobBean?.newPositionName3,
                resumeTemplateSelectBean?.templateId)).name)
            .listener(object : SimpleDownloadListener() {
                override fun onFinish(file: File) {
                    if (isActivityDestroyed) {
                        isLoading = false
                        return
                    }
                    vm_select_view.updateOtherParams(expectJobBean?.id,"${file.length() / 1024}kb", file.absolutePath)
                    loadResumePdf(file)
                }
                override fun onError(error: Int) {
                    super.onError(error)
                    isLoading = false
                    hideLoading()
                }
                override fun onCancel() {
                    super.onCancel()
                    isLoading = false
                    hideLoading()
                }
            })
            .start()
    }

    fun isLoading() = isLoading

    /**
     * 以pdf形式加载附件简历
     */
    private fun loadResumePdf(pdfFile: File) {
        pdf_view.fromFile(pdfFile)
            .onError(object : OnErrorListener {
                override fun onError(t: Throwable) {
                    LogUtils.i("ResumeTemplateCoreActivity", t.message)
                    isLoading = false
                    hideLoading()
                }
            })
            .onLoad(object : OnLoadCompleteListener {
                override fun loadComplete(nbPages: Int) {
                    LogUtils.i("ResumeTemplateCoreActivity", "nbPages = $nbPages")
                    isLoading = false
                    hideLoading()
                }
            })
            .load()
    }

    private fun getPayProxy(): BusinessPayProxy? {
        if (businessPayProxy == null) {
            businessPayProxy = BusinessPayProxy(baseActivity)
        }
        return businessPayProxy
    }

    override fun getPageId(): String {
        return "t1ua6"
    }

    override fun onDestroy() {
        super.onDestroy()
        OkBus.getInstance().unRegister(paySuccessEvent)
        getPayProxy()?.onDestroy()
    }
}