package com.alpha.lagouapk.ui.resume.edit.template

import android.app.Activity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.resume.ResumeTemplateBean
import com.common.adapter.BaseRecycleAdapter
import com.common.adapter.BaseViewHolder
import com.common.adapter.listener.OnItemClick
import com.lagou.commonlibs.imageload.ImageLoadUtils
import com.lagou.commonlibs.utils.ListUtil
import com.lagou.commonlibs.utils.ResourceUtils

/***
 * 简历智能模版核心操作适配器：购买、导出等
 */
class ResumeTemplateCoreAdapter(activity: Activity) :
    BaseRecycleAdapter<ResumeTemplateBean, ResumeTemplateCoreAdapter.ResumeTemplateCoreViewHolder>(
        activity
    ) {

    companion object {
        val DP_91 = ResourceUtils.getDimenSize(R.dimen.dp_91)
        val DP_127 = ResourceUtils.getDimenSize(R.dimen.dp_127)
        val DP_98_05 = ResourceUtils.getDimenSize(R.dimen.dp_98_05)
        val DP_138 = ResourceUtils.getDimenSize(R.dimen.dp_138)
        val DP_95_05 = ResourceUtils.getDimenSize(R.dimen.dp_95_05)
        val DP_128_05 = ResourceUtils.getDimenSize(R.dimen.dp_128_05)
        val DP_103_05 = ResourceUtils.getDimenSize(R.dimen.dp_103_05)
        val DP_145_05 = ResourceUtils.getDimenSize(R.dimen.dp_145_05)
    }


    override fun onCreateViewHolder(parent: ViewGroup, p1: Int): RecyclerView.ViewHolder {
        val itemView =
            LayoutInflater.from(mActivity).inflate(R.layout.adapter_resume_template_core, parent, false)
        return ResumeTemplateCoreViewHolder(itemView)
    }

//    fun setData(
//        list: MutableList<ResumeTemplateBean>?,
//        resumeTemplateSelect: ResumeTemplateBean?
//    ) {
//        this.mList = list
//        notifyDataSetChanged()
//
//        updateSelectTemplate(resumeTemplateSelect)
//    }

    /***
     * 首次设置数据，需要把选中的项移动到第一个
     */
    fun setDataFirst(
        list: MutableList<ResumeTemplateBean>?,
        resumeTemplateSelect: ResumeTemplateBean?
    ) {
        this.mList = list
        mList?.let {
            if (resumeTemplateSelect != null){
                it.remove(resumeTemplateSelect)
                resumeTemplateSelect.isSelected = true
                it.add(0, resumeTemplateSelect)
                notifyDataSetChanged()
                return@let
            }
        }

        notifyDataSetChanged()
        //updateSelectTemplate(resumeTemplateSelect)
    }

    /**
     *  更新选中的样式，把选中的模版滑动到屏幕可见模版的第二位（从左往右计数）
     */
    fun updateSelectTemplate(resumeTemplateBean: ResumeTemplateBean?) {
        resumeTemplateBean?.let {
            if (ListUtil.isEmpty(mList)) {
                return
            }
            //1 先清除原先选中状态
            for ((index, templateBean) in mList.filterNotNull().withIndex()) {
                if (templateBean.isSelected) {
                    templateBean.isSelected = false
                    notifyItemChanged(index)
                    continue
                }
            }
            //2 处理目标待选中的模版
            val selectIndex = mList.indexOf(it)
            if (selectIndex == -1) {
                return
            }
            mList[selectIndex].isSelected = true
            notifyItemChanged(selectIndex)
        }
    }

    inner class ResumeTemplateCoreViewHolder(itemView: View) :
        BaseViewHolder<ResumeTemplateBean>(itemView) {

        private var constrainResumeTemplateContent = itemView.findViewById<ConstraintLayout>(R.id.constrain_content)
        private var imageResumeTemplate =
            itemView.findViewById<ImageView>(R.id.image_resume_template)
        private var imageResumeTemplateSelect =
            itemView.findViewById<ImageView>(R.id.image_resume_template_select)

        override fun onBindViewHolder(
            view: View?,
            t: ResumeTemplateBean?,
            position: Int,
            onItemClick: OnItemClick<ResumeTemplateBean?>?
        ) {
            t?.let {
                ImageLoadUtils
                    .with(imageResumeTemplate.context)
                    .load(it.img)
                    .into(imageResumeTemplate)
                val contentViewParams = constrainResumeTemplateContent.layoutParams
                val imageTemplateParams = imageResumeTemplate.layoutParams

                if (it.isSelected) {
                    //已经选中
                    imageResumeTemplateSelect.visibility = View.VISIBLE
                    constrainResumeTemplateContent.setBackgroundResource(R.drawable.shape_tran_stroke1_00b38a_r4)
                    contentViewParams.width = DP_103_05
                    contentViewParams.height = DP_145_05
                    constrainResumeTemplateContent.layoutParams = contentViewParams

                    imageTemplateParams.width = DP_95_05
                    imageTemplateParams.height = DP_128_05
                    imageResumeTemplate.layoutParams = imageTemplateParams
                    return
                }

                imageResumeTemplateSelect.visibility = View.GONE
                constrainResumeTemplateContent.setBackgroundResource(R.drawable.shape_white_stroke05_f3f3f3_r4)
                contentViewParams.width = DP_98_05
                contentViewParams.height = DP_138
                constrainResumeTemplateContent.layoutParams = contentViewParams

                imageTemplateParams.width = DP_91
                imageTemplateParams.height = DP_127
                imageResumeTemplate.layoutParams = imageTemplateParams
            }

            itemView.setOnClickListener {
                onItemClick?.onClick(t, position)
            }
        }

        override fun getTitle(): String {
            return "简历模版核心操作适配器"
        }
    }
}
