package com.alpha.lagouapk.ui.resume.edit.template

import android.text.TextUtils
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.resume.ResumeTemplateBean
import com.alpha.lagouapk.bean.resume.ResumeTemplateEmailRes
import com.alpha.lagouapk.manager.net.CResumeNetManager
import com.common.widget.CustomTextWatcher
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListenerImpl
import com.lagou.commonlibs.imageload.ImageLoadUtils
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.android.synthetic.main.activity_resume_template_email.*

/**
 * @date： Crate by wangjunwei on 2023/7/31
 * @description：导出简历到邮箱
 **/
class ResumeTemplateEmailActivity : BaseActivity() {

    /**
     * 用户选择的简历模版
     */
    private var resumeTemplateSelect: ResumeTemplateBean? = null

    /***
     * 用户选择的期望id，不选的话默认服务端默认取第一个
     */
    private var exceptId: Int = 0

    /**
     * 生成智能简历文件大小
     */
    private var resumeSize: String? = null


    override fun getContentLayoutId(): Int {
        return R.layout.activity_resume_template_email
    }

    companion object {
        const val KEY_SELECT_RESUME_TEMPLATE = "select_resume_template"
        const val KEY_EXPECT_ID = "expect_id"
        const val KEY_RESUME_SIZE = "resume_size"
    }

    override fun init() {
        resumeTemplateSelect = intent.getParcelableExtra(KEY_SELECT_RESUME_TEMPLATE)
        exceptId = intent.getIntExtra(KEY_EXPECT_ID, 0)
        resumeSize = intent.getStringExtra(KEY_RESUME_SIZE)
        fillUi()
        initListener()
    }

    /**
     * 填充UI
     */
    private fun fillUi() {
        tv_resume_size.text = resumeSize
        //请求接口，获取名称、更新时间、图标 
        getResumeTemplateEmailInfo()
    }

    private fun getResumeTemplateEmailInfo() {
       CResumeNetManager.getResumeTemplateEmailInfo(resumeTemplateSelect?.templateId, exceptId,
           object: OnAdapterCommonListenerImpl<ResumeTemplateEmailRes,
                   HttpResponse<ResumeTemplateEmailRes>>(this@ResumeTemplateEmailActivity) {
               override fun onSuccess(t: ResumeTemplateEmailRes?) {
                   super.onSuccess(t)
                   t?.apply {
                       tv_attach_name.text = fileName
                       tv_resume_update_tips.text = templateUpdateTimeShow
                       ImageLoadUtils
                           .with(this@ResumeTemplateEmailActivity)
                           .load(fileIcon)
                           .into(image_attach_icon)
                   }
               }
           })
    }

    private fun initListener() {
        lbar_title.setLeftImgListener {
            finish()
        }
        lbar_title.setRightTvListener {
            //发送生成好的简历到邮箱
            CResumeNetManager.sendResumeTemplateEmail(resumeTemplateSelect, exceptId, et_input_email.text.toString(),
                object : OnAdapterCommonListenerImpl<Object, HttpResponse<Object>>(this) {
                    override fun onSuccess(t: Object?) {
                        super.onSuccess(t)
                    }
                })
        }
        et_input_email.addTextChangedListener(object : CustomTextWatcher() {
            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
                super.onTextChanged(s, start, before, count)
                if (TextUtils.isEmpty(s.toString())) {
                    lbar_title.tvRight.setTextColor(ResourceUtils.getColor(R.color.color_999999))
                    lbar_title.tvRight.isEnabled = false
                } else {
                    lbar_title.tvRight.setTextColor(ResourceUtils.getColor(R.color.color_00b38a))
                    lbar_title.tvRight.isEnabled = true
                }
            }
        })
    }

}