package com.alpha.lagouapk.ui.resume.edit.template

import android.app.Activity
import android.os.Bundle
import android.view.Gravity
import android.view.View
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.resume.ResumeTemplateBean
import com.alpha.lagouapk.bean.resume.ResumeTemplateEmailRes
import com.alpha.lagouapk.manager.net.CResumeNetManager
import com.alpha.lagouapk.tools.IntentUtils
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListenerImpl
import com.lagou.commonlibs.ui.dialog.BaseBottomSheetDialogFragment
import com.lagou.commonlibs.ui.dialog.BaseDialogFragment
import com.lagou.commonlibs.utils.lgshare.LGShareAction
import com.lagou.commonlibs.utils.lgshare.core.LGShareMedia
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.android.synthetic.main.dialog_resume_template_export.*

/**
 * @date： Crate by wangjunwei on 2023/7/31
 * @description：简历模版导出到邮箱或者微信
 **/
class ResumeTemplateExportDialog : BaseDialogFragment() {

    private var resumeSelectTemplate: ResumeTemplateBean? = null

    /**
     * 用户选择则期望id，适用于多期望的情况（为0接口端默认取第期望）
     */
    private var expectId: Int = 0

    /**
     * 生成智能简历文件大小
     */
    private var resumeSize: String? = null

    /***
     * 简历本地路径
     */
    private var resumePath: String? = null


    private lateinit var resumeTemplateCoreActivity: ResumeTemplateCoreActivity
    private var resumeTemplateEmailRes: ResumeTemplateEmailRes? = null


    override fun getLayoutId(): Int {
       return R.layout.dialog_resume_template_export
    }

    companion object {
        const val KEY_SELECT_TEMPLATE = "select_template"
        const val KEY_EXPECT_ID = "expect_id"
        const val KEY_RESUME_SIZE = "resume_size"
        const val KEY_RESUME_PATH = "resume_path"

        @JvmStatic
        fun getInstance(resumeTemplateBean: ResumeTemplateBean, expectId: Int,
                        resumeSize: String, resumePath: String?): ResumeTemplateExportDialog {
            val bundle = Bundle()
            bundle.putParcelable(KEY_SELECT_TEMPLATE, resumeTemplateBean)
            bundle.putInt(KEY_EXPECT_ID, expectId)
            bundle.putString(KEY_RESUME_SIZE, resumeSize)
            bundle.putString(KEY_RESUME_PATH, resumePath)
            val resumeTemplateExportDialog = ResumeTemplateExportDialog()
            resumeTemplateExportDialog.arguments = bundle
            return resumeTemplateExportDialog
        }
    }

    override fun initConfigAtCreate() {
        super.initConfigAtCreate()
        resumeTemplateCoreActivity = context as ResumeTemplateCoreActivity
    }

    override fun initConfigAtCreateView() {
        super.initConfigAtCreateView()
        //确保应用布局页面能够通到导航栏，并设置导航栏的颜色为透明，保证通到该部分的用用布局能够可
        dialog?.window?.decorView?.systemUiVisibility = View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
    }

    override fun fillUI() {
        super.fillUI()
        arguments?.apply {
            resumeSelectTemplate = getParcelable(KEY_SELECT_TEMPLATE)
            expectId = getInt(KEY_EXPECT_ID)
            resumeSize = getString(KEY_RESUME_SIZE)
            resumePath = getString(KEY_RESUME_PATH)

            getResumeTemplateEmailInfo()
        }
    }

    override fun initListener() {
        super.initListener()
        tv_email_shareboard.setOnClickListener {
            //导出简历到邮箱-跳转到新的页面处理
            IntentUtils.startResumeTemplateEmail(context, resumeSelectTemplate, expectId, resumeSize)
            dismiss()
        }
        tv_wx_shareboard.setOnClickListener {
            //导出简历到微信
            LGShareAction(context as Activity)
                .setShareMedia(LGShareMedia.WEIXIN)
                .setFilePath(resumePath)
                .setTitle(resumeTemplateEmailRes?.fileName)
                .shareFile()
            dismiss()
        }
    }

    private fun getResumeTemplateEmailInfo() {
        CResumeNetManager.getResumeTemplateEmailInfo(resumeSelectTemplate?.templateId, expectId,
            object: OnAdapterCommonListenerImpl<ResumeTemplateEmailRes,
                    HttpResponse<ResumeTemplateEmailRes>>(resumeTemplateCoreActivity) {
                override fun onSuccess(resumeTemplateEmailRes: ResumeTemplateEmailRes?) {
                    super.onSuccess(resumeTemplateEmailRes)
                    this@ResumeTemplateExportDialog.resumeTemplateEmailRes = resumeTemplateEmailRes
                }

                override fun isShowToast() = false
            })
    }

    override fun getGravity(): Int {
        return Gravity.BOTTOM
    }
}