package com.alpha.lagouapk.ui.resume.edit.template

import android.graphics.Rect
import android.os.Message
import android.view.View
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.resume.ResumeTemplateBean
import com.alpha.lagouapk.manager.net.CResumeNetManager
import com.alpha.lagouapk.observer.EventTags
import com.alpha.lagouapk.tools.IntentUtils
import com.common.adapter.listener.OnItemClick
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.bean.json.ClickInfo
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListenerImpl
import com.lagou.commonlibs.event.Event
import com.lagou.commonlibs.event.OkBus
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.android.synthetic.main.activity_resume_template_list.*

/**
 * @date： Crate by wangjunwei on 2023/7/27
 * @description：简历智能模版选择列表页
 **/
class ResumeTemplateListActivity : BaseActivity() {

    private lateinit var resumeTemplateAdapter: ResumeTemplateListAdapter
    private var resumeTemplateBeanList: ArrayList<ResumeTemplateBean>? = null
    private var flushResumeTemplateEvent = Event { getResumeTemplate() }

    override fun getContentLayoutId(): Int {
        return R.layout.activity_resume_template_list
    }

    override fun init() {
        initRecycleView()
        initListener()
        getResumeTemplate()
    }

    private fun initRecycleView() {
        resumeTemplateAdapter = ResumeTemplateListAdapter(this)
        recycle_view.apply {
            layoutManager = GridLayoutManager(this@ResumeTemplateListActivity, 2)
            addItemDecoration(TemplateDividerItemDecoration())
            adapter = resumeTemplateAdapter
        }
    }

    private fun initListener() {
        ltitle_bar.setLeftImgListener {
            finish()
        }
        // 选择模版调到大图模版预览页
        resumeTemplateAdapter.setOnItemClick(object : OnItemClick<ResumeTemplateBean?>{
            override fun onClick(resumeTemplateBean: ResumeTemplateBean?, position: Int) {
                IntentUtils.startResumeTemplateCoreActivity(this@ResumeTemplateListActivity
                    ,resumeTemplateBean
                    ,resumeTemplateBeanList
                )
                val clickInfo = ClickInfo()
                clickInfo.page_id = "r1ua4"
                clickInfo.address_id = "s1ua5"
                clickInfo.addClick_props("templateId", resumeTemplateBean?.templateId)
                AnalyticHelper.getInstance().sendClickInfo(clickInfo)
            }
        })

        OkBus.getInstance().register(EventTags.PAY_SUCCESS_RESUME_TEMPLATE_AUTOMATIC, flushResumeTemplateEvent)
    }

    private fun getResumeTemplate() {
        CResumeNetManager.getResumeTemplate(object :
            OnAdapterCommonListenerImpl<ArrayList<ResumeTemplateBean>, HttpResponse<ArrayList<ResumeTemplateBean>>>(this@ResumeTemplateListActivity) {
            override fun onSuccess(resumeTemplateBeanList: ArrayList<ResumeTemplateBean>?) {
                super.onSuccess(resumeTemplateBeanList)
                this@ResumeTemplateListActivity.resumeTemplateBeanList = resumeTemplateBeanList
                resumeTemplateAdapter.setData(resumeTemplateBeanList)
            }
        })
    }

    class TemplateDividerItemDecoration : RecyclerView.ItemDecoration() {
        private val DP_6 = ResourceUtils.getDimenSize(R.dimen.dp_6)
        private val DP_8 = ResourceUtils.getDimenSize(R.dimen.dp_8)
        private val DP_15 = ResourceUtils.getDimenSize(R.dimen.dp_15)
        private val DP_25 = ResourceUtils.getDimenSize(R.dimen.dp_25)

        override fun getItemOffsets(
            outRect: Rect,
            view: View,
            parent: RecyclerView,
            state: RecyclerView.State
        ) {
            //1 调用父类的方法，保证默认的ItemDecorator效果
            super.getItemOffsets(outRect, view, parent, state)

            //2 获取当前itemView在RecycleView中的位置
            val itemPosition = parent.getChildAdapterPosition(view)
            //根据位置结合设计图，做相应的处理
            //2.1 top处理 位于第一行，top = 25dp
            if (itemPosition <= 1){
                outRect.top = DP_25
            }else {
                outRect.top = DP_8
            }
            //2.2 bottom 处理
            outRect.bottom = DP_8
            //2.3 left right 处理
            if (itemPosition % 2 == 0){
                outRect.left = DP_15
                outRect.right = DP_6
            }else {
                outRect.left = DP_6
                outRect.right = DP_15
            }
        }
    }

    override fun getPageId(): String {
        return "r1ua4"
    }

    override fun onDestroy() {
        super.onDestroy()
        OkBus.getInstance().unRegister(flushResumeTemplateEvent)
    }
}
