package com.alpha.lagouapk.ui.resume.edit.template

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.resume.ResumeTemplateBean
import com.common.adapter.BaseRecycleAdapter
import com.common.adapter.BaseViewHolder
import com.common.adapter.listener.OnItemClick
import com.lagou.commonlibs.imageload.ImageLoadUtils
import com.lagou.commonlibs.utils.DeviceUtils
import com.lagou.commonlibs.utils.SafeTypeConvert

/**
 * 简历智能模版列表适配器类
 */
class ResumeTemplateListAdapter(resumeTemplateActivity: ResumeTemplateListActivity) :
    BaseRecycleAdapter<ResumeTemplateBean, ResumeTemplateListAdapter.ResumeTemplateViewHolder>(
        resumeTemplateActivity
    ) {

    private val IMAGE_WIDTH = DeviceUtils.getScreenWidth() * 0.44
    private val IMAGE_HEIGHT = IMAGE_WIDTH * 1.333

    override fun onCreateViewHolder(p0: ViewGroup, p1: Int): RecyclerView.ViewHolder {
        val itemView =
            LayoutInflater.from(mActivity).inflate(R.layout.adapter_resume_template_list, null)
        return ResumeTemplateViewHolder(itemView)
    }

    inner class ResumeTemplateViewHolder(itemView: View) :
        BaseViewHolder<ResumeTemplateBean>(itemView) {

        private val tvPayState = itemView.findViewById<TextView>(R.id.tv_pay_state)
        private val imageTemplate = itemView.findViewById<ImageView>(R.id.image_resume_template)

        override fun onBindViewHolder(
            view: View?,
            t: ResumeTemplateBean?,
            position: Int,
            onItemClick: OnItemClick<ResumeTemplateBean?>?
        ) {
            val imageParams = imageTemplate.layoutParams
            imageParams.width = SafeTypeConvert.toInt(IMAGE_WIDTH)
            imageParams.height = SafeTypeConvert.toInt(IMAGE_HEIGHT)
            imageTemplate.layoutParams = imageParams

            t?.let {
                tvPayState.text = it.payState
                if (it.payStateBackground == 0) {
                    tvPayState.background = null
                } else {
                    tvPayState.setBackgroundResource(it.payStateBackground)
                }
                ImageLoadUtils
                    .with(imageTemplate.context)
                    .load(it.img)
                    .into(imageTemplate)
            }
            imageTemplate.setOnClickListener {
                onItemClick?.onClick(t, position)
            }
        }

        override fun getTitle(): String {
            return "简历智能模版"
        }
    }
}
