package com.alpha.lagouapk.ui.resume.edit.template

import android.app.Activity
import android.content.Context
import android.graphics.Rect
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.view.View.OnClickListener
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.resume.ResumeTemplateBean
import com.alpha.lagouapk.manager.net.CResumeNetManager
import com.alpha.lagouapk.tools.DialogUtils
import com.alpha.lagouapk.tools.IntentUtils
import com.common.adapter.listener.OnItemClick
import com.common.integrate.pay.PayUseScenesKey
import com.lagou.annotation.aspect.SingleClick
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.bean.json.ClickInfo
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListenerImpl
import com.lagou.baselib.util.BaseIntentUtils.Companion.startWebView
import com.lagou.commonlibs.engine.ThreadUtil
import com.lagou.commonlibs.inter.OnSingleMethodListener
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.web.UrlConfig
import com.lagou.lagouee.bean.jsonbean.business.CreateGoodsOrderReq
import com.lagou.lagouee.manager.net.BusinessNetManager
import com.lagou.pay.common.inter.SpecialPayCallBack
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.android.synthetic.main.view_template_select.view.*
import org.json.JSONObject

/**
 * @date： Crate by wangjunwei on 2023/7/28
 * @description：简历选择面板主页面
 **/
class ResumeTemplateSelectView : ConstraintLayout {
    constructor(context: Context) : this(context, null)
    constructor(context: Context, attrs: AttributeSet?) : this(context, attrs, 0)
    constructor(context: Context, attrs: AttributeSet?, defStyleAttr: Int) : super(
        context,
        attrs,
        defStyleAttr
    ) {
        initRecycleView(context)
        initListener(context)
    }

    init {
        LayoutInflater.from(context).inflate(R.layout.view_template_select, this)
        resumeTemplateCoreActivity = context as ResumeTemplateCoreActivity
    }

    private val DP_2_05 = ResourceUtils.getDimenSize(R.dimen.dp_2_05)
    /**
     * 是否 隐藏可选择的简历模版列表
     */
    private var isCloseRecommend = true
    private var resumeTemplateBeans: ArrayList<ResumeTemplateBean>? = null

    /**
     * 当前选中的模版
     */
    private var resumeTemplateSelect: ResumeTemplateBean? = null
    private lateinit var resumeTemplateCoreAdapter: ResumeTemplateCoreAdapter
    private lateinit var layoutManager: LinearLayoutManager
    private var resumeTemplateCoreActivity: ResumeTemplateCoreActivity

    private var expectId: Int? = 0
    private var resumeSize = ""
    private var resumePath = ""

    /***
     * 导出简历到：微信或者邮箱
     */
    private var exportOnClickListener = OnClickListener {
        exportResumeTemplate()
        exportResumeTemplateAnalytic(resumeTemplateSelect, true)
    }

    /**
     * 去购买简历模版
     */
    private var payOnClickListener = OnClickListener {
        payResumeTemplate()
        exportResumeTemplateAnalytic(resumeTemplateSelect, false)
    }

    private fun initRecycleView(context: Context) {
        resumeTemplateCoreAdapter = ResumeTemplateCoreAdapter(context as Activity)
        layoutManager = LinearLayoutManager(context, LinearLayoutManager.HORIZONTAL, false)
        recycle_view.apply {
            layoutManager = this@ResumeTemplateSelectView.layoutManager
            addItemDecoration(CustomSelectItemDecoration(context))
            adapter = resumeTemplateCoreAdapter
        }
    }

    private fun initListener(context: Context) {
        //1 关闭推荐
        tv_close_recommend.setOnClickListener {
            isCloseRecommend = !isCloseRecommend

            //1.1 清除constrain_content的渐变色背景
            constrain_template_content.background =
                if (isCloseRecommend) ResourceUtils.getDrawable(R.drawable.shape_gradient_ebfffa_ffffff_tl_tr_r10) else null
            //1.2 隐藏模版列表
            recycle_view.visibility = if (!isCloseRecommend) GONE else VISIBLE
            //1.3 修改关闭推荐按钮的文本以及文本颜色
            tv_close_recommend.text = ResourceUtils.getString(
                if (!isCloseRecommend) R.string.see_more_recommend else R.string.close_recommend
            )
            //1.4 更多推荐文案隐藏和展示
            tv_more_recommend_title.text = if (isCloseRecommend) ResourceUtils.getString(R.string.more_recommend_template) else ""

            tv_close_recommend.setTextColor(
                ResourceUtils.getColor(
                    if (!isCloseRecommend) R.color.color_00b38a else R.color.color_999999
                )
            )
        }

        //2 编辑按钮
        tv_edit.setOnClickListener {
            resumeTemplateCoreActivity.setFlushRequest()
            IntentUtils.startResumeEditCommonActivity(context)
        }

        //3 服务协议点击
        tv_online_protocol.setClickableSpanListener(OnSingleMethodListener<String> { v: String? ->
            startWebView(context, UrlConfig.URL_ONLINE_PAY_PROTOCOL)
        })

        //4 切换选择简历模版
        resumeTemplateCoreAdapter?.setOnItemClick(object : OnItemClick<ResumeTemplateBean> {
            override fun onClick(templateBean: ResumeTemplateBean?, position: Int) {
                changeResumeTemplateAnalytic(templateBean)

                //正在loading简历的过程中，中断loading新的简历
                if (resumeTemplateCoreActivity.isLoading()){
                    return
                }

                resumeTemplateSelect = templateBean
                //5.1 更新选中的样式，把选中的模版滑动到屏幕可见模版的第二位（从左往右计数）
                resumeTemplateCoreAdapter.updateSelectTemplate(templateBean)
                //5.2 更新选中的Item到指定位置
                recycle_view.post {
                    val itemView =  layoutManager.findViewByPosition(position)
                    itemView?.let {
                        val offsetX = recycle_view.width / 2  -  it.left - it.width / 2
                        recycle_view.smoothScrollBy(- offsetX, 0)
                    }
//                    layoutManager.getChildAt(0)?.width?.let {
//                        layoutManager.scrollToPositionWithOffset(position, it + DP_2_05 * 2)
//                    }
                }
                //5.3 切换模版之后对于面板底部的影响
                handleSelectBottom(templateBean)

                //5.4 更新预览简历大图
                resumeTemplateCoreActivity.previewResume(templateBean)
            }
        })
    }

    /**
     * 填充显示面板
     */
    fun fillUi(
        resumeTemplateBeans: ArrayList<ResumeTemplateBean>?,
        resumeTemplateSelect: ResumeTemplateBean?
    ) {
        this.resumeTemplateBeans = resumeTemplateBeans
        this.resumeTemplateSelect = resumeTemplateSelect
        ThreadUtil.ui {
            resumeTemplateCoreAdapter?.setDataFirst(resumeTemplateBeans, resumeTemplateSelect)
            //选中用户选择的模版，并把该位置移动到屏幕可见从左往右，第一个位置
            val selectIndex = resumeTemplateBeans?.indexOf(resumeTemplateSelect)
            selectIndex?.let {
                if (it >= 0){
                    recycle_view.post {
                        layoutManager.scrollToPositionWithOffset(it, 0)
                    }
                }
            }

            //更新右边按钮的状态，导出或者购买
            handleSelectBottom(resumeTemplateSelect)
        }
    }

    /**
     * 更新期望id、附件简历带大小、附加简历路径
     */
    fun updateOtherParams(expectId: Int?, resumeSize: String, resumePath: String) {
        this.expectId = expectId
        this.resumeSize = resumeSize
        this.resumePath = resumePath
    }

    fun updateOtherParamsExpectId(expectId: Int?) {
        this.expectId = expectId
    }

    /**
     * 支付成功刷新面板UI，需要保持用户选择模版
     */
    fun fillUIPaySuccess() {
        CResumeNetManager.getResumeTemplate(object :
            OnAdapterCommonListenerImpl<ArrayList<ResumeTemplateBean>, HttpResponse<ArrayList<ResumeTemplateBean>>>(
                resumeTemplateCoreActivity
            ) {
            override fun onSuccess(resumeTemplateList: ArrayList<ResumeTemplateBean>?) {
                super.onSuccess(resumeTemplateList)
                this@ResumeTemplateSelectView.resumeTemplateBeans = resumeTemplateList
                resumeTemplateBeans?.let { it ->
                    resumeTemplateSelect?.let { resumeTemplateBean ->
                        val index = it.indexOf(resumeTemplateBean)
                        if (index == -1) {
                            return@let
                        }
                        resumeTemplateSelect = it[index]
                        resumeTemplateCoreAdapter.updateSelectTemplate(resumeTemplateSelect)

                        //更新右边按钮的状态，导出或者购买
                        handleSelectBottom(resumeTemplateSelect)
                    }
                }
            }
        })
    }

    /**
     * 选中简历模版之后面板按钮区域的显示情况处理
     */
    private fun handleSelectBottom(resumeTemplate: ResumeTemplateBean?) {
        resumeTemplate?.let {
            tv_right_operate.text = it.showText
            tv_right_operate.setBackgroundResource(it.showBackground)
            tv_right_operate.setTextColor(it.showTextColor)
            //根据不同状态配置不同的点击事件
            if (it.isCanUse) {
                tv_right_operate.setOnClickListener(exportOnClickListener)
                return
            }
            tv_right_operate.setOnClickListener(payOnClickListener)
        }
    }

    /***
     * 弹出导出简历弹窗
     */
    @SingleClick(1000)
    private fun exportResumeTemplate() {
        if (context !is BaseActivity) {
            return
        }

        DialogUtils
            .getTemplateExportDialog(resumeTemplateSelect, expectId, resumeSize, resumePath)
            .show(resumeTemplateCoreActivity.supportFragmentManager)
    }

    /**
     * 模版购买流程
     */
    @SingleClick(1000)
    private fun payResumeTemplate() {
        resumeTemplateSelect?.let {

            CResumeNetManager.getGoodsOrderToken(object :
                OnAdapterCommonListenerImpl<String, HttpResponse<String>>(resumeTemplateCoreActivity) {
                override fun onSuccess(token: String?) {
                    super.onSuccess(token)
                    val createGoodsOrderReq = CreateGoodsOrderReq(
                        token, 0, it.price, it.sellGoodsPriceId
                    )
                    //扩展信息
                    val expandJson = JSONObject()
                    expandJson.put("bizType", it.sellStrategy)
                    expandJson.put("bizNo", "TEMPLATE_" + it.templateId)
                    createGoodsOrderReq.expandInfo = expandJson.toString()

                    BusinessNetManager.createAndPayOrder(PayUseScenesKey.RESUME_TEMPLATE_AUTOMATIC_BUY,
                        createGoodsOrderReq, resumeTemplateCoreActivity, object : SpecialPayCallBack() {
                            override fun onCommonPayFail() {
                                super.onCommonPayFail()
                            }

                            override fun onFreePaySuccess() {
                                super.onFreePaySuccess()
                            }
                        })
                }
            })
        }
    }

    inner class CustomSelectItemDecoration(context: Context) : RecyclerView.ItemDecoration() {
        private val DP_10 = ResourceUtils.getDimenSize(R.dimen.dp_10)

        override fun getItemOffsets(
            outRect: Rect,
            view: View,
            parent: RecyclerView,
            state: RecyclerView.State
        ) {
            //1 调用父类的方法，保证默认的ItemDecorator效果
            super.getItemOffsets(outRect, view, parent, state)

            //2 获取当前itemView在RecycleView中的位置
            val itemPosition = parent.getChildAdapterPosition(view)
            val itemCount = state.itemCount

            //outRect.bottom = DP_10
            //根据位置结合设计图，做相应的处理
            //2.1 第一个元素 left=0 right=3.5
            if (itemPosition == 0) {
                outRect.left = 0
                outRect.right = DP_2_05
                return
            }
            //2.2最后一元素right=0
            if (itemPosition == itemCount - 1) {
                outRect.left = DP_2_05
                outRect.right = 0
                return
            }
            //2.3 中间元素
            outRect.left = DP_2_05
            outRect.right = DP_2_05
        }
    }


    /**
     * 切换模版埋点
     */
    private fun changeResumeTemplateAnalytic(templateBean: ResumeTemplateBean?) {
        val clickInfo = ClickInfo()
        clickInfo.page_id = "t1ua6"
        clickInfo.address_id = "u1ua7"
        clickInfo.addClick_props("templateId", templateBean?.templateId)
        AnalyticHelper.getInstance().sendClickInfo(clickInfo)
    }


    /**
     * 导出或者购买按钮点击
     */
    private fun exportResumeTemplateAnalytic(templateBean: ResumeTemplateBean?, isPaid: Boolean) {
        val clickInfo = ClickInfo()
        clickInfo.page_id = "t1ua6"
        clickInfo.address_id = "v1ua8"
        clickInfo.addClick_props("templateId", templateBean?.templateId)
        clickInfo.addClick_props("isPaid", isPaid)
        AnalyticHelper.getInstance().sendClickInfo(clickInfo)
    }
}

