package com.common

import android.content.Context
import android.graphics.Canvas
import android.graphics.Paint
import android.graphics.RectF
import android.util.AttributeSet
import android.view.View
import com.alpha.lagouapk.R
import com.lagou.commonlibs.utils.ResourceUtils

/**
 * 水平进度条
 */
class HorizontalProgressBar(context: Context, attrs: AttributeSet?) : View(context, attrs)  {
    private var progress = 0
    private var max = 100
    private val backgroundPaint = Paint()
    private val progressPaint = Paint()
    private val rect = RectF()

    init {
        backgroundPaint.color = ResourceUtils.getColor(R.color.color_e8e9eb)
        progressPaint.color = ResourceUtils.getColor(R.color.color_00b38a)
    }

    override fun onDraw(canvas: Canvas) {
        super.onDraw(canvas)

        val width = width.toFloat()
        val height = height.toFloat()

        // 绘制背景
        rect.set(0f, 0f, width, height)
        canvas.drawRoundRect(rect, height / 2, height / 2, backgroundPaint)

        // 绘制进度条
        val progressWidth = width * (progress.toFloat() / max)
        rect.set(0f, 0f, progressWidth, height)
        canvas.drawRoundRect(rect, height / 2, height / 2, progressPaint)
    }

    fun setProgress(progress: Int) {
        this.progress = progress
        invalidate() // 通知 View 重绘
    }

    fun setMax(max: Int) {
        this.max = max
        invalidate() // 通知 View 重绘
    }
}



