package com.common.chat.item

import android.content.Context
import android.view.View
import com.alpha.lagouapk.R
import com.common.chat.item.base.BaseStatusChatItem
import com.common.chat.manager.BaseChatOperationManager
import com.common.utils.ViewUtils
import com.lagou.annotation.aspect.SingleClick
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.bean.json.ClickInfo
import com.lagou.baselib.analytic.definition.b.BPageId
import com.lagou.baselib.analytic.definition.c.PageId
import com.lagou.chat.chatlibrary.bean.json.ExchangeWechatMsg
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.socketchannel.util.GsonUtil
import kotlinx.android.synthetic.main.chat_item_exchange_wechat.view.*

class ExchangeWechatItem(context: Context) : BaseStatusChatItem(context) {

    private var mReceiverId: Long = 0
    private var mIsBUser: Boolean = false

    companion object {
        const val BUTTON_AGREE_TYPE = 1
        const val BUTTON_REFUSE_TYPE = 2
    }

    override fun isShouldFetchNewestMsg(): Boolean {
        if (mChatMsg == null) {
            return false
        }
        return mChatMsg.isFinal.not()
    }

    override fun updateUI() {
        val exchangeWechatMsg: ExchangeWechatMsg? =
            GsonUtil.fromJson(mChatMsg?.content, ExchangeWechatMsg::class.java)
        if (exchangeWechatMsg == null) {
            visibility = View.GONE
            return
        }
        visibility = View.VISIBLE
        tvExchangeTitle.text =
            if (exchangeWechatMsg.msg.isNullOrEmpty()) ResourceUtils.getString(R.string.exchange_wechat_request) else exchangeWechatMsg.msg
        val agreeBtnData = exchangeWechatMsg.btns?.find { it.btnType == BUTTON_AGREE_TYPE }
        val refuseBtnData = exchangeWechatMsg.btns?.find { it.btnType == BUTTON_REFUSE_TYPE }
        if (agreeBtnData == null) {
            tvExchangeAgree.isEnabled = false
        } else {
            tvExchangeAgree.isEnabled = agreeBtnData.canClick
            tvExchangeAgree.text =
                if (agreeBtnData.txt.isNullOrEmpty()) ResourceUtils.getString(R.string.agree) else agreeBtnData.txt
        }
        if (refuseBtnData == null) {
            tvExchangeRefuse.isEnabled = false
        } else {
            tvExchangeRefuse.isEnabled = refuseBtnData.canClick
            tvExchangeRefuse.text =
                if (refuseBtnData.txt.isNullOrEmpty()) ResourceUtils.getString(R.string.refuse) else refuseBtnData.txt
        }
    }

    override fun initView() {
        mAvatar = ivExchangeAvatar
        mDate = chat_item_date
        ViewUtils.setViewsClick(this, tvExchangeAgree, tvExchangeRefuse)
    }

    override fun getLayoutId(): Int = R.layout.chat_item_exchange_wechat

    @SingleClick
    override fun onClick(v: View?) {
        super.onClick(v)
        when (v?.id) {
            R.id.tvExchangeAgree -> {
                agreeExchange()
            }
            R.id.tvExchangeRefuse -> {
                refuseExchange()
            }
        }
    }

    /**
     * 同意交换
     * 状态更新依赖推送 153117  status = 267
     */
    private fun agreeExchange() {
        if (mChatMsg == null) {
            return
        }
        BaseChatOperationManager.agreeExchange(
            mContext,
            mChatMsg.serverSessionId,
            mChatMsg.serverMsgId
        )
        val clickInfo = ClickInfo()
            .setPage_id(if (mIsBUser) BPageId._19s1 else PageId._1ksx)
            .setAddress_id(if (mIsBUser) "f1u39" else "d1u37")
            .setContent_id(mReceiverId)
        AnalyticHelper.getInstance().sendClickInfo(clickInfo)
    }

    /**
     * 拒绝交换
     */
    private fun refuseExchange() {
        if (mChatMsg == null) {
            return
        }
        BaseChatOperationManager.refuseExchange(mChatMsg.serverSessionId, mChatMsg.serverMsgId)
    }

    fun setReceiverId(receiverId: Long, isBUser: Boolean): ExchangeWechatItem {
        mReceiverId = receiverId
        mIsBUser = isBUser
        return this
    }

}