package com.common.chat.item

import android.content.Context
import android.graphics.Typeface
import android.util.AttributeSet
import android.view.View
import android.widget.ImageView
import android.widget.RelativeLayout
import android.widget.TextView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.bean.ResumeSummaryBean
import com.alpha.lagouapk.tools.IntentUtils
import com.common.chat.item.base.ChatItem
import com.common.chat.manager.ChatManager
import com.common.utils.CommonIntentUtils
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.chat.chatlibrary.ChatApiSerice
import com.lagou.chat.chatlibrary.bean.ChatMsgBase
import com.lagou.chat.chatlibrary.bean.json.JMsgAddress
import com.lagou.commonlibs.engine.ThreadUtil
import com.lagou.commonlibs.imageload.ImageLoadUtils
import com.lagou.commonlibs.utils.*
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse
import com.lagou.socketchannel.openapi.NetAppConfig
import com.lagou.socketchannel.openapi.config.SocketAppConfig
import com.lagou.socketchannel.util.SPUtil

/**
 * 发送地址
 */
class MapAddressItem @JvmOverloads constructor(context: Context, attr: AttributeSet? = null, defStyleResId: Int = 0):ChatItem(context){
    private var mVipFlag: ImageView? = null
    private var mTvAddressDetail:TextView? = null
    private var tvTitle:TextView? = null
    private var mAddressInfo:JMsgAddress? = null
    private var mIvMap:ImageView? = null
    private var mMapWidth:Int = 0
    private var mMapHeight:Int = 0
    companion object{
        const val MAP_IMG_API = "https://gate.lagou.com/v1/entry/im/chat/address/img"
    }
    init {
        mMapWidth = DeviceUtils.getScreenWidth() - ResourceUtils.getDimenSize(R.dimen.dp_128)
        mMapHeight = (mMapWidth * 100f/238).toInt()
    }
    override fun getLayoutId(): Int = if (mLayoutRight) R.layout.chat_item_address_right else R.layout.chat_item_address_left

    override fun initView() {
        mContentLayout = findViewById(R.id.chat_item_content_layout)
        mDate = findViewById(R.id.chat_item_address_date)
        mAvatar = findViewById(R.id.chat_item_address_avatar)
        mTvAddressDetail = findViewById(R.id.tv_address_detail)
        msgLayout = findViewById(R.id.chat_item_msg_container)
        mRecallWrap = findViewById(R.id.tv_recall_msg_text)
        tvTitle = findViewById(R.id.tv_address_title)
        mIvMap = findViewById(R.id.iv_map_address)

        if (mLayoutRight) {
            mVipFlag = findViewById(R.id.chat_item_vip_flag)
            initState()
        }
        msgLayout.setOnLongClickListener(OnLongClickMsgListener())
        msgLayout.setOnClickListener {
            if (mLayoutRight) {
                CommonIntentUtils.startMapPreviewActivity(context, mAddressInfo?.lat ?: 0.0, mAddressInfo?.lng ?: 0.0,mAddressInfo?.title)
            } else {
                IntentUtils.startMapActivity(context, mTvAddressDetail?.text?.toString(), mAddressInfo?.lat?.toString(), mAddressInfo?.lng?.toString())
            }
        }
        val mapParams = mIvMap?.layoutParams as? RelativeLayout.LayoutParams
        mapParams?.width = mMapWidth
        mapParams?.height = mMapHeight
        mIvMap?.layoutParams = mapParams
    }


    override fun updateView() {
        val addressInfo:JMsgAddress? = mChatMsg.msgAddress
        this.mAddressInfo = addressInfo
        if (addressInfo == null){
            visibility = View.GONE
            return
        }
        visibility = View.VISIBLE
        mTvAddressDetail?.text = StringUtil.jointString(" ",addressInfo.city,addressInfo.district,addressInfo.detailAddress)
        tvTitle?.text = addressInfo.title
        if(mLayoutRight){
            mVipFlag?.visibility = if (SPUtil.isVIP()) VISIBLE else GONE
            updateSendState()
        }
        updateMap()
        updateRecallView()
    }

    private fun updateMap() {
        if (mAddressInfo?.img.isNullOrEmpty().not()){
            showMapImg(mAddressInfo?.img)
        }else{
            if(mChatMsg.serverMsgId <= 0L){
              invokeSDKAPI()
            } else {
                ThreadUtil.io re@{
                    val imgUrl = invokeLaGouApi()
                    if (imgUrl.isNullOrEmpty().not()) {
                        showMapImg(imgUrl)
                        return@re
                    }
                    invokeSDKAPI()
                }

            }
        }
    }

    private fun showMapImg(imgUrl:String?){
        ThreadUtil.ui {
            ImageLoadUtils.with(context).load(imgUrl).placeholder(R.drawable.default_map_bg)
                .error(R.drawable.default_map_bg).dontAnimate().into(mIvMap)
        }
    }

    private fun invokeSDKAPI(){
        val imgUrl = MAP_IMG_API + "?lng=${mAddressInfo?.lng}&lat=${mAddressInfo?.lat}&x_rh_deviceType=${SocketAppConfig.getInstance().appBean.deviceType}&x_rh_userToken=${UriUtils.urlEncode(NetAppConfig.getUserToken())}"
        showMapImg(imgUrl)
    }
    private fun invokeLaGouApi() = ChatApiSerice.api.fetchMapImgUrl(mChatMsg.serverSessionId, mChatMsg.serverMsgId)
        .callback(object : BaseCallback<String>() {
            override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<String>?) {}
            override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<String>?) {}
        }).execute().content

    /**
     * 更新撤回消息
     */
    private fun updateRecallView(){
        mRecallWrap.visibility = if (mChatMsg.state == ChatMsgBase.MsgState.RECALL) VISIBLE else GONE
        mContentLayout.visibility = if (mChatMsg.state == ChatMsgBase.MsgState.RECALL) GONE else VISIBLE
        if (mRecallWrap is TextView) {
            val isSender = ChatManager.getInstance().transIsSender(mChatMsg)
            (mRecallWrap as TextView).text = resources.getString(R.string.msg_recall_text, if (isSender) "你" else "对方")
        }
    }
}