package com.common.chat.item.base

import android.content.Context
import com.ApiService
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.chat.chatlibrary.ChatApiSerice
import com.lagou.chat.chatlibrary.bean.ChatMsgSimple
import com.lagou.chat.chatlibrary.db.ChatDBManager
import com.lagou.commonlibs.engine.ThreadUtil
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse

/**
 *带有状态的item，比如同意、拒绝
 * 当有状态发生变化时，需要重新请求接口，更新UI
 */
abstract class BaseStatusChatItem(context: Context):ChatItem(context) {
    final override fun updateView() {
        if (isShouldFetchNewestMsg()){
            requestTheNewestMsg()
            return
        }
        updateUI()
    }

    /**
     * 是否需要拉取最新消息
     */
    abstract fun isShouldFetchNewestMsg():Boolean

    /**
     * 更新UI
     */
    abstract fun updateUI()
    /**
     * 请求最新的消息
     */
    private fun requestTheNewestMsg(){
       if (mChatMsg == null){
           updateUI()
           return
       }
        ChatApiSerice.api.fetchTheNewestMsg(mChatMsg.serverSessionId,mChatMsg.serverMsgId.toString())
            .enqueue(object: BaseCallback<List<ChatMsgSimple?>>(false){
                override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<List<ChatMsgSimple?>>?) {
                    if (isDestroyed()) {
                        return
                    }
                    val list = response?.content
                    if (list.isNullOrEmpty()) {
                        updateUI()
                        return
                    }
                    val msgWrap = list.first()
                    mChatMsg.isFinal = true
                    mChatMsg.content = msgWrap?.message
                    ThreadUtil.io {
                        ChatDBManager.getInstance().updateMsg(mChatMsg, false)
                    }
                    updateUI()
            }

                override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<List<ChatMsgSimple?>>?) {
                    if (isDestroyed().not()){
                        updateUI()
                    }
                }
        })
    }

    private fun isDestroyed():Boolean {
        return (mContext as? BaseActivity)?.isActivityDestroyed ?: false
    }
}