package com.common.chat.manager

import android.content.Context
import com.alpha.lagouapk.ui.chat.OnChatCallback
import com.common.ui.dialog.SwitchWechatDialog
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.chat.chatlibrary.ChatApiSerice
import com.lagou.chat.chatlibrary.bean.ChatFastOptionButtonsBean
import com.lagou.chat.chatlibrary.bean.ExchangeWechatOperationParams
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse

/**
 * 聊天页底部快捷按键
 */
abstract class BaseChatOperationManager {
    companion object{
        /**
         * 可以聊聊
         */
        fun canTalk(txtMsg:String?){
            if (txtMsg.isNullOrEmpty()){
                return
            }
            ChatManager.getInstance().sendTextMsg(txtMsg)
        }

        /**
         * 不感兴趣
         */
        fun unInterest(sessionId: Long) {
            //不感兴趣
            ChatApiSerice.api.uninterested(sessionId).enqueue()
        }

        /**
         * 交换微信
         */
        fun exchangeWechat(activity:BaseActivity?,sessionId: Long) {
            ChatApiSerice.api.exchangeWechat(sessionId).enqueue(object: BaseCallback<Any?>(false){
                override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<Any?>?) {}
                override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<Any?>?) {
                    super.onFailure(request, response)
                    if (activity?.isActivityDestroyed != false){
                        return
                    }
                    val state = response?.state
                    if (state == 104001){
                        //无微信号，需要弹窗输入微信号
                        SwitchWechatDialog.newInstance().setOnSaveWechatSuccessCallback(object:SwitchWechatDialog.OnSaveWechatSuccessCallback{
                            override fun onSaveWechatSuccess() {
                                //微信号保存成功，重新交换微信
                               exchangeWechat(activity,sessionId)
                            }
                        }).show(activity.supportFragmentManager,"switchDialog")
                    }
                }
            })
        }

        /**
         * 查看微信
         */
        fun seeWechat(sessionId: Long) {
            ChatApiSerice.api.seeWechatNum(sessionId).enqueue()
        }

        /**
         * 聊天页底部按钮
         */
        fun  getTheFastOptionButtons(activity: BaseActivity?,sessionId: Long,callback: OnChatCallback<ChatFastOptionButtonsBean>?){
            ChatApiSerice.api.fetchChatFastOptionButtons(sessionId).enqueue(object:BaseCallback<ChatFastOptionButtonsBean>(false){
                override fun onSuccess(p0: HttpRequest<*>?, response: HttpResponse<ChatFastOptionButtonsBean>?) {
                    if (activity?.isActivityDestroyed != false){
                        return
                    }
                    response?.content?.let { callback?.onCallback(it) }
                }
            })
        }

        /**
         * 同意交换微信
         */
        fun agreeExchange(context:Context?,sessionId:Long,msgId:Long){
            ChatApiSerice.api.agreeExchangeWechat(sessionId, ExchangeWechatOperationParams(msgId.toString())).enqueue(object:BaseCallback<Any>(false){
                override fun onSuccess(p0: HttpRequest<*>?, p1: HttpResponse<Any>?) {}

                override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<Any?>?) {
                    super.onFailure(request, response)
                    if ((context as? BaseActivity)?.isActivityDestroyed != false){
                        return
                    }
                    val state = response?.state
                    if (state == 104001){
                        //无微信号，需要弹窗输入微信号
                        SwitchWechatDialog.newInstance().setOnSaveWechatSuccessCallback(object:SwitchWechatDialog.OnSaveWechatSuccessCallback{
                            override fun onSaveWechatSuccess() {
                                //微信号保存成功，重新同意
                                agreeExchange(context,sessionId,msgId)
                            }
                        }).show(context.supportFragmentManager,"switchDialog")
                    }
                }

            })
        }

        /**
         * 拒绝交换微信
         */
        fun refuseExchange(serverSessionId: Long, serverMsgId: Long) {
            ChatApiSerice.api.refuseExchangeWechat(serverSessionId, ExchangeWechatOperationParams(serverMsgId.toString())).enqueue()
        }
    }
}