package com.common.integrate.pay

import android.os.Message
import android.text.TextUtils
import com.alpha.lagouapk.R
import com.alpha.lagouapk.UserRoleManager
import com.common.CommonEventTags
import com.common.webview.LagouWebviewActivity
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.net.OnUiRelativeListener
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListenerImpl
import com.lagou.commonlibs.event.Event
import com.lagou.commonlibs.event.OkBus
import com.lagou.commonlibs.inter.OnSingleMethodListener
import com.lagou.commonlibs.ui.dialog.inter.ADismissListener
import com.lagou.commonlibs.utils.LogUtils
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.utils.ToastUtil
import com.lagou.lagouee.bean.jsonbean.business.GoodsBuyPushRes
import com.lagou.lagouee.bean.jsonbean.business.NewGoodsPayCheckRes
import com.lagou.lagouee.manager.net.BusinessNetManager
import com.lagou.lagouee.observer.BEventTags
import com.lagou.lagouee.utils.DialogUtils
import com.lagou.lagouee.utils.IntentUtils
import com.lagou.lagouee.widget.FullLoadingDialog
import com.lagou.pay.common.bean.PayExtraParams
import com.lagou.pay.common.moniter.PayMonitorManager
import com.lagou.socketchannel.http.HttpResponse
import com.lagou.socketchannel.util.GsonUtil
import java.net.URLDecoder

/**
 * @date 创建时间：2019-08-15
 * @author 作者：codeman
 * @Description  描述：承载商业化购买webview代理，也是一个支付订单成功后查询具体商品是否兑换成功的过渡页
 *                    H5承接了一部分流程（创建商品订单 尝试用勾豆支付）
 * @version
 */

class BusinessPayWebViewProxy(private val activity: LagouWebviewActivity?) : Event {

    private var loadingDialog: FullLoadingDialog? = null

    /***
     * 复写过滤url类，优先处理购买schema
     */
    fun filterUrl(url: String?): Boolean {
        if (UsePayManager.parsePayUrl(url, activity) == true) {
            return true
        }

        //不符合支付schema规则还走原先的逻辑
        return false
    }

    fun reload() {
        activity?.mWebViewDelegate?.reload()
    }

    fun onDestroy() {
        loadingDialog?.dismiss()
        //关闭页面取消客户端支付成功后注册服务器确认支付消息事件
        OkBus.getInstance().unRegister(this)
    }


    fun initProxyRegister() {
        //业务方注册使用场景需要的等待loading注册
        OkBus.getInstance()
                .registerOneOnOne(BEventTags.PAY_SUCCESS_CALLING_LOADING, this)
                .registerOneOnOne(BEventTags.PAY_SUCCESS_FLUSHPOINT_LOADING, this)
                .registerOneOnOne(BEventTags.PAY_SUCCESS_REPUBLISH_LOADING, this)
                .registerOneOnOne(BEventTags.PAY_SUCCESS_PUBLISH_LOADING, this)
                .registerOneOnOne(BEventTags.PAY_SUCCESS_RENEWAL_LOADING, this)
                .registerOneOnOne(BEventTags.PAY_SUCCESS_COMMUNICATE_LOADING, this)
                .registerOneOnOne(BEventTags.PAY_SUCCESS_VIP_LOADING, this)
                .registerOneOnOne(BEventTags.PAY_SUCCESS_VIP_POSITION_LOADING, this)
                .registerOneOnOne(BEventTags.PAY_SUCCESS_DOWNLOAD_RESUME_LOADING, this)
                .registerOneOnOne(BEventTags.PAY_SUCCESS_HOT_POSITION_LOADING, this)
                .registerOneOnOne(BEventTags.PAY_SUCCESS_BATCH_INVITE_LOADING, this)
                .registerOneOnOne(BEventTags.PAY_SUCCESS_POSITION_EXPOSURE_LOADING, this)
                .registerOneOnOne(BEventTags.PAY_SUCCESS_COMPETE_POSITION_LOADING,this)
    }

    override fun call(msg: Message?) {
        msg?.let {
            var payExtraParams = msg.obj as PayExtraParams?
            //判断当前场景不属于特定的Vip购买，但是购买的商品属于VIP购买，既要刷新当前购买场景，也要刷新和Vip相关的其它场景
            var otherTag = 0
            if (payExtraParams?.sceneContainVip() == true) {
                otherTag = BEventTags.PAY_SUCCESS_VIP
            }
            when (msg.what) {
                BEventTags.PAY_SUCCESS_CALLING_LOADING -> handlePaySuccessLoading(payExtraParams,
                        PaySuccessServerEnsure(payExtraParams), BEventTags.PAY_SUCCESS_CALLING_USER, otherTag)
                BEventTags.PAY_SUCCESS_FLUSHPOINT_LOADING -> handlePaySuccessLoading(payExtraParams,
                        PaySuccessServerEnsure(payExtraParams), BEventTags.PAY_SUCCESS_FLUSHPOINT_USER, otherTag)
                BEventTags.PAY_SUCCESS_REPUBLISH_LOADING -> handlePaySuccessLoading(payExtraParams,
                        PaySuccessServerEnsure(payExtraParams), BEventTags.PAY_SUCCESS_MONTH_USER, otherTag)
                BEventTags.PAY_SUCCESS_PUBLISH_LOADING -> handlePaySuccessLoading(payExtraParams,
                        PaySuccessServerEnsure(payExtraParams), BEventTags.PAY_SUCCESS_MONTH_USER, otherTag)
                BEventTags.PAY_SUCCESS_RENEWAL_LOADING -> handlePaySuccessLoading(payExtraParams,
                        PaySuccessServerEnsure(payExtraParams), BEventTags.PAY_SUCCESS_MONTH_USER, otherTag)
                BEventTags.PAY_SUCCESS_COMMUNICATE_LOADING -> handlePaySuccessLoading(payExtraParams,
                        PaySuccessServerEnsure(payExtraParams), BEventTags.PAY_SUCCESS_COMMUNICATE_USER, otherTag)
                BEventTags.PAY_SUCCESS_VIP_LOADING -> handlePaySuccessLoading(payExtraParams,
                        PaySuccessServerEnsure(payExtraParams), BEventTags.PAY_SUCCESS_VIP, BEventTags.LOW_MEMBER_SUCCESS)
                BEventTags.PAY_SUCCESS_VIP_POSITION_LOADING -> handlePaySuccessLoading(payExtraParams,
                        PaySuccessServerEnsure(payExtraParams), BEventTags.PAY_SUCCESS_VIP)
                BEventTags.PAY_SUCCESS_DOWNLOAD_RESUME_LOADING -> handlePaySuccessLoading(payExtraParams,
                        PaySuccessServerEnsure(payExtraParams), BEventTags.PAY_SUCCESS_DOWNLOAD_RESUME)
                BEventTags.PAY_SUCCESS_HOT_POSITION_LOADING -> {
                    when (payExtraParams?.sceneType) {
                        PayUseScenesKey.SceneType.SCENE_TALENT_DETAIL -> handlePaySuccessLoading(payExtraParams,
                                PaySuccessServerEnsure(payExtraParams), BEventTags.PAY_SUCCESS_HOT_POSITION_DETAIL)
                        PayUseScenesKey.SceneType.SCENE_POSITION_READY -> handlePaySuccessLoading(payExtraParams,
                                PaySuccessServerEnsure(payExtraParams), BEventTags.PAY_SUCCESS_HOT_POSITION_READY)
                        PayUseScenesKey.SceneType.SCENE_POSITION_PUBLISH -> handlePaySuccessLoading(payExtraParams,
                                PaySuccessServerEnsure(payExtraParams), BEventTags.PAY_SUCCESS_HOT_POSITION_PUBLISH)
                        PayUseScenesKey.SceneType.SCENE_POSITION_TRIAL -> handlePaySuccessLoading(payExtraParams,
                                PaySuccessServerEnsure(payExtraParams), BEventTags.PAY_SUCCESS_HOT_POSITION_TRIAL)
                        PayUseScenesKey.SceneType.SCENE_POSITION_REPUBLISH -> handlePaySuccessLoading(payExtraParams,
                                PaySuccessServerEnsure(payExtraParams), BEventTags.PAY_SUCCESS_HOT_POSITION_REPUBLISH)
                        PayUseScenesKey.SceneType.SCENE_MESSAGE_NOTICE -> handlePaySuccessLoading(payExtraParams,
                                PaySuccessServerEnsure(payExtraParams), BEventTags.PAY_SUCCESS_HOT_POSITION_MESSAGE_NOTICE)
                        PayUseScenesKey.SceneType.SCENE_HOME_HINTBAR -> handlePaySuccessLoading(payExtraParams,
                                PaySuccessServerEnsure(payExtraParams), BEventTags.PAY_SUCCESS_HOT_POSITION_HOME_HINTBAR)
                        PayUseScenesKey.SceneType.SCENE_HOME_ALERT -> handlePaySuccessLoading(payExtraParams,
                                PaySuccessServerEnsure(payExtraParams), BEventTags.PAY_SUCCESS_HOT_POSITION_HOME_ALERT)
                        PayUseScenesKey.SceneType.SCENE_POSITION_PROLONG -> handlePaySuccessLoading(payExtraParams,
                                PaySuccessServerEnsure(payExtraParams), BEventTags.PAY_SUCCESS_HOT_POSITION_PROLONG)
                        PayUseScenesKey.SceneType.SCENE_IM_SMS -> handlePaySuccessLoading(payExtraParams,
                                PaySuccessServerEnsure(payExtraParams), BEventTags.PAY_SUCCESS_HOT_POSITION_IM_SMS)
                    }
                }
                BEventTags.PAY_SUCCESS_BATCH_INVITE_LOADING -> handlePaySuccessLoading(payExtraParams,
                        PaySuccessServerEnsure(payExtraParams), BEventTags.PAY_SUCCESS_BATCH_INVITE)
                BEventTags.PAY_SUCCESS_POSITION_EXPOSURE_LOADING -> handlePaySuccessLoading(payExtraParams,
                    PaySuccessServerEnsure(payExtraParams), BEventTags.PAY_SUCCESS_POSITION_EXPOSURE)
                BEventTags.PAY_SUCCESS_COMPETE_POSITION_LOADING -> handlePaySuccessLoading(payExtraParams,
                    PaySuccessServerEnsure(payExtraParams), BEventTags.PAY_SUCCESS_COMPETE_POSITION)
                else -> {
                    LogUtils.i("BusinessPayWebviewActivity", "支付event事件tag异常回调 msg.what = " + msg.what)
                }
            }
        }
    }

    /***
     * 支付订单成功，但是兑换商品还没告知成功，等待服务器处理loading
     */
    private fun handlePaySuccessLoading(payExtraParams: PayExtraParams?, paySeverEvent: Event, vararg successBackUseTag: Int) {
        activity?.let {
            //1 注意注册该事件的时机，客户端支付成功后注册服务器确认支付消息事件
            OkBus.getInstance().register(BEventTags.PAY_SUCCESS_BUSINESS_SERVER, paySeverEvent)

            loadingDialog = FullLoadingDialog.getInstance(ResourceUtils.getString(R.string.in_the_account))
            loadingDialog?.setFailListener(OnSingleMethodListener<Boolean> {
                //2 超时后取消客户端支付成功后注册服务器确认支付消息事件
                OkBus.getInstance().unRegister(paySeverEvent)
                if (payExtraParams != null) {
                    //3 等待超时，主动调取接口拉取
                    BusinessNetManager.checkDirectBusiness(payExtraParams,
                            CheckOnAdapterCommonListenerImpl(activity, payExtraParams, successBackUseTag), this)
                } else {
                    loadingDialog?.dismiss()
                    activity.finish()
                }
            })
            loadingDialog?.show(activity.supportFragmentManager)
        }
    }

    /***
     * 服务器确认支付成功并且兑换对应的商品成功
     * a 收到服务器推送
     * b 等5s没收到推送，自己调用/shop/goodsOrder/check/{orderNo}查询
     */
    private fun handleServerSuccess(userId: Int?, payExtraParams: PayExtraParams?, successBackUseTags: IntArray) {

        activity?.let {
            if (userId?.toLong() != UserRoleManager.getInstance().userId) {
                return
            }

            //1 收到服务端支付确认消息，则关闭loading弹窗
            loadingDialog?.dismiss()

            //2 判断是否使用通用结果页：如果不使用，通过EventBus发送到各个业务方，触发后续操作；
            //  如果使用通用结果页：跳转webView，通过JsCommonBuyResultNotifyHandler处理后续操作
            if (TextUtils.isEmpty(payExtraParams?.useCommonBuyResultUrl)) {
                successBackUseTags.let {
                    for (successTag in successBackUseTags) {
                        successTag.let {
                            if (successTag != 0) {
                                OkBus.getInstance().onEvent(successTag, payExtraParams)
                            }
                        }
                    }
                }
            } else {
                IntentUtils.startLagouWebViewActivityHideTitle(activity,
                    payExtraParams?.useCommonBuyResultUrl + "?extraParams=" + GsonUtil.toJson(payExtraParams))
            }

            //3 收到服务器消息后取消客户端支付成功后注册服务器确认支付消息事件
            OkBus.getInstance().unRegister(this)

            //4 H5如果配置支付成功跳转链接则，在关闭页面之前，跳转到对应的schema页面
            payExtraParams?.let {
                if (!TextUtils.isEmpty(payExtraParams.paySuccessSchema)) {
                    IntentUtils.startLagouWebViewActivity(activity, URLDecoder.decode(payExtraParams.paySuccessSchema))
                }

                //5 成功后的页面销毁和刷新处理
                if (it.isClosePage) {
                    activity.finish()
                    if (it.isRefreshPage) {
                        OkBus.getInstance().onEvent(CommonEventTags.FLUSH_COMMON_WEBVIEW_PAGE)
                    }
                    return
                }

                if (it.isRefreshPage) {
                    activity.mWebViewDelegate?.reload()
                }
            }
        }
    }

    /***
     * 支付成功后，5s内没收到服务器消息，主动查询结果，作进一步处理
     * 之所以用内部类，是考虑到并发支付校验的情况，每种校验必须唯一对应一个回调
     */
    private inner class CheckOnAdapterCommonListenerImpl(relativeListener: OnUiRelativeListener?, val payExtraParams: PayExtraParams?, val successBackTag: IntArray)
        : OnAdapterCommonListenerImpl<NewGoodsPayCheckRes, HttpResponse<NewGoodsPayCheckRes>>(relativeListener) {

        override fun onSuccess(shopGoodsPayCheckRes: NewGoodsPayCheckRes) {
            super.onSuccess(shopGoodsPayCheckRes)
            loadingDialog?.dismiss()
            AnalyticHelper.getInstance().sendShowInfo(payExtraParams?.trackPageId, "1r2m")
            payExtraParams?.payResponse = GsonUtil.toJson(shopGoodsPayCheckRes)
            if (!shopGoodsPayCheckRes.isSuccessDialog()) {
                ToastUtil.show(activity, shopGoodsPayCheckRes.orderResultDesc)
                handleServerSuccess(shopGoodsPayCheckRes.userId, payExtraParams, successBackTag)
            } else {
                //属于VIP购买的都弹窗
                DialogUtils.showPaySuccessDialog(shopGoodsPayCheckRes, activity, object : ADismissListener<String>() {
                    override fun onImageClose() {
                        super.onImageClose()
                        handleServerSuccess(shopGoodsPayCheckRes.userId, payExtraParams, successBackTag)
                    }

                    override fun onDirect() {
                        super.onDirect()
                        handleServerSuccess(shopGoodsPayCheckRes.userId, payExtraParams, successBackTag)
                    }

                    override fun onLeft() {
                        super.onLeft()
                        //H5里边带的也有schema，如果H5里有则以H5为准
                        if (TextUtils.isEmpty(payExtraParams?.paySuccessSchema)) {
                            IntentUtils.startLagouWebViewActivity(activity, shopGoodsPayCheckRes.getSuccessBtnUrl())
                        }
                        handleServerSuccess(shopGoodsPayCheckRes.userId, payExtraParams, successBackTag)
                    }
                })
            }
        }

        override fun onError(errorBean: HttpResponse<NewGoodsPayCheckRes>) {
            super.onError(errorBean)
            loadingDialog?.dismiss()
            PayMonitorManager.queryExchangeGoodFail(payExtraParams?.goodsOrderNo,
                    payExtraParams?.payOrderNo, errorBean)
        }

        override fun onNoInternet() {
            super.onNoInternet()
            loadingDialog?.dismiss()
        }

        override fun isShowToast(): Boolean {
            return false
        }
    }

    /***
     * 服务器确认支付成功并且兑换对应的商品成功
     */
    private inner class PaySuccessServerEnsure(val payExtraParams: PayExtraParams?) : Event {
        override fun call(msg: Message?) {
            msg?.let {
                var goodsBuyResponse = msg.obj as? GoodsBuyPushRes
                goodsBuyResponse?.let {
                    //5s内收到服务端支付确认消息，则不在调用校验兑换商品是否成功接口
                    loadingDialog?.setContinueLCheckPublish(false)
                    handleServerSuccess(goodsBuyResponse.pd_userId, payExtraParams, intArrayOf(goodsBuyResponse.paySuccessState))
                }
            }
        }
    }
}