package com.common.integrate.pay.apply

import com.common.webview.LagouWebviewActivity
import com.lagou.pay.common.base.BasePayDelegate
import com.lagou.pay.common.bean.PayVerify


/**
 * 购买商品增强基类，做了一些通用化处理
 * 比如：刷新H5页面，通知H5支付成功等，实现了一批空方法
 */
open class BaseBuyProductImpl(private var payDelegate: BasePayDelegate?) : BasePackCallback() {

    override fun onFlushRelevant() {
        if (payDelegate == null) {
            return
        }
        //1 刷新当前h5页面
        val webviewActivity = payDelegate?.baseActivity as? LagouWebviewActivity
        webviewActivity?.let {
            if (webviewActivity.isActive) {
                webviewActivity.mWebViewDelegate?.reload()
            }
        }
    }

    override fun onSuccess(verify: PayVerify) {
        if (payDelegate == null) {
            return
        }

        if (payDelegate?.isNotNeedPay == true) {
            //不需要走原生付钱流程(积分或勾豆),直接通知H5显示“支付状态查询中..”，避免中间页闪烁
            notifyH5PurchaseSuccess()
        } else {
            payDelegate?.postSuccessDelayTask {
                notifyH5PurchaseSuccess()
            }
        }
    }

    private fun notifyH5PurchaseSuccess() {
        if (payDelegate?.baseActivity is LagouWebviewActivity) {
            payDelegate?.extraParams?.apply {
                isRefreshPage = true
            }

            val webviewActivity = payDelegate?.baseActivity as LagouWebviewActivity
            webviewActivity.mWebViewDelegate?.callJsFunction("nativePurchaseCallback", "", null)
        }
    }

    override fun onFailure(goodsOrderNo: String, mOrderNo: String) {
    }

    override fun onUnknown(goodsOrderNo: String, mOrderNo: String) {
    }

    override fun getBuyButtonClickAid(): String {
        return ""
    }

    override fun onGetOrderNo(orderNo: String?) {

    }

    override fun onReport() {

    }
}