package com.common.integrate.pay.apply

import android.text.TextUtils
import com.alpha.lagouapk.observer.EventTags
import com.common.integrate.pay.PayUseScenesKey
import com.common.webview.LagouWebviewActivity
import com.lagou.annotation.apt.APayBack
import com.lagou.commonlibs.event.OkBus
import com.lagou.lagouee.observer.BEventTags
import com.lagou.pay.common.base.BasePayDelegate
import com.lagou.pay.common.bean.PayVerify

/**
 * 简历曝光特权购买
 */

@APayBack(
        PayUseScenesKey.ProductType.PRODUCT_RESUME_EXPOSURE,
        PayUseScenesKey.ProductType.PRODUCT_RESUME_EXPOSURE + PayUseScenesKey.SEPARATOR + PayUseScenesKey.SceneType.SCENE_POSITION_HOME,
        PayUseScenesKey.ProductType.PRODUCT_RESUME_EXPOSURE + PayUseScenesKey.SEPARATOR + PayUseScenesKey.SceneType.SCENE_SETTING_HOME)
class BuyResumeExposureImpl(private var payDelegate: BasePayDelegate?) : BasePackCallback() {

    override fun onFlushRelevant() {
        if (payDelegate == null) {
            return
        }
        //1 刷新当前h5页面
        val webviewActivity = payDelegate?.baseActivity as? LagouWebviewActivity
        webviewActivity?.let {
            if (webviewActivity.isActive) {
                webviewActivity.mWebViewDelegate?.reload()
            }
        }
        OkBus.getInstance().onEvent(EventTags.EXPOSURE_RESUME_PAY_REFRESH, payDelegate?.extraParams)
    }

    override fun onSuccess(verify: PayVerify) {
        OkBus.getInstance().onEvent(BEventTags.PAY_SUCCESS_RESUME_EXPOSURE)
        OkBus.getInstance().onEvent(EventTags.EXPOSURE_RESUME_PAY_REFRESH)
        if (payDelegate == null) {
            return
        }

        if (payDelegate?.isNotNeedPay == true) {
            //不需要走原生付钱流程,直接通知H5显示“支付状态查询中..”，避免中间页闪烁
            notifyH5PurchaseSuccess()
        } else {
            payDelegate?.postSuccessDelayTask {
                notifyH5PurchaseSuccess()
            }
        }
    }

    private fun notifyH5PurchaseSuccess() {
        if (payDelegate?.baseActivity is LagouWebviewActivity) {
            payDelegate?.extraParams?.apply {
                isRefreshPage = true
            }

            val webviewActivity = payDelegate?.baseActivity as LagouWebviewActivity
            webviewActivity.mWebViewDelegate?.callJsFunction("nativePurchaseCallback", "", null)
        }
    }

    override fun onFailure(goodsOrderNo: String, mOrderNo: String) {
        OkBus.getInstance().onEvent(EventTags.EXPOSURE_RESUME_PAY_REFRESH)
    }

    override fun onUnknown(goodsOrderNo: String, mOrderNo: String) {
        OkBus.getInstance().onEvent(EventTags.EXPOSURE_RESUME_PAY_REFRESH)
    }

    override fun onGetOrderNo(orderNo: String?) {

    }

    override fun onReport() {
    }


    override fun getBuyPageId(): String {
        return ""
    }

    override fun getBuyButtonClickAid(): String {
        return ""
    }

    private fun isHomeScene():Boolean {
       return TextUtils.equals(payDelegate?.extraParams?.sceneType, PayUseScenesKey.SceneType.SCENE_POSITION_HOME)
    }

    private fun isMineScene() = TextUtils.equals(payDelegate?.extraParams?.sceneType, PayUseScenesKey.SceneType.SCENE_SETTING_HOME)
}