package com.common.integrate.pay.apply

import android.text.TextUtils
import com.common.integrate.pay.PayUseScenesKey
import com.common.webview.LagouWebviewActivity
import com.lagou.annotation.apt.APayBack
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.bean.json.ClickInfo
import com.lagou.commonlibs.event.OkBus
import com.lagou.lagouee.observer.BEventTags
import com.lagou.pay.common.base.BasePayDelegate
import com.lagou.pay.common.bean.PayVerify

/**
 * @date 创建时间：2019-11-14
 * @author 作者：codeman
 * @Description  描述：人才详情联系TA触发，vip会员购买成功
 * @version
 */

@APayBack(
        PayUseScenesKey.BUSINESS_VIP,
        PayUseScenesKey.ProductType.PRODUCT_VIP_MEMBER,
        PayUseScenesKey.ProductType.PRODUCT_VIP_MEMBER + PayUseScenesKey.SEPARATOR + PayUseScenesKey.SceneType.SCENE_RIGHTS,
        PayUseScenesKey.ProductType.PRODUCT_VIP_MEMBER + PayUseScenesKey.SEPARATOR + PayUseScenesKey.SceneType.SCENE_TRIAL_MINIMUM,
        PayUseScenesKey.ProductType.PRODUCT_VIP_MEMBER + PayUseScenesKey.SEPARATOR + PayUseScenesKey.SceneType.SCENE_TRIAL_DISCOUNTS,
        PayUseScenesKey.ProductType.PRODUCT_VIP_MEMBER + PayUseScenesKey.SEPARATOR + PayUseScenesKey.SceneType.SCENE_MALL)
class BuyVipCommonImpl(private var payDelegate: BasePayDelegate?) : BasePackCallback() {

    override fun onFlushRelevant() {
        //1 刷新当前h5页面
        val webviewActivity = payDelegate?.baseActivity as? LagouWebviewActivity
        webviewActivity?.let {
            if (webviewActivity.isActive) {
                webviewActivity.mWebViewDelegate.reload()
            }
        }
    }

    override fun onSuccess(verify: PayVerify) {
        payDelegate?.postSuccessDelayTask {
            payDelegate?.extraParams?.apply {
                isRefreshPage = true
            }
            OkBus.getInstance().onEvent(BEventTags.PAY_SUCCESS_VIP_LOADING, payDelegate?.extraParams)
        }
    }

    override fun onFailure(goodsOrderNo: String, mOrderNo: String) {
    }

    override fun onUnknown(goodsOrderNo: String, mOrderNo: String) {
    }

    override fun onGetOrderNo(orderNo: String?) {

    }

    override fun onReport() {
        var aid = "1p5v"
        when {
            isSixVip() -> aid = "1pan"
            isNineVip() -> aid = "1pav"
            isCommonVip() -> aid = "1paz"
        }
        val clickInfo = ClickInfo()
                .setPage_id(buyPageId)
                .setAddress_id(aid)
                .addClick_props("source_tab", sourceTab)
        AnalyticHelper.getInstance().sendClickInfo(clickInfo)
    }


    override fun getBuyPageId(): String {
        //6元
        if (isSixVip()) {
            return "1pab"
        }

        //9元
        if (isNineVip()) {
            return "1pac"
        }

        //普通App
        if (isCommonVip()) {
            return "1pae"
        }

        return "1mxe"
    }

    override fun getBuyButtonClickAid(): String {
        if (isSixVip()) {
            return "1pap"
        }

        //9元
        if (isNineVip()) {
            return "1paw"
        }

        //普通App
        if (isCommonVip()) {
            return "1pb1"
        }

        return "1p5w"
    }

    private fun isSixVip():Boolean {
       return TextUtils.equals(payDelegate?.extraParams?.sceneType, PayUseScenesKey.SceneType.SCENE_TRIAL_MINIMUM)
    }

    private fun isNineVip() = TextUtils.equals(payDelegate?.extraParams?.sceneType, PayUseScenesKey.SceneType.SCENE_TRIAL_DISCOUNTS)

    private fun isCommonVip() = TextUtils.equals(payDelegate?.extraParams?.sceneType, PayUseScenesKey.SceneType.SCENE_MALL)

}