package com.common.integrate.push

import android.app.Notification
import android.app.NotificationChannel
import android.app.NotificationManager
import android.content.Context
import android.content.Context.NOTIFICATION_SERVICE
import android.os.Build
import android.provider.Settings
import com.lagou.commonlibs.utils.LogUtils
import com.lagou.commonlibs.utils.RomUtil

/**
 * @date： Crate by wangjunwei on 2023/8/23
 * @description：消息分类管理类
 **/
class MessageCategoryManager {

    companion object {

        fun createMessageChannel(context: Context){
            if (RomUtil.isHuaweiDevice() || RomUtil.isHonorDevice() || RomUtil.isPixelDevice()){
                LogUtils.i("MessageCategoryManager", "设备厂商：华为或者荣耀")
                createMessageHuaweiChannel(context)
                return
            }

            if (RomUtil.isOppo() || RomUtil.isOnePlusDevice() || RomUtil.isRealmeDevice()){
                LogUtils.i("MessageCategoryManager", "设备厂商：oppo或者一加或者realme")
                createMessageOppoChannel(context)
                return
            }
        }

        /**
         * 创建华为推送所需的消息通道
         */
        private fun createMessageHuaweiChannel(context: Context){
            val channelNames = listOf<String>("运营信息", "工作事项提醒", "工作事项提醒", "帐号动态", "帐号动态", "即时聊天", "音频、视频通话")
            val channelDescription = listOf<String>(
                "各种运营广告：促投递、促完善信息、促付费、应用升级",
                "待审批电子流提醒、电子流审核进度提醒、工单待处理，活动报名进度提醒",
                "日程安排、会议提醒、考勤打卡提醒",
                "会员开通/到期/过期、加入/退出家庭账户",
                "帐号上下线、帐号状态变化、帐号信息认证",
                "工作类、交易类等必要聊天消息，需明确聊天双方关系，如买卖双方、医患关系等",
                "来电提醒、音频和视频通话")
            val channelIds = listOf<String>("CATEGORY_PROMO", "CATEGORY_REMINDER", "CATEGORY_REMINDER",
                "CATEGORY_SERVICE","CATEGORY_SERVICE",  "CATEGORY_MESSAGE", "CATEGORY_CALL")
            val channelImportance = listOf<Int>(NotificationManager.IMPORTANCE_DEFAULT,
                NotificationManager.IMPORTANCE_DEFAULT, NotificationManager.IMPORTANCE_DEFAULT,
                NotificationManager.IMPORTANCE_DEFAULT, NotificationManager.IMPORTANCE_DEFAULT,
                NotificationManager.IMPORTANCE_DEFAULT, NotificationManager.IMPORTANCE_DEFAULT)

            createMessageChannel(
                context,
                channelNames,
                channelDescription,
                channelImportance,
                channelIds
            )
        }

        /**
         * 创建oppo推送所需的消息通道
         */
       private fun createMessageOppoChannel(context: Context){
            val channelNames = listOf<String>("聊天消息", "升级", "公信通道", "运营消息", "服务提醒")
            val channelDescription = listOf<String>("聊天消息", "App升级专用通知通道", "根据用户的行为等，为用户推荐可能感兴趣的牛人或职位", "", "订单、财务、待办、动态")
            val channelIds = listOf<String>("chat_default", "upgrade", "publish_msg", "operation", "service")
            val channelImportance = listOf<Int>(NotificationManager.IMPORTANCE_DEFAULT, NotificationManager.IMPORTANCE_DEFAULT,
                NotificationManager.IMPORTANCE_DEFAULT, NotificationManager.IMPORTANCE_LOW, NotificationManager.IMPORTANCE_DEFAULT)

            createMessageChannel(
                context,
                channelNames,
                channelDescription,
                channelImportance,
                channelIds
            )
        }


        /***
         * 创建通知消息分类
         */
        private fun createMessageChannel(
            context: Context,
            channelNames: List<String>,
            channelDescription: List<String>,
            channelImportance: List<Int>,
            channelIds: List<String>
        ) {
            if (Build.VERSION.SDK_INT < Build.VERSION_CODES.O) {
               return
            }

            try {
                val notificationManager =
                    context.getSystemService(NOTIFICATION_SERVICE) as NotificationManager
                val channelList = mutableListOf<NotificationChannel>()

                channelNames.forEachIndexed { index, name ->
                    // Create the NotificationChannel
                    val name = name
                    val descriptionText = channelDescription[index]
                    val importance = channelImportance[index]
                    val channel = NotificationChannel(channelIds[index], name, importance)
                    channel.description = descriptionText
                    if (importance == NotificationManager.IMPORTANCE_DEFAULT) {
                        //对于重要的通知出震动效果
                        channel.vibrationPattern =
                            longArrayOf(100, 200, 300, 400, 500, 400, 300, 200, 400)
                        channel.enableVibration(true)
                        channel.setShowBadge(false)
                        channel.setSound(
                            Settings.System.DEFAULT_NOTIFICATION_URI,
                            Notification.AUDIO_ATTRIBUTES_DEFAULT
                        )
                    }
                    channel.lockscreenVisibility = Notification.VISIBILITY_PUBLIC
                    // Register the channel with the system; you can't change the importance
                    // or other notification behaviors after this
                    channelList.add(channel)
                }
                notificationManager.createNotificationChannels(channelList)
            }catch (e: Exception){
                e.printStackTrace()
            }
        }
    }
}