package com.common.integrate.push.getui

import android.app.NotificationManager
import android.content.Context
import com.alpha.lagouapk.LagouApplication
import com.common.integrate.push.MessageCategoryManager
import com.igexin.sdk.PushManager
import com.lagou.commonlibs.utils.LogUtils

/**
 * @date 创建时间：2020/7/1
 * @author 作者：codeman
 * @Description  描述：个推帮助类
 * @version
 */
class GetuiPushManager {

    companion object {
        const val TAG = "GetuiPushManager"
        var mContext: Context? = null

        /***
         * 初始化个推 SDK。多次调用 SDK 初始化并无影响。为了保证 SDK 服务稳定，推荐引导用户授权相关的隐私权限。
         */
        @JvmStatic
        fun init(context: Context) {
            mContext = context
            MessageCategoryManager.createMessageChannel(context)

            PushManager.getInstance().initialize(context)
            if (LogUtils.isDebug()) {
                PushManager.getInstance().setDebugLogger(context) { s ->
                    LogUtils.i(TAG, "setDebugLogger方法  $s")
                }
                //如果无法获取cid，建议调用PushManager.getInstance().checkManifest()接口检查集成结果！
                // 此接口会自动检测运行环境且只在debug状态下时才会工作，线上release版本不会被执行
                //PushManager.getInstance().checkManifest(mContext);
            }
        }

        /***
         * 获取get
         */
        @JvmStatic
        fun getGetuiRegisterId(): String? {
            mContext?.let {
                return PushManager.getInstance().getClientid(mContext)
            }
            return null
        }

        @JvmStatic
        fun clearAllNotification(){
            try {
                val notificationManager = LagouApplication.getLagouApplication().getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
                notificationManager.cancelAll()
                //更新launcher角标数字为初始化状态：退出登录、切换B、C
                setBadgeNum(LagouApplication.getLagouApplication(), 0)
            }catch (e: Exception){
                e.printStackTrace()
            }
        }

        /**
         * 设桌面launcher角标
         * 保持消息页
         */
        @JvmStatic
        fun setBadgeNum(context: Context, badgeNum: Int){
            PushManager.getInstance().setBadgeNum(context, badgeNum)
        }
    }
}