package com.common.integrate.push.getui

import android.content.Context
import android.content.Intent
import android.text.TextUtils
import com.alpha.lagouapk.R
import com.alpha.lagouapk.UserRoleManager
import com.alpha.lagouapk.bean.GetuiPushInfo
import com.alpha.lagouapk.tools.IntentUtils
import com.alpha.lagouapk.ui.position.PositionDetailActivity
import com.common.integrate.push.PushType
import com.common.integrate.push.TriplePushHelper
import com.igexin.sdk.PushManager
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.bean.json.ShowLogInfo
import com.lagou.commonlibs.bean.WebViewExtentBean
import com.lagou.commonlibs.utils.DeviceUtils
import com.lagou.commonlibs.utils.StringUtil
import com.lagou.commonlibs.web.router.Router
import com.lagou.socketchannel.netty.SocketHelper
import com.lagou.socketchannel.util.GsonUtil
import org.json.JSONObject
import java.math.BigInteger
import java.util.*

/**
 * @date 创建时间：2020/7/3
 * @author 作者：codeman
 * @Description  描述：个推厂商通道唤起类
 * @version
 */
class GetuiPushTripleActivity : BaseActivity() {

    override fun getContentLayoutId(): Int = R.layout.activity_triple_push

    override fun init() {

        pushClick(intent)

        // 获取华为平台附带的jpush信息
        var data = intent?.extras?.getString(KEY_EXTRA)
        // 获取fcm、小米、oppo、vivo平台附带的jpush信息
        if (data.isNullOrEmpty()) {
            data = intent?.data?.toString()
        }
        //LogUtils.d(TAG, "JPush-----> --- triple push data:${data}")
        if (data.isNullOrEmpty()) {
            Router.open(this, geHomeSchemaUrl())
            finish()
            return
        }
        try {
            val jsonObject = JSONObject(data)
            val extras: String = jsonObject.optString(KEY_EXTRAS)
            val pushSDKName = getPushSDKName()
            //LogUtils.d(TAG, "JPush-----> --- triple push data:${pushSDKName}")
            val pushInfo = handleNotifyMessageOpened(this, extras)
            // 上报点击事件
            AnalyticHelper.getInstance().sendShowInfo(ShowLogInfo().apply {
                setAddress_Id("1pui")
                pushInfo?.msgId?.let {
                    addShowProp("jpush_id", it)
                }
                addShowProp("push_type", PushType.GEXIN.toString())
                pushSDKName?.let {
                    addShowProp("jpush_channel", pushSDKName)
                }
            })
        } catch (e: Exception) {
            // 使用路由进行跳转，因为路由拦截器处理了未初始化/未登录的逻辑
            Router.open(this, geHomeSchemaUrl())
        }

        finish()
    }

    private fun handleNotifyMessageOpened(context: Context?, extra: String?): GetuiPushInfo? {
        // 注意，此处服务器返回的是uid的md5值,并不是真正的token
        val jPushInfo = GsonUtil.fromJson(extra, GetuiPushInfo::class.java)
        //userIdMd5为空代表未登录，目的登陆成功后能跳转到指定页面
        if (jPushInfo != null && !TextUtils.isEmpty(jPushInfo.jumpUrl)) {
            val extentBean = WebViewExtentBean()
            extentBean.source = PositionDetailActivity.SOURCE.JD_PUSH
            IntentUtils.startLagouWebViewActivity(context, jPushInfo.jumpUrl, extentBean)
            reconnect()
            return jPushInfo
        }
        startHomeBorC(context)
        reconnect()
        return jPushInfo
    }

    /**
     * 启动app首页（自动判断B、C及未登录情况）
     */
    private fun startHomeBorC(context: Context?) {
        // 使用路由进行跳转，因为路由拦截器处理了未初始化/未登录的逻辑
        Router.open(context, geHomeSchemaUrl())
    }

    private fun geHomeSchemaUrl() =
        if (UserRoleManager.getInstance().isCUser)
            "lagou://lagou.com/home"
        else
            "lagouhr://lagou.com/home"

    private fun reconnect() {
        // 启动长连接服务
        if (!SocketHelper.getInstance().isInit
            || SocketHelper.getInstance().isAvailable
        ) {
            return
        }
        SocketHelper.getInstance().reconnect(false)
    }

    /**
     * 由于华为、oppo 无点击数报表返回，vivo无单推点击数报表返回，所以需要您在客户端埋点上报。
     * 点击厂商通知以后，在触发的activity的onCreate()方法里面接收相关参数，上报这 3 个离线厂商消息的点击数据。
     * 开发者可直接使用此方法示例
     * @param intent
     * @return
     */
    fun pushClick(intent: Intent): Boolean {
        var result = false
        try {
            val taskid = intent.getStringExtra(KEY_TASK_ID)
            val gtaction = intent.getStringExtra(KEY_ACTION)
            val clientid: String = PushManager.getInstance().getClientid(this)
            val uuid: String = UUID.randomUUID().toString().replace("-", "")
            // 这里的messageid需要自定义， 保证每条消息汇报的都不相同
            val contentToDigest: String = taskid + clientid + uuid
            val md5s = StringUtil.md5(contentToDigest)
            val messageid: String = BigInteger(md5s, 1).toString(16)
            /***
             * 第三方回执调用接口，可根据业务场景执行
             * 注意：只能用下面回执对应的机型进行上报点击测试，其它机型获取不到 gttask 字段
             * 60020 华为点击
             * 60030 oppo点击
             * 60040 vivo点击
             * 60070 荣耀点击
             * 埋点接口对应填写获取到的actionid值，如果有获取到 actionid 值，就上报埋点，如 果没有则不用上报。
             */
            if (gtaction != null) {
                val actionid = gtaction.toInt()
                result = PushManager.getInstance()
                    .sendFeedbackMessage(this, taskid, messageid, actionid)
            }
        } catch (e: java.lang.Exception) {
            e.printStackTrace()
        }
        return result
    }

    private fun getPushSDKName(): String {
        val deviceManufacturer = DeviceUtils.getDeviceManufacturer()
        when (deviceManufacturer?.toLowerCase()) {
            "vivo" -> return TriplePushHelper.CHANNEL_NAME_VIVO
            "huawei", "华为" -> return TriplePushHelper.CHANNEL_NAME_HUAWEI
            "meizu", "魅族" -> return TriplePushHelper.CHANNEL_NAME_MEIZU
            "oppo", "oneplus" -> return TriplePushHelper.CHANNEL_NAME_OPPO
            "xiaomi", "小米" -> return TriplePushHelper.CHANNEL_NAME_XIAOMI
            else -> return TriplePushHelper.CHANNEL_NAME_GETUI
        }
    }

    override fun enterMustLogin(): Boolean = false

    override fun needCheckInit(): Boolean = false

    companion object {
        private const val TAG = "GetuiPushTripleActivity"

        const val KEY_EXTRA = "JMessageExtra"

        /**任务Id */
        private const val KEY_TASK_ID = "gttask"

        /**该通知的下发通道 */
        private const val KEY_ACTION = "gtaction"

        /**通知附加字段 */
        const val KEY_EXTRAS = "extra"
    }
}