package com.common.integrate.qrcode

import android.view.View
import android.widget.TextView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.tools.IntentUtils
import com.common.bean.json.QrcodeResult
import com.lagou.baselib.privacy.KEY_STORE_PHOTO_BC_SAVE
import com.lagou.commonlibs.inter.SavePictureListener
import com.lagou.commonlibs.ui.dialog.BaseBottomSheetDialogFragment
import com.lagou.commonlibs.utils.BitmapUtils
import com.lagou.commonlibs.utils.ToastUtil
import com.lagou.commonlibs.utils.lgshare.LGShareAction
import com.lagou.commonlibs.utils.lgshare.core.LGShareMedia

/**
 * @date： Crate by wangjunwei on 2023/2/28
 * @description： 图片处理：保存图片、转发到微信、识别二维码
 **/
class QrCodeDetectorDialog : BaseBottomSheetDialogFragment() {

    private var qrcodeResult: QrcodeResult? = null
    private lateinit var tvSavePicture: TextView
    private lateinit var tvForwardWx: TextView
    private lateinit var vmDetectorLine: View
    private lateinit var tvIdentifyQrcode: TextView
    private lateinit var tvCancel: TextView

    companion object {
        fun getInstance(qrcodeResult: QrcodeResult?): QrCodeDetectorDialog {
            val identifyPictureDialog = QrCodeDetectorDialog()
            identifyPictureDialog.qrcodeResult = qrcodeResult
            return identifyPictureDialog
        }
    }

    override fun getLayoutId() = R.layout.dialog_detector_qrcode

    override fun initView(view: View) {
        super.initView(view)
        tvSavePicture = view?.findViewById(R.id.tv_save_picture)
        tvForwardWx = view?.findViewById(R.id.tv_forward_wx)
        vmDetectorLine = view?.findViewById(R.id.tv_identify_qrcode)
        tvIdentifyQrcode = view?.findViewById(R.id.tv_identify_qrcode)
        tvCancel = view?.findViewById(R.id.tv_cancel)

        //1 未检测出包含二维码，包含两种情况其一不包含二维码、其二包含了未能识别出的二维码，此时只有三个选项 保存、转发到微信、取消
        if (qrcodeResult?.codeResult == null){
            tvIdentifyQrcode.visibility = View.GONE
            vmDetectorLine.visibility = View.GONE
            tvForwardWx.setBackgroundResource(R.drawable.ripple_border_none_blr13)
        }else{
            tvIdentifyQrcode.visibility = View.VISIBLE
            vmDetectorLine.visibility = View.VISIBLE
            tvForwardWx.setBackgroundResource(R.drawable.ripple_white)
        }
    }

    override fun initListener() {
        super.initListener()

        //1 保存图片到本地
        tvSavePicture.setOnClickListener {
            BitmapUtils.saveBitmapToGallery(
                activity, qrcodeResult?.bitmap, "",
                object : SavePictureListener {
                    override fun onSuccess() {
                        if (qrcodeResult?.isShowToast == true) {
                            ToastUtil.show(activity, R.string.save_success)
                        }
                    }

                    override fun onFailure() {
                        if (qrcodeResult?.isShowToast == true) {
                            ToastUtil.show(activity, R.string.save_fail)
                        }
                    }
                }, KEY_STORE_PHOTO_BC_SAVE
            )
            dismiss()
        }

        //2 转发图片到微信
        tvForwardWx.setOnClickListener {
            qrcodeResult?.let {
                val shareAction = LGShareAction(getActivity())
                shareAction
                    .setImage(it.bitmap)
                    .setShareMedia(LGShareMedia.WEIXIN)
                    .share()
            }
            dismiss()
        }

        //3 检测出二维码数据，长按或者点击触发弹窗（询问用户是否导航到二维码结果），参考扫码处理逻辑
        tvIdentifyQrcode.setOnClickListener {
            qrcodeResult?.codeResult?.let {
                IntentUtils.startLagouWebViewActivity(getActivity(), it.text)
            }
            dismiss()
        }
        tvCancel.setOnClickListener { dismiss() }
    }

    override fun getWidth(): Int {
        return (0.946 * screenWidth).toInt()
    }

}