package com.common.integrate.site

import android.app.Activity
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewConfiguration
import android.view.ViewGroup
import android.widget.PopupWindow
import android.widget.TextView
import androidx.appcompat.widget.LinearLayoutCompat
import com.alpha.lagouapk.LagouApplication
import com.alpha.lagouapk.R
import com.alpha.lagouapk.tools.IntentUtils
import com.alpha.lagouapk.tools.NotificationUtils
import com.alpha.lagouapk.tools.SPUtils
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.util.BarUtil
import com.lagou.commonlibs.utils.ActivityUtil


/**
 * @date： Crate by wangjunwei on 2023/10/18
 * @description：提醒打开push的站内通知
 **/
class OpenPushAutoWindow : ISitePushForeign {

    val DURATION = 5000
    var mActivity: Activity? = null
    private var mDecorView: View? = null
    private var mStatusHeight = 0

    var mPopupWindow: PopupWindow? = null
    var mContentView: View? = null
    private var scene: String? = null
    private var tipContent:String? = null
    private var mDismissRunnable: Runnable = Runnable {
        dismiss()
    }
    var touchSlop = 0

    companion object {
        const val DELIVER_SUCCESS_OPEN = "deliver_success_open"//投递成功
        const val IM_ANY_OPEN = "im_any_open"//收到IM消息

    }

    constructor(activity: Activity, scene: String?,tipContent:String?) {
        mActivity = activity
        this.scene = scene
        this.tipContent = tipContent
        touchSlop = ViewConfiguration.get(mActivity).scaledTouchSlop
        mDecorView = mActivity?.window?.decorView
        mContentView = initContentView()
        // popup
        mPopupWindow = PopupWindow(
            mContentView,
            ViewGroup.LayoutParams.MATCH_PARENT,
            ViewGroup.LayoutParams.WRAP_CONTENT
        )
        mPopupWindow?.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        mPopupWindow?.isOutsideTouchable = false
        mPopupWindow?.animationStyle = R.style.tip_window_anim
        // other
        mStatusHeight = BarUtil.getStatusBarHeight(activity)
    }

    private fun initContentView(): View? {
        mContentView = LayoutInflater.from(mActivity).inflate(R.layout.layout_open_push_window, null)
        (mContentView?.findViewById(R.id.tv_content) as? TextView)?.text = tipContent
        mContentView?.findViewById<LinearLayoutCompat>(R.id.lin_content)?.setOnClickListener { v: View? ->
            if (!NotificationUtils.isNotificationEnabled(LagouApplication.getLagouApplication())){
                //系统设置关闭&&APP开启--跳转系统设置
                IntentUtils.startPushSettingPage(mActivity)
            }else{
                //非设置页面的弹窗都需要先跳转至设置页面
                IntentUtils.startPushSet(mActivity, scene)
            }
            dismiss()
        }
        mContentView?.setOnTouchListener(TouchListener(this, touchSlop))
        return mContentView
    }

    override fun show() {
        if (mPopupWindow?.isShowing == true) {
            return
        }
        mDecorView?.post {
            if (ActivityUtil.isDestroyed(mActivity)) {
                return@post
            }
            mContentView?.removeCallbacks(mDismissRunnable)
            // 全屏顶到头部
            val y = if (ActivityUtil.isFullScreen(mActivity)) 0 else mStatusHeight
            mPopupWindow?.showAtLocation(mDecorView, Gravity.NO_GRAVITY, 0, y)
            mContentView?.postDelayed(mDismissRunnable, DURATION.toLong())
        }
    }

    override fun clear() {
        mContentView?.removeCallbacks(mDismissRunnable)
        mPopupWindow?.dismiss()
    }

    override fun dismiss() {
        if (ActivityUtil.isActive(mActivity)) {
            mPopupWindow?.dismiss()
        }
    }
}