package com.common.integrate.wx

import com.alpha.lagouapk.ui.login.bean.OauthGetResp
import com.common.manager.net.CommonNetManager
import com.common.manager.net.AccountNetManager
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListenerImpl
import com.lagou.commonlibs.utils.LogUtils
import com.lagou.socketchannel.http.HttpResponse

/**
 * @date 创建时间：2019-11-28
 * @author 作者：codeman
 * @Description  描述：钱包使用微信绑定体现
 * 情况一：当前未绑定，执行去绑定微信操作，绑定完成后去回调到H5
 * 情况二：当前已经绑定微信A，执行去绑定微信操作，a 授权拿到微信code后，解绑微信A，b 然后请求拿到authToken，绑定微信B
 * @version
 */

open class WalletWxBindEvent(activity: BaseActivity?, onWxBindCallback: OnWxBindCallback<OauthGetResp>)
    : BaseWxBindEvent(activity, onWxBindCallback) {

    override fun onCurrentHasBind(code: String) {
        //super.onCurrentHasBind(oauthGetResp)
        //如果当前已经绑定微信，则表示当前是换绑流程
        //1 先拿解绑定微信的
        AccountNetManager.unbindWX(object : OnAdapterCommonListenerImpl<Any, HttpResponse<Any>>(baseActivity){
            override fun onSuccess(t: Any?) {
                super.onSuccess(t)
                //2 解绑成功后，然后在请求/account/oauth/get 拿到authToken
                LogUtils.i("WalletWxBindEvent", " --- unbindWX ---")
                getOauthToken(code)
            }

            override fun isCareResponse(): Boolean {
                return false
            }
        })
    }

    /***
     * 2 请求/account/oauth/get 拿到authToken
     */
    fun getOauthToken(code: String){
        CommonNetManager.checkWxState(code, object : OnAdapterCommonListenerImpl<OauthGetResp, HttpResponse<OauthGetResp>>(baseActivity) {
            override fun onSuccess(oauthGetResp: OauthGetResp?) {
                super.onSuccess(oauthGetResp)
                changeBindWx(oauthGetResp)
            }
        })
    }

    override fun onToBindSuccess(oauthGetResp: OauthGetResp?) {
        super.onToBindSuccess(oauthGetResp)
        //弹出提现引导提示框，弹窗提示统一改成H5来做，绑定成功后回调结果到H5
        //UnitDialogUtils.showWalletTipDialog(baseActivity)
    }

    /***
     *3 换绑执行绑定
     */
    fun changeBindWx(oauthGetResp: OauthGetResp?){
        oauthGetResp?.let {
            LogUtils.i("WalletWxBindEvent", " --- changeBindWx ---")
            CommonNetManager.bindWx(oauthGetResp, object : OnAdapterCommonListenerImpl<Any, HttpResponse<Any>>(baseActivity) {
                override fun onSuccess(t: Any?) {
                    super.onSuccess(t)
                    onWxBindCallback?.onCurrentHasBind(oauthGetResp)
                }
                override fun isCareResponse(): Boolean {
                    return false
                }
            })
        }
    }
}
