package com.common.manager

import android.text.TextUtils
import android.util.Log
import com.lagou.commonlibs.utils.TimeUtils
import com.common.bean.json.InterviewCalendarDayRes
import com.common.bean.json.InterviewCalendarFoldIndex
import com.common.utils.SpUserNotClearUtils
import com.lagou.commonlibs.utils.LogUtils
import com.lagou.lagouee.bean.InviteInterviewTime
import java.text.SimpleDateFormat
import java.util.*

/**
 * @author 作者：codeman
 * @date 创建时间：2020/7/30
 * @Description 描述：
 */
class InterviewCalendarManager {
    companion object {
        private val weeks = listOf("星期日", "星期一", "星期二", "星期三", "星期四", "星期五", "星期六")
        private var serviceTimeStamp = System.currentTimeMillis()
        private var interviewDays: List<String>? = null

        fun createExtendCalendarDays(sTimeStamp: Long, interviewTimes: List<String>?, monthMapIndex: MutableMap<Int, Int>): MutableMap<Int, List<InterviewCalendarDayRes>> {
            serviceTimeStamp = sTimeStamp
            if (interviewTimes != null) {
                interviewDays = interviewTimes
            }

            var calendar: Calendar = Calendar.getInstance()
            //1 调整日历为当月的1号 00时 00分
            calendar.set(Calendar.DAY_OF_MONTH, 1)
            calendar.set(Calendar.HOUR_OF_DAY, 0)
            calendar.set(Calendar.MINUTE, 0)
            calendar.set(Calendar.SECOND, 0)
            calendar.timeInMillis = TimeUtils.convertYearMonthTime(serviceTimeStamp)
            //2 往前推6个月  往后推5个月
            val calendarMap: MutableMap<Int, List<InterviewCalendarDayRes>> = mutableMapOf()
            var index = 0
            for (monthTag in -6..5) {
                calendar.add(Calendar.MONTH, monthTag)
                val monthDay = calendar.getActualMaximum(Calendar.DAY_OF_MONTH)
                val calendarDayList = mutableListOf<InterviewCalendarDayRes>()
                for (dayTag in 0..monthDay - 1) {
                    calendar.add(Calendar.DAY_OF_MONTH, dayTag)
                    //1 其它数据正常处理
                    val interviewCalendarDayRes = InterviewCalendarDayRes()
                    interviewCalendarDayRes?.apply {
                        timeStamp = calendar.timeInMillis
                        year = calendar.get(Calendar.YEAR)
                        month = calendar.get(Calendar.MONTH)
                        day = calendar.get(Calendar.DAY_OF_MONTH)
                        isHasInterview = isHasInterview(calendar.timeInMillis)
                        week = weeks[calendar.get(Calendar.DAY_OF_WEEK) - 1]
                        isCurrentDay = isCurrentDay(calendar.timeInMillis)
                        isSelect = isSameDay(calendar.timeInMillis, serviceTimeStamp)
                        isOccupy = false
                        if (!isCurrentDay) {
                            isBeforeToday = isBeforeToday(calendar.timeInMillis)
                        }
                    }

                    //2 对于每个月的第一天要判断是星期几，前边需要有几个占位
                    if (dayTag == 0) {
                        occupyCalendar(calendar, calendarDayList, interviewCalendarDayRes)
                    }

                    calendarDayList.add(interviewCalendarDayRes)
                    calendar.add(Calendar.DAY_OF_MONTH, -dayTag)
                }

                //占位够6行，为了viewPager展示高度正常统一
                val afterOccupy = 37 - calendarDayList.size
                for (i in 1..afterOccupy) {
                    val interviewCalendarDayRes = InterviewCalendarDayRes()
                    interviewCalendarDayRes.isOccupy = true
                    calendarDayList.add(interviewCalendarDayRes)
                }
                LogUtils.i("createExtendCalendar", "补充占位长度 ${calendarDayList.size}")
                monthMapIndex[calendar.get(Calendar.MONTH)] = index
                calendarMap[index++] = calendarDayList
                calendar.add(Calendar.MONTH, -monthTag)
            }

            return calendarMap
        }

        /***
         * 创建折叠情况下的所有日历的数据
         * schemaTimeStamp schema指定时间戳
         */
        fun createFoldCalendarDays(sTimeStamp: Long, interviewTimes: List<String>?,
                                   monthMapIndex: MutableMap<Int, Int>)
                : MutableMap<Int, MutableMap<Int, List<InterviewCalendarDayRes>>> {
            serviceTimeStamp = sTimeStamp
            interviewDays = interviewTimes

            val calendar: Calendar = Calendar.getInstance()
            //1 调整日历为当月的1号 00时 00分
            calendar.set(Calendar.DAY_OF_MONTH, 1)
            calendar.set(Calendar.HOUR_OF_DAY, 0)
            calendar.set(Calendar.MINUTE, 0)
            calendar.set(Calendar.SECOND, 0)
            calendar.timeInMillis = TimeUtils.convertYearMonthTime(serviceTimeStamp)
            //2 往前推6个月  往后推5个月
            val calendarAllMap: MutableMap<Int, MutableMap<Int, List<InterviewCalendarDayRes>>> = mutableMapOf()
            var index = 0
            for (monthTag in -6..5) {
                calendar.add(Calendar.MONTH, monthTag)
                val monthDay = calendar.getActualMaximum(Calendar.DAY_OF_MONTH)
                var calendarDays: MutableList<InterviewCalendarDayRes>? = null
                val calendarMonthMap: MutableMap<Int, List<InterviewCalendarDayRes>> = mutableMapOf()
                var innerIndex = 0

                for (dayTag in 0..monthDay - 1) {
                    calendar.add(Calendar.DAY_OF_MONTH, dayTag)
                    Log.i("createFoldCalendarDays", "${calendar.get(Calendar.YEAR)}-${calendar.get(Calendar.MONTH) + 1}-${calendar.get(Calendar.DAY_OF_MONTH)}")
                    //1 其它数据正常处理
                    val interviewCalendarDayRes = InterviewCalendarDayRes()
                    interviewCalendarDayRes.apply {
                        timeStamp = calendar.timeInMillis
                        year = calendar.get(Calendar.YEAR)
                        month = calendar.get(Calendar.MONTH)
                        day = calendar.get(Calendar.DAY_OF_MONTH)
                        isHasInterview = isHasInterview(calendar.timeInMillis)
                        week = weeks[calendar.get(Calendar.DAY_OF_WEEK) - 1]
                        isCurrentDay = isCurrentDay(calendar.timeInMillis)

                        isSelect = isSameDay(calendar.timeInMillis, serviceTimeStamp)
                        isOccupy = false
                        if (!isCurrentDay) {
                            isBeforeToday = isBeforeToday(calendar.timeInMillis)
                        }
                    }

                    //2 对于每个月的第一天要判断是星期几，前边需要有几个占位
                    if (dayTag == 0) {
                        calendarDays = mutableListOf()
                        occupyCalendar(calendar, calendarDays, interviewCalendarDayRes)
                    }

                    calendarDays?.add(interviewCalendarDayRes)
                    if (calendarDays?.size == 7 || dayTag == monthDay - 1) {
                        calendarDays?.let {
                            calendarMonthMap[innerIndex++] = it
                        }
                        calendarDays = mutableListOf()
                    }
                    calendar.add(Calendar.DAY_OF_MONTH, -dayTag)
                }
                monthMapIndex[calendar.get(Calendar.MONTH)] = index
                calendarAllMap[index++] = calendarMonthMap
                calendar.add(Calendar.MONTH, -monthTag)
            }

            return calendarAllMap
        }


        /***
         *
         * @param calendarDayRes 对于占位的要赋值月份
         */
        private fun occupyCalendar(calendar: Calendar, calendarDays: MutableList<InterviewCalendarDayRes>, calendarDayRes: InterviewCalendarDayRes) {
            var occupyNum = when (weeks[calendar.get(Calendar.DAY_OF_WEEK) - 1]) {
                "星期日" -> 0
                "星期一" -> 1
                "星期二" -> 2
                "星期三" -> 3
                "星期四" -> 4
                "星期五" -> 5
                "星期六" -> 6
                else -> 0
            }
            for (i in 0..occupyNum - 1) {
                val interviewCalendarDayRes = InterviewCalendarDayRes()
                interviewCalendarDayRes.isOccupy = true
                interviewCalendarDayRes.year = calendarDayRes.year
                interviewCalendarDayRes.month = calendarDayRes.month
                calendarDays.add(interviewCalendarDayRes)
            }
        }

        private fun isHasInterview(timeStamp: Long): Boolean {
            val sdf = SimpleDateFormat("yyyy-MM-dd")
            val goalTime = sdf.format(timeStamp)
            interviewDays?.let {
                for (time in it) {
                    if (TextUtils.equals(goalTime, time)) {
                        return true
                    }
                }
            }
            return false
        }


        private fun isCurrentDay(timeStamp: Long): Boolean {
            var calendar = Calendar.getInstance()
            val sdf = SimpleDateFormat("yyyy-MM-dd")
            return TextUtils.equals(sdf.format(timeStamp), sdf.format(calendar.timeInMillis))
        }

        fun isSameDay(timeStamp: Long, serviceTimeStamp: Long): Boolean {
            val sdf = SimpleDateFormat("yyyy-MM-dd")
            return TextUtils.equals(sdf.format(timeStamp), sdf.format(serviceTimeStamp))
        }

        private fun isBeforeToday(timeStamp: Long): Boolean {
            var calendar = Calendar.getInstance()
            return calendar.timeInMillis > timeStamp
        }

        fun getMonth(calendarDays: List<InterviewCalendarDayRes>?): String? {
            if (calendarDays == null || calendarDays.isEmpty()) {
                return null
            }
            for (calendarDay in calendarDays) {
                if (calendarDay == null) {
                    continue
                }
                return (calendarDay.month + 1).toString()
            }

            return null
        }

        fun getYearMonth(calendarDays: List<InterviewCalendarDayRes>?): String? {
            if (calendarDays == null || calendarDays?.size == 0) {
                return null
            }
            for (calendarDay in calendarDays) {
                if (calendarDay == null) {
                    continue
                }
                return "月" + " /" + calendarDay.year
            }

            return null
        }

        /**
         * 获取当前月份
         */
        fun getCurrentMonth(serverTimStamp: Long): Int {
            var calendar = Calendar.getInstance()
            calendar.timeInMillis = serverTimStamp
            return calendar.get(Calendar.MONTH)
        }

        /**
         * 获取当前月份
         */
        fun getCurrentDay(serverTimStamp: Long): Int {
            var calendar = Calendar.getInstance()
            calendar.timeInMillis = serverTimStamp
            return calendar.get(Calendar.DAY_OF_MONTH)
        }

        /***
         * 获取折叠日历默认选中的页码
         */
        fun getFoldIndexOccupyDay(currentDay: Int, calendarMonthMap: MutableMap<Int, List<InterviewCalendarDayRes>>): Int {
            calendarMonthMap.forEach {
                for (calendarDayRes in it.value) {
                    if (currentDay == calendarDayRes.day) {
                        return it.key
                    }
                }
            }

            return 0
        }


        /***
         * 折叠状态日历，选中某一个月的某一天，其它月份要设成未选中
         */
        fun syncFoldFromExpandCalendar(calendarDay: InterviewCalendarDayRes?, allCalendarsMap:
        MutableMap<Int, MutableMap<Int, List<InterviewCalendarDayRes>>>): InterviewCalendarFoldIndex? {
            val startTime = System.nanoTime()
            var calendarFoldIndex: InterviewCalendarFoldIndex? = null
            allCalendarsMap.forEach {
                it.value.forEach { it2 ->
                    for (calendarDayRes in it2.value) {
                        if (calendarDay != null && calendarDay.isEqualDay(calendarDayRes)) {
                            calendarDayRes.isSelect = true
                            calendarFoldIndex = InterviewCalendarFoldIndex(it.key, it2.key)
                        } else {
                            calendarDayRes.isSelect = false
                        }
                    }
                }
            }
            LogUtils.i("syncFoldFromExpandCalendar", "syncFoldFromExpandCalendar = ${System.nanoTime() - startTime}")
            return calendarFoldIndex
        }

        fun syncExpandFromFoldCalendar(calendarDay: InterviewCalendarDayRes, allCalendarsMap: MutableMap<Int, List<InterviewCalendarDayRes>>?): Int {
            val startTime = System.nanoTime()
            var index = -1
            allCalendarsMap?.forEach {
                for (calendarDayRes in it.value) {
                    if (calendarDay.isEqualDay(calendarDayRes)) {
                        calendarDayRes.isSelect = true
                        index = it.key
                    } else {
                        calendarDayRes.isSelect = false
                    }
                }
            }

            LogUtils.i("syncFoldFromExpandCalendar", "syncExpandFromFoldCalendar = ${System.nanoTime() - startTime}")
            return index
        }


        fun getInviteCalendarStartDay(sTimeStamp: Long): Long {
            serviceTimeStamp = sTimeStamp
            var calendar: Calendar = Calendar.getInstance()
            calendar.set(Calendar.DAY_OF_MONTH, 1)
            calendar.set(Calendar.HOUR_OF_DAY, 0)
            calendar.set(Calendar.MINUTE, 0)
            calendar.set(Calendar.SECOND, 0)
            calendar.timeInMillis = serviceTimeStamp
            calendar.add(Calendar.DAY_OF_MONTH, -getInviteCalendarSundayOccupy(calendar))

            return calendar.timeInMillis
        }


        /***
         * 创建面试邀约所需日历
         * 规则
         * 1 以今天为基础点，用之前的数据填充到周日
         * 2 以今天为基础点，往后推27天，作为可点击区域
         * 3 以今天为基础点，往后推27天后，继续填充（总天数35天），不可点
         */
        fun createInviteCalendarDay(sTimeStamp: Long, interviewTimes: List<String>?): List<InterviewCalendarDayRes>? {
            interviewDays = interviewTimes
            serviceTimeStamp = sTimeStamp
            var calendar: Calendar = Calendar.getInstance()
            calendar.set(Calendar.DAY_OF_MONTH, 1)
            calendar.set(Calendar.HOUR_OF_DAY, 0)
            calendar.set(Calendar.MINUTE, 0)
            calendar.set(Calendar.SECOND, 0)
            calendar.timeInMillis = serviceTimeStamp
            val calendarDays = mutableListOf<InterviewCalendarDayRes>()
            //1 以今天为基础点，用之前的数据填充到最左边（周日）
            occupyInviteCalendar(calendar, calendarDays)
            //2 以今天为基础点，往后推27天，作为可点击区域
            createCalendar(calendar, calendarDays, 1, 27)

            //3 以今天为基础点，往后推27天后，继续填充（总天数35天），不可点
            val beforeSize = calendarDays.size
            createCalendar(calendar, calendarDays, 28, 28 + 35 - beforeSize - 1, isAfterRule = true)
            return calendarDays
        }

        /***
         * 邀约日历占位
         */
        private fun occupyInviteCalendar(calendar: Calendar, calendarDays: MutableList<InterviewCalendarDayRes>) {
            var occupyNum = getInviteCalendarSundayOccupy(calendar)
            for (dayTag in -occupyNum..0) {
                calendar.add(Calendar.DAY_OF_MONTH, dayTag)
                val interviewCalendarDayRes = InterviewCalendarDayRes()
                interviewCalendarDayRes.apply {
                    timeStamp = calendar.timeInMillis
                    year = calendar.get(Calendar.YEAR)
                    month = calendar.get(Calendar.MONTH)
                    day = calendar.get(Calendar.DAY_OF_MONTH)
                    isHasInterview = isHasInterview(calendar.timeInMillis)
                    week = weeks[calendar.get(Calendar.DAY_OF_WEEK) - 1]
                    isCurrentDay = isCurrentDay(calendar.timeInMillis)
                    isOccupy = true
                    if (!isCurrentDay) {
                        isBeforeToday = isBeforeToday(calendar.timeInMillis)
                    }
                }
                calendarDays.add(interviewCalendarDayRes)

                calendar.add(Calendar.DAY_OF_MONTH, -dayTag)
            }
        }


        /***
         * 邀约日历起始时间
         */
        fun getInviteCalendarStartTime(sTimeStamp: Long): Long {
            var calendar: Calendar = Calendar.getInstance()
            calendar.set(Calendar.DAY_OF_MONTH, 1)
            calendar.set(Calendar.HOUR_OF_DAY, 0)
            calendar.set(Calendar.MINUTE, 0)
            calendar.set(Calendar.SECOND, 0)
            calendar.timeInMillis = sTimeStamp
            var occupyNum = getInviteCalendarSundayOccupy(calendar)
            calendar.add(Calendar.DAY_OF_MONTH, -occupyNum)
            return calendar.timeInMillis
        }

        private fun getInviteCalendarSundayOccupy(calendar: Calendar): Int {
            return when (weeks[calendar.get(Calendar.DAY_OF_WEEK) - 1]) {
                "星期日" -> 0
                "星期一" -> 1
                "星期二" -> 2
                "星期三" -> 3
                "星期四" -> 4
                "星期五" -> 5
                "星期六" -> 6
                else -> 0
            }
        }

        private fun createCalendar(calendar: Calendar, calendarDays: MutableList<InterviewCalendarDayRes>, startIndex: Int, endIndex: Int, occupy: Boolean = false, isAfterRule: Boolean = false) {
            for (dayTag in startIndex..endIndex) {
                calendar.add(Calendar.DAY_OF_MONTH, dayTag)
                Log.i("createExtendCalendar", "${calendar.get(Calendar.YEAR)}-${calendar.get(Calendar.MONTH) + 1}-${calendar.get(Calendar.DAY_OF_MONTH)}")
                //1 其它数据正常处理
                val interviewCalendarDayRes = InterviewCalendarDayRes()
                interviewCalendarDayRes.apply {
                    timeStamp = calendar.timeInMillis
                    year = calendar.get(Calendar.YEAR)
                    month = calendar.get(Calendar.MONTH)
                    day = calendar.get(Calendar.DAY_OF_MONTH)
                    isHasInterview = isHasInterview(calendar.timeInMillis)
                    week = weeks[calendar.get(Calendar.DAY_OF_WEEK) - 1]
                    isCurrentDay = isCurrentDay(calendar.timeInMillis)
                    isOccupy = false
                    this.isAfterRuleDay = isAfterRule
                }
                calendarDays.add(interviewCalendarDayRes)
                calendar.add(Calendar.DAY_OF_MONTH, -dayTag)
            }
        }

        /***
         * 创建面试邀约所需要的时间规则
         */
        fun createInviteSelect(): InviteInterviewTime? {
            val serverTimeStamp = SpUserNotClearUtils.getInstance().serverTimeStamp
            val inviteInterviewTime = InviteInterviewTime()
            val calendar = Calendar.getInstance()
            calendar.timeInMillis = serverTimeStamp
            inviteInterviewTime.hour = calendar.get(Calendar.HOUR_OF_DAY)

            //分钟去余数，加20
            inviteInterviewTime.minute = calendar.get(Calendar.MINUTE) / 10 * 10 + 20
            if (inviteInterviewTime.minute - 60 >= 0) {
                if (inviteInterviewTime.hour + 1 > 23) {
                    inviteInterviewTime.isCanSelect = false
                    return inviteInterviewTime
                }
                inviteInterviewTime.hour = inviteInterviewTime.hour + 1
                inviteInterviewTime.minute = inviteInterviewTime.minute - 60
            }

            if (inviteInterviewTime.hour < 7) {
                inviteInterviewTime.hour = 7
                inviteInterviewTime.minute = 0
            }
            inviteInterviewTime.timeStamp = serverTimeStamp
            inviteInterviewTime.date = TimeUtils.secondToYMD(TimeUtils.convertYearMonthDayTime(serverTimeStamp))

            return inviteInterviewTime
        }
    }
}