package com.common.ui

import android.graphics.Color
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.widget.ImageView
import android.widget.TextView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.tools.IntentUtils
import com.amap.api.maps.AMap
import com.amap.api.maps.CameraUpdateFactory
import com.amap.api.maps.MapView
import com.amap.api.maps.model.BitmapDescriptorFactory
import com.amap.api.maps.model.LatLng
import com.amap.api.maps.model.MarkerOptions
import com.amap.api.maps.model.MyLocationStyle
import com.amap.api.services.core.AMapException
import com.amap.api.services.core.PoiItem
import com.amap.api.services.poisearch.PoiResult
import com.amap.api.services.poisearch.PoiSearch.OnPoiSearchListener
import com.lagou.baselib.BaseActivity
import com.lagou.commonlibs.view.LTitleBarLayout

open class MapPreviewActivity:BaseActivity(), OnPoiSearchListener {
    private var titleBar:LTitleBarLayout? = null
    private var mapView:MapView? = null
    private var tvCurrentAddress:TextView? = null
    private var ivCenter:ImageView? = null

    //传递过来的经纬度坐标点
    private var mSelectLatLng: LatLng? = null
    private var aMap: AMap? = null
    override fun getContentLayoutId(): Int = R.layout.activity_map_preview

    override fun init() {
        titleBar = findViewById(R.id.titleBar)
        mapView = findViewById(R.id.map_lbs)
        tvCurrentAddress = findViewById(R.id.tv_current_address)
        ivCenter = findViewById(R.id.iv_center)

        val intent = intent
        val longitude = intent.getDoubleExtra(IntentUtils.KEY_LONGITUDE,0.0)
        val latitude = intent.getDoubleExtra(IntentUtils.KEY_LATITUDE,0.0)
        val title = intent.getStringExtra(IntentUtils.KEY_ADDRESS)
        mSelectLatLng = LatLng(latitude, longitude)
        initMap()
        addMarker(title)
        titleBar?.setLeftImgListener {
            finish()
        }
    }


    private fun initMap() {
        if (aMap == null) {
            aMap = mapView?.map
            val myLocationStyle = MyLocationStyle().myLocationIcon(BitmapDescriptorFactory.fromResource(R.drawable.search_location_marker))
            myLocationStyle.radiusFillColor(Color.argb(80, 73, 169, 238))
            myLocationStyle.strokeColor(Color.argb(80, 73, 169, 238))
            myLocationStyle.strokeWidth(0f)
            myLocationStyle.showMyLocation(true)
            myLocationStyle.myLocationType(MyLocationStyle.LOCATION_TYPE_SHOW)
            aMap?.myLocationStyle = myLocationStyle
            aMap?.uiSettings?.isZoomControlsEnabled = false
            aMap?.uiSettings?.isMyLocationButtonEnabled = false
            aMap?.isMyLocationEnabled = true
        }
        aMap?.setOnMapLoadedListener {
            //地图加载完成回调
            ivCenter?.visibility = View.VISIBLE
            //移动到指定位置
            if (mSelectLatLng != null) {
                //设置中心点和缩放比例
                aMap!!.animateCamera(CameraUpdateFactory.newLatLngZoom(mSelectLatLng, 16f))
                aMap!!.moveCamera(CameraUpdateFactory.changeLatLng(mSelectLatLng))
            }
        }
    }

    private fun addMarker(title:String?) {
        val markerOption = MarkerOptions()
        val makerView = LayoutInflater.from(this).inflate(R.layout.layout_map_marker,null)
        val tvTitle = makerView.findViewById<TextView>(R.id.tv_current_address)
        tvTitle.text = title
        markerOption.position(mSelectLatLng).draggable(false).icon(BitmapDescriptorFactory.fromView(makerView))
        markerOption.isFlat = true
        aMap?.addMarker(markerOption)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        mapView?.onCreate(savedInstanceState)
    }

    override fun onResume() {
        super.onResume()
        mapView?.onResume()
    }

    override fun onPause() {
        super.onPause()
        mapView?.onPause()
    }

    override fun onSaveInstanceState(outState: Bundle) {
        super.onSaveInstanceState(outState)
        mapView?.onSaveInstanceState(outState)
    }

    override fun onDestroy() {
        super.onDestroy()
        mapView?.onDestroy()
    }

    override fun onPoiSearched(poiResult: PoiResult?, resultCode: Int) {
        if (resultCode == AMapException.CODE_AMAP_SUCCESS) {
            val poiItems = poiResult?.pois
            if (poiItems.isNullOrEmpty()){
                tvCurrentAddress?.visibility = View.GONE
            }else{
                tvCurrentAddress?.visibility = View.VISIBLE
                tvCurrentAddress?.text = poiItems[0]?.title
            }
        }else{
            tvCurrentAddress?.visibility = View.GONE
        }
    }

    override fun onPoiItemSearched(p0: PoiItem?, p1: Int) {
    }
}