package com.common.ui.adapter

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.common.bean.VideoRes
import com.common.ui.video.VideoListActivity
import com.common.utils.CommonVideoUtil
import com.lagou.commonlibs.imageload.ImageLoadUtils
import com.lagou.commonlibs.utils.DeviceUtils
import com.lagou.commonlibs.utils.ResourceUtils

/**
 * @date： Crate by wangjunwei on 2022/7/12
 * @description：视频列表对应的适配器
 */
class GalleryVideoAdapter(private val context: Context) : RecyclerView.Adapter<RecyclerView.ViewHolder>() {

    private var videoRes: MutableList<VideoRes>? = null
    private var itemListener: OnItemListener? = null

    /**
     * 高宽比，宽是屏幕（去除边距）三等分
     */
    private val imageThumbScale = 292 * 1f / 247
    /**
     * 视频列表横坐标方向所有间距
     */
    private val SPACE = (VideoListActivity.KEY_SPAN_COUNT - 1) * ResourceUtils.getDimenSize(R.dimen.dp_2) + ResourceUtils.getDimenSize(R.dimen.dp_1)

    /***
     * 缩略图的宽高
     */
    val imageThumbWidth: Int = (DeviceUtils.getScreenWidth() - SPACE) / VideoListActivity.KEY_SPAN_COUNT
    val imageThumbHeight: Int = (imageThumbWidth * imageThumbScale).toInt()


    fun setOnItemListener(itemListener: OnItemListener?){
        this.itemListener = itemListener
    }

    fun setData(videoResS: MutableList<VideoRes>?){
        this.videoRes = videoResS
        notifyDataSetChanged()
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        var itemView = LayoutInflater.from(context).inflate(R.layout.adapter_gallery_video, null, false)
        return VideoViewHolder(itemView)
    }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        videoRes?.let {
            if (it.size > position){
                var videoRes = it[position]
                (holder as? VideoViewHolder)?.bindView(videoRes)
                holder.itemView.setOnClickListener {
                    itemListener?.onItemCheck(position, videoRes)
                }
            }
        }
    }

    override fun getItemCount(): Int {
        videoRes?.let {
            return it.size
        }
        return 0
    }

    inner class VideoViewHolder : RecyclerView.ViewHolder {

        var thumbImage: ImageView = itemView.findViewById(R.id.image_video_thumbnail)
        var tvTime : TextView = itemView.findViewById(R.id.tv_video_time)

        constructor(itemView: View) : super(itemView){
            val layoutParams = thumbImage.layoutParams
            layoutParams.width = imageThumbWidth
            layoutParams.height = imageThumbHeight
            thumbImage.layoutParams = layoutParams
        }

        fun bindView(videoRes: VideoRes?){
            videoRes?.let {
                ImageLoadUtils
                    .with(thumbImage.context)
                    .load(it.uri)
                    .centerCrop()
                    .into(thumbImage)
                tvTime.text = it.handleDuration()
            }
        }
    }

    interface OnItemListener {
        fun onItemCheck(position: Int, videoRes: VideoRes)
    }
}