package com.common.ui.dialog

import android.view.View
import com.ApiService
import com.alpha.lagouapk.R
import com.common.WECHAT_NUM_MAX
import com.common.WECHAT_NUM_MIN
import com.common.utils.KeyboardUtils
import com.lagou.annotation.aspect.SingleClick
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.view.filter.EmojiFilter
import com.lagou.commonlibs.view.filter.NotChineseFilter
import com.lagou.component.ui.dialog.LgBaseDialog
import com.lagou.component.ui.dialog.viewClick
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.android.synthetic.main.dialog_switch_wechat.*

/**
 * 交换微信
 */
class SwitchWechatDialog : LgBaseDialog(), View.OnClickListener {
    companion object {
        fun newInstance(): SwitchWechatDialog = SwitchWechatDialog()
    }

    enum class EditStyle(val resId: Int) {
        NORMAL(R.drawable.shape_f8f9fa_r4),
        ERROR(R.drawable.shape_f8f9fa_stroke05_ff7452_r4),
        VALID(R.drawable.shape_f8f9fa_stroke05_00b38a_r4)
    }

    override fun getLayoutId(): Int = R.layout.dialog_switch_wechat
    override fun initView(view: View) {
        super.initView(view)
        editWechat.filters = arrayOf(EmojiFilter(), NotChineseFilter())
    }

    override fun fillUI() {
        editWechat.setOnFocusChangeListener { _, hasFocus ->
            if (hasFocus) {
                showEditStyle(EditStyle.VALID)
            }
        }

        viewClick(this, tvSwitchWechatCancel, tvSwitchWechatOk)
    }

    private fun showEditStyle(editStyle: EditStyle) {
        editWechat.background = ResourceUtils.getDrawable(editStyle.resId)
        tvShowLimit.visibility = if (editStyle == EditStyle.ERROR) View.VISIBLE else View.INVISIBLE
        if (editStyle == EditStyle.ERROR){
            editWechat.clearFocus()
        }
    }

    override fun getBackgroundResId(): Int = R.drawable.bg_ffffff_corner8

    /**
     * 校验微信输入是否符合规则
     */
    private fun isInputValid(): Boolean {
        val length = getValidWechat().length
        return length in WECHAT_NUM_MIN..WECHAT_NUM_MAX
    }

    /**
     * 获取有效的微信号，去除了所有的空格
     * @return
     */
    private fun getValidWechat(): String {
        val input = editWechat.text?.toString()
        return input?.replace("\\s*".toRegex(), "") ?: ""
    }

    @SingleClick
    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.tvSwitchWechatCancel -> dismiss()
            R.id.tvSwitchWechatOk -> {
                saveWechat()
            }
        }
    }

    override fun isCanceledOnTouchOutside(): Boolean = false

    /**
     * 保存微信号
     */
    private fun saveWechat() {
        if (isInputValid().not()) {
           showEditStyle(EditStyle.ERROR)
            return
        }
        KeyboardUtils.hideSoftInput(editWechat)
        showLoading()
        ApiService.api.saveWechatNum(getValidWechat())
            .enqueue(object : BaseCallback<Any>() {
                override fun onSuccess(p0: HttpRequest<*>?, p1: HttpResponse<Any>?) {
                    if (isActivityDestroyed()) {
                        return
                    }
                    onSaveWechatSuccessCallback?.onSaveWechatSuccess()
                    dismiss()
                }
            })
    }

    private var onSaveWechatSuccessCallback: OnSaveWechatSuccessCallback? = null
    fun setOnSaveWechatSuccessCallback(onSaveWechatSuccessCallback: OnSaveWechatSuccessCallback?): SwitchWechatDialog {
        this.onSaveWechatSuccessCallback = onSaveWechatSuccessCallback
        return this
    }

    interface OnSaveWechatSuccessCallback {
        fun onSaveWechatSuccess()
    }
}