package com.common.ui.question_answer

import android.content.Context
import android.view.Gravity
import android.view.View
import android.widget.LinearLayout
import com.alpha.lagouapk.R
import com.common.bean.PositionQuestionBean
import com.common.bean.QuestionsAndAnswersBean
import com.common.utils.ViewUtils
import com.lagou.commonlibs.ui.dialog.BaseDialogFragment
import com.lagou.commonlibs.utils.InputMethodUtil
import com.lagou.commonlibs.utils.ResourceUtils
import kotlinx.android.synthetic.main.dialog_question_answer.btn_submit_answer
import kotlinx.android.synthetic.main.dialog_question_answer.iv_close_dlg
import kotlinx.android.synthetic.main.dialog_question_answer.ll_question_container
import kotlinx.android.synthetic.main.dialog_question_answer.question_answer_root
import kotlinx.android.synthetic.main.dialog_question_answer.scroll_question_container
import kotlinx.android.synthetic.main.dialog_question_answer.tv_position_name

abstract class BaseQuestionAndAnswerDialog : BaseDialogFragment(){
    var mQuestionAndAnswerInfo: QuestionsAndAnswersBean? = null
    var mQuestionCount:Int = 0
    companion object{
        const val KEY_QUESTION_ANSWER_INFO = "key_question_answer_info"
        const val KEY_ASK_PAPER_ID = "key_ask_paper_id"
        const val KEY_MESSAGE_ID = "key_message_id"
        const val MAX_INPUT_COUNT = 300
    }

    /**
     * 滚动区域的最大高度
     */
    abstract fun getMaxHeight():Int
    /**
     * 是否是查看答案
     */
    open fun isCheckAnswer():Boolean = false
    override fun getLayoutId(): Int = R.layout.dialog_question_answer

    override fun initListener() {
        super.initListener()
        ViewUtils.setViewsClick(this, iv_close_dlg, btn_submit_answer)
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        mQuestionAndAnswerInfo = arguments?.getParcelable(KEY_QUESTION_ANSWER_INFO)
        mQuestionCount = mQuestionAndAnswerInfo?.questionNum ?: 0
    }

    override fun initConfigAtCreateView() {
        super.initConfigAtCreateView()
        dialog?.setCanceledOnTouchOutside(false)
        dialog?.window?.setGravity(Gravity.BOTTOM)
    }
    override fun fillUI() {
        super.fillUI()
        scroll_question_container.setScrollViewMaxHeight(getMaxHeight())
        tv_position_name.text = ResourceUtils.getString(R.string.question_relate_position,mQuestionAndAnswerInfo?.positionName)
        addQuestionCard()
    }

    /**
     * 添加卡片
     */
    private fun addQuestionCard() {
         ll_question_container.removeAllViews()
         val questionList = mQuestionAndAnswerInfo?.questions
         val size = questionList?.size ?: 0
         questionList?.forEachIndexed { index, positionQuestionBean ->
             val questionCard = generateQuestionCard(index + 1,positionQuestionBean)
             questionCard?.let { questionCard ->
                 ll_question_container.addView(questionCard)
                 if (index < size - 1){
                     ll_question_container.addView(generateDivider())
                 }
             }
         }
    }

    private fun generateQuestionCard(sequence:Int,questionBean: PositionQuestionBean):QuestionAnswerCard?{
        context?.let {
            val card = QuestionAnswerCard(it)
            card.layoutParams = LinearLayout.LayoutParams(
                LinearLayout.LayoutParams.MATCH_PARENT,
                LinearLayout.LayoutParams.WRAP_CONTENT)
            card.setData(sequence,questionBean,isCheckAnswer(),mQuestionCount == 1)
            initQuestionCardListener(card)
            return card
        }
        return null
    }

    /**
     * 给卡片设置点击事件
     */
    protected open fun initQuestionCardListener(card:QuestionAnswerCard){

    }

    private fun generateDivider(): View {
        val divider = View(context)
        divider.layoutParams = LinearLayout.LayoutParams(
            LinearLayout.LayoutParams.MATCH_PARENT,
            ResourceUtils.getDimenSize(R.dimen.dp_14))
        return divider
    }

    override fun onClick(v: View?) {
        super.onClick(v)
        when(v?.id){
            R.id.iv_close_dlg -> {
                InputMethodUtil.hideSystemKeyBoard(context,question_answer_root)
                dismiss()
            }
        }
    }

    override fun getGravity(): Int  = Gravity.BOTTOM
}