package com.common.ui.question_answer
import android.content.Context
import android.graphics.Color
import android.os.Bundle
import android.text.InputFilter
import android.view.KeyEvent
import com.ApiService
import com.alpha.lagouapk.R
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.commonlibs.ui.dialog.BaseBottomSheetDialogFragment
import com.lagou.commonlibs.view.MaxInputWatcher
import com.lagou.commonlibs.view.MaxInputWatcher.OnMaxInputListener
import com.lagou.commonlibs.view.filter.EmojiFilter
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.android.synthetic.main.dialog_edit_custom_question.*

class EditCustomQuestionDialog:  BaseBottomSheetDialogFragment(){
    private var mQuestionId:Int = -1
    private var mQuestionName:String? = null
    private var mSaveCallback:((String?,Int)->Unit)? = null
    companion object{
        const val KEY_QUESTION_CONTENT = "key_question_content"
        const val KEY_QUESTION_ID = "key_question_id"
        const val MAX_INPUT_COUNT = 200
        @JvmStatic
        fun getInstance(questionContent:String?,questionId:Int,saveCallback:((String?,Int)->Unit)?):EditCustomQuestionDialog{
            val dialog = EditCustomQuestionDialog()
            val bundle = Bundle()
            bundle.putString(KEY_QUESTION_CONTENT,questionContent)
            bundle.putInt(KEY_QUESTION_ID,questionId)
            dialog.arguments = bundle
            dialog.setSaveCallback(saveCallback)
            return dialog
        }
    }

    override fun getTheme(): Int {
        return R.style.BottomSheetDialog_SoftInput
    }

    private fun setSaveCallback(saveCallback: ((String?,Int) -> Unit)?) {
        this.mSaveCallback = saveCallback
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        mQuestionId = arguments?.getInt(KEY_QUESTION_ID) ?: -1
        mQuestionName = arguments?.getString(KEY_QUESTION_CONTENT)
    }
    override fun getLayoutId(): Int = R.layout.dialog_edit_custom_question

    override fun fillUI() {
        super.fillUI()
        btn_save_question.isEnabled = mQuestionName.isNullOrEmpty().not()
        edit_question.apply {
            filters = arrayOf<InputFilter>(EmojiFilter())
            setText(mQuestionName)
            setSelection(mQuestionName?.length ?: 0)
            requestFocus()
        }

    }

    override fun initListener() {
        super.initListener()
        iv_close_dlg.setOnClickListener { dismiss() }
        btn_save_question.setOnClickListener {
            saveQuestion()
        }
        val maxInputWatcher = MaxInputWatcher(MAX_INPUT_COUNT).apply{
            setIsCalTextLength(true)
            setInputType(MaxInputWatcher.INPUT_TYPE_DOUBLE)
            setMaxInputListener(object : OnMaxInputListener{
                override fun equalMaxLength(isEqual: Boolean) {
                    val color = if (isEqual) Color.parseColor("#FF3333") else Color.parseColor("#cccccc")
                    tv_input_count.setTextColor(color)
                }

                override fun onTextChanged(s: String?) {
                    btn_save_question.isEnabled = s?.trim()?.isNotEmpty() ?: false
                }

                override fun showLengthTip(length: Int, text: String?) {
                    tv_input_count.text = "$length/$MAX_INPUT_COUNT"
                }

            })
        }
        edit_question.addTextChangedListener(maxInputWatcher)
    }

    private fun saveQuestion() {
        val request = if (mQuestionId <= 0) ApiService.api.saveCustomQuestion(edit_question?.text?.toString()) else ApiService.api.saveCustomQuestion(edit_question?.text?.toString(),mQuestionId)
        request.enqueue(object: BaseCallback<Int>(){
            override fun onSuccess(request: com.lagou.socketchannel.http.HttpRequest<*>?, response: HttpResponse<Int>?) {
               mSaveCallback?.invoke(edit_question?.text?.toString(),response?.content ?: -1)
                dismiss()
            }
        })
    }

    override fun isDraggable() = false
    private fun setBackDismiss() {
        if (dialog == null) {
            return
        }
        dialog?.setOnKeyListener { _, keyCode, _ ->
            keyCode == KeyEvent.KEYCODE_BACK
        }
    }
    override fun initConfigAtCreateView() {
        super.initConfigAtCreateView()
        dialog?.setCanceledOnTouchOutside(false)
    }
    override fun onStart() {
        super.onStart()
        setBackDismiss()
    }
}