package com.common.ui.question_answer

import android.content.Context
import android.os.Message
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.ApiService
import com.alpha.lagouapk.R
import com.alpha.lagouapk.widget.decoration.DividerItemDecoration
import com.common.bean.PositionRelateQuestionsBean
import com.common.utils.ViewUtils
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.commonlibs.event.Event
import com.lagou.commonlibs.event.OkBus
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.component.ui.dialog.LgAlertDialog
import com.lagou.component.ui.dialog.listener.LgDialogListener
import com.lagou.lagouee.observer.BEventTags
import com.lagou.lagouee.utils.IntentUtils
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.android.extensions.LayoutContainer
import kotlinx.android.synthetic.main.activity_question_answer_set.*
import kotlinx.android.synthetic.main.card_position_relate_question.*
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.async
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

/**
 * 职位问答开关设置页
 */
class PositionQuestionOpenActivity:BaseActivity() {
    private var mPositionId:Long = 0
    private var mSessionId:Long = 0
    private var mSwitchState:Int = State.INIT.ordinal
    companion object{
        const val KEY_POSITION_ID = "key_position_id"
        const val KEY_SESSION_ID = "key_session_id"
    }
    enum class State(value:Int){
        INIT(0),//未配置过
        CLOSED(1),//已关闭
        OPENED(2)//已开启
    }
    override fun getContentLayoutId(): Int = R.layout.activity_question_answer_set

    override fun init() {
        titleBar.setLeftImgListener { finish() }
        sbQuestion.setOnClickListener{
            saveSwitchStateAndFetchQuestions(sbQuestion.isOn.not())
        }
        mPositionId = intent.getLongExtra(KEY_POSITION_ID,-1)
        mSessionId = intent.getLongExtra(KEY_SESSION_ID,-1)
        btnAddQuestion.setOnClickListener { IntentUtils.startQuestionRelateActivity(this,false,mPositionId,mSessionId) }
        btnBottomAddQuestion.setOnClickListener { btnAddQuestion.performClick() }
        rvRelateQuestions.apply {
            layoutManager = LinearLayoutManager(this@PositionQuestionOpenActivity)
            addItemDecoration(DividerItemDecoration(this@PositionQuestionOpenActivity,DividerItemDecoration.HORIZONTAL_LIST,ResourceUtils.getDimenSize(R.dimen.dp_14),ResourceUtils.getColor(R.color.transparent)))
        }
        fetchData()
        OkBus.getInstance().register(BEventTags.UPDATE_POSITION_RELATE_QUESTIONS,updateRelateQuestions )
    }

    /**
     * 更新职位关联问题列表
     */
    private val updateRelateQuestions = Event { _ ->
        lifecycleScope.launch(Dispatchers.Main){
            val relateJob = async { fetchPositionRelateQuestions() }
            val relateQuestions = relateJob.await()
            showQuestions(relateQuestions)
        }
    }

    /**
     * 保存开关状态并请求问题列表
     */
    private fun saveSwitchStateAndFetchQuestions(isOpen:Boolean){
        showLoading()
        ApiService.api.saveQuestionSwitchState(isOpen).enqueue(object:BaseCallback<Any?>(false){
            override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<Any?>?) {
                if (isActivityDestroyed){
                    return
                }
                sbQuestion.setState(isOpen)
                mSwitchState = if (isOpen) State.OPENED.ordinal else State.CLOSED.ordinal
                hideLoading()
                updateRelateQuestions.call(Message.obtain())
            }

            override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<Any?>?) {
                super.onFailure(request, response)
                hideLoading()
            }
        })
    }

    private fun fetchData() {
        lifecycleScope.launch(Dispatchers.Main){
            showLoading()
            val relateJob = async { fetchPositionRelateQuestions() }
            val switchJob = async { fetchSwitchState() }
            val switchState = switchJob.await()
            val relateQuestions = relateJob.await()
            mSwitchState = switchState ?: State.INIT.ordinal
            hideLoading()
            showQuestions(relateQuestions)
        }

    }

    private fun showQuestions(relateQuestions:MutableList<PositionRelateQuestionsBean>?){
        if (relateQuestions.isNullOrEmpty() || mSwitchState != State.OPENED.ordinal) {
            noRelateQuestionLogic()
        } else {
            relateQuestionLogic(relateQuestions!!)
        }
    }

    /**
     * 获取所有职位关联的问题
     */
    private suspend fun fetchPositionRelateQuestions():MutableList<PositionRelateQuestionsBean>? {
        return withContext(Dispatchers.IO){
            ApiService.api.fetchPositionRelateQuestionList().callback(object : BaseCallback<MutableList<PositionRelateQuestionsBean>>(){
                override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<MutableList<PositionRelateQuestionsBean>>?) {}
            }).execute()?.content
        }

    }

    /**
     * 职位没有关联问题
     */
    private fun noRelateQuestionLogic(){
        ViewUtils.setViewGone(rvRelateQuestions,btnBottomAddQuestion)
        ViewUtils.setViewVisibility(llNoRelateQuestion)
        sbQuestion.setState(mSwitchState == State.OPENED.ordinal)
        btnAddQuestion.visibility = if (mSwitchState == State.OPENED.ordinal) View.VISIBLE else View.GONE
    }
    /**
     * 已经设置过职位关联问题
     */
    private fun relateQuestionLogic(list:MutableList<PositionRelateQuestionsBean>){
        ViewUtils.setViewVisibility(rvRelateQuestions,btnBottomAddQuestion)
        ViewUtils.setViewGone(llNoRelateQuestion)
        sbQuestion.setState(mSwitchState == State.OPENED.ordinal)
        rvRelateQuestions.adapter = PositionRelateQuestionAdapter(this,list,mSessionId) { isEmpty ->
            if (isEmpty){
                noRelateQuestionLogic()
            }
        }
    }


    /**
     * 查看开关状态
     */
    private suspend fun fetchSwitchState():Int? {
        return withContext(Dispatchers.IO){
            ApiService.api.checkQuestionSwitchState().callback(object : BaseCallback<Int>(){
                override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<Int>?) {}
            }).execute()?.content
        }
    }

    class PositionRelateQuestionAdapter(val ctx: Context, val data:MutableList<PositionRelateQuestionsBean>, val sessionId:Long,
                                        private val deleteCallback:((Boolean) -> Unit)? = null): RecyclerView.Adapter<PositionRelateQuestionAdapter.MVHolder>() {
        class MVHolder(override val containerView: View):RecyclerView.ViewHolder(containerView),LayoutContainer

        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): MVHolder {
            val holder = MVHolder(LayoutInflater.from(ctx).inflate(R.layout.card_position_relate_question,parent,false))
            holder.iv_edit_relate.setOnClickListener {
                val bean = data[holder.adapterPosition]
                IntentUtils.startQuestionRelateActivity(ctx,true,bean.lgPositionId,sessionId)
            }
            holder.iv_delete_relate.setOnClickListener {
                val bean = data[holder.adapterPosition]
                LgAlertDialog.Builder()
                    .content("请确认是否删除？")
                    .leftButtonText(ResourceUtils.getString(R.string.cancel))
                    .rightButtonText(ResourceUtils.getString(R.string.ensure))
                    .create().listener(object:LgDialogListener(){
                        override fun onRight() {
                            super.onRight()
                            doDelete(holder.adapterPosition,bean.lgPositionId)
                        }
                    }).show((ctx as PositionQuestionOpenActivity).supportFragmentManager,"PositionQuestionOpenActivity")

            }
            return holder
        }

        /**
         * 删除关联问题
         */
        private fun doDelete(adapterPosition: Int,positionId:Long) {
            data.removeAt(adapterPosition)
            notifyItemRemoved(adapterPosition)
            notifyItemRangeChanged(adapterPosition, data.size - adapterPosition)
            ApiService.api.deletePositionRelateQuestions(positionId).enqueue()
            deleteCallback?.invoke(data.isEmpty())
        }

        override fun getItemCount(): Int = data.size

        override fun onBindViewHolder(holder: MVHolder, position: Int) {
            val bean = data[position]
            if (bean.questions.isNullOrEmpty() || bean.questions.size > 1){
                holder.tv_relate_question.text = ResourceUtils.getString(R.string.relate_question_num,bean.questionNum)
            }else{
                holder.tv_relate_question.text =  bean.questions[0].content
            }
            holder.tv_relate_position.text = ResourceUtils.getString(R.string.relate_position,bean.positionName)
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        OkBus.getInstance().unRegister(updateRelateQuestions)
    }
}