package com.common.ui.question_answer

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.ApiService
import com.alpha.lagouapk.R
import com.alpha.lagouapk.widget.decoration.DividerItemDecoration
import com.common.bean.PositionQuestionBean
import com.common.bean.PositionsCategory
import com.common.bean.TYPE_CUSTOM
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.commonlibs.event.OkBus
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.utils.StringUtil
import com.lagou.commonlibs.utils.ToastUtil
import com.lagou.component.ui.dialog.LgAlertDialog
import com.lagou.component.ui.dialog.listener.LgDialogListener
import com.lagou.lagouee.observer.BEventTags
import com.lagou.lagouee.utils.IntentUtils
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.android.extensions.LayoutContainer
import kotlinx.android.synthetic.main.activity_position_relate_question_main.*
import kotlinx.android.synthetic.main.card_position_question_select_custom.clCustomQuestionRoot
import kotlinx.android.synthetic.main.card_position_question_select_custom.ivCustomDelete
import kotlinx.android.synthetic.main.card_position_question_select_custom.ivCustomEdit
import kotlinx.android.synthetic.main.card_position_question_select_custom.ivCustomQuestionCheck
import kotlinx.android.synthetic.main.card_position_question_select_custom.tvCustomQuestionName
import kotlinx.android.synthetic.main.card_position_question_select_system.clSystemQuestionRoot
import kotlinx.android.synthetic.main.card_position_question_select_system.flSystemQuestionLabel
import kotlinx.android.synthetic.main.card_position_question_select_system.ivSystemQuestionCheck
import kotlinx.android.synthetic.main.card_position_question_select_system.tvSystemQuestionName
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.async
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

/**
 * 职位关联问题设置页
 */
class PositionSetRelateQuestionActivity : BaseActivity() {
    private var mPositionId: Long = -1
    private var mQuestionList = mutableListOf<PositionQuestionBean>()
    private var mPositionList = ArrayList<PositionsCategory>()
    private lateinit var mQuestionAdapter: QuestionSelectAdapter
    private var mSelectPositionIds: String? = null
    private var mIsEdit: Boolean = false
    private var mSessionId:Long = 0
    companion object {
        const val KEY_POSITION_ID = "key_position_id"
        const val KEY_IS_EDIT = "key_is_edit"
        const val KEY_SESSION_ID = "key_session_id"
        const val MAX_SELECT_QUESTION_NUM = 3//最多关联问题数量
        const val MAX_CUSTOM_QUESTION_NUM = 3//自定义问题最大数量
    }

    override fun getContentLayoutId(): Int = R.layout.activity_position_relate_question_main

    override fun init() {
        initView()
        initListener()
        fetchRelateData()
    }

    private fun fetchRelateData() {
        lifecycleScope.launch(Dispatchers.Main) {
            showLoading()
            val questionJob = async { fetchRelateQuestions() }
            val positionsJob = async { fetchPositions() }
            val questions = questionJob.await()
            val positions = positionsJob.await()
            hideLoading()
            if (positions.isNullOrEmpty()) {
                emptyView.visibility = View.VISIBLE
                rlQuestionContainer.visibility = View.GONE
                return@launch
            }
            mPositionList.addAll(positions)
            emptyView.visibility = View.GONE
            rlQuestionContainer.visibility = View.VISIBLE
            setPositionName(getPositionNameById())
            if (questions.isNullOrEmpty().not()) {
                mQuestionList.addAll(questions!!)
                updateCustomAndSaveButton()
                mQuestionAdapter.notifyDataSetChanged()
            }
        }
    }

    private fun getPositionNameById(): String? {
        mPositionList.forEach { categoryBean ->
            categoryBean.positions?.forEach {
                if (mPositionId == it.lgPositionId) {
                    return it.positionName
                }
            }
        }
        return null
    }

    private fun initView() {
        mIsEdit = intent.extras?.getBoolean(KEY_IS_EDIT, false) ?: false
        mPositionId = intent.extras?.getLong(KEY_POSITION_ID) ?: -1L
        mSessionId = intent.extras?.getLong(KEY_SESSION_ID) ?: -1L
        if (mPositionId > 0) {
            mSelectPositionIds = mPositionId.toString()
        }
        tvSelectPositionTitle.setTextColor(
            if (mIsEdit) ResourceUtils.getColor(R.color.color_cccccc) else ResourceUtils.getColor(
                R.color.color_333333
            )
        )
        rlSelectPosition.isEnabled = mIsEdit.not()
        ivSelectPositionIndicator.visibility = if (mIsEdit) View.GONE else View.VISIBLE
        mQuestionAdapter =
            QuestionSelectAdapter(this@PositionSetRelateQuestionActivity, mQuestionList)
        mQuestionAdapter.setOnQuestionChangeListener(object :
            QuestionSelectAdapter.OnQuestionChangeListener {
            override fun onEdit(position: Int) {
                val bean = mQuestionList[position]
                showAddOrEditQuestionDialog(bean.content, bean.questionId)
            }

            override fun onDelete(position: Int) {
                LgAlertDialog.Builder()
                    .content("请确认是否删除？")
                    .leftButtonText(ResourceUtils.getString(R.string.cancel))
                    .rightButtonText(ResourceUtils.getString(R.string.ensure))
                    .create().listener(object:LgDialogListener(){
                        override fun onRight() {
                            super.onRight()
                            val questionId = mQuestionList[position].questionId
                            mQuestionList.removeAt(position)
                            mQuestionAdapter.notifyItemRemoved(position)
                            mQuestionAdapter.notifyItemRangeChanged(position, mQuestionList.size - position)
                            ApiService.api.deleteCustomPositionQuestion(questionId).enqueue(object : BaseCallback<Any?>(false){
                                override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<Any?>?) {
                                    if (isActivityEnable){
                                        OkBus.getInstance().onEvent(BEventTags.UPDATE_POSITION_RELATE_QUESTIONS)
                                    }
                                }
                            })
                            updateCustomAndSaveButton()
                        }
                    }).show(supportFragmentManager,"DeleteCustomQuestion")
            }

            override fun onSelectChanged(isSelect: Boolean) {
                btnSave.isEnabled = isSelect && tvPositionName.text.isNullOrEmpty().not()
            }

        })
        rvQuestions.apply {
            layoutManager = LinearLayoutManager(this@PositionSetRelateQuestionActivity)
            addItemDecoration(
                DividerItemDecoration(
                    this@PositionSetRelateQuestionActivity,
                    DividerItemDecoration.HORIZONTAL_LIST,
                    ResourceUtils.getDimenSize(R.dimen.dp_14),
                    ResourceUtils.getColor(R.color.transparent)
                )
            )
            adapter = mQuestionAdapter
        }
    }

    private fun initListener() {
        titleBar.setLeftImgListener { finish() }
        rlSelectPosition.setOnClickListener {
            SelectPositionDialog.getInstance(
                mPositionList,
                mSelectPositionIds
            ) { positionNames, positionIds ->
                this.mSelectPositionIds = positionIds
                setPositionName(positionNames)
                updateSaveButton()
            }.show(supportFragmentManager)
        }
        tvCustomQuestion.setOnClickListener {
            if (it.alpha != 1f) {
                ToastUtil.show(
                    this,
                    ResourceUtils.getString(R.string.max_relate_question, MAX_CUSTOM_QUESTION_NUM)
                )
                return@setOnClickListener
            }
            showAddOrEditQuestionDialog()
        }
        ivCustomQuestionAdd.setOnClickListener { tvCustomQuestion.performClick() }
        emptyView.setOnClickBtnListener {
            IntentUtils.startReleaseActivity(this)
        }
        btnSave.setOnClickListener {
            val iSelectRelatePositions : Boolean = mPositionList.any {category -> category.positions?.any { it.hasQuestion && it.isSelect} == true}
            if (iSelectRelatePositions){
                LgAlertDialog.Builder()
                    .title("是否更新已关联的职位？")
                    .content("您选择的部分职位已关联过其他问题，确认后将覆盖原有关联")
                    .leftButtonText(ResourceUtils.getString(R.string.cancel))
                    .rightButtonText(ResourceUtils.getString(R.string.ensure))
                    .create().listener(object: LgDialogListener(){
                        override fun onRight() {
                            super.onRight()
                            commitRelateQuestions()
                        }
                    }).show(supportFragmentManager,this@PositionSetRelateQuestionActivity::class.java.name)
            }else{
                commitRelateQuestions()
            }

        }
    }

    private fun setPositionName(positionName: String?) {
        tvPositionName.text = positionName
        tvPositionName.visibility = if (positionName.isNullOrEmpty()) View.GONE else View.VISIBLE
        tvPositionName.setTextColor(
            if (mIsEdit) ResourceUtils.getColor(R.color.color_cccccc) else ResourceUtils.getColor(
                R.color.color_333333
            )
        )
    }

    /**
     * 提交
     */
    private fun commitRelateQuestions() {
        showLoading()
        val questionIds = mQuestionList.filter { it.choosed }.joinToString(",") { it.questionId.toString() }
        ApiService.api.savePositionRelateQuestion(questionIds, mSelectPositionIds,mSessionId)
            .enqueue(object : BaseCallback<Any?>() {
                override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<Any?>?) {
                    hideLoading()
                    if (isActivityDestroyed){
                        return
                    }
                    OkBus.getInstance().onEvent(BEventTags.UPDATE_POSITION_RELATE_QUESTIONS)
                    finish()
                }

                override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<Any?>?) {
                    super.onFailure(request, response)
                    hideLoading()
                }
            })
    }

    /**
     * 请求职位关联的问题
     */
    private suspend fun fetchRelateQuestions(): List<PositionQuestionBean>? {
        return withContext(Dispatchers.IO) {
            val request =
                if (mPositionId <= 0) ApiService.api.fetchAllQuestions() else ApiService.api.fetchQuestionsByPositionId(
                    mPositionId
                )
            request.callback(object : BaseCallback<List<PositionQuestionBean>>() {
                override fun onSuccess(
                    request: HttpRequest<*>?,
                    response: HttpResponse<List<PositionQuestionBean>>?
                ) {
                }
            }).execute()?.content
        }
    }

    /**
     * 获取在线职位
     */
    private suspend fun fetchPositions(): List<PositionsCategory>? {
        return withContext(Dispatchers.IO) {
            ApiService.api.fetchPositionsByCategory()
                .callback(object : BaseCallback<List<PositionsCategory>?>() {
                    override fun onSuccess(
                        request: HttpRequest<*>?,
                        response: HttpResponse<List<PositionsCategory>?>?
                    ) {
                    }
                }).execute()?.content
        }
    }

    /**
     * 展示添加或者编辑自定义问题弹窗
     */
    private fun showAddOrEditQuestionDialog(questionName: String? = null, questionId: Int = -1) {
        EditCustomQuestionDialog.getInstance(questionName, questionId) { newQuestionName, newQuestionId ->
            if(newQuestionId > 0){
                val questionBean =  mQuestionList.find { it.questionId == questionId }
                //编辑再保存的时候，后端返回的questionId发生变化了，所以需要将之前的那个删除
                if (questionBean != null){
                    mQuestionList.remove(questionBean)
                }
                val newQuestionBean = PositionQuestionBean(newQuestionId, newQuestionName, TYPE_CUSTOM, null, null, false)
                mQuestionList.add(0, newQuestionBean)
                updateCustomAndSaveButton()
                mQuestionAdapter.notifyDataSetChanged()
            }
        }.show(supportFragmentManager)
    }

    private fun updateCustomAndSaveButton() {
        updateSaveButton()
        ivCustomQuestionAdd.alpha =
            if (mQuestionList.filter { it.type == TYPE_CUSTOM }.size >= MAX_CUSTOM_QUESTION_NUM) 0.3f else 1f
        tvCustomQuestion.alpha = ivCustomQuestionAdd.alpha
    }

    private fun updateSaveButton() {
        btnSave.isEnabled =
            mQuestionList.any { it.choosed } && tvPositionName.text.isNullOrEmpty().not()
    }

    class QuestionSelectAdapter(val ctx: Context, val data: MutableList<PositionQuestionBean>) :
        RecyclerView.Adapter<RecyclerView.ViewHolder>() {
        private var mListener: OnQuestionChangeListener? = null

        class CustomQuestionViewHolder(override val containerView: View) :
            RecyclerView.ViewHolder(containerView), LayoutContainer {
            fun bindData(bean: PositionQuestionBean) {
                this.ivCustomQuestionCheck.isSelected = bean.choosed
                this.tvCustomQuestionName.text = bean.content
                this.clCustomQuestionRoot.isSelected = bean.choosed
            }

        }

        class SystemQuestionViewHolder(override val containerView: View) :
            RecyclerView.ViewHolder(containerView), LayoutContainer {
            fun bindData(bean: PositionQuestionBean) {
                this.ivSystemQuestionCheck.isSelected = bean.choosed
                this.tvSystemQuestionName.text = bean.content
                this.clSystemQuestionRoot.isSelected = bean.choosed
                this.flSystemQuestionLabel.labels = bean.tags?.split(",")
            }
        }

        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
            if (viewType == R.layout.card_position_question_select_custom) {
                val customViewHolder = CustomQuestionViewHolder(
                    LayoutInflater.from(ctx).inflate(viewType, parent, false)
                )
                customViewHolder.ivCustomEdit.setOnClickListener {
                    mListener?.onEdit(customViewHolder.adapterPosition)
                }
                customViewHolder.ivCustomDelete.setOnClickListener {
                    mListener?.onDelete(customViewHolder.adapterPosition)
                }
                return customViewHolder
            }
            return SystemQuestionViewHolder(
                LayoutInflater.from(ctx).inflate(viewType, parent, false)
            )
        }

        override fun getItemCount(): Int = data.size

        override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
            val bean = data[position]
            if (holder is CustomQuestionViewHolder) {
                holder.bindData(bean)

            } else {
                (holder as SystemQuestionViewHolder).bindData(bean)
            }
            holder.itemView.setOnClickListener {
                if (bean.choosed) {
                    bean.choosed = false
                    notifyItemChanged(position)
                    mListener?.onSelectChanged(getSelectQuestion().isEmpty().not())
                } else {
                    //判断选中的是否已经超过3个，超过三个则不能再选中
                    if (getSelectQuestion().size >= MAX_SELECT_QUESTION_NUM) {
                        ToastUtil.show(
                            ctx,
                            ResourceUtils.getString(
                                R.string.max_relate_question,
                                MAX_SELECT_QUESTION_NUM
                            )
                        )
                        return@setOnClickListener
                    }
                    bean.choosed = true
                    notifyItemChanged(position)
                    mListener?.onSelectChanged(true)
                }

            }
        }

        /**
         * 获取选中的wenti
         */
        private fun getSelectQuestion(): List<PositionQuestionBean> {
            return data.filter { it.choosed }
        }

        override fun getItemViewType(position: Int): Int {
            val bean = data[position]
            if (bean.type == TYPE_CUSTOM) {
                return R.layout.card_position_question_select_custom
            }
            return R.layout.card_position_question_select_system
        }

        fun setOnQuestionChangeListener(listener: OnQuestionChangeListener) {
            this.mListener = listener
        }

        interface OnQuestionChangeListener {
            fun onEdit(position: Int)
            fun onDelete(position: Int)
            fun onSelectChanged(isSelect: Boolean)
        }
    }

}