package com.common.ui.question_answer

import android.content.Context
import android.graphics.Rect
import android.text.Html
import android.text.InputFilter
import android.text.SpannableStringBuilder
import android.util.AttributeSet
import android.view.View
import android.view.View.OnClickListener
import android.widget.Button
import android.widget.EditText
import android.widget.FrameLayout
import android.widget.LinearLayout
import android.widget.RelativeLayout
import android.widget.TextView
import com.alpha.lagouapk.R
import com.common.bean.PositionQuestionBean
import com.common.utils.ViewUtils
import com.common.widget.NewExpandableTextView
import com.lagou.baselib.BaseActivity
import com.lagou.commonlibs.utils.InputMethodUtil
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.utils.VerticalCenterImageSpan
import com.lagou.commonlibs.view.MaxInputWatcher
import com.lagou.commonlibs.view.filter.EmojiFilter

/**
 * 问题与回答展示卡片
 */
class QuestionAnswerCard @JvmOverloads constructor(context: Context, attrs:AttributeSet? = null, defStyle:Int = 0):FrameLayout(context,attrs,defStyle),OnClickListener {
    private var tvQuestionName:TextView
    private var tvToAnswer:TextView
    private var tvShowAnswer:NewExpandableTextView
    private var mState:State = State.C_NOT_ANSWERED
    private var mRootView:LinearLayout
    private var mEditAnswerContainer:RelativeLayout
     var mEditAnswer: EditText
    private var mTvShowCount: TextView
    private lateinit var mQuestionBean:PositionQuestionBean
    private var mEditCallback:((QuestionAnswerCard)->Unit)? = null
    private var mEditTextIsEmptyCallback:((Boolean,Boolean)->Unit)? = null
    enum class State{
        C_NOT_ANSWERED,//C端查看-未回答过
        ANSWERED_NOT_COMMIT,//回答但是未提交
        COMMIT, //已提交
        ANSWER_EDIT_ING,//编辑中
        B_NOT_ANSWERED,//B端查看-未回答过
        B_ANSWERED//B端查看-已回答
    }
    init {
        inflate(context, R.layout.card_question_answer,this)
        tvQuestionName = findViewById(R.id.tv_question_name)
        tvToAnswer = findViewById(R.id.tv_to_answer_question)
        tvShowAnswer = findViewById(R.id.tv_show_answer)
        mRootView = findViewById(R.id.root_question_card)
        mEditAnswerContainer = findViewById(R.id.rl_answer_edit)
        mEditAnswer = findViewById(R.id.edit_answer)
        mTvShowCount = findViewById(R.id.tv_input_count)
        ViewUtils.setViewsClick(this,tvToAnswer,tvShowAnswer)
        mEditAnswer.filters = arrayOf<InputFilter>(EmojiFilter())
        mEditAnswer.addTextChangedListener(MaxInputWatcher(BaseQuestionAndAnswerDialog.MAX_INPUT_COUNT).apply{
            setIsCalTextLength(true)
            setInputType(MaxInputWatcher.INPUT_TYPE_DOUBLE)
            setMaxInputListener(object : MaxInputWatcher.OnMaxInputListener {
                override fun equalMaxLength(isEqual: Boolean) {}

                override fun onTextChanged(s: String?) {
                    mQuestionBean.editAnswer = s
                    mEditTextIsEmptyCallback?.invoke(s.isNullOrEmpty(),mQuestionBean.answer.isNullOrEmpty().not())
                }

                override fun showLengthTip(length: Int, text: String?) {
                    val text: String = ResourceUtils.getString(R.string.edit_answer_count_tip,length,BaseQuestionAndAnswerDialog.MAX_INPUT_COUNT)
                    mTvShowCount.text = Html.fromHtml(text)
                }

            })
        })

    }


    override fun onClick(v: View?) {
        when(v?.id){
            R.id.tv_show_answer -> {
                if(mState != State.ANSWERED_NOT_COMMIT){
                    return
                }
                setViewByState(State.ANSWER_EDIT_ING)
                mEditCallback?.invoke(this)
            }
            R.id.tv_to_answer_question ->{
                setViewByState(State.ANSWER_EDIT_ING)
                mEditCallback?.invoke(this)
            }
        }
    }

    fun setEditCallback(callback:((QuestionAnswerCard)->Unit)?){
        this.mEditCallback = callback
    }
    fun setEditTextIsEmpty(callback: ((Boolean,Boolean) -> Unit)?){
        this.mEditTextIsEmptyCallback = callback
        this.mEditTextIsEmptyCallback?.invoke(mQuestionBean.editAnswer.isNullOrEmpty(),mQuestionBean.answer.isNullOrEmpty().not())
    }

    fun setData(sequence:Int,questionBean: PositionQuestionBean,isCheckAnswer:Boolean,isOnlyOneQuestion:Boolean){
        this.mQuestionBean = questionBean
        val dp4 = ResourceUtils.getDimenSize(R.dimen.dp_4)
        val builder = SpannableStringBuilder()
        VerticalCenterImageSpan.inject(context, builder, R.drawable.green_dot_4,dp4,dp4, Rect(0,0,dp4,0))
        builder.append("问题").append(if (isOnlyOneQuestion) "" else sequence.toString()).append("：").append(questionBean.content)

        tvQuestionName.text = builder
        mEditAnswer.setText(questionBean.editAnswer)
        mEditAnswer.setSelection(questionBean.editAnswer?.length ?: 0)
        var state = if (questionBean.answer.isNullOrEmpty()){
            if (isCheckAnswer){
                State.B_NOT_ANSWERED
            } else{
                if (isOnlyOneQuestion) State.ANSWER_EDIT_ING else (if (questionBean.editAnswer.isNullOrEmpty()) State.C_NOT_ANSWERED else State.ANSWERED_NOT_COMMIT)
            }
        }else{
            if (isCheckAnswer) State.B_ANSWERED else State.COMMIT
        }
        setViewByState(state)
    }

    private fun setViewByState(state: State) {
        this.mState = state
        when(mState){
            State.C_NOT_ANSWERED -> {
               ViewUtils.setViewVisibility(tvToAnswer)
               ViewUtils.setViewGone(tvShowAnswer,mEditAnswerContainer)
            }
            State.ANSWERED_NOT_COMMIT -> {
                ViewUtils.setViewVisibility(tvShowAnswer)
                ViewUtils.setViewGone(tvToAnswer,mEditAnswerContainer)
                tvShowAnswer.text = mQuestionBean.editAnswer
                tvShowAnswer.setTextColor(ResourceUtils.getColor(R.color.color_666666))
            }
            State.COMMIT -> {
                ViewUtils.setViewVisibility(tvShowAnswer)
                ViewUtils.setViewGone(tvToAnswer,mEditAnswerContainer)
                tvShowAnswer.text = mQuestionBean.answer
                tvShowAnswer.setTextColor(ResourceUtils.getColor(R.color.color_BBBBBB))
            }
            State.ANSWER_EDIT_ING ->{
                ViewUtils.setViewVisibility(mEditAnswerContainer)
                ViewUtils.setViewGone(tvToAnswer,tvShowAnswer)
            }
            State.B_NOT_ANSWERED ->{
                tvShowAnswer.text = "暂未回答"
                ViewUtils.setViewVisibility(tvShowAnswer)
                ViewUtils.setViewGone(tvToAnswer,mEditAnswerContainer)
                tvShowAnswer.setTextColor(ResourceUtils.getColor(R.color.color_666666))
            }
            State.B_ANSWERED ->{
                ViewUtils.setViewVisibility(tvShowAnswer)
                ViewUtils.setViewGone(tvToAnswer,mEditAnswerContainer)
                tvShowAnswer.text = mQuestionBean.answer
                tvShowAnswer.setTextColor(ResourceUtils.getColor(R.color.color_666666))
            }
        }
    }

    fun onUpdateEditAnswer() {
        if (mQuestionBean.editAnswer.isNullOrEmpty()){
            setViewByState(State.C_NOT_ANSWERED)
        }else{
            setViewByState(State.ANSWERED_NOT_COMMIT)
        }
    }

}