package com.common.ui.question_answer

import android.content.Context
import android.os.Bundle
import android.view.View
import android.view.WindowManager
import com.ApiService
import com.alpha.lagouapk.R
import com.alpha.lagouapk.UserRoleManager
import com.common.bean.QuestionsAndAnswersBean
import com.common.utils.ViewUtils
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.commonlibs.ui.dialog.BaseBottomSheetDialogFragment
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.lagouee.utils.IntentUtils
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.android.synthetic.main.dialog_question_answer_intro.*

/**
 * 职位问答卷介绍弹窗
 */
class QuestionAnswerIntroDialog : BaseBottomSheetDialogFragment(){
    private var mAnsweredCount:Int = 0
    private var mTotalQuestionCount:Int = 0
    private var mAskPaperId:Int = 0
    private var mChatMsgId:Long = 0
    private var mPositionId:Long = 0
    private var mSessionId:Long = 0
    companion object{
        const val KEY_ANSWERED_QUESTION_COUNT = "key_answered_question_count"
        const val KEY_TOTAL_QUESTION_COUNT = "key_total_question_count"
        const val KEY_ASK_PAPER_ID = "key_ask_paper_id"
        const val KEY_CHAT_MSG_ID = "key_chat_msg_id"
        const val KEY_POSITION_ID = "key_position_id"
        const val KEY_SESSION_ID = "key_session_id"
        @JvmStatic
        @JvmOverloads
        fun getInstance(answeredCount:Int = 0,totalQuestionCount:Int = 0,chatMsgId:Long = 0,askPaperId:Int = 0,positionId:Long = 0,sessionId:Long = 0):QuestionAnswerIntroDialog{
            val dialog = QuestionAnswerIntroDialog()
            val argument = Bundle()
            argument.putInt(KEY_ANSWERED_QUESTION_COUNT,answeredCount)
            argument.putInt(KEY_TOTAL_QUESTION_COUNT,totalQuestionCount)
            argument.putLong(KEY_CHAT_MSG_ID,chatMsgId)
            argument.putLong(KEY_POSITION_ID,positionId)
            argument.putInt(KEY_ASK_PAPER_ID,askPaperId)
            argument.putLong(KEY_SESSION_ID,sessionId)
            dialog.arguments = argument
            return dialog
        }
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        mAnsweredCount = arguments?.getInt(KEY_ANSWERED_QUESTION_COUNT) ?: 0
        mTotalQuestionCount = arguments?.getInt(KEY_TOTAL_QUESTION_COUNT) ?: 0
        mPositionId = arguments?.getLong(KEY_POSITION_ID) ?: 0
        mChatMsgId = arguments?.getLong(KEY_CHAT_MSG_ID) ?: 0
        mAskPaperId = arguments?.getInt(KEY_ASK_PAPER_ID) ?: 0
        mSessionId = arguments?.getLong(KEY_SESSION_ID) ?: 0
    }
    override fun getLayoutId(): Int = R.layout.dialog_question_answer_intro

    override fun isDraggable(): Boolean  = false

    override fun initListener() {
        super.initListener()
        iv_close_dlg.setOnClickListener { dismiss() }
        ViewUtils.setViewsClick(this,iv_close_dlg,btn_close_intro,btn_intro_action)
    }

    override fun onClick(v: View?) {
        super.onClick(v)
        when(v?.id){
            R.id.iv_close_dlg,R.id.btn_close_intro -> dismiss()
            R.id.btn_intro_action -> doAction()
        }
    }

    private fun doAction() {
        val isCRole = UserRoleManager.getInstance().isCUser
        if (isCRole){
            if (activity is BaseActivity){
                ApiService.api.fetchQuestionsAndAnswers(mAskPaperId)
                    .enqueue(object : BaseCallback<QuestionsAndAnswersBean?>() {
                        override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<QuestionsAndAnswersBean?>?) {
                            if ((activity as BaseActivity).isActivityDestroyed){
                                return
                            }
                            ToAnswerDialog.getInstance(response?.content,mAskPaperId,mChatMsgId).show((activity as BaseActivity).supportFragmentManager)
                        }
                    })
            }
        }else{
            IntentUtils.startQuestionSetActivity(activity,mPositionId,mSessionId)
        }
        dismiss()
    }

    override fun getWidth(): Int = WindowManager.LayoutParams.MATCH_PARENT

    override fun fillUI() {
        super.fillUI()
        val isCRole = UserRoleManager.getInstance().isCUser
        tv_intro_desc.text =  ResourceUtils.getString(if (isCRole) R.string.question_answer_intro_desc_c else R.string.question_answer_intro_desc_b)
        iv_intro_bg.setImageResource(if (isCRole) R.drawable.bg_question_answer_intro_c else R.drawable.bg_question_answer_intro_b)
        if (isCRole){
            var answerInfo:String? = null
            if (mTotalQuestionCount > 1){
                if (mAnsweredCount > 0){
                    answerInfo = ResourceUtils.getString(R.string.question_answered_num,mAnsweredCount,mTotalQuestionCount)
                }else{
                    answerInfo = ResourceUtils.getString(R.string.question_answer_intro_action_c,mAnsweredCount,mTotalQuestionCount)
                }
            }else{
               answerInfo = if (mAnsweredCount == mTotalQuestionCount) ResourceUtils.getString(R.string.answered) else ResourceUtils.getString(R.string.now_answer)
            }
            btn_intro_action.text = answerInfo
        }else{
            btn_intro_action.text = ResourceUtils.getString(R.string.go_set)
        }

    }

}