package com.common.ui.question_answer

import android.annotation.SuppressLint
import android.content.Context
import android.content.DialogInterface
import android.os.Bundle
import android.view.KeyEvent
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.widget.decoration.DividerItemDecoration
import com.common.bean.Position
import com.common.bean.PositionsCategory
import com.lagou.commonlibs.ui.dialog.BaseBottomSheetDialogFragment
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.utils.StringUtil
import com.lagou.lagouee.utils.DataServer
import kotlinx.android.extensions.LayoutContainer
import kotlinx.android.synthetic.main.dialog_select_relate_position.btnSaveQuestion
import kotlinx.android.synthetic.main.dialog_select_relate_position.iv_close_dlg
import kotlinx.android.synthetic.main.dialog_select_relate_position.rvPositionCategory
import kotlinx.android.synthetic.main.dialog_select_relate_position.rvPositions
import kotlinx.android.synthetic.main.item_select_position_cate.categoryIndicator
import kotlinx.android.synthetic.main.item_select_position_cate.tvPositionCategory
import kotlinx.android.synthetic.main.item_select_position_position.ivPositionCheck
import kotlinx.android.synthetic.main.item_select_position_position.tvPositionName
import kotlinx.android.synthetic.main.item_select_position_position.tvPositionSalary
import kotlinx.android.synthetic.main.item_select_position_position.tvRelateMark
import kotlin.text.StringBuilder

/**
 * 选择职位
 */
class SelectPositionDialog: BaseBottomSheetDialogFragment(){
    private lateinit var mPositionAdapter:PositionAdapter
    private var mSelectPositionIds:String? = null
    private var mPositionList:ArrayList<PositionsCategory>? = null
    private var mSelectPositionNum:Int = 0//选中职位的数量
    private var mSaveCallback:((String,String) -> Unit)? = null
    companion object{
        const val KEY_SELECT_POSITION_IDS = "key_select_position_ids"
        const val KEY_POSITIONS_LIST = "key_positions_list"
        @JvmStatic
        fun getInstance(positionList:ArrayList<PositionsCategory>,positionIds:String? = null,saveCallback:((String,String) -> Unit)?):SelectPositionDialog{
            val bundle = Bundle()
            bundle.putString(KEY_SELECT_POSITION_IDS,positionIds)
            bundle.putParcelableArrayList(KEY_POSITIONS_LIST,positionList)
            val dialog = SelectPositionDialog()
            dialog.setSaveCallback(saveCallback)
            dialog.arguments = bundle
            return dialog
        }
    }

    private fun setSaveCallback(saveCallback: ((String, String) -> Unit)?) {
        this.mSaveCallback = saveCallback
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        mSelectPositionIds = arguments?.getString(KEY_SELECT_POSITION_IDS)
        mPositionList = arguments?.getParcelableArrayList(KEY_POSITIONS_LIST)
    }
    override fun getLayoutId(): Int = R.layout.dialog_select_relate_position
    override fun isDraggable() = false

    override fun initConfigAtCreateView() {
        super.initConfigAtCreateView()
        dialog?.setCanceledOnTouchOutside(false)
    }

    override fun onStart() {
        super.onStart()
        if (dialog == null) {
            return
        }
        dialog?.setOnKeyListener { _, keyCode, _ ->
            keyCode == KeyEvent.KEYCODE_BACK
        }
    }

    @SuppressLint("NotifyDataSetChanged")
    override fun initListener() {
        super.initListener()
        iv_close_dlg.setOnClickListener { dismiss() }
        rvPositionCategory.setOnItemClickListener { _, adapter, position ->
            mPositionList?.forEachIndexed { index, bean ->
                bean.isSelect = position == index
            }
            adapter.notifyDataSetChanged()
            mPositionAdapter.setPositions(mPositionList?.get(position)?.positions)
        }
        rvPositions.setOnItemClickListener { _, adapter, position ->
            val bean = (adapter as PositionAdapter).data!![position]
            bean.isSelect = !bean.isSelect
            adapter.notifyItemChanged(position)
            if (bean.isSelect){
                mSelectPositionNum++
            }else{
                mSelectPositionNum--
            }
            setSaveButton()
        }
        btnSaveQuestion.setOnClickListener {
            val positionNames = StringBuilder()
            val positionIds = StringBuilder()
            val positionNameSplit = "、"
            val positionIdSplit = ","
            mPositionList?.forEach {
                it.positions?.forEach {positionBean ->
                    if (positionBean.isSelect){
                        positionNames.append(positionBean.positionName).append(positionNameSplit)
                        positionIds.append(positionBean.lgPositionId).append(positionIdSplit)
                    }
                }
            }
            val lastPositionNameSplitIndex = positionNames.lastIndexOf(positionNameSplit)
            val lastPositionIdSplitIndex = positionIds.lastIndexOf(positionIdSplit)
            if (lastPositionNameSplitIndex >= 0){
                positionNames.deleteCharAt(lastPositionNameSplitIndex)
            }
            if (lastPositionIdSplitIndex >= 0){
                positionIds.deleteCharAt(lastPositionIdSplitIndex)
            }
            mSaveCallback?.invoke(positionNames.toString(),positionIds.toString())
            dismiss()
        }
    }

    override fun fillUI() {
        super.fillUI()
        if(mPositionList.isNullOrEmpty()){
            return
        }
        val defaultSelectCate = getInitSelectCateIndex()
        rvPositionCategory.apply {
            setInitLoading(false)
            layoutManager = LinearLayoutManager(context)
            addItemDecoration(DividerItemDecoration(context, DividerItemDecoration.HORIZONTAL_LIST,ResourceUtils.getDimenSize(R.dimen.dp_30), ResourceUtils.getColor(R.color.transparent)))
            adapter = CategoryAdapter(context,mPositionList!!)
        }
        rvPositions.apply {
            setInitLoading(false)
            mPositionAdapter = PositionAdapter(context,mPositionList!![defaultSelectCate].positions)
            layoutManager = LinearLayoutManager(context)
            addItemDecoration(DividerItemDecoration(context, DividerItemDecoration.HORIZONTAL_LIST,ResourceUtils.getDimenSize(R.dimen.dp_30), ResourceUtils.getColor(R.color.transparent)))
            adapter = mPositionAdapter
        }
        setSaveButton()
    }

    private fun getInitSelectCateIndex():Int{
        //初始化选中
        var defaultSelectCate = -1
        mSelectPositionNum = 0
        mPositionList?.forEachIndexed{index,categoryBean ->
            categoryBean.isSelect = false
            categoryBean.positions?.forEach {positionBean ->
                if (mSelectPositionIds?.split(",")?.contains(positionBean.lgPositionId.toString()) == true){
                    if (defaultSelectCate == -1){
                        defaultSelectCate = index
                    }
                    positionBean.isSelect = true
                    mSelectPositionNum++
                }else{
                    positionBean.isSelect = false
                }
            }
        }
        mPositionList?.get(0.coerceAtLeast(defaultSelectCate))?.isSelect = true
        return 0.coerceAtLeast(defaultSelectCate)
    }

    private fun setSaveButton(){
        btnSaveQuestion.isEnabled = mSelectPositionNum > 0
        btnSaveQuestion.text = if(mSelectPositionNum > 0) ResourceUtils.getString(R.string.ok_num,mSelectPositionNum) else ResourceUtils.getString(R.string.ok)
    }


    class CategoryAdapter(val ctx: Context, val data:List<PositionsCategory>): RecyclerView.Adapter<CategoryAdapter.CateViewHolder>() {
        class CateViewHolder(override val containerView: View):RecyclerView.ViewHolder(containerView),LayoutContainer

        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): CateViewHolder {
            return CateViewHolder(LayoutInflater.from(ctx).inflate(R.layout.item_select_position_cate,parent,false))
        }

        override fun getItemCount(): Int = data.size

        override fun onBindViewHolder(holder: CateViewHolder, position: Int) {
            val bean:PositionsCategory = data[position]
            holder.tvPositionCategory.text = bean.category
            holder.tvPositionCategory.isSelected = bean.isSelect
            holder.categoryIndicator.visibility = if (bean.isSelect) View.VISIBLE else View.GONE
        }
    }

    class PositionAdapter(val ctx:Context,var data:List<Position>?): RecyclerView.Adapter<PositionAdapter.PositionViewHolder>() {
        class PositionViewHolder(override val containerView: View):RecyclerView.ViewHolder(containerView),LayoutContainer

        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): PositionViewHolder {
            return PositionViewHolder(LayoutInflater.from(ctx).inflate(R.layout.item_select_position_position,parent,false))
        }

        override fun getItemCount(): Int = data?.size ?: 0

        override fun onBindViewHolder(holder: PositionViewHolder, position: Int) {
            val bean = data!![position]
            holder.tvPositionName.text = bean.positionName
            holder.tvPositionSalary.text =  DataServer.getSalaryText(bean.salaryMin.toString().plus("-").plus(bean.salaryMax).plus("K"), bean.salaryMonth)
            holder.tvRelateMark.visibility = if (bean.hasQuestion) View.VISIBLE else View.GONE
            holder.ivPositionCheck.isSelected = bean.isSelect
            holder.tvPositionName.isSelected = bean.isSelect
            holder.tvPositionSalary.isSelected = bean.isSelect
        }

        fun setPositions(positions: List<Position>?) {
            this.data = positions
            notifyDataSetChanged()
        }
    }
}