package com.common.ui.question_answer

import android.content.Context
import android.os.Bundle
import android.view.View
import com.ApiService
import com.alpha.lagouapk.R
import com.common.bean.CommitAnswerReq
import com.common.bean.QuestionsAndAnswersBean
import com.common.chat.activity.ChatActivityBase
import com.common.chat.activity.ChatActivityBase.OnSoftInputStateListener
import com.lagou.annotation.aspect.SingleClick
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.commonlibs.utils.DeviceUtils
import com.lagou.commonlibs.utils.InputMethodUtil
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.android.synthetic.main.dialog_question_answer.btn_submit_answer
import kotlinx.android.synthetic.main.dialog_question_answer.scroll_question_container


class ToAnswerDialog:BaseQuestionAndAnswerDialog(), OnSoftInputStateListener {
    private var mAskPaperId:Int = 0
    private var mMessageId:Long = 0
    private var mCurEditQuestionCard: QuestionAnswerCard? = null
    companion object{
        @JvmStatic
        fun getInstance(bean: QuestionsAndAnswersBean?,askPaperId:Int,messageId:Long):ToAnswerDialog{
            val bundle = Bundle()
            bundle.putParcelable(KEY_QUESTION_ANSWER_INFO,bean)
            bundle.putInt(KEY_ASK_PAPER_ID,askPaperId)
            bundle.putLong(KEY_MESSAGE_ID,messageId)
            val dialog = ToAnswerDialog()
            dialog.arguments = bundle
            return dialog
        }
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        mAskPaperId = arguments?.getInt(KEY_ASK_PAPER_ID) ?: 0
        mMessageId = arguments?.getLong(KEY_MESSAGE_ID) ?: 0
    }

    override fun initView(view: View?) {
        super.initView(view)
        (activity as? ChatActivityBase)?.setOnSoftInputStateListener(this)
    }

    override fun getTheme(): Int = R.style.BottomSheetDialog_SoftInput_Visible

    override fun onClick(v: View?) {
        super.onClick(v)
        when(v?.id){
            R.id.btn_submit_answer -> {
                commitAnswer()
            }
        }
    }

    /**
     * 提交答案
     */
    @SingleClick
    private fun commitAnswer() {
        val answerList = mutableListOf<CommitAnswerReq.CommitQuestionBean>()
        mQuestionAndAnswerInfo?.questions?.forEach {
            answerList.add(CommitAnswerReq.CommitQuestionBean(it.questionId,it.editAnswer))
        }
        val requestParams = CommitAnswerReq(mAskPaperId,mMessageId,answerList)
        ApiService.api.commitAnswer(requestParams).enqueue(object: BaseCallback<Any?>() {
            override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<Any?>?) {
                dismiss()
            }
        })
    }

    override fun getMaxHeight(): Int = (DeviceUtils.getRealScreenHeight(activity) * 0.8f).toInt() - ResourceUtils.getDimenSize(R.dimen.dp_125)

    override fun initQuestionCardListener(card: QuestionAnswerCard) {
        super.initQuestionCardListener(card)
        if (mQuestionCount > 1){
            card.setEditCallback { questionAnswerCard ->
                this.mCurEditQuestionCard = questionAnswerCard
                InputMethodUtil.showKeyboard(activity,questionAnswerCard.mEditAnswer)
                questionAnswerCard.mEditAnswer.requestFocus()
            }
        }else{
            this.mCurEditQuestionCard = card
            card.setEditTextIsEmpty {isEmpty,isAnswered ->
                btn_submit_answer.isEnabled = isEmpty.not() && isAnswered.not()
            }
        }
    }

    override fun fillUI() {
        super.fillUI()
        if (mQuestionCount == 1 && (mQuestionAndAnswerInfo?.questions?.any { it.answer.isNullOrEmpty() } == true)){
            mCurEditQuestionCard?.mEditAnswer?.requestFocus()
            InputMethodUtil.showKeyboard(activity,mCurEditQuestionCard?.mEditAnswer)
        }
        setCommitEnable()
    }


    private fun setCommitEnable(){
        btn_submit_answer.isEnabled = mQuestionAndAnswerInfo?.questions?.any { it.editAnswer.isNullOrEmpty().not() && it.answer.isNullOrEmpty() } == true
    }

    override fun onSoftInputState(keyboardHeight: Int) {
        if (keyboardHeight > 0){
            if (btn_submit_answer == null){
                return
            }
            if (mQuestionCount > 1){
                btn_submit_answer.visibility = View.GONE
            }
            scroll_question_container.setScrollable(false)
            scroll_question_container?.setScrollViewMaxHeight(mCurEditQuestionCard?.height ?: ResourceUtils.getDimenSize(R.dimen.dp_208))
            scroll_question_container.post {
                scroll_question_container.smoothScrollTo(0,mCurEditQuestionCard?.top ?: 0)

            }
        }else{
            if (btn_submit_answer == null){
                return
            }
            if (mQuestionCount > 1){
                btn_submit_answer.visibility = View.VISIBLE
                mCurEditQuestionCard?.onUpdateEditAnswer()
                setCommitEnable()
            }
            scroll_question_container?.setScrollViewMaxHeight(getMaxHeight())
            scroll_question_container?.setScrollable(true)
        }

    }

    override fun onDestroyView() {
        super.onDestroyView()
        (activity as? ChatActivityBase)?.setOnSoftInputStateListener(null)
    }
}