package com.common.ui.video

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.graphics.Rect
import android.view.View
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.tools.IntentUtils
import com.common.bean.VideoRes
import com.common.ui.adapter.GalleryVideoAdapter
import com.common.utils.CommonVideoUtil
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.privacy.KEY_STORE_SCENE_POSITION_VIDEO
import com.lagou.baselib.privacy.checkPrivacy
import com.lagou.commonlibs.engine.ThreadUtil
import com.lagou.commonlibs.privacy.AdapterPrivacyRightListener
import com.lagou.commonlibs.privacy.PrivacyPermissionKey
import com.lagou.commonlibs.privacy.PrivacyRightManager
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.utils.ToastUtil
import com.lagou.commonlibs.view.LTitleBarLayout
import kotlinx.android.synthetic.main.activity_video_list.*
import java.util.*

/**
 * @date： Crate by wangjunwei on 2022/7/11
 * @description：视频列表页面：视频检索、视频选取
 **/
class VideoListActivity : BaseActivity() {

    private var lTitleBar: LTitleBarLayout? = null
    private var recyclerView: RecyclerView? = null
    private var videoAdapter: GalleryVideoAdapter? = null

    companion object {
        /***
         * 选择视频requestCode 和 resultCode
         */
        const val SELECT_REQUEST_CODE = 0x01
        const val SELECT_RESULT_CODE = 0x02
        const val KEY_SELECT_VIDEO = "select_video"

        /**
         * 定义列数
         */
        const val KEY_SPAN_COUNT = 3

        /***
         * 打开本地视频列表页面
         */
        fun startVideoListActivity(activity: Activity) {
            checkPrivacy(activity, arrayListOf(KEY_STORE_SCENE_POSITION_VIDEO),object : AdapterPrivacyRightListener() {
                override fun onAgreeRight() {
                    var intent = Intent(activity,  VideoListActivity::class.java)
                    IntentUtils.startActivityForResultNoException(activity, intent, SELECT_REQUEST_CODE)
                }
            },PrivacyPermissionKey.EXTERNAL_STORAGE)
        }
    }

    override fun getContentLayoutId() = R.layout.activity_video_list

    override fun init() {
        recyclerView = findViewById(R.id.recycle_view)
        lTitleBar = findViewById(R.id.ltitle_bar)
        initRecycleView()
        requestData()
        lTitleBar?.setLeftImgListener {
            finish()
        }
    }

    private fun initRecycleView() {
        videoAdapter = GalleryVideoAdapter(this@VideoListActivity)
        videoAdapter?.setOnItemListener(object : GalleryVideoAdapter.OnItemListener {
            override fun onItemCheck(position: Int, videoRes: VideoRes) {
                if (videoRes.isExceedSize()) {
                    ToastUtil.show(this@VideoListActivity, ResourceUtils.getString(R.string.video_exceed_size_tip))
                    return
                }
                if (videoRes.isUnSupportFormat()){
                    ToastUtil.show(this@VideoListActivity, ResourceUtils.getString(R.string.video_unsupport_format))
                    return
                }
                VideoPlayActivity.startPlayActivity(this@VideoListActivity, videoRes, false)
            }
        })
        recyclerView?.apply {
            layoutManager = GridLayoutManager(this@VideoListActivity, KEY_SPAN_COUNT)
            addItemDecoration(VideoDecoration(this@VideoListActivity))
            adapter = videoAdapter
        }
    }

    private fun requestData() {
        ThreadUtil.io{
            val videos = CommonVideoUtil.getGalleryVideos(this)
            runOnUiThread {
                videoAdapter?.setData(videos)
                if (videos == null || videos?.size == 0){
                    lin_empty_tip.visibility = View.VISIBLE
                    recyclerView?.visibility = View.GONE
                }else {
                    lin_empty_tip.visibility = View.GONE
                    recyclerView?.visibility = View.VISIBLE
                }
            }
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == SELECT_REQUEST_CODE
            && resultCode == SELECT_RESULT_CODE
        ) {
            val videoInfo = data?.getParcelableExtra(KEY_SELECT_VIDEO) as? VideoRes
            var intent = Intent()
            intent.putExtra(KEY_SELECT_VIDEO, videoInfo)
            setResult(SELECT_RESULT_CODE, intent)
            finish()
        }
    }

    class VideoDecoration(context: Context) : RecyclerView.ItemDecoration() {

        private val DP2: Int = context.resources.getDimension(R.dimen.dp_2).toInt()
        private val DP1: Int = context.resources.getDimension(R.dimen.dp_1).toInt()

        override fun getItemOffsets(
            outRect: Rect,
            view: View,
            parent: RecyclerView,
            state: RecyclerView.State
        ) {
            super.getItemOffsets(outRect, view, parent, state)
            val position = parent.getChildAdapterPosition(view)
            //1 第一行顶部不设置间距
            if (position <= 2) {
                // 第一列左边设置间距为1dp
                if (position % KEY_SPAN_COUNT == 0) {
                    outRect.set(DP1, 0, 0, 0)
                    return
                }
                outRect.set(DP2, 0, 0, 0)
                return
            }
            //2 第一列左边间距dp2（已经排除一行）
            if (position % KEY_SPAN_COUNT == 0) {
                outRect.set(DP1, DP2, 0, 0)
                return
            }

            outRect.set(DP2, DP2, 0, 0)
        }
    }

    override fun getStatusBarColor(): Int {
        return R.color.color_ff000000
    }
}