package com.common.utils

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.net.Uri
import android.os.Build
import android.os.Parcelable
import android.provider.MediaStore
import android.text.TextUtils
import com.alpha.lagouapk.LagouApplication
import com.alpha.lagouapk.service.AppDataInitService
import com.alpha.lagouapk.service.AppDataInitService26
import com.alpha.lagouapk.tools.IntentUtils
import com.alpha.lagouapk.tools.SPUtils
import com.common.ui.MapPreviewActivity
import com.common.ui.SafeTipsWebActivity
import com.common.ui.privacy.RecommendProtocolActivity
import com.common.ui.privacy.TouristWebActivity
import com.common.upgrade.AppUpgradeActivity
import com.common.webview.LagouWebviewActivity
import com.common.webview.WalletWebViewActivity
import com.common.webview.WebViewDelegate
import com.lagou.baselib.privacy.checkPrivacy
import com.lagou.baselib.util.BaseIntentUtils
import com.lagou.commonlibs.privacy.AdapterPrivacyRightListener
import com.lagou.commonlibs.privacy.PrivacyPermissionKey
import com.lagou.commonlibs.privacy.PrivacyRightManager
import com.lagou.commonlibs.web.WebviewUtils
import com.lagou.commonlibs.web.router.RouteUtils

/**
 * @date 创建时间：2019-12-02
 * @author 作者：codeman
 * @Description  描述：B C共用的的IntentUtils
 * @version
 */

class CommonIntentUtils : BaseIntentUtils() {

    companion object {

        const val KEY_NEXT_INTENT = "key_target_intent"

        /**
         * 加载指定url，并显示指定title
         *
         * @param context
         * @param url     url链接
         * @param title   指定标题
         */
        fun startLagouWebViewActivity(context: Context?, url: String?, title: String?) {
            val intent = Intent(context, LagouWebviewActivity::class.java)
            intent.putExtra(WebViewDelegate.INTENT_KEY_URL, url)
            intent.putExtra(WebViewDelegate.INTENT_TITLE, title)
            IntentUtils.startActivityNoException(context, intent)
        }

        /**
         * 加载指定url
         *
         * @param context
         * @param url     url连接
         */
        fun startLagouWebViewActivity(context: Context?, url: String?) {
            val intent = Intent(context, LagouWebviewActivity::class.java)
            intent.putExtra(WebViewDelegate.INTENT_KEY_URL, url)
            IntentUtils.startActivityNoException(context, intent)
        }

        fun startWalletWebViewActivity(context: Context?, hasTitle: Boolean) {
            startWalletWebViewActivity(context, "", hasTitle)
        }

        /***
         * 启动拉勾钱包页面，B C共用
         * @param context
         * @param url
         */
        fun startWalletWebViewActivity(context: Context?, url: String, hasTitle: Boolean) {
            if (context == null) {
                return
            }
            val intent = Intent(context, WalletWebViewActivity::class.java)
            intent.putExtra(WalletWebViewActivity.INTENT_KEY_URL, url)
            intent.putExtra(WebViewDelegate.INTENT_HAS_TITLE, hasTitle)
            startActivity(context, intent)
        }

        /***
         * 启动游客模式web页面
         * @param context
         */
        fun startTouristWebActivity(context: Context) {
            //清理cookie，该游客模式不能带cookieSi
            //用户同意过隐私弹窗才能清理，CookieManager.getInstance()会触发对getPackInfo的调用，小米市场判定不合规
            if (SPUtils.isPrivacyDlgAgreed() || SPUtils.isPrivacyDlgConsumed()) {
                WebviewUtils.clearAllCookie()
            }
            val intent = Intent(context, TouristWebActivity::class.java)
            intent.putExtra(WebViewDelegate.INTENT_HAS_TITLE, false)
            startActivityNoException(context, intent)
        }

        /***
         * 启动个性化推荐政策web页面
         * @param context
         */
        fun startRecommendProtocolWebActivity(context: Context) {
            WebviewUtils.clearAllCookie()
            val intent = Intent(context, RecommendProtocolActivity::class.java)
            startActivityNoException(context, intent)
        }

        fun startInitDataService() {
            //https://bugly.qq.com/v2/crash-reporting/crashes/2390cd07d3/153572?pid=1  加try/catch是为了解决一个bug 没有想到好的解决方案 暂时先这么处理吧
            try {
                val context = LagouApplication.getLagouApplication()
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                    AppDataInitService26.enqueueWork(
                        context,
                        AppDataInitService26::class.java,
                        AppDataInitService26.JOB_ID,
                        Intent(context, AppDataInitService26::class.java)
                    )
                } else {
                    IntentUtils.startServiceNoException(
                        context,
                        Intent(context, AppDataInitService::class.java)
                    )
                }
            } catch (e: java.lang.Exception) {
                e.printStackTrace()
            }
        }

        /***
         * 启动外部url安全提示
         * @param context
         */
        fun startSafeTipsWebActivity(context: Context, url: String) {
            if (TextUtils.isEmpty(url)) {
                return
            }
            val intent = Intent(context, SafeTipsWebActivity::class.java)
            intent.putExtra(SafeTipsWebActivity.INTENT_KEY_URL, url)
            startActivityNoException(context, intent)
        }

        /**
         * 跳转到系统拍照页面
         */
        fun startSystemCameraActivity(activity: Activity, requestCode: Int, fileUri: Uri?,sceneRightsKey:String?) {
            if (sceneRightsKey.isNullOrEmpty()) {
                //场景权限为空，只检测系统权限
                PrivacyRightManager.showNewPrivacyStrategy(
                    activity,
                    object : AdapterPrivacyRightListener() {
                        override fun onAgreeRight() {
                            onSystemRightsAgree(activity,requestCode,fileUri)
                        }
                    },
                    PrivacyPermissionKey.CAMERA
                )
                return
            }
            checkPrivacy(activity, arrayListOf(sceneRightsKey), object : AdapterPrivacyRightListener() {
                override fun onAgreeRight() {
                    onSystemRightsAgree(activity,requestCode,fileUri)
                }
            }, PrivacyPermissionKey.CAMERA)

        }

        private fun onSystemRightsAgree(activity: Activity,requestCode: Int, fileUri: Uri?){
            val intent = Intent(MediaStore.ACTION_IMAGE_CAPTURE)
            if (intent.resolveActivity(activity.packageManager) != null) {
                intent.putExtra(MediaStore.EXTRA_OUTPUT, fileUri)
                startActivityForResultNoException(
                    activity,
                    intent,
                    requestCode
                )
            }
        }

        /**
         * 获取初始化之后该跳网的原始activity对应的intent
         *
         * @param srcIntent
         * @return
         */
        fun getOriginalIntent(srcIntent: Intent?): Intent? {
            var nextIntent: Intent? = null
            if (srcIntent == null) {
                return nextIntent
            }
            val extra = srcIntent.getParcelableExtra<Parcelable>(KEY_NEXT_INTENT)
            if (extra is Intent) {
                nextIntent = extra
            }
            return nextIntent
        }

        /**
         * 封装初始化数据前原始的intent
         *
         * @param initDataIntent
         * @param originalIntent
         */
        fun wrapOriginalIntent(initDataIntent: Intent, originalIntent: Intent?) {
            initDataIntent.putExtra(KEY_NEXT_INTENT, originalIntent)
        }

        /**
         * 无异常启动activity
         *
         * @param context
         * @param intent
         */
        fun startActivityNoException(context: Context, intent: Intent?) {
            try {
                RouteUtils.checkIntent(context, intent)
                context.startActivity(intent)
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }

        fun startActivityForResultNoException(
            activity: Activity?,
            intent: Intent?,
            requestCode: Int
        ) {
            if (intent == null || activity == null) {
                return
            }
            try {
                activity.startActivityForResult(intent, requestCode)
            } catch (e: java.lang.Exception) {
                e.printStackTrace()
            }
        }

        fun startMapPreviewActivity(context:Context?,latitude:Double,longitude:Double,title:String?){
            if (context == null){
                return
            }
            val intent = Intent(context,MapPreviewActivity::class.java)
            intent.putExtra(IntentUtils.KEY_LATITUDE,latitude)
            intent.putExtra(IntentUtils.KEY_LONGITUDE,longitude)
            intent.putExtra(IntentUtils.KEY_ADDRESS,title)
            startActivityNoException(context,intent)
        }
        @JvmStatic
        fun startUpgradeActivity(context:Context?,upgradeDesc:String?,downloadUrl:String?){
            if (context == null){
                return
            }
            val intent = Intent(context,AppUpgradeActivity::class.java)
            intent.putExtra(AppUpgradeActivity.KEY_UPGRADE_DESC,upgradeDesc)
            intent.putExtra(AppUpgradeActivity.KEY_UPGRADE_URL,downloadUrl)
            startActivityNoException(context,intent)
        }

    }
}