package com.common.utils

import android.content.ContentUris
import android.content.Context
import android.net.Uri
import android.os.Build
import android.provider.MediaStore
import com.common.bean.VideoRes
import java.util.concurrent.TimeUnit

/**
 * @date： Crate by wangjunwei on 2022/7/11
 * @description：视频处理工具类：本地视频检索等
 **/
class CommonVideoUtil {

    companion object {

        /**
         * 支持的视频格式
         */
        private val suffixMap = mapOf("mp4" to true,
            "MP4" to true, "mov" to true, "MOV" to true)

        /**
         * 选中后返回的视频
         */
        const val KEY_SELECTED_IMAGE = "selected_video"

        /**
         * 查询本地，符合条件的视频列表
         */
        fun getGalleryVideos(context: Context?): MutableList<VideoRes> {
            val videoList = mutableListOf<VideoRes>()
            val collection = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
                MediaStore.Video.Media.getContentUri(MediaStore.VOLUME_EXTERNAL)
            } else {
                MediaStore.Video.Media.EXTERNAL_CONTENT_URI
            }
            val projection = arrayOf(
                MediaStore.Video.Media._ID,
                MediaStore.Video.Media.DISPLAY_NAME,
                MediaStore.Video.Media.DURATION,
                MediaStore.Video.Media.SIZE
            )
            //筛选出时长小于等于60s的视频
            val selection = "${MediaStore.Video.Media.DURATION} <= ?"
            val selectionArgs = arrayOf(
                TimeUnit.MILLISECONDS.convert(61, TimeUnit.SECONDS).toString()
            )
            //按时间倒序排列视频
            val sortOrder = "${MediaStore.Video.Media.DATE_ADDED} DESC"

            val query = context?.contentResolver?.query(
                collection,
                projection,
                selection,
                selectionArgs,
                sortOrder
            )

            query?.use { cursor ->
                // Cache column indices.
                val idColumn = cursor.getColumnIndexOrThrow(MediaStore.Video.Media._ID)
                val nameColumn =
                    cursor.getColumnIndexOrThrow(MediaStore.Video.Media.DISPLAY_NAME)
                val durationColumn =
                    cursor.getColumnIndexOrThrow(MediaStore.Video.Media.DURATION)
                val sizeColumn = cursor.getColumnIndexOrThrow(MediaStore.Video.Media.SIZE)

                while (cursor.moveToNext()) {
                    // Get values of columns for a given video.
                    val id = cursor.getLong(idColumn)
                    val name = cursor.getString(nameColumn)
                    val duration = cursor.getInt(durationColumn)
                    val size = cursor.getInt(sizeColumn)

                    //只支持mp4或者mov视频格式，在选择视频时拦截，和iOS保持一致
                    //if (!isSupportFormat(name)){
                    //    continue
                    //}
                    val contentUri: Uri = ContentUris.withAppendedId(
                        MediaStore.Video.Media.EXTERNAL_CONTENT_URI, id
                    )
                    // Stores column values and the contentUri in a local object
                    // that represents the media file.
                    val videoLocalRes = VideoRes(duration.toLong(), size.toLong(), contentUri, name)
                    videoList += videoLocalRes
                }
            }

            return videoList
        }

        /**
         * 判断当前的视频是否是支持的格式
         */
        fun isSupportFormat(name: String?): Boolean {
            name?.let {
                val list = name.split(".")
                if (list == null || list.size < 2){
                    return false;
                }
                if (list[list.size - 1] == null){
                    return false
                }
                return suffixMap.containsKey(list[list.size - 1]);
            }

            return false
        }
    }
}