package com.common.widget.drawable

import android.graphics.Bitmap
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.ColorFilter
import android.graphics.Paint
import android.graphics.PixelFormat
import android.graphics.PorterDuff
import android.graphics.PorterDuffXfermode
import android.graphics.RectF
import android.graphics.drawable.Drawable
import android.os.Build
import androidx.annotation.RequiresApi
import com.alpha.lagouapk.R
import com.lagou.commonlibs.utils.ResourceUtils

/**
 * @date： Crate by wangjunwei on 2023/12/26
 * @description：优惠券镂空背景色
 **/
class CouponXfermodeDrawable(
     private val iColor: Int, private val xfermodeOrientation: XfermodeOrientation
) : Drawable() {

    enum class XfermodeOrientation(val value: String) {
        LEFT("left"), RIGHT("right")
    }

    private val paint = Paint(Paint.ANTI_ALIAS_FLAG)
    private val xfermode = PorterDuffXfermode(PorterDuff.Mode.DST_OUT)
    private val DP_2 = ResourceUtils.getDimenSize(R.dimen.dp_2)

    /**
     * 绘制区域范围
     */
    private lateinit var boundsCustom: RectF

    /***
     * 圆角矩形
     */
    private lateinit var squareBitmap: Bitmap

    /***
     * 镂空所需要的圆
     */
    private lateinit var circleBitmapTop: Bitmap
    private lateinit var circleBitmapBottom: Bitmap

    private var isInit = false

    init {
        paint.style = Paint.Style.FILL
    }

    @RequiresApi(Build.VERSION_CODES.LOLLIPOP)
    private fun init() {
        if (isInit) {
            return
        }
        isInit = true

        boundsCustom = RectF(
            bounds.left.toFloat(),
            bounds.top.toFloat(),
            bounds.right.toFloat(),
            bounds.bottom.toFloat()
        )
        squareBitmap = Bitmap.createBitmap(
            bounds.width(), bounds.height(), Bitmap.Config.ARGB_8888
        )
        circleBitmapTop = Bitmap.createBitmap(
            bounds.width(), bounds.height(), Bitmap.Config.ARGB_8888
        )
        circleBitmapBottom = Bitmap.createBitmap(
            bounds.width(), bounds.height(), Bitmap.Config.ARGB_8888
        )

        //1 绘制圆角矩形
        val canvas = Canvas(squareBitmap)
        paint.color = iColor
        canvas.drawRoundRect(
            bounds.left.toFloat(),
            bounds.top.toFloat(),
            bounds.right.toFloat(),
            bounds.bottom.toFloat(),
            DP_2.toFloat(),
            DP_2.toFloat(),
            paint
        )

        //2 绘制圆，用来镂空操作
        paint.color = Color.parseColor("#ffffffff")
        if (xfermodeOrientation == XfermodeOrientation.LEFT) {
            canvas.setBitmap(circleBitmapTop)
            canvas.drawCircle(bounds.left.toFloat(), bounds.top.toFloat(), DP_2.toFloat(), paint)
            canvas.setBitmap(circleBitmapBottom)
            canvas.drawCircle(bounds.left.toFloat(), bounds.bottom.toFloat(), DP_2.toFloat(), paint)
        } else {
            canvas.setBitmap(circleBitmapTop)
            canvas.drawCircle(bounds.right.toFloat(), bounds.top.toFloat(), DP_2.toFloat(), paint)
            canvas.setBitmap(circleBitmapBottom)
            canvas.drawCircle(bounds.right.toFloat(), bounds.bottom.toFloat(), DP_2.toFloat(), paint)
        }
    }

    @RequiresApi(Build.VERSION_CODES.LOLLIPOP)
    override fun draw(canvas: Canvas) {
        try {

            init()

            val count = canvas.saveLayer(boundsCustom, null)
            canvas.drawBitmap(squareBitmap, 0f, 0f, paint)
            paint.xfermode = xfermode
            if (xfermodeOrientation == XfermodeOrientation.LEFT) {
                canvas.drawBitmap(circleBitmapTop, 0f, 0f, paint)
                canvas.drawBitmap(
                    circleBitmapBottom, 0f, 0f, paint
                )
            } else {
                canvas.drawBitmap(circleBitmapTop, 0f, 0f, paint)
                canvas.drawBitmap(circleBitmapBottom, 0f, 0f, paint)
            }
            paint.xfermode = null
            canvas.restoreToCount(count)

        }catch (e: Exception){
            e.printStackTrace()
        }
    }

    override fun setAlpha(alpha: Int) {
        paint.alpha = alpha
    }

    override fun setColorFilter(colorFilter: ColorFilter?) {
        paint.colorFilter = colorFilter
    }

    override fun getOpacity(): Int {
        return when (alpha) {
            0 -> PixelFormat.TRANSPARENT
            0xff -> PixelFormat.OPAQUE
            else -> PixelFormat.TRANSLUCENT
        }
    }
}