package com.common.widget.interview

import android.content.Context
import android.graphics.Rect
import androidx.recyclerview.widget.RecyclerView
import android.util.AttributeSet
import android.view.LayoutInflater
import android.widget.FrameLayout
import androidx.recyclerview.widget.GridLayoutManager
import com.alpha.lagouapk.R
import com.lagou.commonlibs.utils.TimeUtils
import com.common.bean.json.InterviewCalendarDayRes
import com.common.bean.json.InterviewTimeRes
import com.common.manager.InterviewCalendarManager
import com.common.ui.adapter.InterviewInviteCalendarDayAdapter
import com.common.ui.adapter.InterviewWeekAdapter
import com.common.utils.SpUserNotClearUtils
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListenerImpl
import com.lagou.lagouee.bean.jsonbean.interview.InterviewTimeReq
import com.lagou.lagouee.manager.net.InterviewNetManager
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.android.synthetic.main.card_interview_calendar_invite.view.*

/**
 * @date 创建时间：2020/7/30
 * @author 作者：codeman
 * @Description  描述：邀约日历
 * @version
 */
class InterviewInviteCalendarView(
    context: Context, attrs: AttributeSet? = null
) : FrameLayout(context, attrs) {

    /***
     * 服务器时间戳
     */
    private var serverTimeStamp = SpUserNotClearUtils.getInstance().serverTimeStamp

    /***
     * 有面试的时间列表
     */
    private var interviewTimes: List<String>? = null

    /**
     * 日历选中回调
     */
    private var iInterviewCalendar: InterviewInviteCalendarDayAdapter.IInnerInviteCalendar? = null

    /***
     * 1 日历Adapter
     */
    private var calendarAdapter: InterviewInviteCalendarDayAdapter? = null

    /***
     * 1 被选中日历
     */
    private var selectCalendarDayRes: InterviewCalendarDayRes? = null

    private val baseActivity = context as BaseActivity

    private val onInterviewTimesListener = object :
            OnAdapterCommonListenerImpl<InterviewTimeRes, HttpResponse<InterviewTimeRes>>(baseActivity) {

        override fun onSuccess(times: InterviewTimeRes?) {
            super.onSuccess(times)
            interviewTimes = times?.days
            fillUi()
        }

        override fun onError(errorBean: HttpResponse<InterviewTimeRes>?) {
            super.onError(errorBean)
            fillUi()
        }

        override fun onNoInternet() {
            super.onNoInternet()
            fillUi()
        }

        override fun isShowToast() = false
    }

    init {
        LayoutInflater.from(context).inflate(R.layout.card_interview_calendar_invite, this, true)
        initWeekRecycle()
        initCalendarViewPager()
        requestInterviewTimes()
    }

    fun setSelectTimeListener(iInterviewCalendar: InterviewInviteCalendarDayAdapter.IInnerInviteCalendar?) {
        this.iInterviewCalendar = iInterviewCalendar
    }

    private fun initWeekRecycle() {
        recycle_week?.apply {
            isNestedScrollingEnabled = false
            layoutManager = GridLayoutManager(context, 7)
            itemAnimator = null
            adapter = InterviewWeekAdapter(context).bigTxt()
        }
    }

    private fun initCalendarViewPager() {
        recycle_calendar_day?.apply {
            isNestedScrollingEnabled = false
            layoutManager = GridLayoutManager(context, 7)
            itemAnimator = null
            addItemDecoration(CalendarDecoration(context))
            calendarAdapter = InterviewInviteCalendarDayAdapter(context)
            adapter = calendarAdapter
        }
        calendarAdapter?.setInterviewCalendarListener(object : InterviewInviteCalendarDayAdapter.IInnerInviteCalendar{
            override fun onSelectTime(calendarDayRes: InterviewCalendarDayRes?) {
                selectCalendarDayRes = calendarDayRes
                iInterviewCalendar?.onSelectTime(calendarDayRes)
            }
        })
    }

    /***
     * 填充日历UI
     */
    private fun fillUi() {
        calendarAdapter?.apply {
            val calendarDays = InterviewCalendarManager.createInviteCalendarDay(
                    serverTimeStamp,
                    interviewTimes
            )
            setData(calendarDays)
        }
    }

    class CalendarDecoration(context: Context) : RecyclerView.ItemDecoration() {
        private val dp_13: Int = context.resources.getDimension(R.dimen.dp_13).toInt()

        override fun getItemOffsets(outRect: Rect, itemPosition: Int, parent: RecyclerView) {
            super.getItemOffsets(outRect, itemPosition, parent)
            if (itemPosition > 6) {
                outRect.set(0, dp_13, 0, 0)
            }
        }
    }

    /***
     * 有面试的时间段
     */
    private fun requestInterviewTimes() {
        val interviewTimeReq = InterviewTimeReq()
        interviewTimeReq.startTime = InterviewCalendarManager.getInviteCalendarStartTime(
                TimeUtils.convertYearMonthDayTime(InterviewCalendarManager.getInviteCalendarStartDay(serverTimeStamp)))
        interviewTimeReq.endTime = TimeUtils.getGoalTime( 35, interviewTimeReq.startTime)
        InterviewNetManager.getCInterviewTimes(interviewTimeReq, onInterviewTimesListener)
    }
}