package com.common.widget.popup

import android.content.Context
import android.graphics.Color
import android.graphics.Rect
import android.graphics.drawable.ColorDrawable
import android.os.Build
import android.view.View
import android.view.ViewGroup
import android.widget.PopupWindow
import com.common.bean.json.InterviewCalendarDayRes
import com.common.widget.interview.InterviewCalenderExpandView
import com.common.widget.interview.InterviewCalenderFoldView
import com.lagou.baselib.util.BarUtil
import com.lagou.lagouee.widget.inter.DialogListener

/**
 * @date 创建时间：2020/7/28
 * @author 作者：codeman
 * @Description  描述：面试日历popup
 * @version
 */
class InterviewCalenderPopup(ctx: Context, locationView: View, calenderFoldView: InterviewCalenderFoldView) : PopupWindow(), DialogListener {


    private var context = ctx
    private var locationView = locationView

    private var popupHeight = 0
    private var calenderExpandView: InterviewCalenderExpandView? = null
    private var calenderFoldView: InterviewCalenderFoldView = calenderFoldView

    /***
     * 面试日历相关的回调，包含选中面试时间、切换日历、收起日历
     */
    interface IInterviewCalendar {
        fun onScrollCalendar(pageNum: Int, calendarDays: List<InterviewCalendarDayRes>?, month: String?, year: String?)
        /***
         * 收起日历，包含选中和手动收起
         */
        fun onFoldCalendar(calendarDayRes: InterviewCalendarDayRes?, calendarDays: List<InterviewCalendarDayRes>?)
    }

    init {
        initConfig()
        initView()
        initPopup()
        initListener()
    }

    private fun initPopup() {
        locationView?.let {
            it.rootView?.addOnLayoutChangeListener { _: View?, _: Int, _: Int, _: Int,
                                                  _: Int, _: Int, _: Int, _: Int, _: Int ->
                it.rootView?.let { it2 ->
                    if (it2.height > 0) {
                        val positionHeight = getPopupPositionSearchHeight()
                        if (height != positionHeight) {
                            width = ViewGroup.LayoutParams.MATCH_PARENT
                            height = positionHeight
                            if (isShowing) {
                                update(ViewGroup.LayoutParams.MATCH_PARENT, positionHeight
                                )
                            }
                        }
                    }
                }
            }
        }
    }

    private fun getPopupPositionSearchHeight(): Int {
        locationView?.let {
            it.rootView?.height?.let {
                locationView?.let { it2 ->
                    return it - it2.height - BarUtil.getStatusBarHeight(context)
                }
            }
        }
        return 0
    }

    fun delayCalendarUi() {
       calenderExpandView?.fillUi()
    }

    fun showAnim() {
        calenderExpandView?.showAnim()
    }

    fun hideAnim() {
        calenderExpandView?.hideAnim()
    }

    fun beforeCalendar(isExpand: Boolean){
        calenderExpandView?.beforeCalendar(isExpand)
    }
    fun afterCalendar(isExpand: Boolean){
        calenderExpandView?.afterCalendar(isExpand)
    }

    fun syncFoldCalendar(calendarDay: InterviewCalendarDayRes, isApplyFold: Boolean){
        calenderExpandView?.syncFoldCalendar(calendarDay, isApplyFold)
    }

    fun onlySyncFoldCalendar(calendarDay: InterviewCalendarDayRes, isApplyFold: Boolean){
        calenderExpandView?.syncFoldCalendar(calendarDay, isApplyFold)
    }
    override fun setHeight(height: Int) {
        super.setHeight(height)
        if (height > 0) {
            popupHeight = height
            calenderExpandView?.initAnim(popupHeight)
        }
    }

    override fun show() {
        if (isShowing) {
            return
        }
        try {
            showAsDropDown(locationView)
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    override fun hide() {
        try {
            dismiss()
        } catch (e: java.lang.Exception) {
            e.printStackTrace()
        }
    }

    private fun initView() {
        calenderExpandView = InterviewCalenderExpandView(context)
        calenderExpandView?.setCalenderFoldView(calenderFoldView);
        contentView = calenderExpandView
    }

    private fun initConfig() {
        setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        animationStyle = 0
    }

    private fun initListener() {
        calenderExpandView?.setListener(this)
    }

    override fun showAsDropDown(anchor: View) {
        //适配android 7.1.1系统上显示位置不正确的问题解决
        if (Build.VERSION.SDK_INT == 25) {
            val rect = Rect()
            anchor.getGlobalVisibleRect(rect)
            val h = anchor.resources.displayMetrics.heightPixels - rect.bottom
            height = h
        }
        super.showAsDropDown(anchor)
    }

}