package com.lagou.lagouee.manager.net

import com.ApiService
import com.alpha.lagouapk.R
import com.alpha.lagouapk.observer.EventTags
import com.lagou.baselib.net.httpnetback.AdapterBaseCallBack
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListener
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListenerImpl
import com.lagou.commonlibs.event.OkBus
import com.lagou.commonlibs.utils.ToastUtil
import com.lagou.lagouee.bean.jsonbean.company.CompanyAddress
import com.lagou.lagouee.bean.jsonbean.company.CompanyAddressBean
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse

/**
 * @date 创建时间：2020/6/19
 * @author 作者：codeman
 * @Description  描述：地址网路请求管理类
 * @version
 */
class AddressNetManager {

    companion object {
        /**
         * 获取地址信息
         */
        fun getAddressList(
            isCompany: Boolean,
            listener: OnAdapterCommonListenerImpl<List<CompanyAddress>, HttpResponse<List<CompanyAddress>>>
        ) {
            if (listener == null || !listener.isActive) {
                return
            }

            val httpRequest = if (isCompany) ApiService.api.companyAddressInfo
            else ApiService.api.positionAddressInfo
            httpRequest.enqueue(object : AdapterBaseCallBack<List<CompanyAddressBean>>(listener) {
                override fun onAdapterSuccess(
                    request: HttpRequest<*>,
                    response: HttpResponse<List<CompanyAddressBean>>
                ) {
                    listener.onSuccess(CompanyAddressBean.convertToCompanyAddress(response.content))
                }
            })
        }

        /**
         * 删除地址
         */
        fun delAddress(
            isCompany: Boolean, companyAddress: CompanyAddress,
            listener: OnAdapterCommonListener<Any?, HttpResponse<Any>?>
        ) {
            if (listener == null || !listener.isActive) {
                return
            }
            listener.showLoading()
            val httpRequest =
                if (isCompany) ApiService.api.deleteCompanyAddress(companyAddress.getPd_id())
                else ApiService.api.deletePositionAddress(companyAddress.getPd_id())
            httpRequest.enqueue(object : AdapterBaseCallBack<Any>(listener) {
                override fun onAdapterSuccess(
                    request: HttpRequest<*>?,
                    response: HttpResponse<Any>?
                ) {
                    listener.onSuccess(Any())
                    ToastUtil.show(listener.baseActivity, R.string.delete_address_success)
                }
            })
        }

        /**
         * 添加/编辑地址
         */
        @JvmOverloads
        fun addAddress(
            isCompany: Boolean, isUpdate: Boolean, companyAddress: CompanyAddress,isFinishActivity:Boolean = true,
            listener: OnAdapterCommonListenerImpl<CompanyAddress, HttpResponse<CompanyAddress>>
        ) {
            if (listener == null || !listener.isActive) {
                return
            }
            listener.showLoading()
            val companyAddressBean = companyAddress.convertCompanyAddressBean()
            if (isUpdate) {
                val httpRequest =
                    if (isCompany) ApiService.api.updateCompanyAddress(companyAddressBean)
                    else ApiService.api.updatePositionAddress(companyAddressBean)
                httpRequest.enqueue(object : AdapterBaseCallBack<Any>(listener) {
                    override fun onAdapterSuccess(
                        request: HttpRequest<*>,
                        response: HttpResponse<Any>
                    ) {
                        OkBus.getInstance().onEvent(EventTags.ADD_COMPANY_ADDRESS, companyAddress)
                        listener.onSuccess(companyAddress)
                        if (isFinishActivity){
                            listener.finishActivity()
                        }
                    }
                })
                return
            }

            val httpRequest =
                if (isCompany) ApiService.api.addCompanyAddress(companyAddressBean)
                else ApiService.api.addPositionAddress(companyAddressBean)

            httpRequest.enqueue(object : AdapterBaseCallBack<CompanyAddressBean>(listener) {
                override fun onAdapterSuccess(
                    request: HttpRequest<*>,
                    response: HttpResponse<CompanyAddressBean>
                ) {
                    companyAddress.setPd_id(response.content.id)
                    OkBus.getInstance().onEvent(EventTags.ADD_COMPANY_ADDRESS, companyAddress)
                    listener.onSuccess(companyAddress)
                    if (isFinishActivity){
                        listener.finishActivity()
                    }
                }
            })
        }
    }


}