package com.lagou.lagouee.manager.net

import com.ApiService
import com.alpha.lagouapk.bean.interview.CCommentDetail
import com.alpha.lagouapk.bean.interview.CCommentDetailReq
import com.alpha.lagouapk.bean.interview.CInterviewOperateReq
import com.alpha.lagouapk.bean.interview.InterviewWaitRes
import com.common.bean.json.InterviewTimeRes
import com.common.utils.PageHelper
import com.lagou.baselib.net.httpnetback.AdapterBaseCallBack
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListListenerImpl
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListenerImpl
import com.lagou.commonlibs.engine.ThreadUtil
import com.lagou.lagouee.bean.InterviewList
import com.lagou.lagouee.bean.InterviewList.InterviewArrangeVosBean
import com.lagou.lagouee.bean.jsonbean.interview.BInterviewDetailReq
import com.lagou.lagouee.bean.jsonbean.interview.BInterviewDetailRes
import com.lagou.lagouee.bean.jsonbean.interview.InterviewListReq
import com.lagou.lagouee.bean.jsonbean.interview.InterviewTimeReq
import com.lagou.lagouee.ui.resume.list.interview.MinuteGroup
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse
import java.text.SimpleDateFormat
import java.util.*

/**
 * @date 创建时间：2020/7/22
 * @author 作者：codeman
 * @Description  描述：面试相关的接口请求
 * @version
 */
class InterviewNetManager {

    companion object {

        /***
         * 获取B端面试详情
         */
        fun getInterviewDetail(interviewId: Long?, commonListenerImpl: OnAdapterCommonListenerImpl<BInterviewDetailRes, HttpResponse<BInterviewDetailRes>>?) {
            if (commonListenerImpl == null || !commonListenerImpl.isActive) {
                return
            }
            if (interviewId == null) {
                return
            }

            commonListenerImpl.showLoading()
            ApiService.api.getNewInterviewDetail(BInterviewDetailReq(interviewId)).enqueue(object : AdapterBaseCallBack<BInterviewDetailRes>(commonListenerImpl) {
                override fun onAdapterSuccess(request: HttpRequest<*>?, response: HttpResponse<BInterviewDetailRes>?) {
                    commonListenerImpl.onSuccess(response?.content)
                }
            })
        }

        /***
         * 面试时间列表
         */
        fun getBInterviewTimes(interviewTimeReq: InterviewTimeReq?, commonListenerImpl: OnAdapterCommonListenerImpl<InterviewTimeRes, HttpResponse<InterviewTimeRes>>?) {
            ApiService.api.getInterviewTimes(interviewTimeReq).enqueue(object : AdapterBaseCallBack<InterviewTimeRes>(commonListenerImpl) {
                override fun onAdapterSuccess(request: HttpRequest<*>?, response: HttpResponse<InterviewTimeRes>?) {
                    commonListenerImpl?.onSuccess(response?.content)
                }
            })
        }


        /***
         * 面试时间列表
         */
        fun getCInterviewTimes(interviewTimeReq: InterviewTimeReq?, commonListenerImpl: OnAdapterCommonListenerImpl<InterviewTimeRes, HttpResponse<InterviewTimeRes>>?) {
            ApiService.api.getCInterviewTimes(interviewTimeReq).enqueue(object : AdapterBaseCallBack<InterviewTimeRes>(commonListenerImpl) {
                override fun onAdapterSuccess(request: HttpRequest<*>?, response: HttpResponse<InterviewTimeRes>?) {
                    commonListenerImpl?.onSuccess(response?.content)
                }
            })
        }

        /**
         * 面试待确认信息
         */
        fun getCInterviewWaitInfo(commonListenerImpl: OnAdapterCommonListenerImpl<InterviewWaitRes, HttpResponse<InterviewWaitRes>>?) {
            ApiService.api.getCInterviewWaitInfo().enqueue(object : AdapterBaseCallBack<InterviewWaitRes>(commonListenerImpl) {
                override fun onAdapterSuccess(request: HttpRequest<*>?, response: HttpResponse<InterviewWaitRes>?) {
                    commonListenerImpl?.onSuccess(response?.content)
                }
            })
        }

        /**
         * 接受面试
         */
        fun cAcceptInterview(interviewId: Int?, commonListenerImpl: OnAdapterCommonListenerImpl<Any, HttpResponse<Any>>?) {
            if (commonListenerImpl?.isActive == false) {
                return
            }
            if (interviewId != null) {
                commonListenerImpl?.showLoading()
                ApiService.api.cAcceptInterview(CInterviewOperateReq(interviewId)).enqueue(object : AdapterBaseCallBack<Object>(commonListenerImpl) {
                    override fun onAdapterSuccess(request: HttpRequest<*>?, response: HttpResponse<Object>?) {
                        commonListenerImpl?.onSuccess(response?.content)
                    }
                })
            }
        }

        /**
         * 拒绝面试
         */
        fun cRefuseInterview(interviewId: Int?, commonListenerImpl: OnAdapterCommonListenerImpl<Any, HttpResponse<Any>>?) {
            if (commonListenerImpl?.isActive == false) {
                return
            }
            if (interviewId != null) {
                commonListenerImpl?.let {
                    it.showLoading()
                }
                ApiService.api.cRefuseInterview(CInterviewOperateReq(interviewId)).enqueue(object : AdapterBaseCallBack<Object>(commonListenerImpl) {
                    override fun onAdapterSuccess(request: HttpRequest<*>?, response: HttpResponse<Object>?) {
                        commonListenerImpl?.onSuccess(response?.content)
                    }
                })
            }
        }


        /**
         * 取消面试
         */
        fun cCancelInterview(interviewId: Int?, commonListenerImpl: OnAdapterCommonListenerImpl<Any, HttpResponse<Any>>?) {
            if (commonListenerImpl?.isActive == false) {
                return
            }
            if (interviewId != null) {
                commonListenerImpl?.let {
                    it.showLoading()
                }
                ApiService.api.cCancelInterview(CInterviewOperateReq(interviewId)).enqueue(object : AdapterBaseCallBack<Object>(commonListenerImpl) {
                    override fun onAdapterSuccess(request: HttpRequest<*>?, response: HttpResponse<Object>?) {
                        commonListenerImpl?.onSuccess(response?.content)
                    }
                })
            }
        }


        /**
         * C端面试评价详情
         */
        fun getCCommentDetail(orderId: Long?, commonListenerImpl: OnAdapterCommonListenerImpl<CCommentDetail, HttpResponse<CCommentDetail>>?) {
            if (commonListenerImpl?.isActive == false) {
                return
            }
            if (orderId != null) {
                commonListenerImpl?.let {
                    it.showLoading()
                }
                ApiService.api.getCCommentDetail(CCommentDetailReq(orderId)).enqueue(object : AdapterBaseCallBack<CCommentDetail>(commonListenerImpl) {
                    override fun onAdapterSuccess(request: HttpRequest<*>?, response: HttpResponse<CCommentDetail>?) {
                        commonListenerImpl?.onSuccess(response?.content)
                    }
                })
            }
        }

        /**
         * 重新装配数据
         */
        fun newList(set: Set<InterviewArrangeVosBean>): List<Any>? {
            val list: MutableList<Any> = ArrayList()
            val dayMillis: Long = -1
            var minuteMillis: Long = -1
            val num = 0
            val sdf = SimpleDateFormat("HH:mm")
            for (bean in set) {
                var time = "";
                try {
                    time = sdf.format(bean.interviewTimeStamp);
                } catch (e: Exception) {
                    e.printStackTrace()
                }
                list.add(MinuteGroup(bean.orderResumeId, bean.interviewTimeStamp, dayMillis, num, time))
                // 添加item
                bean.num = num
                list.add(bean)
            }
            return list
        }

    }

    class InterviewListRequest {
        private var mPageNo = 1

        /**
         * 时间排序
         *
         *
         * 排序规则见[InterviewList.InterviewArrangeVosBean.compareTo]方法
         */
        private val mSet = TreeSet<InterviewArrangeVosBean>()

        /**
         * 记录唯一id
         *
         *
         * 使用此map避免重复数据
         */
        private val mIdMap: HashMap<Long, InterviewArrangeVosBean> = HashMap()

        fun getInterviewList(interviewListReq: InterviewListReq?, isReverseOrder: Boolean?,
                             first: Boolean, positionIds: ArrayList<Int>?,
                             adapterCommonListener: OnAdapterCommonListListenerImpl<List<Any>, HttpResponse<List<Any>>, Boolean>?) {
            if (adapterCommonListener == null || !adapterCommonListener.isActive) {
                return
            }
            if (interviewListReq == null) {
                callbackNoData(first, adapterCommonListener)
                return
            }
            if (first) {
                adapterCommonListener.showLoading()
                mPageNo = 1
            }
            interviewListReq.pageNo = mPageNo
            interviewListReq.pageSize = PageHelper.COUNT_20;
            positionIds?.let {
                interviewListReq.positionIds = it
            }

            ApiService.api.getNewInterviewList(interviewListReq).enqueue(object : AdapterBaseCallBack<InterviewList?>(adapterCommonListener) {
                override fun onAdapterSuccess(request: HttpRequest<*>?, response: HttpResponse<InterviewList?>?) {
                    handleInterviewListResult(first, isReverseOrder, request, response, adapterCommonListener)
                }

            })
        }

        private fun handleInterviewListResult(first: Boolean, isReverseOrder: Boolean?,
                                              request: HttpRequest<*>?, response: HttpResponse<InterviewList?>?,
                                              adapterCommonListener: OnAdapterCommonListListenerImpl<List<Any>, HttpResponse<List<Any>>, Boolean>?) {
            val list = response?.content
            val hasNext = list != null && list.isHasMore
            if (hasNext) {
                mPageNo++
            }
            var subList: MutableList<InterviewArrangeVosBean>? = list?.interviewArrangeVos
            if (subList == null || subList.isEmpty()) {
                if (first) {
                    adapterCommonListener?.onNoData(null, first)
                    return
                }

                if (hasNext) {
                    adapterCommonListener?.onSuccess(null, first)
                    return
                }

                adapterCommonListener?.onNoMore(null, first)
                return
            }

            ThreadUtil.compute {
                synchronized(this@InterviewListRequest) {
                    if (first) {
                        mSet.clear()
                    }
                    // 更新面试日程
                    val calendar = Calendar.getInstance()
                    var pre: InterviewArrangeVosBean?
                    for (bean in subList) {
                        handleBean(bean, calendar, isReverseOrder)
                        pre = mIdMap.put(bean.id, bean)
                        pre?.let {
                            // 此id数据需要更新，故先移除
                            mSet.remove(pre)
                        }
                        // 添加并排序
                        mSet.add(bean)
                    }
                    // 再处理数据
                    val result: List<Any>? = newList(mSet)
                    ThreadUtil.ui {
                        if (adapterCommonListener == null || !adapterCommonListener.isActive) {
                            return@ui
                        }
                        if (hasNext) {
                            adapterCommonListener.onSuccess(result, first)
                        } else {
                            adapterCommonListener.onNoMore(result, first)
                        }
                    }
                }
            }
        }

        private fun callbackNoData(first: Boolean, adapterCommonListener: OnAdapterCommonListListenerImpl<List<Any>, HttpResponse<List<Any>>, Boolean>?) {
            if (first) {
                adapterCommonListener?.onNoData(null, first)
            } else {
                adapterCommonListener?.onNoMore(null, first)
            }
        }


        /**
         * 对bean自定义逻辑字段进行赋值
         */
        private fun handleBean(bean: InterviewArrangeVosBean, calendar: Calendar, isReverseOrder: Boolean?) {
            calendar.timeInMillis = bean.interviewTimeStamp
            calendar[Calendar.SECOND] = 0
            calendar[Calendar.MILLISECOND] = 0
            // 分-起始时间戳
            bean.minuteMillis = calendar.timeInMillis
            calendar[Calendar.HOUR_OF_DAY] = 0
            calendar[Calendar.MINUTE] = 0
            // 天-起始时间戳
            bean.dayMillis = calendar.timeInMillis
            isReverseOrder?.let {
                bean.isReversed = it
            }
        }
    }
}