package com.lagou.lagouee.share

import android.app.Activity
import android.content.DialogInterface
import android.graphics.Bitmap
import android.graphics.drawable.Drawable
import android.os.Bundle
import android.text.TextUtils
import android.view.View
import android.widget.LinearLayout
import android.widget.TextView
import android.widget.Toast
import androidx.constraintlayout.widget.ConstraintLayout
import com.alpha.lagouapk.R
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.bean.json.ClickInfo
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListenerImpl
import com.lagou.baselib.privacy.KEY_STORE_PHOTO_BC_SAVE
import com.lagou.commonlibs.imageload.ImageLoadUtils
import com.lagou.commonlibs.imageload.inter.ALoadImgListener
import com.lagou.commonlibs.inter.SavePictureListener
import com.lagou.commonlibs.ui.share.BaseShareDialog
import com.lagou.commonlibs.utils.BitmapUtils
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.utils.ToastUtil
import com.lagou.commonlibs.utils.lgshare.LGShareAction
import com.lagou.commonlibs.utils.lgshare.LGShareListener
import com.lagou.commonlibs.utils.lgshare.core.LGShareMedia
import com.lagou.lagouee.manager.net.CommonNetManager
import com.lagou.socketchannel.http.HttpResponse

/**
 * @author 作者：ryan.lei
 * @date 创建时间：2021/9/16 15:43
 * @Description 描述：HR成就卡/排行榜分享弹窗
 **/
class HrUrgeShareDialog : BaseShareDialog() {
    private var mShareAction: LGShareAction? = null
    private var mPageId: String = ""
    private var mAId: String = ""
    private var mCardId: String = ""
    private var mRankType: String = ""

    /**
     * 是否分享过,默认为已分享过,不显示100积分的提示
     */
    private var mShareState: Boolean = true
    private var mImageUrl: String = ""

    private var integralShareListener: IntegralShareListener? = null

    private var mLlWxShareBoard: LinearLayout? = null
    private var mClWxCircleShareBoard: ConstraintLayout? = null
    private var mLlPhotoShareBoard: LinearLayout? = null
    private var mTvShareCancel: TextView? = null
    private var mTvWxCircleIntegralNotice: TextView? = null

    private val mTransformAchievementTypeMap = HashMap<String, Int>().apply {
        put(TYPE_ACHIEVEMENT_NIGHT, 1)
        put(TYPE_ACHIEVEMENT_PERSERVE, 2)
        put(TYPE_ACHIEVEMENT_HARDWORK, 3)
    }

    private val mTransformRankingTypeMap = HashMap<String, Int>().apply {
        put(TYPE_RANKING_POPULAR, 1)
        put(TYPE_RANKING_IM, 2)
        put(TYPE_RANKING_INFLUENCE, 3)
        put(TYPE_RANKING_POTENTIAL, 4)
        put(TYPE_RANKING_HARDWORK, 5)
    }

    override fun getLayoutId(): Int = R.layout.fragment_shareboard_hr_task

    companion object {
        private const val HR_URGE_SHARE_PAGE_ID = "hr_urge_share_page_id"
        private const val HR_URGE_SHARE_AID = "hr_urge_share_aid"
        private const val HR_URGE_SHARE_CARD_ID = "hr_urge_share_card_id"
        private const val HR_URGE_SHARE_RANK_TYPE = "hr_urge_share_rank_type"
        private const val HR_URGE_SHARE_SHARE_STATE = "hr_urge_share_share_state"
        private const val HR_URGE_SHARE_IMAGE_URL = "hr_urge_share_image_url"
        private const val STATE_SAVE_INTEGRAL_SUCCESS = 1

        /**
         * 成就卡——披星戴月
         */
        private const val TYPE_ACHIEVEMENT_NIGHT = "ACHIEVEMENT_NIGHT"

        /**
         * 成就卡——持之以恒
         */
        private const val TYPE_ACHIEVEMENT_PERSERVE = "ACHIEVEMENT_PERSERVE"

        /**
         * 成就卡——不辞辛劳
         */
        private const val TYPE_ACHIEVEMENT_HARDWORK = "ACHIEVEMENT_HARDWORK"

        /**
         * 周榜——最受欢迎招聘者
         */
        private const val TYPE_RANKING_POPULAR = "RANKING_POPULAR"

        /**
         * 周榜——最会撩招聘者（积极主动）
         */
        private const val TYPE_RANKING_IM = "RANKING_IM"

        /**
         * 周榜——最具影响力招聘者
         */
        private const val TYPE_RANKING_INFLUENCE = "RANKING_INFLUENCE"

        /**
         * 周榜——最具潜力招聘者
         */
        private const val TYPE_RANKING_POTENTIAL = "RANKING_POTENTIAL"

        /**
         * 周榜——最辛劳招聘者
         */
        private const val TYPE_RANKING_HARDWORK = "RANKING_HARDWORK"


        /**
         * 弹窗的初始化入口
         *
         * @param shareState 是否分享过
         * @param cardId    成就卡/排行榜ID
         * @param rankType  类型:成就卡还是排行榜
         * @return
         */
        fun newInstance(
            pageId: String,
            aid: String,
            shareState: Boolean,
            cardId: String,
            rankType: String,
            imageUrl: String,
            listener: IntegralShareListener
        ): HrUrgeShareDialog {
            val dialog = HrUrgeShareDialog()
            val bundle = Bundle()
            bundle.putString(HR_URGE_SHARE_PAGE_ID, pageId)
            bundle.putString(HR_URGE_SHARE_AID, aid)
            bundle.putString(HR_URGE_SHARE_CARD_ID, cardId)
            bundle.putString(HR_URGE_SHARE_RANK_TYPE, rankType)
            bundle.putBoolean(HR_URGE_SHARE_SHARE_STATE, shareState)
            bundle.putString(HR_URGE_SHARE_IMAGE_URL, imageUrl)
            dialog.setIntegralShareListener(listener)
            dialog.arguments = bundle
            return dialog
        }
    }

    override fun initView(view: View?) {
        super.initView(view)
        mLlWxShareBoard = view?.findViewById(R.id.ll_wx_shareboard)
        mClWxCircleShareBoard = view?.findViewById(R.id.cl_wxcircle_shareboard)
        mLlPhotoShareBoard = view?.findViewById(R.id.ll_photo_shareboard)
        mTvShareCancel = view?.findViewById(R.id.tv_share_cancel)
        mTvWxCircleIntegralNotice = view?.findViewById(R.id.tv_wxcircle_integral_notice)
    }

    override fun fillUI() {
        super.fillUI()
        val bundle = arguments
        mPageId = bundle?.getString(HR_URGE_SHARE_PAGE_ID) ?: ""
        mAId = bundle?.getString(HR_URGE_SHARE_AID) ?: ""
        mCardId = bundle?.getString(HR_URGE_SHARE_CARD_ID) ?: ""
        mRankType = bundle?.getString(HR_URGE_SHARE_RANK_TYPE) ?: ""
        mShareState = bundle?.getBoolean(HR_URGE_SHARE_SHARE_STATE) ?: true
        mImageUrl = bundle?.getString(HR_URGE_SHARE_IMAGE_URL) ?: ""

        mTvWxCircleIntegralNotice?.visibility = if (mShareState) View.GONE else View.VISIBLE
    }

    override fun initListener() {
        super.initListener()
        mLlWxShareBoard?.setOnClickListener(this)
        mClWxCircleShareBoard?.setOnClickListener(this)
        mLlPhotoShareBoard?.setOnClickListener(this)
        mTvShareCancel?.setOnClickListener(this)
    }

    override fun onClick(view: View?) {
        super.onClick(view)
        var shareWay: Int = -1
        var shareMedia: LGShareMedia? = null
        when (view?.id) {
            R.id.ll_wx_shareboard -> {
                shareWay = 1
                if (mShareAction == null) {
                    mShareAction = LGShareAction(activity)
                }
                if (mShareAction?.isInstall(LGShareMedia.WEIXIN) == false) {
                    ToastUtil.show(
                        activity,
                        ResourceUtils.getString(R.string.no_wechat_install_notice),
                        Toast.LENGTH_SHORT
                    )
                    return
                }
                shareMedia = LGShareMedia.WEIXIN
            }
            R.id.cl_wxcircle_shareboard -> {
                shareWay = 2
                if (mShareAction == null) {
                    mShareAction = LGShareAction(activity)
                }
                if (mShareAction?.isInstall(LGShareMedia.WEIXIN) == false) {
                    ToastUtil.show(
                        activity,
                        ResourceUtils.getString(R.string.no_wechat_install_notice),
                        Toast.LENGTH_SHORT
                    )
                    return
                }
                shareMedia = LGShareMedia.WEIXIN_CIRCLE
            }
            R.id.ll_photo_shareboard -> {
                shareWay = 3
                if (mShareAction == null) {
                    mShareAction = LGShareAction(activity)
                }
                shareMedia = LGShareMedia.BITMAP
            }
            R.id.tv_share_cancel -> {
                //点击取消按钮
                dismiss()
                return
            }
        }

        val lgShareListener = HrUrgeShareListener(activity)
        if (shareMedia == LGShareMedia.WEIXIN || shareMedia == LGShareMedia.WEIXIN_CIRCLE) {
            //分享到微信或微信朋友圈
            mShareAction?.setShareMedia(shareMedia)
                ?.setImage(mImageUrl)
                ?.setDefaultImage(R.drawable.icon_share_c_change)
                ?.setListener(lgShareListener)
                ?.share()
        } else {
            //生成图片并保存
            ImageLoadUtils.with(activity)
                .load(mImageUrl)
                .dontAnimate()
                .into(object : ALoadImgListener<Bitmap>() {
                    override fun onResourceReady(resource: Bitmap?) {
                        saveBitmap(resource)
                        dismiss()
                    }

                    override fun onLoadFailed(e: Exception?, errorDrawable: Drawable?) {
                        super.onLoadFailed(e, errorDrawable)
                        ToastUtil.show(activity, ResourceUtils.getString(R.string.save_bitmap_fail))
                        dismiss()
                    }
                })

        }
        analyticClick(mPageId, mAId, mRankType, shareWay, mShareState)
    }

    fun setIntegralShareListener(listener: IntegralShareListener) {
        integralShareListener = listener
    }

    inner class HrUrgeShareListener(activity: Activity) : LGShareListener(activity) {
        override fun onError(shareMedia: LGShareMedia?, throwable: Throwable?) {
            super.onError(shareMedia, throwable)
            dismiss()
        }

        override fun onStart(shareMedia: LGShareMedia?) {
            super.onStart(shareMedia)
            //产品要求,只有分享到朋友圈才算积分
            //且点击调起微信即算,实测通过LGShareListener已无法监听微信的调起,故直接在点击分享成功发起调起微信请求时去请求积分接口
            if (shareMedia == LGShareMedia.WEIXIN_CIRCLE) {
                getHrAchievementIntegral(mCardId, mRankType)
            } else {
                dismiss()
            }
        }
    }

    /**
     * 保存生成的图片
     *
     * @param bitmap
     */
    private fun saveBitmap(bitmap: Bitmap?) {
        BitmapUtils.saveBitmapToGallery(activity, bitmap, "",
            object : SavePictureListener {
                override fun onSuccess() {
                    ToastUtil.show(activity, R.string.save_bitmap_success)
                }

                override fun onFailure() {
                    ToastUtil.show(activity, R.string.save_fail)
                }
            }, KEY_STORE_PHOTO_BC_SAVE)
    }

    /**
     * 分享组件点击埋点
     */
    private fun analyticClick(
        pageId: String,
        aid: String,
        type: String,
        shareWay: Int,
        shareState: Boolean
    ) {
        val clickInfo = ClickInfo()
        clickInfo.page_id = pageId
        clickInfo.address_id = aid
        clickInfo.addClick_props("ranking_list", mTransformRankingTypeMap[type])
        clickInfo.addClick_props("achievement_list", mTransformAchievementTypeMap[type])
        clickInfo.addClick_props("share_way", shareWay)
        clickInfo.addClick_props(
            "is_give_integral",
            if (shareWay == 2) (if (shareState) 0 else 1) else 0
        )
        AnalyticHelper.getInstance().sendClickInfo(clickInfo)
    }

    override fun onDismiss(dialog: DialogInterface) {
        super.onDismiss(dialog)
        if (activity is HrUrgeShareOpenActivity) {
            activity.finish()
        }
    }

    /**
     *分享得积分
     */
    private fun getHrAchievementIntegral(cardId: String, rankType: String) {
        if (mShareState) {
            dismiss()
            return
        }
        CommonNetManager.getHrAchievementIntegral(cardId, rankType,
            object :
                OnAdapterCommonListenerImpl<String, HttpResponse<String?>?>(activity as BaseActivity) {
                override fun onSuccess(msg: String) {
                    if (!TextUtils.isEmpty(msg)) {
                        Toast.makeText(activity, msg, Toast.LENGTH_SHORT).show()
                    }
                    mTvWxCircleIntegralNotice?.visibility = View.GONE
                    integralShareListener?.onSuccess()
                    dismiss()
                }

                override fun onError(errorBean: HttpResponse<String?>?) {
                    dismiss()
                }

                override fun onNoInternet() {
                    dismiss()
                }
            })
    }
}