package com.lagou.lagouee.share

import android.text.TextUtils
import com.alpha.lagouapk.R
import com.common.CommonEventTags
import com.lagou.annotation.apt.RouterRule
import com.lagou.baselib.BaseActivity
import com.lagou.commonlibs.event.OkBus
import com.lagou.commonlibs.web.router.RouteUtils
import com.lagou.lagouee.utils.DialogUtils

/**
 * @author 作者：ryan.lei
 * @date 创建时间：2021/9/23 12:18
 * @Description 描述：H5打开HR激励分享的中转页面
 *                  shareImgUrl:海报图片URL,
 *                  playbillType：海报类型- 0:招聘者周榜. 1:成就海报,
 *                  playbillDetailType:每种海报具体 1=最受欢迎榜 2=最会撩榜 3=最具潜力榜 4=最具影响力榜 5=最辛劳榜 6=披星戴月卡 7=持之以恒卡 8=不辞辛劳卡,
 *                  playbillPid:上报pagID,
 *                  playbillAid:上报aid
 **/
@RouterRule(
    "lagouhr://lagou.com/playbillShareAlert_schedule?playbillID={playbillID}&shareImgUrl={shareImgUrl}&" +
            "playbillType={playbillType}&playbillDetailType={playbillDetailType}&playbillPid={playbillPid}&playbillAid={playbillAid}&shareState={shareState}"
)
class HrUrgeShareOpenActivity : BaseActivity() {
    /**
     * 海报Id
     */
    private var mId: String? = null

    /**
     * 海报图片URL
     */
    private var mShareImageUrl: String? = null

    /**
     * 要打开的弹窗类型 0:招聘者周榜, 1:成就海报 (对应RouteUrl中的playbillType)
     */
    private var mPlaybillType: String? = null

    /**
     * 每种海报具体:
     * 1=最受欢迎榜
     * 2=最会撩榜
     * 3=最具潜力榜
     * 4=最具影响力榜
     * 5=最辛劳榜
     * 6=披星戴月卡
     * 7=持之以恒卡
     * 8=不辞辛劳卡
     */
    private var mPlaybillDetailType: String? = ""

    /**
     * 上报pagID
     */
    private var mPlaybillPid: String? = null

    /**
     * 上报aid
     */
    private var mPlaybillAid: String? = null

    private var mShareState: String? = null

    override fun getContentLayoutId(): Int = R.layout.activity_hr_urge_share_open

    override fun getStatusBarColor(): Int = R.color.color_cc000000

    override fun init() {
        if (intent == null) {
            finish()
            return
        }
        val routeUrl = RouteUtils.getRouteUrl(intent)
        if (TextUtils.isEmpty(routeUrl)) {
            finish()
            return
        }
        mId = RouteUtils.getQueryParameter(routeUrl, "playbillID", String::class.java)
        mShareImageUrl = RouteUtils.getQueryParameter(routeUrl, "shareImgUrl", String::class.java)
        if (TextUtils.isEmpty(mShareImageUrl)) {
            finish()
            return
        }
        mPlaybillType = RouteUtils.getQueryParameter(routeUrl, "playbillType", String::class.java)
        mPlaybillDetailType =
            RouteUtils.getQueryParameter(routeUrl, "playbillDetailType", String::class.java)
        mPlaybillPid = RouteUtils.getQueryParameter(routeUrl, "playbillPid", String::class.java)
        mPlaybillAid = RouteUtils.getQueryParameter(routeUrl, "playbillAid", String::class.java)
        mShareState = RouteUtils.getQueryParameter(routeUrl, "shareState", String::class.java)
        DialogUtils.getHrUrgeShareDialog(
            this,
            mPlaybillPid,
            mPlaybillAid,
            mShareState == "true",
            mId,
            mPlaybillDetailType,
            mShareImageUrl,
            object : IntegralShareListener {
                override fun onSuccess() {
                    OkBus.getInstance().onEvent(CommonEventTags.FLUSH_COMMON_WEBVIEW_PAGE)
                }
            });
    }

    override fun finish() {
        super.finish()
        overridePendingTransition(R.anim.fade_in, R.anim.fade_out)
    }
}