package com.lagou.lagouee.ui.adapter

import android.content.Context
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.appcompat.widget.LinearLayoutCompat
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.tools.SystemUtils
import com.common.adapter.viewholder.BaseVHAdapter
import com.common.adapter.viewholder.BaseViewHolder
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.chat.chatlibrary.view.CircleImageView
import com.lagou.commonlibs.engine.ThreadUtil
import com.lagou.commonlibs.imageload.ImageLoadUtils
import com.lagou.commonlibs.inter.OnSingleMethodListener
import com.lagou.commonlibs.inter.OnSingleMethodNoParameterListener
import com.lagou.commonlibs.inter.OnSingleTwoMdPsListener
import com.lagou.commonlibs.utils.ListUtil
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.utils.StringUtil
import com.lagou.lagouee.bean.jsonbean.aigc.AIGCReleaseRes
import com.lagou.lagouee.manager.MineManager
import com.lagou.lagouee.widget.ItemAIGCReleaseChangeView
import com.lagou.socketchannel.http.event.HttpEventResponse

/**
 * 发布职位AIGC 生成文本适配器
 * by codemanwang 2023.07.10
 */
class AIGCReleaseAdapter(context: Context?, dialogCallback: OnSingleMethodListener<Int>) : BaseVHAdapter<AIGCReleaseRes>(context) {

    /***
     * ItemView的的类型
     */
    val GENERATE_TYPE = 1
    val FIX_TYPE = 2

    /***
     * 开启新问题类型
     */
    val OPEN_TYPE = 3

    /**
     * 使用该描述回调到目标方（比如"发布职位页"）
     */
    var useCallback: OnSingleTwoMdPsListener<AIGCReleaseRes, AIGCReleaseRes>? = null

    /***
     * 使用描述、复制按钮回调事件，回调到弹窗页
     */
    var dialogCallback: OnSingleMethodListener<Int>? = null
    val DP_10 = ResourceUtils.getDimenSize(R.dimen.dp_10)
    val DP_20 = ResourceUtils.getDimenSize(R.dimen.dp_20)

    init {
        this.dialogCallback = dialogCallback
    }

    override fun setData(isFirst: Boolean, mBeans: MutableList<AIGCReleaseRes>?) {
        //移除为空的成员
        super.setData(isFirst, mBeans?.filterNotNull())
    }

    /***
     * AIGC生成内容显示新增和更新（招呼语、用户提、正常生成、异常文案等）
     */
    fun handleAiContent(aigcReleaseRes: AIGCReleaseRes?){
        //新的生成内容
        aigcReleaseRes?.apply {
            if (mBeans == null){
                mBeans = mutableListOf()
            }
            val index = mBeans.indexOf(this)
            //1 不包含aigcReleaseRes，直接新增
            if (index == -1){
                mBeans.add(aigcReleaseRes)
                notifyItemInserted(mBeans.size - 1)
                return
            }

            //2 包含aigcReleaseRes，根据isAppend的状态来决定是追加还是其它操作
            val currentAIGCReleaseRes = mBeans[index]
            currentAIGCReleaseRes.updateAigcContent(this)
            notifyItemChanged(index, 0)
        }
    }

    /**
     * AIGC换个问题卡片显示处理
     * 处理下是否显示换个问题，在adapter内实现该方法，前提：换个问题显示在每次AIGC生成结束之后
     * 1 未点击过换个问题时，只在AIGC生成结束之后追加一个换个问题的item
     * 2 点击过换个问题，换个问题卡片保留，在最后一行追加更换问题卡片
     */
    fun handleAiChangeQuestion(aigcReleaseChange: AIGCReleaseRes) {
        if (ListUtil.isEmpty(mBeans)){
            return
        }
        //1 先移除现有其它默认状态的换个问题的数据
        val originalIterator = mBeans.iterator()
        var index = 0
        while (originalIterator.hasNext()){
            val aigcReleaseRes = originalIterator.next()
            if (aigcReleaseRes.isShowOpenDefault){
                originalIterator.remove()
                notifyItemRemoved(index)
            }
            index++
        }

        //2 在列表的最后追加，更换问题的数据体
        mBeans.add(aigcReleaseChange)
        notifyItemInserted(mBeans.size - 1)
        notifyDataSetChanged()
    }

    /**
     * 发送文案之后，立马移除换个问题
     */
    fun removeAiChangeQuestionLast(){
        if (ListUtil.isEmpty(mBeans)){
            return
        }
        val lastIndex = mBeans.size - 1
        val aigcReleaseRes = mBeans[lastIndex]
        aigcReleaseRes?.apply {
            if (isShowOpenDefault){
                mBeans.removeAt(lastIndex)
                notifyItemRemoved(lastIndex)
            }
        }
    }

    /**
     * 异常中断或者关闭链接，导致接口回传的数据为空，更新Ai列表中最后一项未完成状态
     */
    fun updateAiContentLast(){
        mBeans?.apply {
            val size = mBeans.size
            if (size == 0){
                return
            }
            val currentAIGCReleaseRes = mBeans[size - 1] ?: return
            currentAIGCReleaseRes.code = HttpEventResponse.State.FINISH
            notifyItemChanged(size - 1)
        }
    }

    /**
     * 获取要所有生成文本的上下文
     */
    fun getContextIds(): String? {
        if (ListUtil.isEmpty(mBeans)){
            return ""
        }
        val contextList = mutableListOf<String>()
        for(index in mBeans.indices.reversed()){
            val aigcBean = mBeans[index]
            //倒序遍历&遇到分线结束
            if (aigcBean.isShowOpenClick){
                break
            }
            if (!TextUtils.isEmpty(aigcBean.contextId)){
                contextList.add(0, aigcBean.contextId)
            }
        }
        return StringUtil.jointString(",", contextList)
    }

    /***
     * 是否有提问过问题，埋点使用
     */
    fun hasSendQuestion(): Boolean {
        mBeans?.filter { it?.contentType == AIGCReleaseRes.ContentType.INPUT_TYPE}?.size?.let {
            if (it > 0){
                return true
            }
        }
        return false
    }

    override fun getItemViewType(position: Int): Int {
        if (!ListUtil.isListIndexLegal(mBeans, position)) {
            return FIX_TYPE
        }

        if (mBeans[position].isGenerateType) {
            return GENERATE_TYPE
        }

        if (mBeans[position].isShowOpenDefault || mBeans[position].isShowOpenClick){
            return OPEN_TYPE
        }

        return FIX_TYPE
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {

        if (viewType == GENERATE_TYPE) {
            val view = LayoutInflater.from(mContext)
                .inflate(R.layout.adapter_aigc_release_generate_content, parent, false)
            return GenerateViewHolder(view)
        }

        if (viewType == OPEN_TYPE){
            val view = ItemAIGCReleaseChangeView(mContext)
            val groupParams = ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT)
            view.layoutParams = groupParams
            return OpenViewHolder(view)
        }

        val viewFix = LayoutInflater.from(mContext)
            .inflate(R.layout.adapter_aigc_release_fix_content, parent, false)
        return FixViewHolder(viewFix)
    }

    override fun onBindViewHolder(
        holder: RecyclerView.ViewHolder,
        position: Int,
        payloads: MutableList<Any?>
    ) {
        if (holder is GenerateViewHolder) {
            holder.onBindViewHolder(
                holder.itemView,
                getItem(position) as AIGCReleaseRes,
                position
            )
            return
        }
        if (holder is FixViewHolder) {
            holder.onBindViewHolder(
                holder.itemView,
                getItem(position) as AIGCReleaseRes,
                position
            )
            return
        }
        if (holder is OpenViewHolder){
            holder.onBindViewHolder(
                holder.itemView,
                getItem(position) as AIGCReleaseRes,
                position
            )
            return
        }

        super.onBindViewHolder(holder, position, payloads)
    }


    /***
     * 生成式viewholder
     */
     inner class GenerateViewHolder(v: View) : BaseViewHolder<AIGCReleaseRes>(v) {

        private val linAiGenerateContent = v.findViewById<LinearLayoutCompat>(R.id.lin_ai_generate_content)
        private val imageAiGenerateLoading = v.findViewById<ImageView>(R.id.image_ai_loading)
        private val tvAiGenerateContent = v.findViewById<TextView>(R.id.tv_ai_generate_content)
        private val tvAiGenerateTips = v.findViewById<TextView>(R.id.tv_ai_generate_tips)
        private val vmAiGenerateLine = v.findViewById<View>(R.id.vm_ai_generate_btn_line)
        private val linAiGenerateBtn = v.findViewById<LinearLayoutCompat>(R.id.lin_ai_generate_btn)
        private val tvAiGenerateBtnLeft = v.findViewById<TextView>(R.id.tv_ai_generate_btn_left)
        private val tvAiGenerateBtnRight = v.findViewById<TextView>(R.id.tv_ai_generate_btn_right)
        private val imageHead = v.findViewById<CircleImageView>(R.id.image_ai_head)

        override fun onBindViewHolder(view: View, obj: AIGCReleaseRes, position: Int) {
            if (obj.isRobotRole){
                imageHead.setImageResource(R.drawable.ai_im_head)
                //Ai和自己背景色不一样
                linAiGenerateContent.setBackgroundResource(R.drawable.bg_ai_msg_left)
            }else {
                ImageLoadUtils
                    .with(mContext)
                    .load(MineManager.getInstance().userImageUrl)
                    .error(R.drawable.icon_login_head)
                    .into(imageHead)
                linAiGenerateContent.setBackgroundResource(R.drawable.bg_chat_msg_left)
            }

            linAiGenerateContent.setPadding(0, DP_10, 0, 0)
            val linContentParams = linAiGenerateContent.layoutParams
            //1 loading状态
            if (obj.isLoading){
                imageAiGenerateLoading.visibility = View.VISIBLE
                tvAiGenerateContent.visibility = View.GONE
                tvAiGenerateTips.visibility = View.GONE
                vmAiGenerateLine.visibility = View.GONE
                linAiGenerateBtn.visibility = View.GONE
                linContentParams.width = ViewGroup.LayoutParams.MATCH_PARENT
                linContentParams.height = ViewGroup.LayoutParams.WRAP_CONTENT
                linAiGenerateContent.layoutParams = linContentParams
                ImageLoadUtils
                    .with(mContext)
                    .load(R.drawable.ai_loading)
                    .asGif()
                    .into(imageAiGenerateLoading)
                return
            }

            linContentParams.width = ViewGroup.LayoutParams.WRAP_CONTENT
            linContentParams.height = ViewGroup.LayoutParams.WRAP_CONTENT
            linAiGenerateContent.layoutParams = linContentParams
            tvAiGenerateContent.text = obj.content
            //2 生成中页面样式
            if (!obj.isFinishMoe || !obj.isGenerateType) {
                imageAiGenerateLoading.visibility = View.GONE
                tvAiGenerateContent.visibility = View.VISIBLE
                tvAiGenerateTips.visibility = View.GONE
                vmAiGenerateLine.visibility = View.GONE
                linAiGenerateBtn.visibility = View.GONE
                return
            }

            //3 生成结束
            imageAiGenerateLoading.visibility = View.GONE
            tvAiGenerateContent.visibility = View.VISIBLE
            tvAiGenerateTips.visibility = View.VISIBLE
            vmAiGenerateLine.visibility = View.VISIBLE
            linAiGenerateBtn.visibility = View.VISIBLE
            tvAiGenerateBtnLeft.setOnClickListener {
                //调用复制到剪贴板方法
                SystemUtils.copyClipboard(view.context, tvAiGenerateContent.text as String?)
                //dialogCallback?.onCallBack(obj)
                useCallback?.onLeftCallBack(obj)
                //复制点击买点
                AnalyticHelper.getInstance().sendClickInfo("i1uam", "p1uas")
            }
            tvAiGenerateBtnRight.setOnClickListener {
                //使用该描述，触发回调覆盖到发布职位职位描述、职位名称、工作经验、学历、薪资范围等
                useCallback?.onRightCallBack(obj)
                //使用该描述点击回调到弹窗
                dialogCallback?.onCallBack(position)
            }
        }
    }

    /***
     * 固定式viewholder
     */
   inner class FixViewHolder(v: View) : BaseViewHolder<AIGCReleaseRes>(v) {
        private val tvAiFixContent = v.findViewById<TextView>(R.id.tv_ai_fix_content)
        private val imageHead = v.findViewById<CircleImageView>(R.id.image_ai_head)

        override fun onBindViewHolder(view: View, obj: AIGCReleaseRes, position: Int) {
            val tvLayoutParams = tvAiFixContent.layoutParams
            tvLayoutParams.height = ViewGroup.LayoutParams.WRAP_CONTENT
            tvAiFixContent.text = obj.content
            if (obj.isRobotRole){
                imageHead.setImageResource(R.drawable.ai_im_head)
                tvAiFixContent.setBackgroundResource(R.drawable.bg_ai_msg_left)
                tvLayoutParams.width = ViewGroup.LayoutParams.MATCH_PARENT
            }else {
                ImageLoadUtils
                    .with(mContext)
                    .load(MineManager.getInstance().userImageUrl)
                    .error(R.drawable.icon_login_head)
                    .into(imageHead)
                tvAiFixContent.setBackgroundResource(R.drawable.bg_chat_msg_left)
                tvLayoutParams.width = ViewGroup.LayoutParams.WRAP_CONTENT
            }

            tvAiFixContent.layoutParams = tvLayoutParams
            tvAiFixContent.setPadding(DP_20, DP_10, DP_10, DP_10)
        }
    }

    inner class OpenViewHolder(v: View?) : BaseViewHolder<AIGCReleaseRes>(v) {
        private val aigcReleaseChangeView = v as ItemAIGCReleaseChangeView

        override fun onBindViewHolder(view: View?, obj: AIGCReleaseRes, position: Int) {
            aigcReleaseChangeView.fillUiData(obj)
            aigcReleaseChangeView.setOpenCallback(object : OnSingleMethodNoParameterListener{
                override fun onCallBack() {
                    //更换问题点击，需要更新当前实体数据
                    obj?.setShowOpenQuestionClick()
                }
            })
        }
    }
}



