package com.lagou.lagouee.ui.adapter

import android.app.Activity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import android.widget.RelativeLayout
import android.widget.TextView
import androidx.appcompat.widget.LinearLayoutCompat
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.tools.IntentUtils
import com.common.adapter.BaseRecycleAdapter
import com.common.adapter.BaseViewHolder
import com.common.adapter.listener.OnItemClick
import com.common.widget.drawable.CouponXfermodeDrawable
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.utils.SafeTypeConvert
import com.lagou.lagouee.bean.jsonbean.business.CouponReceiveBean

/**
 * @date： Crate by wangjunwei on 2023/12/14
 * @description：首页优惠券弹窗适配器
 **/
class CouponReceiveAdapter(activity: Activity?) :
    BaseRecycleAdapter<CouponReceiveBean.CouponVoListBean, CouponReceiveAdapter.CouponReceiveViewHolder>(
        activity
    ) {

    override fun onCreateViewHolder(p0: ViewGroup, p1: Int): RecyclerView.ViewHolder {
        val rootView =
            LayoutInflater.from(mActivity).inflate(R.layout.adapter_coupon_receive, p0, false)
        return CouponReceiveViewHolder(rootView)
    }

    inner class CouponReceiveViewHolder(itemView: View) :
        BaseViewHolder<CouponReceiveBean.CouponVoListBean>(itemView) {

        private val linLeft = itemView.findViewById<LinearLayoutCompat>(R.id.linear_left)
        private val tvAmount = itemView.findViewById<TextView>(R.id.tv_amount)
        private val tvUnit = itemView.findViewById<TextView>(R.id.tv_unit)
        private val tvType = itemView.findViewById<TextView>(R.id.tv_type)

        private val relRight = itemView.findViewById<RelativeLayout>(R.id.rel_right)
        private val tvPosition = itemView.findViewById<TextView>(R.id.tv_position)
        private val tvValidityTime = itemView.findViewById<TextView>(R.id.tv_validity_time)
        private val tvGoUse = itemView.findViewById<TextView>(R.id.tv_go_use)

        override fun onBindViewHolder(
            view: View,
            couponVoListBean: CouponReceiveBean.CouponVoListBean?,
            position: Int,
            onItemClick: OnItemClick<CouponReceiveBean.CouponVoListBean>?
        ) {
            //1 设置背景色
            var leftColor = SafeTypeConvert.toColor(couponVoListBean?.bgColor)
            val relColor = ResourceUtils.getColor(R.color.white)
            if (leftColor == -1){
                leftColor = ResourceUtils.getColor(R.color.color_f67618)
            }
            linLeft.background = CouponXfermodeDrawable(leftColor, CouponXfermodeDrawable.XfermodeOrientation.RIGHT)
            relRight.background = CouponXfermodeDrawable(relColor, CouponXfermodeDrawable.XfermodeOrientation.LEFT)

            couponVoListBean?.apply {
                tvAmount.text = showAmount
                tvUnit.text = unit
                tvType.text = showMeetAmount()
                tvPosition.text = couponName
                tvValidityTime.text = showUseEndDate

                tvGoUse.setOnClickListener {
                    IntentUtils.startLagouWebViewActivity(mActivity, jumpUrl)
                    onItemClick?.onClick(couponVoListBean, position)
                }
            }
        }

        override fun getTitle(): String {
            return "多优惠券领取弹窗"
        }
    }
}