package com.lagou.lagouee.ui.adapter

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.lagou.commonlibs.inter.OnSingleMethodListener
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.lagouee.bean.PositionExposureInfoBean
import kotlinx.android.extensions.LayoutContainer
import kotlinx.android.synthetic.main.item_position_exposure_duration.*

/**
 * Created by mihao on 11/29/22
 * 强力曝光时长选择适配器
 */
class PositionExposureDurationAdapter(
    private val mContext: Context,
    private val mDurationList: List<PositionExposureInfoBean.Durations>
) : RecyclerView.Adapter<PositionExposureDurationAdapter.ViewHolder>() {

    private var onCheckListener: OnSingleMethodListener<PositionExposureInfoBean.Durations>? = null

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder =
        ViewHolder(
            LayoutInflater.from(mContext)
                .inflate(R.layout.item_position_exposure_duration, parent, false)
        )

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        mDurationList[position].let {
            holder.iv_label.visibility = if (it.isDefault) View.VISIBLE else View.INVISIBLE
            holder.tv_exposure_duration.apply {
                text = it.durationText()
                setTextColor(
                    if (it.isSelect) ResourceUtils.getColor(R.color.color_bc6600)
                    else ResourceUtils.getColor(R.color.color_333333)
                )
                setBackgroundResource(
                    if (it.isSelect) R.drawable.shape_fbf5ef_stroke05_a47b52_r2
                    else R.drawable.shape_tran_stroke05_d9d9d9_r2
                )
                setOnClickListener(View.OnClickListener { view ->
                    if (it.isSelect) {
                        return@OnClickListener
                    }
                    for (duration in mDurationList) {
                        duration.isSelect = false
                    }
                    it.isSelect = true
                    notifyDataSetChanged()
                    onCheckListener?.onCallBack(it)
                })
            }
            holder.iv_exposure_check.visibility =
                if (it.isSelect) View.VISIBLE else View.GONE
        }
    }

    override fun getItemCount(): Int {
        return if (mDurationList.isNullOrEmpty()) 0 else mDurationList.size
    }

    class ViewHolder(override val containerView: View) : RecyclerView.ViewHolder(containerView), LayoutContainer

    fun setOnItemCheckListener(onCheckListener: OnSingleMethodListener<PositionExposureInfoBean.Durations>?) {
        this.onCheckListener = onCheckListener
    }

}