package com.lagou.lagouee.ui.adapter

import android.app.Activity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.CheckBox
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.common.adapter.BaseRecycleAdapter
import com.common.adapter.BaseViewHolder
import com.common.adapter.listener.OnItemClick
import com.common.widget.CustomTextWatcher
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.view.ClearEditText
import com.lagou.commonlibs.view.filter.EmojiFilter
import com.lagou.lagouee.bean.position.RefuseCompeteFeedbackRes

/**
 * @date： Crate by wangjunwei on 2024/1/11
 * @description：竞招职位拒绝购买反馈适配器
 **/
class RefuseCompetePayFeedbackAdapter(activity: Activity?) : BaseRecycleAdapter<RefuseCompeteFeedbackRes.ContentBean,
        BaseViewHolder<RefuseCompeteFeedbackRes.ContentBean>>(activity) {


    override fun onCreateViewHolder(rootView: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        if (viewType == R.layout.adapter_refuse_compete_other){
            val otherView = LayoutInflater.from(mActivity).inflate(R.layout.adapter_refuse_compete_other, rootView, false)
            return RefuseCompetePayOtherHolder(otherView)
        }

        val commonView = LayoutInflater.from(mActivity).inflate(R.layout.adapter_refuse_compete_common, rootView, false)
        return RefuseCompetePayCommonHolder(commonView)
    }

    override fun getItemViewType(position: Int): Int {
        if (mList.isNullOrEmpty()){
            return R.layout.adapter_refuse_compete_common
        }

        if (position == mList.size - 1){
            return R.layout.adapter_refuse_compete_other
        }

        return R.layout.adapter_refuse_compete_common
    }

    /***
     * 获取被选中的原因
     */
    fun getSelectBean(): List<RefuseCompeteFeedbackRes.ContentBean>?{
        if (mList.isNullOrEmpty()){
            return null
        }
        return mList.filter { it.isChecked }
    }

    inner class RefuseCompetePayCommonHolder(itemView: View) :
        BaseViewHolder<RefuseCompeteFeedbackRes.ContentBean>(itemView) {

        private val checkBox: CheckBox = itemView as CheckBox

        override fun onBindViewHolder(
            view: View,
            contentBean: RefuseCompeteFeedbackRes.ContentBean?,
            position: Int,
            onItemClick: OnItemClick<RefuseCompeteFeedbackRes.ContentBean>?
        ) {
            checkBox.text = contentBean?.decription
            checkBox.setOnCheckedChangeListener { buttonView, isChecked -> //选中和取消选中
                contentBean?.isChecked = isChecked
                onItemClick?.onClick(contentBean, position)
            }
        }

        override fun getTitle(): String {
            return "竞招职位拒绝购买反馈适配器-普通"
        }
    }

    inner class RefuseCompetePayOtherHolder(itemView: View) :
        BaseViewHolder<RefuseCompeteFeedbackRes.ContentBean>(itemView) {

        private val checkBox: CheckBox = itemView.findViewById(R.id.cb_check)
        private val editContent: ClearEditText = itemView.findViewById(R.id.edit_content)

        override fun onBindViewHolder(
            view: View,
            contentBean: RefuseCompeteFeedbackRes.ContentBean?,
            position: Int,
            onItemClick: OnItemClick<RefuseCompeteFeedbackRes.ContentBean>?
        ) {
            checkBox.text = ResourceUtils.getString(R.string.other_reason)
            editContent.filters = arrayOf(EmojiFilter())
            checkBox.setOnCheckedChangeListener { buttonView, isChecked -> //选中和取消选中
                contentBean?.isChecked = isChecked
                onItemClick?.onClick(contentBean, position)
            }
            editContent.addTextChangedListener(object : CustomTextWatcher() {
                override fun onTextChanged(s: CharSequence, start: Int, before: Int, count: Int) {
                    super.onTextChanged(s, start, before, count)
                    checkBox.isChecked = s.isNotEmpty()
                    //选中和取消选中
                    contentBean?.isChecked = checkBox.isChecked
                    contentBean?.decription = s?.toString()
                    onItemClick?.onClick(contentBean, position)
                }
            })
        }

        override fun getTitle(): String {
            return "竞招职位拒绝购买反馈适配器-其它"
        }
    }

}