package com.lagou.lagouee.ui.business

import android.graphics.Rect
import android.text.SpannableStringBuilder
import android.view.View
import android.widget.LinearLayout
import android.widget.TextView
import androidx.recyclerview.widget.GridLayoutManager
import butterknife.OnClick
import com.alpha.lagouapk.R
import com.alpha.lagouapk.net.HttpResult
import com.alpha.lagouapk.observer.EventTags
import com.common.widget.decoration.GridItemDecoration
import com.lagou.annotation.apt.RouterRule
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListenerImpl
import com.lagou.commonlibs.event.Event
import com.lagou.commonlibs.event.OkBus
import com.lagou.commonlibs.inter.OnSingleMethodListener
import com.lagou.commonlibs.utils.ActivityStackHelper
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.utils.VerticalCenterImageSpan
import com.lagou.commonlibs.web.router.RouteUtils
import com.lagou.component.ui.dialog.listener.LgDialogListener
import com.lagou.lagouee.bean.CreatePositionExposureBean
import com.lagou.lagouee.bean.PositionExposureInfoBean
import com.lagou.lagouee.manager.net.BusinessNetManager
import com.lagou.lagouee.observer.BEventTags
import com.lagou.lagouee.ui.HomeActivity
import com.lagou.lagouee.ui.adapter.PositionExposureDurationAdapter
import com.lagou.lagouee.ui.rights.RightsApplyActivity
import com.lagou.lagouee.ui.rights.RightsScene
import com.lagou.lagouee.utils.DialogUtils
import com.lagou.lagouee.utils.IntentUtils
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.android.synthetic.main.activity_position_exposure.*

/**
 * Created by mihao on 11/29/22
 * 职位强力曝光
 */
@RouterRule(PositionExposureActivity.ROUTER_URL)
class PositionExposureActivity : BaseActivity() {

    /**
     * 简招职位ID
     */
    private var mdsPositionId = 0

    /**
     * 选择的曝光时长
     */
    private var selectLength = 0

    /**
     * 选择的曝光时长消耗的权益数
     */
    private var selectConsumeCount = 0

    private var mPositionExposureInfoBean: PositionExposureInfoBean? = null

    private var mDurationList: ArrayList<PositionExposureInfoBean.Durations>? = null
    private var positionExposureDurationAdapter: PositionExposureDurationAdapter? = null

    /**
     * 购买强力曝光成功，重新发起强力曝光
     */
    private val buyPositionExposureEvent = Event { getPositionExposureInfo(true) }

    companion object {
        const val ROUTER_URL = "lagouhr://lagou.com/power_expose?mdsPositionID={mdsPositionID}"
        const val KEY_POSITION_ID = "position_id"
    }

    override fun getContentLayoutId() = R.layout.activity_position_exposure

    override fun init() {
        mdsPositionId = if (RouteUtils.matchRule(intent, ROUTER_URL)) {
            RouteUtils.getQueryParameter(intent, "mdsPositionID", Int::class.java)
        } else {
            intent.getIntExtra(KEY_POSITION_ID, 0)
        }
        initListener()
        initRecyclerView()
        getPositionExposureInfo(false)
    }

    private fun initListener() {
        title_bar.setLeftImgListener { finish() }
        wifi_view.setReLoadListener { getPositionExposureInfo(false) }
        OkBus.getInstance().register(
            BEventTags.PAY_SUCCESS_POSITION_EXPOSURE,
            buyPositionExposureEvent
        )
    }

    private fun initRecyclerView() {
        mDurationList = ArrayList()
        positionExposureDurationAdapter =
            PositionExposureDurationAdapter(baseActivity, mDurationList!!)
        positionExposureDurationAdapter!!.setOnItemCheckListener(OnSingleMethodListener {
            if (it != null) {
                selectLength = it.length
                selectConsumeCount = it.consumeCount
                updateRightsInfo()
            }
        })
        rv_exposure_time.adapter = positionExposureDurationAdapter
        rv_exposure_time.layoutManager = GridLayoutManager(baseActivity, 3)
        rv_exposure_time.addItemDecoration(
            GridItemDecoration(
                ResourceUtils.getDimenSize(R.dimen.dp_15),
                3
            )
        )
    }

    private fun getPositionExposureInfo(isPaySuccess: Boolean) {
        BusinessNetManager.getPositionExposureInfo(mdsPositionId, object :
            OnAdapterCommonListenerImpl<PositionExposureInfoBean, HttpResponse<PositionExposureInfoBean?>?>(
                this
            ) {
            override fun onSuccess(positionExposureInfo: PositionExposureInfoBean) {
                mPositionExposureInfoBean = positionExposureInfo
                if (!isPaySuccess) {
                    wifi_view.visibility = View.GONE
                    tv_expose_guide.text = positionExposureInfo.guideTips
                    tv_position_name.text = positionExposureInfo.positionName
                    tv_position_salary.text = positionExposureInfo.salary
                    tv_position_education.text = positionExposureInfo.education
                    setGoodsTips()
                    tv_exposure_time_subtitle.text = positionExposureInfo.durationTips
                    for (duration in positionExposureInfo.durations) {
                        if (duration == null) {
                            continue
                        }
                        if (duration.isDefault) {
                            selectLength = duration.length
                            selectConsumeCount = duration.consumeCount
                            duration.isSelect = true
                        }
                    }
                    mDurationList?.clear()
                    mDurationList?.addAll(positionExposureInfo.durations)
                    positionExposureDurationAdapter?.notifyDataSetChanged()
                } else {
                    positionExposureClick()
                }
                updateRightsInfo()
            }

            override fun onError(errorBean: HttpResponse<PositionExposureInfoBean?>?) {
                wifi_view.visibility = View.VISIBLE
            }

            override fun onNoInternet() {
                wifi_view.visibility = View.VISIBLE
            }
        })
    }

    private fun setGoodsTips() {
        ll_position_exposure_advantage_container.removeAllViews()
        var goodsTips =  mPositionExposureInfoBean?.goodsTips?.split("\n")
        if (goodsTips.isNullOrEmpty()){
            view_expose_card_divider.visibility = View.GONE
            ll_position_exposure_advantage_container.visibility = View.GONE
            return
        }
        view_expose_card_divider.visibility = View.VISIBLE
        ll_position_exposure_advantage_container.visibility = View.VISIBLE
        goodsTips.forEachIndexed { index, s ->
            val builder = SpannableStringBuilder()
            VerticalCenterImageSpan.inject(
                this, builder, R.drawable.exposure_check_ic, ResourceUtils.getDimenSize(R.dimen.dp_11), ResourceUtils.getDimenSize(R.dimen.dp_11),
                Rect(0, 0, ResourceUtils.getDimenSize(R.dimen.dp_3), 0)
            )
            builder.append(s)

            val params = LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT,LinearLayout.LayoutParams.WRAP_CONTENT)
            params.topMargin = if (index == 0) 0 else ResourceUtils.getDimenSize(R.dimen.dp_10)
            val textView = TextView(this).apply {
                textSize = 12f
                setTextColor(ResourceUtils.getColor(R.color.color_84512a))
                text = builder
                layoutParams = params
            }
            ll_position_exposure_advantage_container.addView(textView)
        }
    }

    private fun updateRightsInfo() {
        tv_exposure_rights_tips.text =
            mPositionExposureInfoBean?.getExposureRightsTips(selectConsumeCount)
        tv_exposure_bottom_left.visibility = View.GONE
        tv_exposure_bottom_right.visibility = View.VISIBLE
        if (!isBuyRights()) {
            tv_exposure_bottom_right.text =
                ResourceUtils.getString(R.string.position_exposure_create)
            return
        }
        if (mPositionExposureInfoBean?.isApplyGoods!!) {
            tv_exposure_bottom_left.visibility = View.VISIBLE
            tv_exposure_bottom_left.text =
                ResourceUtils.getString(R.string.apply_from_manager)
        }
        if (mPositionExposureInfoBean?.isBuyPlus!!) {
            tv_exposure_bottom_left.visibility = View.VISIBLE
            tv_exposure_bottom_left.text =
                ResourceUtils.getString(R.string.apply_lagou_plus_guide)
        }
        tv_exposure_bottom_right.text =
            ResourceUtils.getString(R.string.immediately_buy)
    }

    @OnClick(R.id.tv_exposure_bottom_right)
    fun positionExposureClick() {
        if (mPositionExposureInfoBean == null) {
            return
        }
        if (isBuyRights()) {
            IntentUtils.startLagouWebViewActivity(mContext, mPositionExposureInfoBean!!.buyUrl)
            return
        }

        BusinessNetManager.createPositionExposure(mdsPositionId, selectLength, object :
            OnAdapterCommonListenerImpl<CreatePositionExposureBean, HttpResponse<CreatePositionExposureBean?>?>(
                this
            ) {
            override fun onSuccess(createPositionExposureBean: CreatePositionExposureBean) {
                DialogUtils.showPositionExposureSuccessDialog(
                    baseActivity, createPositionExposureBean.remainRights,
                    object : LgDialogListener() {
                        override fun onRight() {
                            OkBus.getInstance().onEvent(EventTags.B_POSITION_FLUSH_ONLINE)
                            IntentUtils.startPositionMain(baseActivity)
                            ActivityStackHelper.clearAndFinishOtherActivity(HomeActivity::class.java)
                        }
                    })
            }

            override fun onError(errorBean: HttpResponse<CreatePositionExposureBean?>?) {
                if (errorBean?.state == HttpResult.POSITION_EXPOSURE_BUY_RIGHTS) {
                    IntentUtils.startLagouWebViewActivity(
                        mContext,
                        mPositionExposureInfoBean?.buyUrl
                    )
                }
            }
        })
    }

    @OnClick(R.id.tv_exposure_bottom_left)
    fun applyRightsClick() {
        if (mPositionExposureInfoBean == null) {
            return
        }
        if (mPositionExposureInfoBean!!.isBuyPlus) {
            IntentUtils.startLagouWebViewActivity(mContext, mPositionExposureInfoBean?.getGoodsUrl)
            return
        }
        if (mPositionExposureInfoBean!!.isApplyGoods) {
            IntentUtils.startRightsApplyActivity(
                mContext,
                RightsScene.POSITION_EXPOSE,
                RightsApplyActivity.NOT_NEED_ID.toLong()
            )
        }
    }

    private fun isBuyRights(): Boolean {
        if (mPositionExposureInfoBean?.remainRights is Int) {
            return selectConsumeCount > mPositionExposureInfoBean?.remainRights!!
        }
        return true
    }

    override fun onDestroy() {
        super.onDestroy()
        OkBus.getInstance().unRegister(buyPositionExposureEvent)
    }

}


