package com.lagou.lagouee.ui.candidate.detail

import android.content.Context
import android.os.Bundle
import android.text.TextUtils
import android.view.View
import android.widget.LinearLayout
import android.widget.TextView
import androidx.core.widget.NestedScrollView
import com.ApiService
import com.alpha.lagouapk.R
import com.blankj.utilcode.util.StringUtils
import com.common.integrate.bugly.BuglyHelper
import com.common.widget.NewExpandableTextView
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.BaseFragment
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.definition.b.BPageId
import com.lagou.baselib.net.OnUiRelativeListener
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.commonlibs.engine.ThreadUtil
import com.lagou.commonlibs.utils.*
import com.lagou.lagouee.bean.candidate.CandidateCountParamsBean
import com.lagou.lagouee.bean.jsonbean.problem.AnswerQuestionBean
import com.lagou.lagouee.bean.jsonbean.resume.ResumeState
import com.lagou.lagouee.bean.jsonbean.talent.ResumeCoreInfoBeanX
import com.lagou.lagouee.bean.jsonbean.talent.SimilarCandidateBean
import com.lagou.lagouee.bean.jsonbean.talent.TalentDetailBean
import com.lagou.lagouee.manager.net.TalentNetManager
import com.lagou.lagouee.ui.candidate.constants.CandidateConstants
import com.lagou.lagouee.ui.candidate.inter.ICandidateResumeListener
import com.lagou.lagouee.ui.candidate.inter.InvitePortfolioClickListener
import com.lagou.lagouee.ui.candidate.view.talent.*
import com.lagou.lagouee.ui.candidate.view.talent.BTalentWorkExpectView.OnSeeCompanyInfoListener
import com.lagou.lagouee.widget.QuotationMarkLayout
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse
import com.lagou.socketchannel.util.GsonUtil

/**
 * @author 作者：ryan.lei
 * @date 创建时间：2022/3/28 20:01
 * @Description 描述：人才详情在线简历
 **/
class CandidateOnlineResumeFragment : BaseFragment() {
    private var talentDetailBean: TalentDetailBean? = null

    /**
     * 埋点数据参数
     */
    private var mCountParams: CandidateCountParamsBean? = null

    private var onSeeCompanyInfoListener: OnSeeCompanyInfoListener? = null

    private var mIsChatInviteInterview: Boolean = false

    /**
     * C端用户id
     */
    private var mCUserId: Long = -1

    /**
     * 拉勾职位id
     */
    private var mPositionId: Int = 0

    /**
     * 用户求职期望职位id
     */
    private var mExpectJobId: String? = null

    /**
     * 给面试，面试状态
     */
    private var mChatInviteState: Int = -1

    /**
     * 用户名
     */
    private var mUserName: String = ""

    private var mCandidateResumeListener: ICandidateResumeListener? = null

    private var llUserExpectGroup: LinearLayout? = null
    private var tvExpectPositionCity: TextView? = null
    private var tvExpectJobSalary: TextView? = null
    private var tvExpectLabel: TextView? = null
    private var tvExpectJobCity: TextView? = null
    private var tvExpectJobTime: TextView? = null
    private var quotationExpectMark: QuotationMarkLayout? = null
    private var bTalentInvitePortfolio: BTalentInvitePortfolio? = null
    private var llAdviserComment: LinearLayout? = null
    private var lineAdviserComment: View? = null
    private var tvAdviserComment: TextView? = null
    private var llUserAdvantage: LinearLayout? = null
    private var lineUserAdvantage: View? = null
    private var tvUserAdvantageTitle: TextView? = null
    private var tvUserAdvantage: NewExpandableTextView? = null
    private var bTalentWorkExpectList: BTalentWorkExpectView? = null
    private var lineWorkExpect: View? = null
    private var bTalentProjectExpectList: BTalentProjectExpectView? = null
    private var lineProjectExpect: View? = null
    private var bTalentEducationExpectList: BTalentEducationExpectView? = null
    private var lineEducationExpect: View? = null
    private var bTalentSocialExpectList: BTalentSocialListView? = null
    private var lineSocialExpect: View? = null
    private var bTalentImageExpectList: BTalentImageExpectView? = null
    private var lineImageExpect: View? = null
    private var bTalentRecommendList: BTalentRecommendListView? = null
    private var svRoot: NestedScrollView? = null

    companion object {
        private const val TAG = "CandidateOnlineResumeFragment"
        private const val KEY_TALENT_DETAIL_BEAN = "talentDetailBean"
        private const val KEY_CANDIDATE_COUNT_PARAMS_BEAN = "candidateCountParamsBean"

        fun newInstance(
            talentDetailBean: TalentDetailBean,
            mCountParams: CandidateCountParamsBean
        ): CandidateOnlineResumeFragment {
            val bundle = Bundle()
            bundle.putString(KEY_TALENT_DETAIL_BEAN, GsonUtil.toJson(talentDetailBean))
            bundle.putSerializable(KEY_CANDIDATE_COUNT_PARAMS_BEAN, mCountParams)
            val fragment = CandidateOnlineResumeFragment()
            fragment.arguments = bundle
            return fragment
        }
    }

    override fun getLayoutId(): Int =
        R.layout.fragment_candidate_online_resume

    override fun onAttach(context: Context) {
        super.onAttach(context)
        val bundle = arguments
        bundle?.apply {
            // 线上异常 android.os.BadParcelableException: ClassNotFoundException when unmarshalling: at android.os.Parcel.readParcelableCreator(Parcel.java:3050)，
            // 页面在后台被回收，Parcelable序列化传递数据classLoader未发现导致。
            // 解决方案：暂时用Gson转换成json字符串传递数据，避免Parcelable序列化

            talentDetailBean = GsonUtil.fromJson(
                getString(KEY_TALENT_DETAIL_BEAN),
                TalentDetailBean::class.java
            )
            mCountParams =
                getSerializable(KEY_CANDIDATE_COUNT_PARAMS_BEAN) as CandidateCountParamsBean?
        }
    }

    override fun init() {
        initParams()
        initView()
        initData()
    }

    private fun initParams() {
        //解析bundle参数，尽量复用之前的逻辑
        mCountParams?.apply {
            mIsChatInviteInterview = isChatInviteInterview
            mCUserId = userId
            mPositionId = positionId
            mExpectJobId = expectJobId
            mUserName = userName
        }
    }

    private fun initView() {
        svRoot = findViewById(R.id.sv_root)
        llUserExpectGroup = findViewById(R.id.ll_user_expect_group)
        tvExpectPositionCity = findViewById(R.id.tv_expect_position_city)
        tvExpectJobSalary = findViewById(R.id.tv_expect_job_salary)
        tvExpectLabel = findViewById(R.id.tv_expect_label)
        tvExpectJobCity = findViewById(R.id.tv_expect_job_city)
        tvExpectJobTime = findViewById(R.id.tv_expect_job_time)
        quotationExpectMark = findViewById(R.id.quotation_expect_mark)
        bTalentInvitePortfolio = findViewById(R.id.b_talent_invite_portfolio)
        llAdviserComment = findViewById(R.id.ll_adviser_comment)
        lineAdviserComment = findViewById(R.id.line_adviser_comment)
        tvAdviserComment = findViewById(R.id.tv_adviser_comment)
        llUserAdvantage = findViewById(R.id.ll_user_advantage)
        lineUserAdvantage = findViewById(R.id.line_user_advantage)
        tvUserAdvantageTitle = findViewById(R.id.tv_user_advantage_title)
        tvUserAdvantage = findViewById(R.id.tv_user_advantage)
        bTalentWorkExpectList = findViewById(R.id.b_talent_work_expect_list)
        lineWorkExpect = findViewById(R.id.line_work_expect)
        bTalentProjectExpectList = findViewById(R.id.b_talent_project_expect_list)
        lineProjectExpect = findViewById(R.id.line_project_expect)
        bTalentEducationExpectList = findViewById(R.id.b_talent_education_expect_list)
        lineEducationExpect = findViewById(R.id.line_education_expect)
        bTalentSocialExpectList = findViewById(R.id.b_talent_social_expect_list)
        lineSocialExpect = findViewById(R.id.line_social_expect)
        bTalentImageExpectList = findViewById(R.id.b_talent_image_expect_list)
        lineImageExpect = findViewById(R.id.line_image_expect)
        bTalentRecommendList = findViewById(R.id.b_talent_recommend_list)
    }

    private fun initData() {
        //设置简历各模块信息
        talentDetailBean?.apply {
            //设置求职期望
            setUserExpectData(resumeCoreInfo)
            //人才详情邀请投递简历
            setTalentInviteResume(this, mCountParams)
            //顾问说
            setAdviserComment(this)
            //我的优势
            setUserAdvantageData(resumeCoreInfo)
            //工作经历
            bTalentWorkExpectList?.setCountParams(mCountParams)
            bTalentWorkExpectList?.registerOnSeeCompanyInfoListener(onSeeCompanyInfoListener)
            bTalentWorkExpectList?.updateView(this)
            //项目经历
            bTalentProjectExpectList?.setCountParams(mCountParams)
            bTalentProjectExpectList?.updateView(this)
            //教育信息
            bTalentEducationExpectList?.setCountParams(mCountParams)
            bTalentEducationExpectList?.updateView(this)
            //社交主页
            bTalentSocialExpectList?.setCountParams(mCountParams)
            bTalentSocialExpectList?.updateView(this)
            //作品展示
            bTalentImageExpectList?.setCountParams(mCountParams)
            bTalentImageExpectList?.updateView(this)
            //相似人才列表(猜你喜欢）
            setTalentRecommendList(mCountParams)

            setLayoutLineVisible(this)
        }
        //请求相似职位和chat
        requestRecommendAndChatData()
        //获取求职问答数据
        getProblemAnswer(mCUserId)
    }

    private fun requestRecommendAndChatData() {
        if (mIsChatInviteInterview) {
            getChatInviteState(mCUserId.toInt(), mPositionId)
        } else {
            //获取推荐人才列表
            getRecommendList(mCUserId, mExpectJobId, this)
        }
    }

    private fun setTalentRecommendList(countParams: CandidateCountParamsBean?) {
        bTalentRecommendList?.visibility =
            if (countParams == null || countParams.isChatInviteInterview) View.GONE else View.VISIBLE
    }

    /**
     * 设置求职期望
     */
    private fun setUserExpectData(mCoreInfo: ResumeCoreInfoBeanX?) {
        //求职期望模块默认先置为GONE
        llUserExpectGroup?.visibility = View.GONE
        if (mCoreInfo == null) {
            return
        }
        val mExpectJob = mCoreInfo.expectJobVO
        //根据mCoreInfo的值来判断是否展示求职期望模块,以及填充模块数据
        mExpectJob?.apply {
            //职位、薪资
            tvExpectJobSalary?.text = if (TextUtils.isEmpty(salaries)) "" else salaries
            //职能标签，此处有个开关，开关打开时采用新的职位名称及标签
            tvExpectLabel?.visibility =
                if (ListUtil.isEmpty<String>(positionTags)) View.GONE else View.VISIBLE
            tvExpectLabel?.text = StringUtil.jointString("  ", positionTags)
            //当开关打开时，如果newPositionType为空，则使用原来的期望作为兜底
            tvExpectPositionCity?.text =
                if (TextUtils.isEmpty(newPositionType)) positionName else newPositionType
            tvExpectJobCity?.text = city
            tvExpectJobCity?.visibility = if (TextUtils.isEmpty(city)) View.GONE else View.VISIBLE
            var arrivalTime: String = arrivalTime
            if (TextUtils.equals("--", arrivalTime)) {
                arrivalTime = ""
            }
            tvExpectJobTime?.text = StringUtil.jointString("，", status, arrivalTime)
            //如果三项都是空 则进行占位
            if (tvExpectPositionCity?.visibility != View.VISIBLE && tvExpectJobSalary?.visibility != View.VISIBLE && tvExpectJobTime?.visibility != View.VISIBLE) {
                llUserExpectGroup?.visibility = View.INVISIBLE
            } else {
                llUserExpectGroup?.visibility = View.VISIBLE
            }
        }
    }

    /**
     * 人才详情邀请投递简历
     */
    private fun setTalentInviteResume(
        talentDetailBean: TalentDetailBean?,
        countParams: CandidateCountParamsBean?
    ) {
        if (isResumeNotOwn(talentDetailBean) && countParams != null && !countParams.isChatInviteInterview) {
            if (talentDetailBean != null && talentDetailBean.isHasNearby) {
                AnalyticHelper.getInstance().sendShowInfo("fU00", "c1sy3")
                if (llAdviserComment?.visibility == View.VISIBLE) {
                    lineAdviserComment?.visibility = View.GONE
                    return
                }
                if (llUserAdvantage?.visibility == View.VISIBLE) {
                    lineUserAdvantage?.visibility = View.GONE
                    return
                }
                if (llAdviserComment?.visibility == View.GONE && llUserAdvantage?.visibility == View.GONE) {
                    val layoutParams =
                        bTalentInvitePortfolio?.layoutParams as LinearLayout.LayoutParams
                    layoutParams.bottomMargin = ResourceUtils.getDimenSize(R.dimen.dp_24)
                    bTalentInvitePortfolio?.layoutParams = layoutParams
                }
            }
        }
    }

    /**
     * 判断人才详情邀请投递简历模块是否有展示,有则需按最高优先级显示(此时不展示邀投作品集模块)
     *
     * @return
     */
    private fun isResumeNotOwn(talentDetailBean: TalentDetailBean?): Boolean {
        return talentDetailBean != null && talentDetailBean.resumeStageCode == ResumeState.STATE_RESUME_NOTOWN
    }

    /**
     * 顾问说
     */
    private fun setAdviserComment(talentDetailBean: TalentDetailBean?) {
        if (talentDetailBean == null || !talentDetailBean.isShowAdviserComment
            || TextUtils.isEmpty(talentDetailBean.adviserComment)
        ) {
            llAdviserComment?.visibility = View.GONE
            return
        }
        llAdviserComment?.visibility = View.VISIBLE
        tvAdviserComment?.text = talentDetailBean.adviserComment
    }

    /**
     * 我的优势
     */
    private fun setUserAdvantageData(coreInfo: ResumeCoreInfoBeanX?) {
        if (coreInfo == null || StringUtils.isEmpty(coreInfo.selfDescription)
            || StringUtils.isEmpty(BaseUtils.getHtmlStr(coreInfo.selfDescription))
        ) {
            llUserAdvantage?.visibility = View.GONE
            return
        }
        llUserAdvantage?.visibility = View.VISIBLE
        tvUserAdvantage?.text = BaseUtils.getHtmlStr(coreInfo.selfDescription)
    }

    /**
     * 设置分割线展示或隐藏
     */
    private fun setLayoutLineVisible(detailBean: TalentDetailBean?) {
        if (detailBean == null || detailBean.resumeCoreInfo == null) {
            svRoot?.visibility = View.GONE
            return
        }
        //没有工作经历隐藏分割线
        if (ListUtil.isEmpty(detailBean.resumeCoreInfo.workExperienceVOs)) {
            lineWorkExpect?.visibility = View.GONE
        }

        //没有工作经验 隐藏分割线
        if (ListUtil.isEmpty(detailBean.resumeCoreInfo.projectExperienceVOs)) {
            lineProjectExpect?.visibility = View.GONE
        }
        //没有教育经历 隐藏分割线
        if (ListUtil.isEmpty(detailBean.resumeCoreInfo.educationExperienceVOs)) {
            lineEducationExpect?.visibility = View.GONE
        }
        //没有社交内容、隐藏分割线
        if (ListUtil.isEmpty(detailBean.resumeCoreInfo.socialAccountVOs)) {
            lineEducationExpect?.visibility = View.GONE
            lineSocialExpect?.visibility = View.GONE
        }
        //没有图片作品隐藏、分割线
        if (ListUtil.isEmpty(detailBean.resumeCoreInfo.pictures)) {
            lineSocialExpect?.visibility = View.GONE
        }
    }

    fun registerOnSeeCompanyInfoListener(onSeeCompanyInfoListener: OnSeeCompanyInfoListener?) {
        this.onSeeCompanyInfoListener = onSeeCompanyInfoListener
    }

    fun unRegisterOnSeeCompanyInfoListener() {
        bTalentWorkExpectList?.unRegisterOnSeeCompanyInfoListener()
        onSeeCompanyInfoListener = null
    }

    /**
     * 获取相似人才列表
     */
    private fun getRecommendList(
        referUserId: Long,
        expectJobId: String?,
        listener: OnUiRelativeListener
    ) {
        val userId = SafeTypeConvert.toInt(referUserId.toString())
        if (userId == -1) {
            return
        }
        TalentNetManager.getSimilarCandidates(
            referUserId,
            expectJobId,
            object : BaseCallback<List<SimilarCandidateBean>?>() {
                override fun onSuccess(
                    request: HttpRequest<*>?,
                    response: HttpResponse<List<SimilarCandidateBean>?>?
                ) {
                    if (!isActive) {
                        return
                    }
                    if (response == null || response.content == null) {
                        return
                    }
                    setRecommendList(response.content)
                }
            })
    }

    /**
     * 设置相似人才列表数据
     */
    private fun setRecommendList(list: List<SimilarCandidateBean>?) {
        if (ListUtil.isEmpty(list)) {
            LogUtils.i(TAG, "设置推荐数据条件不成立 mSimilarCandidateList is empty")
            return
        }
        val activity = activity as BaseActivity?
        if (activity is CandidateDetailActivity && !activity.isEnableRecommendListClick) {
            //相似人才点击进入人才详情页面后不再展示相似人才推荐
            LogUtils.i(TAG, "相似人才点击进入人才详情页面后不再展示相似人才推荐")
            return
        }
        lineImageExpect?.visibility = View.VISIBLE
        bTalentRecommendList?.setCountParams(mCountParams)
        bTalentRecommendList?.updateView(list)
    }

    /**
     * 给面试，面聊状态
     *
     * @param positionId
     */
    private fun getChatInviteState(cUserId: Int, positionId: Int) {
        if (cUserId <= 0) {
            return
        }
        ApiService.api.fetchChatInviteState(positionId.toLong(), cUserId)
            .enqueue(object : BaseCallback<Int>() {
                override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<Int>?) {
                    if (!isActive) {
                        return
                    }
                    if (response == null || response.content == null) {
                        return
                    }
                    mCandidateResumeListener?.fetchChatInviteState(response.content)
                }
            })
    }

    fun setCandidateResumeListener(listener: ICandidateResumeListener) {
        mCandidateResumeListener = listener
    }

    private fun getProblemAnswer(cUserId: Long) {
        ApiService.api.getProblemAnswer(cUserId)
            .enqueue(object : BaseCallback<AnswerQuestionBean>() {
                override fun onSuccess(
                    request: HttpRequest<*>?,
                    response: HttpResponse<AnswerQuestionBean>?
                ) {
                    LogUtils.i(TAG, "getProblemAnswer : onAdapterSuccess ")
                    if (!isActive) {
                        return
                    }
                    if (response == null || response.content == null) {
                        return
                    }
                    val bean = response.content
                    LogUtils.i(TAG, "getProblemAnswer : onAdapterSuccess bean = $bean")
                    //人才卡片内部存在 recyclerview 绘制需要时间所以此处刷新求职问答 需要进行 post 到消息队列中
                    ThreadUtil.ui {
                        updateProblem(bean)
                    }
                }

            })
    }

    private fun updateProblem(questionListBean: AnswerQuestionBean?) {
        LogUtils.i(TAG, "updateProblem() questionListBean = $questionListBean")
        if (questionListBean == null || ListUtil.isEmpty(questionListBean.questionList)) {
            return
        }
        try {
            updateProblemList(questionListBean)
        } catch (e: Exception) {
            e.printStackTrace()
            BuglyHelper.postCaughtException(e)
        }
    }

    //更新求职问答展示
    private fun updateProblemList(questionListBean: AnswerQuestionBean) {
        val listBeans = questionListBean.questionList
        var bean: AnswerQuestionBean.QuestionListBean?
        for (i in listBeans.indices) {
            bean = listBeans[i]
            if (bean == null || StringUtils.isEmpty(bean.userAnswer)) {
                continue
            }
            val questionScore = bean.questionScore
            if (questionScore == CandidateConstants.QuestionShowType.USER_CORE) {    //求职意向
                quotationExpectMark?.updateProblem(bean)
            } else if (questionScore == CandidateConstants.QuestionShowType.USER_WORK) {  //工作经历
                bTalentWorkExpectList?.updateProblem(bean)
            } else if (questionScore == CandidateConstants.QuestionShowType.USER_EDUCATION) { //教育经历
                bTalentEducationExpectList?.updateProblem(bean)
            }
        }
    }
}