package com.lagou.lagouee.ui.candidate.viewholder

import android.content.Context
import android.text.TextUtils
import android.view.View
import android.widget.ImageView
import android.widget.TextView
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.tools.IntentUtils
import com.lagou.commonlibs.imageload.ImageLoadUtils
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.lagouee.bean.SuperFilterConfigBean

/**
 * B端高级筛选
 */
class SuperFilterHolder(val context: Context, itemView: View): RecyclerView.ViewHolder(itemView){
    private val frequencyOfChangeJobView = arrayOf<TextView>(itemView.findViewById(R.id.tv_frequency_all),itemView.findViewById(R.id.tv_frequency_53),itemView.findViewById(R.id.tv_frequency_1))
    private val recent30DaysActionView = arrayOf<TextView>(itemView.findViewById(R.id.tv_30day_all),itemView.findViewById(R.id.tv_30day_no_browse),itemView.findViewById(R.id.tv_30day_no_chat))
    private var mIconView:ImageView = itemView.findViewById(R.id.iv_super_filter_type_logo)
    private var mDescView:TextView = itemView.findViewById(R.id.tv_super_filter_type_desc)
    private var mMaskView:ConstraintLayout = itemView.findViewById(R.id.cl_super_filter_mask_container)
    private var mMaskDescView:TextView = itemView.findViewById(R.id.tv_mask_super_filter_type_desc)
    private var mMaskOpen:TextView = itemView.findViewById(R.id.tv_mask_super_filter_type_open)
    private var mSuperFilterConfigBean:SuperFilterConfigBean? = null
    private var mSelectFrequencyChangeJobList:ArrayList<String>? = null
    private var mSelectRecentActionList:ArrayList<String>? = null
    private var closeFilterWindowListener: (() -> Unit)? = null
    companion object{
        val OPTIONS_FREQUENCY: Array<String> = ResourceUtils.getStringArray(R.array.frequency_change_job_options)
        val OPTIONS_RECENT_ACTION: Array<String> = ResourceUtils.getStringArray(R.array.recent_30days_action_options)
        const val TYPE_FREQUENCY_CHANGE_JOB = 1
        const val TYPE_RECENT_ACTION = 2
    }
    init {
        frequencyOfChangeJobView.forEachIndexed { index, textView ->
            textView.text = OPTIONS_FREQUENCY.getOrNull(index)
            textView.setOnClickListener(OnItemCLickListener(TYPE_FREQUENCY_CHANGE_JOB))
        }

        recent30DaysActionView.forEachIndexed { index, textView ->
            textView.text = OPTIONS_RECENT_ACTION.getOrNull(index)
            textView.setOnClickListener(OnItemCLickListener(TYPE_RECENT_ACTION))
        }

        mMaskOpen.setOnClickListener {
            closeFilterWindowListener?.invoke()
            IntentUtils.startLagouWebViewActivity(context,mSuperFilterConfigBean?.fbtnJumpUrl)
        }
    }

    inner class OnItemCLickListener(val type:Int):View.OnClickListener{

        override fun onClick(v: View?) {
            if (mSuperFilterConfigBean?.canPlusFilter == false){
                //不可使用高级筛选
                mMaskView.visibility = View.VISIBLE
                return
            }
            v?.isSelected = (v?.isSelected == true).not()
            mMaskView.visibility = View.GONE
            if (type == TYPE_FREQUENCY_CHANGE_JOB){
                onSelect(v as TextView,mSelectFrequencyChangeJobList,frequencyOfChangeJobView)
            }else if (type == TYPE_RECENT_ACTION){
                onSelect(v as TextView,mSelectRecentActionList,recent30DaysActionView)
            }
        }
    }

    fun onSelect(textView: TextView,selectList:ArrayList<String>?,itemViews:Array<TextView>){
        val noLimitStr = ResourceUtils.getString(R.string.search_filter_any)
        val isClickNoLimit = TextUtils.equals(textView.text.toString(),noLimitStr)
        if (isClickNoLimit){
            //点击的不限
            selectList?.clear()
            itemViews.filter { it.text != noLimitStr}.forEach { it.isSelected = false }
        }else{
            removeSelectSpec(ResourceUtils.getString(R.string.search_filter_any),selectList)
            itemViews.find { it.text == noLimitStr}?.isSelected = false
        }

        if (textView.isSelected){
            selectList?.add(textView.text.toString())
        }else{
            removeSelectSpec(textView.text.toString(),selectList)
        }
    }

    private fun removeSelectSpec(removeStr:String,selectList:ArrayList<String>?){
        val iterator = selectList?.iterator()
        while (iterator?.hasNext() == true){
            val text = iterator.next()
            if (TextUtils.equals(text,removeStr)){
                iterator.remove()
                return
            }
        }
    }

    fun onFilterWindowClosed(){
        mMaskView.visibility = View.GONE
    }

    fun bindSuperFilter(selectFrequencyChangeJob:ArrayList<String>,selectRecentAction:ArrayList<String>,superFilterBean: SuperFilterConfigBean?){
        this.mSuperFilterConfigBean = superFilterBean
        this.mSelectFrequencyChangeJobList = selectFrequencyChangeJob
        this.mSelectRecentActionList = selectRecentAction
        setSelect()
        if (superFilterBean?.fimageUrl.isNullOrEmpty()){
            mIconView.visibility = View.GONE
        }else{
            mIconView.visibility = View.VISIBLE
            ImageLoadUtils.with(context).load(superFilterBean?.fimageUrl).placeholder(R.drawable.icon_lagou_plus).error(R.drawable.icon_lagou_plus).dontAnimate().into(mIconView)
        }

        if (superFilterBean?.frightsText.isNullOrEmpty()){
            mDescView.visibility = View.GONE
        }else{
            mDescView.visibility = View.VISIBLE
            mDescView.text = superFilterBean?.frightsText
            mMaskDescView.text = superFilterBean?.frightsText
        }

    }

    private fun setSelect() {
        frequencyOfChangeJobView.forEach { textView ->
            textView.isSelected = mSelectFrequencyChangeJobList?.contains(textView.text.toString()) == true
        }
        recent30DaysActionView.forEach { textView ->
            textView.isSelected = mSelectRecentActionList?.contains(textView.text.toString()) == true
        }
    }

    fun initFilter(openCallback:(()->Unit)?):SuperFilterHolder{
        this.closeFilterWindowListener = openCallback
        return this
    }
}