package com.lagou.lagouee.ui.chat.item

import android.content.Context
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.text.SpannableStringBuilder
import android.text.TextUtils
import com.alpha.lagouapk.R
import com.alpha.lagouapk.tools.UserHeadUtils
import com.common.chat.item.base.ChatItem
import com.lagou.chat.chatlibrary.bean.ChatCUserInfoBean
import com.lagou.commonlibs.utils.ListUtil
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.utils.VerticalCenterImageSpan
import kotlinx.android.synthetic.main.chat_item_business_card_b.view.*

/**
 * @date： Crate by wangjunwei on 2022/9/19
 * @description：名片用户的，IM用户卡片
 **/
class BChatUserInfoItemV2Simple(context: Context?) : ChatItem(context) {

    //是否正在进行卡片动画
    private var isExecuteCardAnim = false

    override fun initView() {
        //名片卡片不能点击，无需设置点击事件
    }

    override fun updateView() {
        if (mChatMsg == null){
            return
        }
        val bean = mChatMsg.chatCUserInfoBean
        bean?.let {
            setBaseInfo(it)
            setWorkExperience(it)
            setEducationExperience(it)

            //开启卡片动画
//            if (!isExecuteCardAnim) {
//               startCardAnim()
//            }
        }
    }

    override fun getLayoutId(): Int {
        return R.layout.chat_item_business_card_b
    }

    /**
     * 基本信息
     */
    private fun setBaseInfo(model: ChatCUserInfoBean) {

        UserHeadUtils.setUserHeader(iv_avatar, model.headPic)
        var gender = model.sex
        gender = gender ?: ""
        when (gender) {
            "男" -> iv_chat_sex.setImageResource(R.drawable.icon_male_flag)
            "女" -> iv_chat_sex.setImageResource(R.drawable.icon_female_flag)
            else -> iv_chat_sex.setImageDrawable(ColorDrawable(Color.TRANSPARENT))
        }
        tv_chat_name.text = model.nickname
        tv_salary.text = model.salary
        // 3年 大专 26岁 北京
        val builder = SpannableStringBuilder()
        VerticalCenterImageSpan.inject(
            context, builder, R.drawable.layer_chat_position_item_gap,
            model.timeDescription, model.expectCity, model.expectPosition
        )
        tv_chat_personal_detail.text = builder
        if (TextUtils.isEmpty(model.communicationPosition)) {
            tv_communication_position.visibility = INVISIBLE
        } else {
            tv_communication_position.visibility = VISIBLE
            tv_communication_position.text = "沟通职位：" + model.communicationPosition
        }
    }


    /**
     * 工作经历
     */
    private fun setWorkExperience(model: ChatCUserInfoBean) {
        val list = model.workExperiences
        if (list == null || list.isEmpty()) {
            tv_work_content.visibility = GONE
            tv_work_company.text = ResourceUtils.getString(R.string.no_work_exp)
        }else{
            val experience = list[0] ?: return
            if (!TextUtils.isEmpty(experience.companyName)){
                tv_work_company.text = experience.companyName
            }else {
                tv_work_company.text = ResourceUtils.getString(R.string.no_work_exp)
            }

            if (!TextUtils.isEmpty(experience.positionName)){
                tv_work_content.visibility = VISIBLE
                tv_work_content.text = experience.positionName
            }else {
                tv_work_content.visibility = GONE
            }
        }
    }

    /**
     * 教育经历
     */
    private fun setEducationExperience(model: ChatCUserInfoBean) {
        val e = ListUtil.first(model.educationExperiences)
        if (e == null) {
            image_education_icon.visibility = GONE
            tv_education_content.visibility = GONE
            tv_education_content_after.visibility = GONE
        } else {
            image_education_icon.visibility = VISIBLE
            tv_education_content.visibility = VISIBLE
            tv_education_content_after.visibility = VISIBLE
            tv_education_content.text = e.schoolName + "·"
            tv_education_content_after.text = model.highestEducationShowTxt
        }
    }

    //开启卡片动画
//    private fun startCardAnim() {
//
//        isExecuteCardAnim = true
//
//        val transformationView = ivAnimCard
//
//        val alphaRootView = fl_alpha_root_view
//        val alpha1View = iv_card_anim_star_1
//        val alpha3View = iv_card_anim_star_3
//        val alpha2View = iv_card_anim_star_2
//
//        //卡片平移动画
//        transformationView.visibility = View.VISIBLE
//        val transformationAnim = ObjectAnimator.ofFloat(transformationView, "translationX", 0.0f, constrain_business_card.width * 1.0f)
//        transformationAnim.duration = 1000
//
//        //星星渐变动画
//        val alpha1 = getAlphaAnimator(alpha1View)
//        val alpha2 = getAlphaAnimator(alpha3View)
//        val alphaEnd = getAlphaAnimator(alpha2View)
//
//        alpha1.addListener(object : AnimatorListenerAdapter() {
//            override fun onAnimationEnd(animation: Animator?) {
//                super.onAnimationEnd(animation)
//                alpha1View?.visibility = View.GONE
//            }
//        })
//        alpha2.addListener(object : AnimatorListenerAdapter() {
//            override fun onAnimationEnd(animation: Animator?) {
//                super.onAnimationEnd(animation)
//                alpha3View?.visibility = View.GONE
//            }
//        })
//        alphaEnd.addListener(object : AnimatorListenerAdapter() {
//            override fun onAnimationEnd(animation: Animator?) {
//                super.onAnimationEnd(animation)
//
//                alpha2View?.visibility = View.GONE
//                alphaRootView?.visibility = View.GONE
//            }
//        })
//
//        transformationAnim.addListener(object : AnimatorListenerAdapter() {
//            override fun onAnimationEnd(animation: Animator?) {
//                transformationView.visibility = View.GONE
//
//                alphaRootView?.visibility = View.VISIBLE
//                alpha1View?.visibility = View.VISIBLE
//                alpha3View?.visibility = View.VISIBLE
//                alpha1.start()
//                alpha2.start()
//
//                //间隔动画 主要是为了延迟时间
//                val animDelay = ValueAnimator.ofFloat(0f, 1f)
//                animDelay.duration = 500
//                animDelay.addListener(object : AnimatorListenerAdapter() {
//                    override fun onAnimationEnd(animation: Animator?) {
//                        super.onAnimationEnd(animation)
//                        alpha2View?.visibility = View.VISIBLE
//                        alphaEnd.start()
//                    }
//                })
//                animDelay.start()
//            }
//        })
//        transformationAnim.start()
//    }
//
//    //返回渐变动画
//    private fun getAlphaAnimator(view: View): ObjectAnimator {
//        return ObjectAnimator.ofFloat(view, "alpha", 0.0f, 1.0f, 0.0f)
//            .also {
//                it.duration = 1000
//            }
//    }
}