package com.lagou.lagouee.ui.dialog

import android.graphics.Rect
import android.os.Bundle
import android.text.TextUtils
import android.view.*
import android.view.View.OnTouchListener
import androidx.fragment.app.FragmentManager
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.blankj.utilcode.util.KeyboardUtils
import com.blankj.utilcode.util.MapUtils
import com.common.SensitiveLogic
import com.common.SensitiveLogic.CheckSensitiveCallback
import com.common.SensitiveLogic.checkExistSensitiveKey
import com.common.widget.CustomTextWatcher
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.bean.json.ClickInfo
import com.lagou.baselib.net.httpnetback.BaseEventCallback
import com.lagou.commonlibs.engine.ThreadUtil
import com.lagou.commonlibs.inter.OnSingleMethodListener
import com.lagou.commonlibs.inter.OnSingleMethodNoParameterListener
import com.lagou.commonlibs.inter.OnSingleTwoMdPsListener
import com.lagou.commonlibs.ui.dialog.BaseDialogFragment
import com.lagou.commonlibs.utils.*
import com.lagou.lagouee.bean.jsonbean.aigc.AIGCGenerateConfig
import com.lagou.lagouee.bean.jsonbean.aigc.AIGCReleaseRes
import com.lagou.lagouee.manager.net.PositionNetManager
import com.lagou.lagouee.ui.adapter.AIGCReleaseAdapter
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.event.HttpEventHelper
import com.lagou.socketchannel.http.event.HttpEventResponse
import kotlinx.android.synthetic.main.dialog_aigc_release.*
import pl.droidsonroids.gif.GifDrawable
import java.util.*


/**
 * @date： Crate by wangjunwei on 2023/7/7
 *
 * @description：AIGC相关的发布职位文本生成弹窗
 **/
class AIGCReleaseDialog : BaseDialogFragment() {

    val DP_6_05 = ResourceUtils.getDimenSize(R.dimen.dp_6_05)
    val DP_10 = ResourceUtils.getDimenSize(R.dimen.dp_10)
    val DP_15 = ResourceUtils.getDimenSize(R.dimen.dp_15)
    val DP_25 = ResourceUtils.getDimenSize(R.dimen.dp_25)

    /**
     * 输入字数限制：中文50 英文100
     * 目前的策略计数都按照英文字符计数（中文换算成两个英文）
     */
    private val CN_BOUND = 50
    private val EN_BOUND = 100

    private var adapterReleaseAIGC: AIGCReleaseAdapter? = null
    private val example = "Hi，我是您在拉勾的智能AI招聘助手HRMind，非常高兴为您提供服务！请告诉我您的招聘岗位的核心要求，我来帮您写职位描述。\n例如：我想招一个高级Java工程师。要求5-10年工作经验，具备架构能力，沟通能力出色，大厂经验。"
    private val aiTips = ResourceUtils.getString(R.string.ai_input_tips)
    private var aigcGenerateConfig: AIGCGenerateConfig? = null

    private var maxSendCount = 3
    /**
     * 可以发送的次数
     */
    private var canSendCount = maxSendCount

    /**
     * 打字效果处理器
     */
    private var displayTextDisplayOffset: DelayedTextDisplayOffset? = null

    /**
     * AI是否正在生成中
     */
    private var isAiGenerating = false
    private var layoutManagerCompact: LinearLayoutManager? = null

    /**
     * timer定时器管理类
     */
    private val timerList = mutableListOf<Timer?>()

    /**
     * 使用该描述回调到目标方（比如"发布职位页"）
     */
    private var useCallback: OnSingleTwoMdPsListener<AIGCReleaseRes, AIGCReleaseRes>? = null

    /**
     * 当前问题
     */
    private var currentQuestion: String = ""

    /**
     * 是否有手动滑动意图，有的话中断自动滑动（smoothToNewestContent），等到生成结束的时候在滑动到最后
     */
    private var isHandSlide = false

    private var httpRequest: HttpRequest<String>? = null

    companion object {
        const val KEY_CONFIG = "config"

        @JvmStatic
        fun getInstance(aigcGenerateConfig: AIGCGenerateConfig, useCallback: OnSingleTwoMdPsListener<AIGCReleaseRes, AIGCReleaseRes>?): AIGCReleaseDialog {
            val bundle = Bundle()
            bundle.putSerializable(KEY_CONFIG, aigcGenerateConfig)
            val aiGeneralDialog = AIGCReleaseDialog()
            aiGeneralDialog.arguments = bundle
            aiGeneralDialog.useCallback = useCallback
            return aiGeneralDialog
        }
    }

    override fun getLayoutId(): Int {
        return R.layout.dialog_aigc_release
    }

    override fun getGravity(): Int {
        return Gravity.BOTTOM
    }

    override fun initConfigAtCreateView() {
        super.initConfigAtCreateView()
        try {
            val softInputMode = WindowManager.LayoutParams.SOFT_INPUT_ADJUST_RESIZE
            dialog?.window?.setSoftInputMode(softInputMode)
            dialog?.setCanceledOnTouchOutside(true)
            dialog?.setCancelable(true)
        }catch (e: Exception){
            e.printStackTrace()
        }
    }

    override fun fillUI() {
        super.fillUI()
        rl_ai_gen_root.setPadding(0,getContentTopMargin(),0,0)
        aigcGenerateConfig = arguments?.get(KEY_CONFIG) as? AIGCGenerateConfig
        aigcGenerateConfig?.let {
            maxSendCount = it.limitTimes
            canSendCount = maxSendCount
            tv_ai_send.text = ResourceUtils.getString(R.string.send_ai_txt, canSendCount, maxSendCount)
        }

        //1 Ai机器人动起来
        // 将gif图资源转化为GifDrawable
        val animationDrawable = GifDrawable(
            resources, R.drawable.ai_robot
        )
        image_role_ai_dog.setImageDrawable(animationDrawable)
        animationDrawable.loopCount = 0

        //2 初始化recycleview
        initRecycleView()

        //3 监听事件 initListener
        adapterReleaseAIGC?.useCallback = useCallback
    }


    private fun initRecycleView() {
        adapterReleaseAIGC = AIGCReleaseAdapter(context
            ,OnSingleMethodListener<Int> { position ->
                //dismiss()
                useAnalyticClick(position)
            })
        layoutManagerCompact = LinearLayoutManager(context)
        recycle_ai_content.apply {
            layoutManager = layoutManagerCompact
            adapter = adapterReleaseAIGC
            addItemDecoration(object : RecyclerView.ItemDecoration() {
                override fun getItemOffsets(
                    outRect: Rect,
                    view: View,
                    parent: RecyclerView,
                    state: RecyclerView.State
                ) {
                    super.getItemOffsets(outRect, view, parent, state)
                    outRect.set(DP_15, DP_15, DP_15, DP_6_05)
                }
            })
        }

        //1 样例灌入
        fillExampleShow()

        //2 发送问题事件
        tv_ai_send.setOnClickListener {
            //发送按钮点击
              sendAnalyticClick()

            //-1 单词弹窗能发送会话的最大次数超过限制，则toast提示
            if (canSendCount <= 0) {
                ToastUtil.show(context,
                    ResourceUtils.getString(R.string.ai_limit_txt, aigcGenerateConfig?.limitTimes)
                )
                return@setOnClickListener
            }

            //0 Ai是否正在生成中
            if (isAiGenerating) {
                ToastUtil.show(context, "主人请不要着急, 先等我把话说完, 再提出新的问题")
                return@setOnClickListener
            }
            if  (displayTextDisplayOffset != null && displayTextDisplayOffset?.isDisplayTextEnd() == false) {
                ToastUtil.show(context, "主人请不要着急, 先等我把话说完, 再提出新的问题")
                return@setOnClickListener
            }

            //1 空校验
            var text = ""
            if (et_bottom_question.text != null){
                text = et_bottom_question.text.toString().trim()
            }
            if (TextUtils.isEmpty(text)) {
                ToastUtil.show(context, "请输入内容")
                return@setOnClickListener
            }

            if (!NetUtils.isNetworkAvailable(context)){
                ToastUtil.show(context, ResourceUtils.getString(R.string.no_network))
                return@setOnClickListener
            }

            //2 超限文本截取前100个发送：先校验敏感词，通过之后继续后边的流程
            val endIndex = StringUtil.countCnIndex(text, EN_BOUND)
            val trimText = StringUtil.trimText(text, endIndex)
            currentQuestion = trimText
            isAiGenerating = true

            checkSensitiveLogic(trimText, text)
        }
    }
    private fun getContentTopMargin(): Int = (DeviceUtils.getRealScreenHeight(context) * 0.02).toInt()

    override fun initListener() {
        super.initListener()
        //1 文本输入框，只要有新的输入，提示文案回复原始（更新敏感词的显示提示文案）
        et_bottom_question.addTextChangedListener(object : CustomTextWatcher() {
            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
                super.onTextChanged(s, start, before, count)
                //敏感词错误提示: 你输入的内容包含敏感词"XXX"，请重新输入。显示在输入框上方, 用户编辑输入区后, 消失, 并恢复显示原有内容
                tv_ai_tips.text = aiTips
                tv_ai_tips.setCompoundDrawablesWithIntrinsicBounds(null, null, null, null)
                //字数限制控制中文50 英文100
//                val result = s.toString().trim()
//                val length = StringUtil.countLength(result)
//                if (length > EN_BOUND) {
//                    //计算出可以截取的endIndex（不管英文中文都算一个）
//                    val endIndex = StringUtil.countCnIndex(result, EN_BOUND)
//                    et_bottom_question.setText(StringUtil.trimText(result, endIndex))
//                    et_bottom_question.setSelection(et_bottom_question.text.length)
//                }
            }
        })

        //2 空白处关闭弹窗
        frame_ai_header.setOnClickListener {
            dismiss()
            dismissAnalyticClick()
        }
        //只监听弹窗的返回事件，不做拦截处理
        dialog?.setOnKeyListener { dialog, keyCode, event ->
            if (keyCode == KeyEvent.KEYCODE_BACK) {
                //防止回调多次
                getDialog()?.setOnKeyListener(null)
                dismissAnalyticClick()
            }
            return@setOnKeyListener false
        }

        //屏蔽头部非空白部分的点击事件
        image_ai_bg.setOnClickListener {  }
        frame_role_ai_dog.setOnClickListener { dismiss() }
        image_close.setOnClickListener { dismiss() }

        //3 recycleview列表滚动监听，人一旦触碰到屏幕终止列表自动滑动
        recycle_ai_content.setOnTouchListener(OnTouchListener { v, event ->
            val action = event.action
            if (action == MotionEvent.ACTION_DOWN || action == MotionEvent.ACTION_MOVE) {
                // 手指按下或者滑动，人手触碰滑动开始
                isHandSlide = true
            }
            false
        })
    }

    /***
     * 敏感词校验
     */
    private fun checkSensitiveLogic(trimText: String, originalContent: String) {
        val map: MutableMap<String, String> = HashMap()
        val positionNameKey = SensitiveLogic.SensitiveKey.RESUME_WORK_POSITION_NAME_WARN.name
        map[positionNameKey] = trimText
        checkExistSensitiveKey(
            map, object  : CheckSensitiveCallback {
                override fun checkFinish(
                    keyList: ArrayList<String>?,
                    keyAndErrorMsgMap: HashMap<String, String>?) {
                    if (MapUtils.isNotEmpty(keyAndErrorMsgMap)) {
                        isAiGenerating = false
                        //ToastUtil.showCenter(context, ResourceUtils.getString(R.string.content_warn))
                        tv_ai_tips.text = StringUtil.jointString(",", keyAndErrorMsgMap?.values?.toList())
                        tv_ai_tips.setCompoundDrawablesWithIntrinsicBounds(ResourceUtils.getDrawable(R.drawable.ai_im_remind_ic), null, null, null)
                        return
                    }

                    //敏感词校验通过，开始执行Ai生成流程
                    //2.1 更新到列表显示原始文本
                    filInputShow(originalContent, OnSingleMethodNoParameterListener {
                        //3 执行接口请求
                        httpRequest = fillRequestInput(trimText)
                        httpRequest?.apply {
                            val aigcReleaseRes = AIGCReleaseRes()
                            aigcReleaseRes.id = SafeTypeConvert.toLong(tag)
                            aigcReleaseRes.isLoading = true
                            aigcReleaseRes.contentType = AIGCReleaseRes.ContentType.GENERATE_TYPE
                            adapterReleaseAIGC?.handleAiContent(aigcReleaseRes)
                        }
                    })
                }
            })
    }

    /***
     * 样例灌入，模拟打字的效果
     */
    private fun fillExampleShow(noParameterListener : OnSingleMethodNoParameterListener? = null) {
        val aigcReleaseResList =
            AIGCReleaseRes.uponTextCreate(
                example,
                AIGCReleaseRes.ContentType.EXAMPLE_TYPE,
                System.currentTimeMillis()
            )
        DelayedTextDisplayShuffle(aigcReleaseResList, noParameterListener).startDelayedTextDisplay()
    }

    /***
     * 用户输入更新到页面显示
     */
    private fun filInputShow(text: String, parameterListener: OnSingleMethodNoParameterListener? = null) {
        //1 移除最后一行是换个问题，始终保持只有一个换个问题在最底部
        adapterReleaseAIGC?.removeAiChangeQuestionLast()

        //2 用户发的问题插入聊天列表中
        val aigcReleaseRes =
            AIGCReleaseRes.uponUserTextCreate(
                text,
                AIGCReleaseRes.ContentType.INPUT_TYPE,
                System.currentTimeMillis()
            )
        adapterReleaseAIGC?.handleAiContent(aigcReleaseRes)

        //2.1 隐藏软键盘
        et_bottom_question?.let {
            KeyboardUtils.hideSoftInput(it)
        }

        //3 回调下一步紧接着发送问题到接口
        parameterListener?.onCallBack()

        //4 用户输入东西更新到会话列表显示时，才清空输入框文本
        et_bottom_question?.setText("")

        //提问问题更新之后，跳转到最新的位置（没有打字效果，需要手动滑动到最底部）
        smoothToNewestContent()
    }

    /***
     * Ai 响应结果更新页面
     */
    private fun fillRequestInput(trimText: String?): HttpRequest<String>? {
        displayTextDisplayOffset = null
       
         return PositionNetManager.chatOpenJd(trimText, adapterReleaseAIGC?.getContextIds(),
            object : BaseEventCallback<String>() {
                override fun onSuccess(
                    httpRequest: HttpRequest<*>,
                    httpResponse: HttpEventResponse<String>) {
                    if (httpResponse.isFinish) {
                        isAiGenerating = false
                        canSendCount--
                    }
                    if (activity?.isDestroyed == true) {
                        return
                    }

                    onRequestCall(httpRequest, httpResponse)
                }

                override fun onFailure(
                    httpRequest: HttpRequest<*>,
                    httpEventResponse: HttpEventResponse<String>) {
                    //101的情况不出toast，直接在列表中以会话的形式出来
                    if (httpEventResponse.code != HttpEventResponse.State.FAIL){
                        super.onFailure(httpRequest, httpEventResponse)
                    }
                    isAiGenerating = false
                    if (activity?.isDestroyed == true) {
                        return
                    }

                    onRequestCall(httpRequest, httpEventResponse)
                }

                override fun onClose(
                    httpRequest: HttpRequest<*>,
                    httpEventResponse: HttpEventResponse<String>) {
                    super.onClose(httpRequest, httpEventResponse)
                    isAiGenerating = false
                    if (activity?.isDestroyed == true) {
                        return
                    }

                    onRequestCall(httpRequest, httpEventResponse)
                }

                private fun onRequestCall(httpRequest: HttpRequest<*>, httpResponse: HttpEventResponse<String>) {

                    if (tv_ai_send == null){
                        return
                    }

                    //更新发送进度展示
                    tv_ai_send.text = ResourceUtils.getString(R.string.send_ai_txt, canSendCount, maxSendCount)

                    if (httpResponse == null) {
                        adapterReleaseAIGC?.updateAiContentLast()
                        return
                    }

                    val aigcReleaseRes = AIGCReleaseRes()
                    aigcReleaseRes.contextId = httpResponse.contextId
                    aigcReleaseRes.id = SafeTypeConvert.toLong(httpRequest.tag)
                    aigcReleaseRes.code = httpResponse.code
                    aigcReleaseRes.content = httpResponse.content
                    aigcReleaseRes.positionName = httpResponse.positionName
                    aigcReleaseRes.filterContent = httpResponse.filterContent
                    //如果是101情况，且content为空则取message，因为这个异常的文案要出现在会话列表中
                    if (aigcReleaseRes.isFailMode && TextUtils.isEmpty(aigcReleaseRes.content)){
                        aigcReleaseRes.content = httpResponse.message
                    }
                    aigcReleaseRes.contentType = aigcReleaseRes.uponCodeType()

                    if (displayTextDisplayOffset == null) {
                        displayTextDisplayOffset = DelayedTextDisplayOffset(aigcReleaseRes)
                        displayTextDisplayOffset?.startDelayedTextDisplay()
                        return
                    }

                    //有新的流式文本更新
                    displayTextDisplayOffset?.updateGoalContent(aigcReleaseRes)
                }
            });
    }

    /***
     * 自动平滑到底部，方便用户看到最新的输出
     */
    private fun smoothToNewestContent() {
        var lastItemIndex = adapterReleaseAIGC?.itemCount
        if (lastItemIndex != null && lastItemIndex < 0) {
            lastItemIndex = 0
        }
        //val offset: Int = recycle_ai_content.computeVerticalScrollRange() - recycle_ai_content.computeVerticalScrollExtent()
        lastItemIndex?.let {
            //layoutManagerCompact?.smoothScrollToPosition(recycle_ai_content, null, lastItemIndex)
            //recycle_ai_content?.scrollBy(0, offset)
            recycle_ai_content?.smoothScrollToPosition(lastItemIndex)
        }
    }

    /**
     * 通过操作下标偏移的方式，实现打字机效果，更适合流式逐步给出内容
     * 不确定：接口请求流式出文案，速度不定
     * 匀速：本地控制显示匀速出
     * 注意：流式给出第一条文案就要及时显示到列表，后续有更新及时更新本地匀速出的数据源
     * 保障：timer 定时器不关闭，除非本地匀速出文案结束（currentIndex >= aigcReleaseRes.content.length）&接口出文案也结束的情况下（finishReleaseRes不为空）
     */
    inner class DelayedTextDisplayOffset(aigcRes: AIGCReleaseRes) {

        private var aigcReleaseRes: AIGCReleaseRes
        private var finishReleaseRes: AIGCReleaseRes? = null
        private var timer: Timer? = null

        /**
         * 打字效果的当前下标
         */
        private var currentIndex: Int = 0

        /***
         * 当前偏移量，注意判断不要越界
         */
        private var offsetCount = 0

        init {
            aigcReleaseRes = aigcRes
        }

        /**
         * 流式过程中不停地更新文本，避免重复开启任务流，判断只有当当前执行流程结束时才重新激活
         */
        fun updateGoalContent(aigcRes: AIGCReleaseRes) {
            if (aigcRes.isGeneratingMode){
                aigcReleaseRes = aigcRes
            }else {
                //有异常，直接丢弃后续的更新
                //if (aigcReleaseRes != null && aigcReleaseRes.isFailMode){
                //    return
                //}
                //正常结束先缓存起来等到打字效果偏移量走完了在更新
                finishReleaseRes = aigcRes
                //aigcReleaseRes.code = aigcRes.code
            }
        }

        fun startDelayedTextDisplay() {
            isHandSlide = false
            displayTextWithDelay()
        }

        /**
         * 判断当次的提问回答是否结束：元数据为空或者打字过程结束
         */
        fun isDisplayTextEnd(): Boolean{
            if (aigcReleaseRes == null || aigcReleaseRes.content == null || aigcReleaseRes.content.isEmpty()){
                return true
            }
            return currentIndex >= aigcReleaseRes.content.length
        }

        /***
         * 是否是异常数据，是的话丢弃当次timer任务，等待下次timer任务执行
         */
        private fun isExceptionData(): Boolean {
            if (aigcReleaseRes == null || TextUtils.isEmpty(aigcReleaseRes.content)) {
                return true
            }
            return false
        }

        private fun displayTextWithDelay() {
            timer = Timer()
            timerList.add(timer)
            timer?.scheduleAtFixedRate(object : TimerTask() {
                override fun run() {
                    if (isExceptionData()) {
                        return
                    }
                    try {
                        //fix Attempt to invoke virtual method 'int java.lang.String.length()' on a null object reference
                        if (currentIndex < aigcReleaseRes.content.length) {
                            // 在此处处理显示文本的逻辑，例如将 text 设置到 TextView 中显示
                            offsetCount = StringUtil.generateRandomNumber(1, 3)
                            if (currentIndex + offsetCount > aigcReleaseRes.content.length) {
                                offsetCount = aigcReleaseRes.content.length - currentIndex
                            }
                            val resultTxt =
                                aigcReleaseRes.content.substring(currentIndex, currentIndex + offsetCount)
                            //LogUtils.i("displayTextWithDelay","$currentIndex  $offsetCount  resultTxt = $resultTxt")

                            val aigcReleaseRes =
                                AIGCReleaseRes.createOffsetText(resultTxt, aigcReleaseRes)

                            ThreadUtil.ui {
                                adapterReleaseAIGC?.handleAiContent(aigcReleaseRes)
                                currentIndex += offsetCount
                                //如果是有手动滑动的意图，则中断当前生成内容会话的自动滑动，直到当前内容生成结束才重置isHandSlide状态
                                if (!isHandSlide){
                                    smoothToNewestContent()
                                }
                            }
                            return
                        }

                        if (currentIndex >= aigcReleaseRes.content.length){
                            finishReleaseRes?.apply {
                                aigcReleaseRes?.contentType = contentType
                                aigcReleaseRes.code = code
                                aigcReleaseRes.contextId = contextId
                                aigcReleaseRes.id = id
                                aigcReleaseRes.positionName = positionName
                                aigcReleaseRes.filterContent = filterContent
                                finishReleaseRes = null
                                ThreadUtil.ui {
                                    adapterReleaseAIGC?.handleAiContent(aigcReleaseRes)
                                    smoothToNewestContent()

                                    //处理下是否显示换个问题，在adapter内实现该方法，前提：换个问题显示在每次AIGC生成结束之后
                                    //1 未点击过换个问题时，只在AIGC生成结束之后追加一个换个问题的item
                                    //2 点击过换个问题，换个问题卡片保留，在最后一行追加更换问题卡片
                                    //3 最后一个AIGCS生成文案之后不展示"换个问题"
                                    if (canSendCount > 0){
                                        val aigcReleaseChange = AIGCReleaseRes()
                                        aigcReleaseChange.setShowOpenQuestionDefault()
                                        aigcReleaseChange.id = System.currentTimeMillis()
                                        adapterReleaseAIGC?.handleAiChangeQuestion(aigcReleaseChange)
                                    }

                                    isHandSlide = false
                                    timer?.cancel()
                                    timerList.remove(timer)
                                }
                            }
                        }
                    }catch (e:Exception){
                        e.printStackTrace()
                    }
                }
            }, 0, 25)
        }
    }

    /**
     * 通过打散文本的方式实现打字机效果，适合于非流式出文案的方式（用打字、异常情况、Ai固定文案）
     */
    inner class DelayedTextDisplayShuffle(private val list: List<AIGCReleaseRes>,
                                          var parameterListener: OnSingleMethodNoParameterListener?) {

        private var currentIndex: Int = 0

        fun startDelayedTextDisplay() {
            if (list.isNotEmpty()) {
                displayTextWithDelay()
            }
        }

        private fun displayTextWithDelay() {
            if (currentIndex < list.size) {
                ThreadUtil.uiDelay({ // 在此处处理显示文本的逻辑，例如将 text 设置到 TextView 中显示
                    val aigcReleaseRes = list[currentIndex]
                    adapterReleaseAIGC?.handleAiContent(aigcReleaseRes)
                    smoothToNewestContent()
                    currentIndex++
                    displayTextWithDelay()
                    //结束之后回调其它事件，用于在用户发送的问题页面更新完之后，要处理的事情
                    if (currentIndex >= list.size){
                        parameterListener?.onCallBack()
                    }
                }, 25) // 延迟时间为25毫秒
            }
        }
    }

    override fun show(manager: FragmentManager?) {
        super.show(manager)
        //上报浏览信息
        AnalyticHelper.getInstance().sendExploreInfo("i1uam")
    }


    override fun onDestroy() {
        super.onDestroy()
        try {
            timerList.forEach { it?.cancel()}
        }catch (e: Exception){
            e.printStackTrace()
        }
        try {
            HttpEventHelper.getInstance().cancel(httpRequest)
        }catch (e: Exception){
            e.printStackTrace()
        }
    }


    /**
     * 提问问题按钮点击埋点
     * "{sendTimes:1, questionContent:""我需要招一个厨子, 必须会做宫保鸡丁"" },
     *  sendTimes: 当次会话发送次数, 1为第一次, 2为第二次....
     *  questionContent:当前答案的对应问题"
     */
    private fun sendAnalyticClick() {
        val clickInfo = ClickInfo()
        clickInfo.page_id = "i1uam"
        clickInfo.address_id = "m1uaq"
        clickInfo.addClick_props("sendTimes", (maxSendCount - canSendCount))
        clickInfo.addClick_props("questionContent", currentQuestion)
        AnalyticHelper.getInstance().sendClickInfo(clickInfo)
    }


    /**
     * 使用描述点击
     * "{askIndex:0, totalSendTimes:2, questionContent:""我想招一个java...""  },
     *  askIndex使用的第几个答复(0为第一个);
     *  totalSendTimes:总共发送问题次数
     *  questionContent:当前答案的对应问题"
     */
    private fun useAnalyticClick(position: Int) {
        val clickInfo = ClickInfo()
        clickInfo.page_id = "i1uam"
        clickInfo.address_id = "n1uar"
        clickInfo.addClick_props("askIndex", position)
        clickInfo.addClick_props("totalSendTimes", maxSendCount)
        clickInfo.addClick_props("questionContent", currentQuestion)
        AnalyticHelper.getInstance().sendClickInfo(clickInfo)
    }

    /***
     * 无意义的关闭弹窗点击事件上报
     */
    private fun dismissAnalyticClick() {
        val clickInfo = ClickInfo()
        clickInfo.page_id = "i1uam"
        clickInfo.address_id = "i1uc3"
        //0 表示未发送过问题 1发送过问题
        if (adapterReleaseAIGC?.hasSendQuestion() == true){
            clickInfo.addClick_props("hasSent", 1)
        }else {
            clickInfo.addClick_props("hasSent", 0)
        }
        AnalyticHelper.getInstance().sendClickInfo(clickInfo)
    }
}