package com.lagou.lagouee.ui.dialog

import android.content.Context
import android.graphics.Color
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.WindowManager
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.bean.json.ClickInfo
import com.lagou.baselib.analytic.bean.json.ShowLogInfo
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.utils.TextViewUtils
import com.lagou.commonlibs.view.SpaceItemDecoration
import com.lagou.component.ui.dialog.LgBaseDialog
import com.lagou.lagouee.bean.jsonbean.business.ReachItem
import com.lagou.lagouee.utils.IntentUtils
import kotlinx.android.extensions.LayoutContainer
import kotlinx.android.synthetic.main.dialog_compete_position_reach_multiple.btnContinueHiring
import kotlinx.android.synthetic.main.dialog_compete_position_reach_multiple.mIvClose
import kotlinx.android.synthetic.main.dialog_compete_position_reach_multiple.rvReachPosition
import kotlinx.android.synthetic.main.dialog_compete_position_reach_multiple.tvDeliverNum
import kotlinx.android.synthetic.main.dialog_compete_position_reach_single.btnNotUpdateCompete
import kotlinx.android.synthetic.main.dialog_compete_position_reach_single.btnUpdateCompete
import kotlinx.android.synthetic.main.dialog_compete_position_reach_single.mIvSingleClose
import kotlinx.android.synthetic.main.dialog_compete_position_reach_single.tvSingleDeliverNum
import kotlinx.android.synthetic.main.dialog_compete_position_reach_single.tvSingleTalentNum
import kotlinx.android.synthetic.main.item_compete_position_reach.view.tvDeliverNum
import kotlinx.android.synthetic.main.item_compete_position_reach.view.tvPositionName

class CompetePositionReachDialog private constructor():LgBaseDialog() {
    private lateinit var positionList:ArrayList<ReachItem>
    private var jumpUrl:String? = null
    private var isMultiplePosition:Boolean = false
    companion object{
        const val KEY_COMPETE_POSITION_REACHED_LIST = "key_compete_position_reached_list"
        const val KEY_JUMP_URL = "key_jump_url"
        @JvmStatic
        fun getInstance(positionList:ArrayList<ReachItem>, jumpUrl:String?):CompetePositionReachDialog{
            val bundle = Bundle()
            bundle.putParcelableArrayList(KEY_COMPETE_POSITION_REACHED_LIST,positionList)
            bundle.putString(KEY_JUMP_URL,jumpUrl)
            val dialog = CompetePositionReachDialog()
            dialog.arguments = bundle
            return dialog
        }
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        positionList = arguments?.getParcelableArrayList(KEY_COMPETE_POSITION_REACHED_LIST) ?: ArrayList()
        jumpUrl = arguments?.getString(KEY_JUMP_URL)
        isMultiplePosition = positionList.size > 1
    }
    override fun getLayoutId(): Int = if (isMultiplePosition) R.layout.dialog_compete_position_reach_multiple else R.layout.dialog_compete_position_reach_single

    override fun fillUI() {
        context?.let { ctx ->
            if (isMultiplePosition){
                val totalDeliverNum:Int = positionList.sumBy { it.deliverNum }
                tvDeliverNum.text = getString(R.string.compete_position_reach_dialog_title,totalDeliverNum)
                TextViewUtils.setGradientText(tvDeliverNum,Color.parseColor("#874A37"),Color.parseColor("#1B2038"),0,tvDeliverNum.text.length)
                rvReachPosition.layoutManager = LinearLayoutManager(ctx)
                rvReachPosition.addItemDecoration(SpaceItemDecoration(ResourceUtils.getDimenSize(R.dimen.dp_14_05),SpaceItemDecoration.SpaceOrientation.VERTICAL))
                rvReachPosition.adapter = MultiplePositionAdapter(ctx,positionList)
                TextViewUtils.setGradientText(btnContinueHiring,Color.parseColor("#FFE8D0"),Color.parseColor("#E0985B"),0,tvDeliverNum.text.length)
                val positionIds = positionList.joinToString(",", transform = {item: ReachItem -> item.lgPositionId.toString()})
                btnContinueHiring.setOnClickListener {
                    IntentUtils.startLagouWebViewActivity(ctx,jumpUrl)
                    dismiss()
                    AnalyticHelper.getInstance().sendClickInfo(ClickInfo().apply {
                        page_id = "u1udu"
                        address_id = "v1udv"
                        addClick_props("content_id",positionIds)
                        addClick_props("num",positionList.size)
                    })
                }
                mIvClose.setOnClickListener{ dismiss() }
                AnalyticHelper.getInstance().sendShowInfo(ShowLogInfo().apply {
                    setPage_Id("u1udu")
                    setAddress_Id("v1udv")
                    addShowProp("num",positionList.size)
                    addShowProp("contentid",positionIds)
                })
            }else{
                val totalDeliverNum:Int = positionList.sumBy { it.deliverNum }
                tvSingleDeliverNum.text = getString(R.string.compete_position_reach_dialog_title,totalDeliverNum)
                TextViewUtils.setGradientText(tvSingleDeliverNum,Color.parseColor("#874A37"),Color.parseColor("#1B2038"),0,tvSingleDeliverNum.text.length)
                tvSingleTalentNum.text = ResourceUtils.getString(R.string.compete_position_reach_dialog_talent_num,positionList.elementAtOrNull(0)?.talentNum ?: 0)
                TextViewUtils.setGradientText(btnUpdateCompete,Color.parseColor("#FFE8D0"),Color.parseColor("#E0985B"),0,btnUpdateCompete.text.length)
                btnNotUpdateCompete.setOnClickListener { dismiss() }
                mIvSingleClose.setOnClickListener{ dismiss() }
                btnUpdateCompete.setOnClickListener {
                    AnalyticHelper.getInstance().sendClickInfo(ClickInfo().apply {
                        page_id = "k1ucv"
                        address_id = "a1uda"
                        addClick_props("content_id",positionList.elementAtOrNull(0)?.lgPositionId ?: 0)
                    })
                    IntentUtils.startLagouWebViewActivity(ctx,jumpUrl)
                    dismiss()
                }
                AnalyticHelper.getInstance().sendShowInfo(ShowLogInfo().apply {
                    setPage_Id("k1ucv")
                    setAddress_Id("a1uda")
                    addShowProp("contentid",positionList.elementAtOrNull(0)?.lgPositionId ?: 0)
                })
            }
        }
    }

    override fun getWidth(): Int = WindowManager.LayoutParams.MATCH_PARENT

    override fun isCanceledOnTouchOutside(): Boolean = false

     class MultiplePositionAdapter(val context: Context,val positionList:List<ReachItem>): RecyclerView.Adapter<MultiplePositionAdapter.MultiplePositionViewHolder>() {
         class MultiplePositionViewHolder(override val containerView: View):RecyclerView.ViewHolder(containerView),LayoutContainer

        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): MultiplePositionViewHolder {
           return MultiplePositionViewHolder(LayoutInflater.from(context).inflate(R.layout.item_compete_position_reach,parent,false))
        }

        override fun getItemCount(): Int = positionList.size

        override fun onBindViewHolder(holder: MultiplePositionViewHolder, position: Int) {
            val reachItemBean = positionList[position]
            holder.containerView.tvDeliverNum.text = ResourceUtils.getString(R.string.compete_position_reach_dialog_deliver_num,reachItemBean.deliverNum)
            holder.containerView.tvPositionName.text = reachItemBean.positionName
        }
    }
}