package com.lagou.lagouee.ui.dialog.business

import android.os.Bundle
import android.view.WindowManager
import androidx.recyclerview.widget.GridLayoutManager
import butterknife.OnClick
import com.alpha.lagouapk.R
import com.common.ui.adapter.InterviewWeekAdapter
import com.google.gson.reflect.TypeToken
import com.lagou.commonlibs.inter.OnSingleMdTwoPsListener
import com.lagou.commonlibs.ui.dialog.BaseBottomSheetDialogFragment
import com.lagou.commonlibs.utils.ResourceUtils.getStringArray
import com.lagou.lagouee.bean.jsonbean.business.DateDetailListBean
import com.lagou.lagouee.ui.adapter.TopPositionSelectDateAdapter
import com.lagou.socketchannel.util.GsonUtil
import kotlinx.android.synthetic.main.dialog_top_select_date.*
import java.util.*

/**
 * Created by mihao on 2/3/23
 * 置顶职位选择日期弹窗
 */
class TopSelectDateDialog : BaseBottomSheetDialogFragment() {

    private var mDateDetailList: ArrayList<DateDetailListBean>? = null
    private var mListener: OnSingleMdTwoPsListener<ArrayList<DateDetailListBean>, Boolean>? = null
    private var isChangeSelect: Boolean = false

    companion object {
        const val KEY_DATE_LIST = "date_list"
    }

    fun newInstance(
        dateDetailList: ArrayList<DateDetailListBean>?,
        onConfirmListener: OnSingleMdTwoPsListener<ArrayList<DateDetailListBean>, Boolean>?
    ): TopSelectDateDialog {
        val topSelectDateDialog = TopSelectDateDialog()
        val bundle = Bundle()
        bundle.putSerializable(KEY_DATE_LIST, dateDetailList)
        topSelectDateDialog.arguments = bundle
        topSelectDateDialog.mListener = onConfirmListener
        return topSelectDateDialog
    }

    override fun getLayoutId() = R.layout.dialog_top_select_date

    override fun fillUI() {
        super.fillUI()
        isCancelable = false
        if (arguments?.getSerializable(KEY_DATE_LIST) is ArrayList<*>) {
            val dateDetailList =
                GsonUtil.toJson(arguments?.getSerializable(KEY_DATE_LIST) as ArrayList<*>?)
            mDateDetailList = GsonUtil.from<ArrayList<DateDetailListBean>>(
                dateDetailList,
                object : TypeToken<ArrayList<DateDetailListBean>?>() {})
        }
        initWeekRecycleView()
        initDateRecycleView()
    }

    private fun initWeekRecycleView() {
        rv_week?.apply {
            layoutManager = GridLayoutManager(context, 7)
            adapter = InterviewWeekAdapter(
                activity,
                getStringArray(R.array.weeks_1)
            ).bigTxt()
        }
    }

    private fun initDateRecycleView() {
        rv_date?.apply {
            layoutManager = GridLayoutManager(context, 7)
            val topPositionSelectDateAdapter =
                TopPositionSelectDateAdapter(activity, mDateDetailList)
            adapter = topPositionSelectDateAdapter
            topPositionSelectDateAdapter.setOnItemSelectListener(OnSingleMdTwoPsListener { date, position ->
                if (date == null) {
                    return@OnSingleMdTwoPsListener
                }
                date.isSelected = !date.isSelected
                topPositionSelectDateAdapter.notifyItemChanged(position)
                isChangeSelect = true
            })
        }
    }

    override fun getWidth(): Int {
        return WindowManager.LayoutParams.MATCH_PARENT
    }

    override fun getTheme(): Int {
        return R.style.BottomSheetDialog
    }

    @OnClick(R.id.iv_dialog_close)
    fun closeDialog() {
        dismiss()
    }

    @OnClick(R.id.tv_dialog_confirm)
    fun confirm() {
        dismiss()
        mListener?.onCallBack(mDateDetailList, isChangeSelect)
    }
}