package com.lagou.lagouee.ui.interview

import androidx.recyclerview.widget.LinearLayoutManager
import android.view.View
import com.alpha.lagouapk.R
import com.lagou.commonlibs.utils.TimeUtils
import com.common.bean.json.InterviewCalendarDayRes
import com.common.bean.json.InterviewTimeRes
import com.common.utils.SpUserNotClearUtils
import com.common.widget.interview.CalenderExpandPopupFactory
import com.common.widget.popup.InterviewCalenderPopup
import com.lagou.annotation.apt.RouterRule
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListListenerImpl
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListenerImpl
import com.lagou.commonlibs.event.Event
import com.lagou.commonlibs.event.OkBus
import com.lagou.commonlibs.utils.LogUtils
import com.lagou.commonlibs.web.router.RouteUtils
import com.lagou.lagouee.bean.InterviewList
import com.lagou.lagouee.bean.jsonbean.interview.InterviewListReq
import com.lagou.lagouee.bean.jsonbean.interview.InterviewTimeReq
import com.lagou.lagouee.manager.net.InterviewNetManager
import com.lagou.lagouee.observer.BEventTags
import com.lagou.lagouee.ui.interview.adapter.BInterviewScheduleAdapter
import com.lagou.lagouee.ui.resume.list.interview.InterviewDecoration
import com.lagou.lagouee.utils.IntentUtils
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.android.synthetic.main.activity_interview_schedule_b.*
import kotlinx.android.synthetic.main.layout_interview_calendar_change.*


/**
 * @date 创建时间：2020/7/21
 * @author 作者：codeman
 * @Description  描述：B端 新面试日程独立页面
 * @version
 */
@RouterRule(BInterviewScheduleActivity.URI)
class BInterviewScheduleActivity : BaseActivity() {

    private var interviewDays = listOf<String>()
    private var serverTimeStamp = System.currentTimeMillis()

    /**
     * schema指定的时间戳
     */
    private var schemaTimeStamp: Long? = null

    /***
     * 当前日历viewpager页码
     */
    private var currentIndex = -1


    private val eventFlush = Event { requestNextPage(true) }

    companion object {
        const val URI = "lagouhr://lagou.com/interview_schedule?[date]={date}"
    }

    init {
        if (SpUserNotClearUtils.getInstance().serverTimeStamp != 0L) {
            serverTimeStamp = SpUserNotClearUtils.getInstance().serverTimeStamp
        }
    }

    private var scheduleAdapter: BInterviewScheduleAdapter? = null
    private val interviewListRequest = InterviewNetManager.InterviewListRequest()
    private var selectCalendarDay: InterviewCalendarDayRes? = null



    private val onAdapterCommonListener = object :
            OnAdapterCommonListListenerImpl<List<Any>, HttpResponse<List<Any>>, Boolean>(this) {

        override fun onSuccess(t: List<Any>?, first: Boolean?) {
            super.onSuccess(t, first)
            showNormalStyle()
            recycle_interview.completeOnSuccess(true)
            first?.let { scheduleAdapter?.updateList(t, it) }
        }

        override fun onNoMore(t: List<Any>?, first: Boolean?) {
            super.onNoMore(t, first)
            showNormalStyle()
            recycle_interview.completeOnSuccess(false)
            first?.let { scheduleAdapter?.updateList(t, it) }
        }

        override fun onNoData(t: List<Any>?, first: Boolean?) {
            super.onNoData(t, first)
            showNormalStyle()
            recycle_interview.completeOnSuccess(false)
            first?.let { scheduleAdapter?.updateList(t, it) }
        }

        override fun onError(errorBean: HttpResponse<List<Any>>?) {
            super.onError(errorBean)
            showNormalStyle()
            if (isActive) {
                recycle_interview.completeOnFailure()
            }
        }

        override fun onNoInternet() {
            super.onNoInternet()
            if (isActive) {
                recycle_interview.completeOnFailure()
                wifi.visibility = View.VISIBLE
                coordinator_content.visibility = View.GONE
            }
        }
    }

    private val onInterviewTimesListener = object :
            OnAdapterCommonListenerImpl<InterviewTimeRes, HttpResponse<InterviewTimeRes>>(this) {

        override fun onSuccess(interviewTimes: InterviewTimeRes?) {
            super.onSuccess(interviewTimes)
            interviewTimes?.let {
                it.days?.apply {
                    interviewDays = this
                }
            }
            initInterviewCalendar()
        }

        override fun onError(errorBean: HttpResponse<InterviewTimeRes>?) {
            super.onError(errorBean)
            initInterviewCalendar()
        }

        override fun onNoInternet() {
            super.onNoInternet()
            initInterviewCalendar()
        }

        override fun isShowToast() = false
    }

    override fun getContentLayoutId() = R.layout.activity_interview_schedule_b

    override fun init() {
        if (RouteUtils.matchRule(intent, URI)) {
            schemaTimeStamp = RouteUtils.getQueryParameter(intent, "date", Long::class.javaPrimitiveType)
        }
        initRecycleView()
        initListener()
        //initInterviewCalendar()
        requestInterviewTimes()
        requestNextPage(true)

    }

    private fun initRecycleView() {
        recycle_interview.apply {
            layoutManager = LinearLayoutManager(context)
            itemAnimator = null
            addItemDecoration(InterviewDecoration(context))
            //面试日程列表点击，
            setOnItemClickListener { _, _, position ->
                AnalyticHelper.getInstance().sendClickInfo("1qqb")
                scheduleAdapter?.let {
                    val interviewBean: InterviewList.InterviewArrangeVosBean? =
                            scheduleAdapter?.getItem(position) as? InterviewList.InterviewArrangeVosBean
                    interviewBean?.apply {
                        IntentUtils.startInterviewDetailActivity(this@BInterviewScheduleActivity, id)
                    }
                }
            }

            setOnLoadMoreListener {
                // 加载更多
                requestNextPage(false)
            }

            setEmptyView(recycle_interview.generateChildView(R.layout.layout_no_interview_record_new))
            scheduleAdapter = BInterviewScheduleAdapter(context)
            adapter = scheduleAdapter
        }
    }

    private fun initListener() {
        ltitle_bar.setLeftImgListener {
            AnalyticHelper.getInstance().sendClickInfo("1qpu")
            finish()
        }
        wifi.setReLoadListener {
            requestNextPage(true)
        }
        calendar_fold_view.setIInterviewCalendar(object : InterviewCalenderPopup.IInterviewCalendar {
            override fun onScrollCalendar(
                    pageNum: Int,
                    calendarDays: List<InterviewCalendarDayRes>?,
                    month: String?, year: String?) {
                LogUtils.i("setIInterviewCalendar", "pageNum = $pageNum")
                tv_select_calendar_left.text = month
                tv_select_calendar.text = year

                if (currentIndex != -1){
                    if (pageNum < currentIndex){
                        handleLeftChange()
                    }else if (pageNum > currentIndex){
                        handleRightChange()
                    }
                }
                currentIndex = pageNum
            }

            override fun onFoldCalendar(
                    calendarDayRes: InterviewCalendarDayRes?,
                    calendarDays: List<InterviewCalendarDayRes>?) {
                selectCalendarDay = calendarDayRes
                requestNextPage(true)
            }
        })
        //calendar_fold_view.setLocationView(ltitle_bar)
        calendar_fold_view.setCalendarExpandView(CalenderExpandPopupFactory(InterviewCalenderPopup(
            this, ltitle_bar, calendar_fold_view
        )))
        frame_before_month.setOnClickListener {
            AnalyticHelper.getInstance().sendClickInfo("1qq6")
            calendar_fold_view.beforeCalendarMonth()
        }

        frame_after_month.setOnClickListener {
            AnalyticHelper.getInstance().sendClickInfo("1qq7")
            calendar_fold_view.afterCalendarMonth()
        }

        OkBus.getInstance()
                .register(BEventTags.INTERVIEW_INVITE_SUCCESS, eventFlush)
                .register(BEventTags.INTERVIEW_AGAIN_FINISH, eventFlush)
                .register(BEventTags.FLUSH_INTERVIEW_LIST, eventFlush)
    }

    private fun initInterviewCalendar() {
        calendar_fold_view?.fillUi(serverTimeStamp, interviewDays, schemaTimeStamp)
    }

    private fun showNormalStyle(){
        wifi.visibility = View.GONE
        coordinator_content.visibility = View.VISIBLE
    }


    /***
     * 请求下一页
     */
    private fun requestNextPage(firstPage: Boolean) {
        var interviewListReq = InterviewListReq()
        if (selectCalendarDay == null) {
            interviewListReq.startTime = TimeUtils.convertYearMonthDayTime(serverTimeStamp)
        } else {
            selectCalendarDay?.let { it2 ->
                interviewListReq.startTime = TimeUtils.convertYearMonthDayTime(it2.timeStamp)
            }
        }
        interviewListReq.endTime = TimeUtils.getGoalTime(1, interviewListReq.startTime)
        //interviewListReq.endTime = TimeUtils.getGoalTime(1000, interviewListReq.startTime)
        //interviewListReq.startTime = TimeUtils.getGoalTime(-1000, interviewListReq.startTime)
        interviewListRequest.getInterviewList(interviewListReq, false, firstPage,
                null, onAdapterCommonListener)
    }

    private fun requestInterviewTimes() {
        val interviewTimeReq = InterviewTimeReq()
        interviewTimeReq.startTime = TimeUtils.getGoalMonthStamp(serverTimeStamp, -6)
        interviewTimeReq.endTime = TimeUtils.getGoalMonthStamp(serverTimeStamp, 5)
        InterviewNetManager.getBInterviewTimes(interviewTimeReq, onInterviewTimesListener)
    }

    private fun handleRightChange() {
        if (calendar_fold_view.isMaxIndex()) {
            image_after_month.setImageResource(R.drawable.schedule_month_ic_arrowright_unclick)
            frame_after_month.isEnabled = false
        } else {
            image_after_month.setImageResource(R.drawable.schedule_month_ic_arrowright_nor)
            frame_after_month.isEnabled = true
        }

        image_before_month.setImageResource(R.drawable.schedule_month_ic_arrowleft_nor)
        frame_before_month.isEnabled = true
    }

    private fun handleLeftChange() {
        if (calendar_fold_view.isMinIndex()) {
            image_before_month.setImageResource(R.drawable.schedule_month_ic_arrowleft_unclick)
            frame_before_month.isEnabled = false
        } else {
            image_before_month.setImageResource(R.drawable.schedule_month_ic_arrowleft_nor)
            frame_before_month.isEnabled = true
        }

        image_after_month.setImageResource(R.drawable.schedule_month_ic_arrowright_nor)
        frame_after_month.isEnabled = true
    }


    override fun getPageId(): String {
        return "1qpt"
    }

    override fun onDestroy() {
        super.onDestroy()
        OkBus.getInstance().unRegister(eventFlush)
    }
}