package com.lagou.lagouee.ui.interview

import android.animation.ObjectAnimator
import android.os.Bundle
import android.os.Message
import android.view.View
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.ApiService
import com.alpha.lagouapk.R
import com.alpha.lagouapk.observer.EventTags
import com.alpha.lagouapk.ui.home.v2.refresh.HomeRefresh
import com.common.bean.json.InterviewCalendarDayRes
import com.common.bean.json.InterviewTimeRes
import com.common.utils.SpUserNotClearUtils
import com.common.widget.interview.CalenderExpandViewFactory
import com.common.widget.popup.InterviewCalenderPopup
import com.lagou.baselib.BaseFragment
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListListenerImpl
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListenerImpl
import com.lagou.commonlibs.event.Event
import com.lagou.commonlibs.event.OkBus
import com.lagou.commonlibs.utils.ListUtil
import com.lagou.commonlibs.utils.LogUtils
import com.lagou.commonlibs.utils.TimeUtils
import com.lagou.component.ui.emptyPage.DefaultEmptyLayout
import com.lagou.component.ui.refreshLayout.NewPullToRefreshLayout
import com.lagou.lagouee.bean.ChangeResumeState
import com.lagou.lagouee.bean.InterviewList.InterviewArrangeVosBean
import com.lagou.lagouee.bean.ResumeSnapshot
import com.lagou.lagouee.bean.jsonbean.interview.InterviewListReq
import com.lagou.lagouee.bean.jsonbean.interview.InterviewTimeReq
import com.lagou.lagouee.manager.net.InterviewNetManager
import com.lagou.lagouee.observer.BEventTags
import com.lagou.lagouee.ui.interview.adapter.BResumeManagerInterviewAdapter
import com.lagou.lagouee.ui.resume.list.ResumeBaseFragment
import com.lagou.lagouee.ui.resume.list.ResumeManagerFragment
import com.lagou.lagouee.ui.resume.list.interview.InterviewTabPageDecoration
import com.lagou.lagouee.utils.IntentUtils
import com.lagou.lagouee.widget.inter.DialogListener
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.android.synthetic.main.activity_interview_schedule_b.calendar_fold_view
import kotlinx.android.synthetic.main.fragment_interview_schedule_content.*
import kotlinx.android.synthetic.main.layout_interview_calendar_change.*


/**
 * @date 创建时间：2020/7/21
 * @author 作者：codeman
 * @Description  描述：B端 新面试日程独立页面
 * @version
 */
class BInterviewScheduleFragment : BaseFragment(),Event {

    private var calenderExpandViewFactory: CalenderExpandViewFactory? = null
    private var mSelectPositionId: Int = ResumeManagerFragment.ALL_POSITION_ID
    private var interviewDays = listOf<String>()
    private var serverTimeStamp = System.currentTimeMillis()
    private var mHomeRefresh: HomeRefresh? = null
    private var mIsLoading = false
    private var pastVisiblesItems = 0
    private var visibleItemCount:Int = 0
    private var totalItemCount:Int = 0
    private var listener: DialogListener? = null
    private var showLayerAlphaAnim: ObjectAnimator? = null
    private var hideLayerAlphaAnim: ObjectAnimator? = null

    /***
     * 当前日历viewpager页码
     */
    private var currentIndex = -1

    private val eventFlush = Event {
        //requestNextPage(true)
        startRefresh()
    }

    companion object {

        fun newInstance(
            type: Int,
            positionId: Int,
            mdsPositionId: Int
        ): BInterviewScheduleFragment {
            val bundle = Bundle()
            bundle.putInt(ResumeBaseFragment.KEY_TYPE, type)
            bundle.putInt(ResumeBaseFragment.KEY_POSITION_ID, positionId)
            bundle.putInt(ResumeBaseFragment.KEY_MDS_POSITION_ID, mdsPositionId)
            val fragment = BInterviewScheduleFragment()
            fragment.arguments = bundle
            return fragment
        }
    }

    init {
        if (SpUserNotClearUtils.getInstance().serverTimeStamp != 0L) {
            serverTimeStamp = SpUserNotClearUtils.getInstance().serverTimeStamp
        }
    }

    private var scheduleAdapter: BResumeManagerInterviewAdapter? = null
    private val interviewListRequest = InterviewNetManager.InterviewListRequest()
    private var selectCalendarDay: InterviewCalendarDayRes? = null
    private var mEmptyView: DefaultEmptyLayout? = null

    private val onAdapterCommonListener = object :
            OnAdapterCommonListListenerImpl<List<Any>, HttpResponse<List<Any>>, Boolean>(this) {

        override fun onSuccess(t: List<Any>?, first: Boolean?) {
            super.onSuccess(t, first)
            mIsLoading = false
            showNormalStyle()
            scheduleAdapter?.setFooterStatus(BResumeManagerInterviewAdapter.FOOTER_LOADING)
            //recycle_interview?.completeOnSuccess(true)
            first?.let { scheduleAdapter?.updateList(t, it) }
            stopRefreshing()
        }

        override fun onNoMore(t: List<Any>?, first: Boolean?) {
            super.onNoMore(t, first)
            mIsLoading = false
            showNormalStyle()
            scheduleAdapter?.setFooterStatus(BResumeManagerInterviewAdapter.FOOTER_NO_MORE)
            //recycle_interview?.completeOnSuccess(false)
            first?.let {
                scheduleAdapter?.updateList(t, it)
                if (ListUtil.isEmpty(t)) {
                    showEmptyView()
                }
            }
            stopRefreshing()
        }

        override fun onNoData(t: List<Any>?, first: Boolean?) {
            super.onNoData(t, first)
            mIsLoading = false
            showNormalStyle()
            scheduleAdapter?.setFooterStatus(BResumeManagerInterviewAdapter.FOOTER_NO_MORE)
            //recycle_interview?.completeOnSuccess(false)
            first?.let {
                scheduleAdapter?.updateList(t, it)
                if (ListUtil.isEmpty(t)) {
                    showEmptyView()
                }
            }
            stopRefreshing()
        }

        override fun onError(errorBean: HttpResponse<List<Any>>?) {
            super.onError(errorBean)
            mIsLoading = false
            showNormalStyle()
            if (isActive) {
                //recycle_interview?.completeOnFailure()
                if (scheduleAdapter?.isEmpty == true) {
                    //当前是空的
                    showNoNetView()
                } else {
                    scheduleAdapter?.setFooterStatus(BResumeManagerInterviewAdapter.FOOTER_ERROR)
                }
            }
            stopRefreshing()
        }

        override fun onNoInternet() {
            super.onNoInternet()
            if (isActive) {
                //recycle_interview?.completeOnFailure()
                showNoNetView()
                //coordinator_content.visibility = View.GONE
            }
            stopRefreshing()
        }
    }

    private val onInterviewTimesListener = object :
            OnAdapterCommonListenerImpl<InterviewTimeRes, HttpResponse<InterviewTimeRes>>(this) {

        override fun onSuccess(interviewTimes: InterviewTimeRes?) {
            super.onSuccess(interviewTimes)
            interviewTimes?.let {
                it.days?.apply {
                    interviewDays = this
                }
            }
            initInterviewCalendar()
        }

        override fun onError(errorBean: HttpResponse<InterviewTimeRes>?) {
            super.onError(errorBean)
            initInterviewCalendar()
        }

        override fun onNoInternet() {
            super.onNoInternet()
            initInterviewCalendar()
        }

        override fun isShowToast() = false
    }
    //private var recyclerView: LmRecyclerView? = null
    private val mClickPositionTitleEvent = Event {
        calenderExpandViewFactory?.hideAnim()
    }

    override fun getLayoutId(): Int {
        return R.layout.fragment_interview_schedule_content
    }

    override fun init() {
        OkBus.getInstance().register(BEventTags.RESUME_MANAGER_POSITION_TITLE_CLICK, mClickPositionTitleEvent)
        OkBus.getInstance().registerTags(this, EventTags.RESUME_MANAGER_CHANGE_POSITION,
                EventTags.FORCE_FLUSH_ALL_RESUME, EventTags.REQUEST_RESUMELIST_POSITION,
                EventTags.INVITE_OR_OUST_SUCCESS, EventTags.RESUME_MANAGER_STATE_CHANGE,
                BEventTags.RESUME_MANAGER_CLOSE_INTERVIEW_LAYER)

        initBundleData()
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initEmpty()
        initRecycleView()
        initListener()
        //initInterviewCalendar()
        requestInterviewTimes()
        //requestNextPage(true)
        startRefresh()
    }

    private fun initBundleData() {
        val args = arguments
        if (args != null) {
            mSelectPositionId = args.getInt(ResumeBaseFragment.KEY_POSITION_ID)
        }
    }

    private fun initEmpty() {
        mEmptyView = mContentView.findViewById(R.id.wifi_view)
        mEmptyView?.setImageResource(R.drawable.blank_job_360x292)
        mEmptyView?.setSubTitle("")
        mEmptyView?.setContent(getString(R.string.no_interview_today))
        mEmptyView?.setShowButton(false)
        mEmptyView?.setOnTouchListener { v, event -> true }
    }

    private fun initRecycleView() {
        refresh_layout?.apply {
            mHomeRefresh = HomeRefresh(mContext, refresh_layout, null)
            mHomeRefresh?.setListener(object:HomeRefresh.OnRefreshListener{
                override fun onRefresh() {
                    requestNextPage(true)
                }

                override fun onRefreshViewReset() {
                }

            })
            //解决pullToRefreshLayout的滑动冲突
            setOnChildScrollUpCallback(NewPullToRefreshLayout.OnChildScrollUpCallback setOnChildScrollUpCallback@{ parent: NewPullToRefreshLayout?, child: View? ->
                if (recycle_interview?.visibility == View.VISIBLE) {
                    return@setOnChildScrollUpCallback recycle_interview.canScrollVertically(-1)
                }
                false
            })
        }

        val mLayoutManager = LinearLayoutManager(baseActivity);
        recycle_interview?.apply {
            layoutManager = mLayoutManager
            itemAnimator = null
            addItemDecoration(InterviewTabPageDecoration(baseActivity))
            scheduleAdapter = BResumeManagerInterviewAdapter(baseActivity)
            adapter = scheduleAdapter
            scheduleAdapter?.setOnLoadMoreClickListener{
                // 点击重新加载更多
                requestNextPage(false)
            }
            //面试日程列表点击，
            scheduleAdapter?.setOnItemClickListener { interviewBean -> IntentUtils.startResumeActivityByResumeId(mContext, interviewBean.orderResumeId) }
            addOnScrollListener(object : RecyclerView.OnScrollListener() {
                override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
                    if (dy > 0) { //check for scroll down
                        visibleItemCount = mLayoutManager.getChildCount()
                        totalItemCount = mLayoutManager.getItemCount()
                        pastVisiblesItems = mLayoutManager.findFirstVisibleItemPosition()
                        if (!mIsLoading) {
                            if (visibleItemCount + pastVisiblesItems >= totalItemCount) {
                                onLoadMore();
                            }
                        }
                    }
                }
            })
        }
    }

    private fun onLoadMore() {
        if (scheduleAdapter?.status != BResumeManagerInterviewAdapter.FOOTER_NO_MORE) {
            scheduleAdapter?.setFooterStatus(BResumeManagerInterviewAdapter.FOOTER_LOADING)
            // 加载更多
            requestNextPage(false)
        }
    }

    private fun initListener() {
        wifi?.setReLoadListener {
            requestNextPage(true)
        }
        calendar_fold_view?.setBackgroundResource(R.drawable.shape_white_bl_br_r10);
        calendar_fold_view?.setIInterviewCalendar(object : InterviewCalenderPopup.IInterviewCalendar {
            override fun onScrollCalendar(
                    pageNum: Int,
                    calendarDays: List<InterviewCalendarDayRes>?,
                    month: String?, year: String?) {
                LogUtils.i("setIInterviewCalendar", "pageNum = $pageNum")
                tv_select_calendar_left?.text = month
                tv_select_calendar?.text = year

                if (currentIndex != -1) {
                    if (pageNum < currentIndex) {
                        handleLeftChange()
                    } else if (pageNum > currentIndex) {
                        handleRightChange()
                    }
                }
                currentIndex = pageNum
            }

            override fun onFoldCalendar(
                    calendarDayRes: InterviewCalendarDayRes?,
                    calendarDays: List<InterviewCalendarDayRes>?) {
                selectCalendarDay = calendarDayRes
                calendarDayRes?.let {
                    serverTimeStamp = it.timeStamp
                }
                //requestNextPage(true)
                startRefresh()
            }
        })
        calendar_expand_view?.setCalenderFoldView(calendar_fold_view)
        calendar_expand_view.setListener(listener)
        calendar_expand_view?.setTabLayerAnimation(showLayerAlphaAnim, hideLayerAlphaAnim)
        calenderExpandViewFactory = CalenderExpandViewFactory(calendar_expand_view)
        calendar_fold_view?.setCalendarExpandView(calenderExpandViewFactory)
        //calendar_fold_view.setLocationView(calendar_fold_view)
        frame_before_month.setOnClickListener {
            AnalyticHelper.getInstance().sendClickInfo("1qq6")
            calendar_fold_view?.beforeCalendarMonth()
        }

        frame_after_month?.setOnClickListener {
            AnalyticHelper.getInstance().sendClickInfo("1qq7")
            calendar_fold_view?.afterCalendarMonth()
        }
    }

    private fun initInterviewCalendar() {
        calendar_fold_view?.fillUi(serverTimeStamp, interviewDays, 0L)
    }

    private fun startRefresh() {
        refresh_layout?.postDelayed({ refresh_layout?.isRefreshing = true }, 200)
    }

    /**
     * 下拉刷新结束
     */
    private fun stopRefreshing() {
        if (refresh_layout?.isRefreshing == true) {
            refresh_layout?.postDelayed(Runnable {
                refresh_layout?.isRefreshing = false
            }, 100)
        }
    }

    private fun showNormalStyle(){
        recycle_interview?.let {
            if (it.visibility != View.VISIBLE) {
                it.visibility = View.VISIBLE
            }
        }
        mEmptyView?.let {
            if (it.visibility != View.GONE) {
                it.visibility = View.GONE
            }
        }
        wifi?.let {
            if (it.visibility != View.GONE) {
                it.visibility = View.GONE
            }
        }
    }

    private fun showEmptyView() {
        recycle_interview?.let {
            if (it.visibility != View.GONE) {
                it.visibility = View.GONE
            }
        }
        mEmptyView?.let {
            if (it.visibility != View.VISIBLE) {
                it.visibility = View.VISIBLE
            }
        }
        wifi?.let {
            if (it.visibility != View.GONE) {
                it.visibility = View.GONE
            }
        }
    }

    private fun showNoNetView() {
        recycle_interview?.let {
            if (it.visibility != View.GONE) {
                it.visibility = View.GONE
            }
        }
        mEmptyView?.let {
            if (it.visibility != View.GONE) {
                it.visibility = View.GONE
            }
        }
        wifi?.let {
            if (it.visibility != View.VISIBLE) {
                it.visibility = View.VISIBLE
            }
        }
    }

    /***
     * 请求下一页
     */
    private fun requestNextPage(firstPage: Boolean) {
        mIsLoading = true
        var interviewListReq = InterviewListReq()
        var isReverseOrder = false
        if (selectCalendarDay == null) {
            interviewListReq.startTime = TimeUtils.convertYearMonthDayTime(serverTimeStamp)
            isReverseOrder = true
        } else {
            selectCalendarDay?.let { it2 ->
                isReverseOrder = !it2.isBeforeToday || it2.isCurrentDay
                interviewListReq.startTime = TimeUtils.convertYearMonthDayTime(it2.timeStamp)
            }
        }
        interviewListReq.endTime = TimeUtils.getGoalTime(1, interviewListReq.startTime)
        //interviewListReq.endTime = TimeUtils.getGoalTime(1000, interviewListReq.startTime)
        //interviewListReq.startTime = TimeUtils.getGoalTime(-1000, interviewListReq.startTime)
        //first: Boolean, positionIds: ArrayList<Int>, type: Int,
        var positionIds: ArrayList<Int> = ArrayList();
        positionIds.add(mSelectPositionId)
        interviewListRequest.getInterviewList(interviewListReq, isReverseOrder, firstPage, positionIds, onAdapterCommonListener)
    }

    private fun requestInterviewTimes() {
        val interviewTimeReq = InterviewTimeReq()
        interviewTimeReq.startTime = TimeUtils.getGoalMonthStamp(serverTimeStamp, -6)
        interviewTimeReq.endTime = TimeUtils.getGoalMonthStamp(serverTimeStamp, 5)
        InterviewNetManager.getBInterviewTimes(interviewTimeReq, onInterviewTimesListener)
    }

    private fun handleRightChange() {
        if (calendar_fold_view == null || image_after_month == null) {
            return
        }
        if (calendar_fold_view.isMaxIndex()) {
            image_after_month.setImageResource(R.drawable.schedule_month_ic_arrowright_unclick)
            frame_after_month.isEnabled = false
        } else {
            image_after_month.setImageResource(R.drawable.schedule_month_ic_arrowright_nor)
            frame_after_month.isEnabled = true
        }

        image_before_month.setImageResource(R.drawable.schedule_month_ic_arrowleft_nor)
        frame_before_month.isEnabled = true
    }

    private fun handleLeftChange() {
        if (calendar_fold_view == null || frame_before_month == null) {
            return
        }
        if (calendar_fold_view.isMinIndex()) {
            image_before_month.setImageResource(R.drawable.schedule_month_ic_arrowleft_unclick)
            frame_before_month.isEnabled = false
        } else {
            image_before_month.setImageResource(R.drawable.schedule_month_ic_arrowleft_nor)
            frame_before_month.isEnabled = true
        }

        image_after_month.setImageResource(R.drawable.schedule_month_ic_arrowright_nor)
        frame_after_month.isEnabled = true
    }


    override fun getPageId(): String {
        return "1qpt"
    }

    override fun onDestroy() {
        super.onDestroy()
        OkBus.getInstance().unRegister(eventFlush)
        OkBus.getInstance().unRegister(mClickPositionTitleEvent)
        OkBus.getInstance().unRegister(this)
    }

    override fun call(msg: Message?) {
        if (msg == null) {
            return
        }
        when (msg.what) {
            EventTags.RESUME_MANAGER_CHANGE_POSITION -> applyChangePosition(msg)
            EventTags.FORCE_FLUSH_ALL_RESUME -> applyForceChangePosition(msg)
            EventTags.INVITE_OR_OUST_SUCCESS -> applyInviteOrOustSuccess(msg)
            EventTags.RESUME_MANAGER_STATE_CHANGE -> applyUpdateState(msg)
            BEventTags.RESUME_MANAGER_CLOSE_INTERVIEW_LAYER -> calenderExpandViewFactory?.hideAnim()
            else -> {
            }
        }
    }

    private fun applyChangePosition(msg: Message) {
        if (msg.obj !is Int) {
            return
        }
        val transferPositionId = msg.obj as Int
        if (mSelectPositionId == ResumeManagerFragment.ALL_POSITION_ID || mSelectPositionId != transferPositionId) {
            mSelectPositionId = transferPositionId
            //requestInterviewTimes()
            //requestNextPage(true)
            startRefresh()
        }
    }

    private fun applyForceChangePosition(msg: Message) {
        if (msg.obj !is Int) {
            return
        }
        mSelectPositionId = msg.obj as Int
        requestInterviewTimes()
        startRefresh()
    }

    private fun applyPositionFail() {

    }

    private fun applyInviteOrOustSuccess(msg: Message) {
        if (msg.obj !is ResumeSnapshot) {
            return
        }
        val snapshot = msg.obj as ResumeSnapshot
        val resumeId = snapshot.resumeId
        when (snapshot.type) {
            ResumeSnapshot.INTERVIEW_TYPE, ResumeSnapshot.INTERVIEW_AGAIN_TYPE, ResumeSnapshot.MODIFY_TIME_TYPE -> {
                //updateItemState(resumeId, ResumeState.STATE_RESUME_INTERVIEW)
                modifyInterviewInfo(snapshot)
            }
            ResumeSnapshot.OUST_TYPE -> removeItem(resumeId)
            else -> {
            }
        }
    }

    private fun applyUpdateState(msg: Message) {
        if (msg.obj is ChangeResumeState) {
            val crs = msg.obj as ChangeResumeState
            //updateItemState(crs.resumeId, crs.newStage)
            var resumeId = crs.resumeId
            scheduleAdapter?.updateItemById(resumeId, crs.oldStage, crs.newStage)
        }
    }

    private fun removeItem(resumeId: Long) {
        requestInterviewTimes()
        scheduleAdapter?.removeItem(resumeId)
    }

    private fun modifyInterviewInfo(snapshot: ResumeSnapshot) {
        requestInterviewTimes()
//        startRefresh()
        getInterviewArrangeVosBean(snapshot.resumeId, snapshot.parentPositionId, snapshot.interviewTime)
    }

    private fun getInterviewArrangeVosBean(orderResumeId: Long, positionId: Long, interviewTime: Long) {
        ApiService.api.getInterviewArrangeVosBean(orderResumeId, positionId, interviewTime).enqueue(object : BaseCallback<InterviewArrangeVosBean?>(false) {
            override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<InterviewArrangeVosBean?>?) {
                response?.let {
                    scheduleAdapter?.updateItemChanged(selectCalendarDay, it.content)
                }
            }
        })
    }

    fun setListener(listener: DialogListener?) {
        this.listener = listener
    }

    fun setTabLayerAnimation(
        showLayerAlphaAnim: ObjectAnimator?,
        hideLayerAlphaAnim: ObjectAnimator?
    ) {
        this.showLayerAlphaAnim = showLayerAlphaAnim
        this.hideLayerAlphaAnim = hideLayerAlphaAnim
    }
}