package com.lagou.lagouee.ui.mine

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.tools.BaseUtils
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.commonlibs.imageload.ImageLoadUtils
import com.lagou.commonlibs.utils.ListUtil
import kotlinx.android.extensions.LayoutContainer
import kotlinx.android.synthetic.main.item_b_my_page_setup.*

/**
 * @description ：V783 Sara需求 B端 我页 通用入口适配器
 * Created by @author YongliangWang on 2021/11/11 14:14
 */
class BMyPageAdapter(val context: Context?, val list: MutableList<SettingBean?>) : RecyclerView.Adapter<BMyPageAdapter.IconHolder>() {

    private val listRedDot: MutableList<Int> = mutableListOf()
    private val analyticShowIds: MutableList<String?> = mutableListOf()

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): IconHolder = IconHolder(LayoutInflater.from(context).inflate(R.layout.item_b_my_page_setup, parent, false))


    override fun getItemCount(): Int = if (list.isNullOrEmpty()) 0 else list.size

    override fun onBindViewHolder(holder: IconHolder, position: Int) {
        list[position]?.let { bean ->

            holder.let {
                ImageLoadUtils.with(context)
                        .load(if (bean.isOptTypeIntent) bean.picture else bean.pictureResId)
                        .into(it.iv_icon)
                it.tv_name.text = bean.name

                //默认隐藏两类红点，优先显示 tagPicture（如NEW）， 其次是 小圆点
                it.iv_tag_picture.visibility = View.GONE
                it.tv_red_dot.visibility = View.GONE
                it.tv_red_dot_num.visibility = View.GONE
                if (!bean.tagPicture.isNullOrEmpty()) {
                    it.iv_tag_picture.visibility = View.VISIBLE
                    ImageLoadUtils.with(context)
                            .load(bean.tagPicture)
                            .into(it.iv_tag_picture)
                }else if (bean.showNum > 0){
                   BaseUtils.showMsgRedDotNum(it.tv_red_dot_num,if (bean.showNum > 0) bean.showNum else -1)
                } else if (listRedDot.contains(position)) {
                    it.tv_red_dot.visibility = View.VISIBLE
                }
                it.rl_root_item_b_my_page.setOnClickListener { listener?.onClick(holder.adapterPosition) }
            }
            analyticShow(bean.pageId, bean.showId)
        }
    }

    private fun analyticShow(pageId: String?, showId: String?) {
        if (pageId.isNullOrEmpty() || showId.isNullOrEmpty()) {
            return
        }
        if (!analyticShowIds.contains(showId)) {
            AnalyticHelper.getInstance().sendShowInfo(pageId, showId)
            analyticShowIds.add(showId)
        }
    }

    fun setRedDotVis(positionValue: Int, isShow: Boolean) {
        if (!ListUtil.isListIndexLegal(list, positionValue)) {
            return
        }
        //特别注意：positionValue是list的下标（是listRedDot的元素），index才是listRedDot的下标；
        val index = listRedDot.indexOf(positionValue)
        if (isShow) {
            if (index < 0) {
                listRedDot.add(positionValue)//末位追加，那就只刷新末位
                notifyItemChanged(positionValue)
            }
        } else if (index >= 0) {
            listRedDot.remove(positionValue) //按照value删除，而不是index，注意区分removeAt(index)
            notifyItemChanged(positionValue)
        }
    }

    interface SettingOnListener {
        /**
         * @param position 下标
         */
        fun onClick(position: Int)
    }

    private var listener: SettingOnListener? = null

    fun setItemListener(listener: SettingOnListener?) {
        this.listener = listener
    }

    class IconHolder(override val containerView: View) : RecyclerView.ViewHolder(containerView), LayoutContainer

}