package com.lagou.lagouee.ui.mine

import android.content.Context
import android.os.Handler
import androidx.recyclerview.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.CheckBox
import android.widget.TextView
import com.ApiService
import com.alpha.lagouapk.R
import com.alpha.lagouapk.tools.BaseUtils
import com.alpha.lagouapk.ui.login.perfect.EditInfoCode
import com.common.chat.manager.GreetManager
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.chat.chatlibrary.bean.json.GreetInfo
import com.lagou.chat.chatlibrary.holder.GreetSwitchHolder
import com.lagou.commonlibs.utils.TextViewUtils
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse

class RefuseAdapter(val context: Context?, val list: MutableList<GreetInfo>, var autoRefuse: Boolean) : RecyclerView.Adapter<RecyclerView.ViewHolder>() {
    val inflater: LayoutInflater = LayoutInflater.from(context)
    val handler = Handler()

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        if (viewType == TEXT) {
            val view: View = inflater.inflate(R.layout.item_refuse, parent, false)
            val holder = ItemHolder(view)

            holder.cb?.setOnCheckedChangeListener { _, isChecked ->
                if (isChecked) {
                    if (null == curDefault) {
                        curDefault = holder
                        return@setOnCheckedChangeListener
                    }
                    //修改当次的bean为true
                    holder.mInfoBean?.isDefault = true
                    //修改前一个的bean为false
                    curDefault?.mInfoBean?.isDefault = false
                    //修改当次的checkbox为true
                    holder.cb.isChecked = true
                    //修改前一个的checkbox为false
                    curDefault?.cb?.isChecked = false
                    curDefault = holder
                    holder.mInfoBean?.let { updateDefault(it) }
                } else {
                    holder.mInfoBean?.isDefault = false
                    if (isNoneCheck()) { //如果一个都没有勾选, 则给最后操作的item和bean置为true状态(最少勾选一个)
                        holder.mInfoBean?.isDefault = true
                        holder.cb.isChecked = true
                        curDefault = holder
                    }
                }
            }
            //长按复制
            TextViewUtils.setLongClickCopy(holder.tvContent)
            holder.tvEdit?.setOnClickListener { com.alpha.lagouapk.tools.IntentUtils.startEditGreet(context, EditInfoCode.TYPE_GREET_REFUSE, holder.mInfoBean) }
            return holder
        } else {
            val holder = GreetSwitchHolder(inflater.inflate(R.layout.item_switch_greeting, parent, false))
            holder.bind(autoRefuse, true)
            holder.lineBottom.visibility = if (autoRefuse) View.VISIBLE else View.GONE
            holder.mSwitch.setOnStateChangeListener { isOn ->
                autoRefuse = isOn
                holder.tvSubTitle.text = if (autoRefuse) "选中项将在下次回复时生效" else "已关闭，系统将不再为你自动发出不合适回复"
                holder.lineBottom.visibility = if (autoRefuse) View.GONE else View.VISIBLE
                handler.post(::notifyDataSetChanged)
                GreetManager.getInstance().setAutoRefuse(autoRefuse)
            }
            return holder
        }
    }

    fun updateDefault(greetInfo: GreetInfo) {
        ApiService.api.updateRefuseReply(greetInfo).enqueue(object : BaseCallback<Any>(false) {
            override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<Any>?) {
            }

            override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<Any>?) {
                super.onFailure(request, response)
            }
        })
    }

    fun isNoneCheck() = null == list.find { it.isDefault }


    companion object {
        const val TEXT = 0
        const val SWITCH = 1
    }

    override fun getItemCount(): Int = if (!autoRefuse) 1 else list.size + 1

    override fun getItemViewType(position: Int): Int = if (0 == position) SWITCH else TEXT


    /**
     * 当前是默认文案的holder
     *
     * 直接记录holder，但这里不会存在复用导致不准确的问题
     */
    var curDefault: ItemHolder? = null
    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        if (holder is ItemHolder) {
            holder.bind(list[position - 1])
        } else if (holder is GreetSwitchHolder) {
            holder.bind(autoRefuse, true)
        }
    }

    inner class ItemHolder(view: View) : RecyclerView.ViewHolder(view) {
        val cb: CheckBox? = view.findViewById<CheckBox>(R.id.cb_item_refuse)
        val tvEdit: TextView? = view.findViewById<TextView>(R.id.tv_edit)
        val tvContent: TextView? = view.findViewById<TextView>(R.id.tv_item_refuse)
        var mInfoBean: GreetInfo? = null
        fun bind(refuseBean: GreetInfo) {
            this.mInfoBean = refuseBean
            tvContent?.text = BaseUtils.getHtmlStr(refuseBean.content)
            cb?.isChecked = refuseBean.isDefault
        }
    }
}