package com.lagou.lagouee.ui.mine.card

import android.content.Context
import android.util.AttributeSet
import android.view.Gravity
import android.view.LayoutInflater
import android.widget.LinearLayout
import com.alpha.lagouapk.R
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.definition.b.BAid
import com.lagou.baselib.analytic.definition.b.BPageId
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.utils.TextViewUtils
import com.lagou.lagouee.bean.UserData
import com.lagou.lagouee.bean.UserInfoBusinessBean
import com.lagou.lagouee.utils.IntentUtils
import kotlinx.android.synthetic.main.view_vip_flag.view.*
import pl.droidsonroids.gif.GifDrawable

/**
 * @date： Crate by wangjunwei on 2023/5/30
 * @description：B端我的页面自定义Vip bar
 **/
class BVipFlagView @JvmOverloads constructor(
    context: Context, attrs: AttributeSet? = null
) : LinearLayout(context, attrs) {

    /**
     * 等级图片
     *
     * 0为占位
     */
    private val gradeIds = intArrayOf(
        0,
        R.drawable.xxhdpi_vip_number_1,
        R.drawable.xxhdpi_vip_number_2,
        R.drawable.xxhdpi_vip_number_3,
        R.drawable.xxhdpi_vip_number_4,
        R.drawable.xxhdpi_vip_number_5,
        R.drawable.xxhdpi_vip_number_6
    )

    init {
        LayoutInflater.from(context).inflate(R.layout.view_vip_flag, this)
        gravity = Gravity.CENTER_VERTICAL
    }
    fun setUserData(userData: UserData?) {
        userData?.userInfoBusiness?.let {
            visibility = VISIBLE
            setRootInfo(it.isVipUser)
            setVipDrawable(it)
            setVipRightsInfo(it.vipRightsText)
            setVipButton(it.isVipUser, it.vipBtnText)
            setOnClickListener(it)
            return
        }
        visibility = GONE
    }

    private fun setOnClickListener(userInfoBusinessBean: UserInfoBusinessBean) {
        setOnClickListener {
            IntentUtils.startLagouWebViewActivity(context, userInfoBusinessBean.vipBtnJumpUrl)
            AnalyticHelper.getInstance().sendClickInfo(
                BPageId.fS00, if (userInfoBusinessBean.isBuyOnlineVip) "k1u3e" else BAid._1nw5
            )
        }
    }

    private fun setRootInfo(isVip: Boolean) {
        if (!isVip){
            setBackgroundResource(R.drawable.xxhdpi_mine_openvip_bg)
        }else {
            setBackgroundResource(R.drawable.xxhdpi_mine_vip_bg)
        }
        setPadding(
            paddingLeft,
            paddingTop,
            paddingRight,
            paddingBottom
        )
    }

    private fun setVipDrawable(infoBusinessBean: UserInfoBusinessBean) {
        if (infoBusinessBean.isHitHotPositionFlag) {
            mIvFlag.visibility = GONE
            mIvGrade.visibility = VISIBLE
            mIvGrade.setImageResource(R.drawable.package_ic_57)
            return
        }
        try {
            val gifDrawable = GifDrawable(
                resources, R.drawable.profile_ic_vip_b
            )
            gifDrawable.setSpeed(2.0f)
            mIvFlag.setImageDrawable(gifDrawable)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        mIvFlag.visibility = VISIBLE
        mIvGrade.visibility = if (infoBusinessBean.isHideVipLevel) INVISIBLE else VISIBLE
        mIvGrade.setImageResource(
            if (infoBusinessBean.isHideVipLevel) 0 else gradeIds[infoBusinessBean.userVipLevel]
        )
    }

    /**
     * 设置vip权益信息
     * @param vip
     * @param vipRightsText
     */
    private fun setVipRightsInfo(vipRightsText: String?) {
        tvVipRightsInfo.text = vipRightsText
    }

    /**
     * 设置按钮
     * <>开通VIP会员>
     * <>续费VIP会员>
     * <>查看会员特权>
     */
    private fun setVipButton(vip: Boolean, text: String?) {
        mTvOpen.text = text
        if (vip) {
            mTvOpen.setBackgroundResource(R.drawable.shape_gradient_d69950_e59a3f_r4)
            TextViewUtils.setRightDrawable(mTvOpen, null)
            mTvOpen.compoundDrawablePadding = 0
            mTvOpen.textSize = 11.0f
            mTvOpen.setTextColor(ResourceUtils.getColor(R.color.color_FFF4E4))
            mTvOpen.setPadding(
                ResourceUtils.getDimenSize(R.dimen.dp_7), mTvOpen.paddingTop,
                ResourceUtils.getDimenSize(R.dimen.dp_7), mTvOpen.paddingBottom
            )
        } else {
            mTvOpen.background = null
            TextViewUtils.setRightDrawable(mTvOpen, R.drawable.profile_ic_arrow_b)
            mTvOpen.compoundDrawablePadding = ResourceUtils.getDimenSize(R.dimen.dp_1_05)
            mTvOpen.setTextColor(ResourceUtils.getColor(R.color.color_8d5a27))
            mTvOpen.textSize = 13.0f
            mTvOpen.setPadding(0, mTvOpen.paddingTop, 0, mTvOpen.paddingBottom)
        }
    }
}