package com.lagou.lagouee.ui.position

import android.content.Context
import android.graphics.Rect
import android.util.AttributeSet
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import androidx.recyclerview.widget.RecyclerView.ItemDecoration
import androidx.recyclerview.widget.StaggeredGridLayoutManager
import com.alpha.lagouapk.R
import com.common.bean.TypeKeyValue
import com.lagou.commonlibs.utils.ResourceUtils


class PositionDescBubbleView @JvmOverloads constructor(
    private val mContext: Context,
    attributeSet: AttributeSet? = null,
    defAttarStyleId: Int = 0
) : LinearLayout(mContext, attributeSet, defAttarStyleId) {
    private lateinit var mRecyclerView: RecyclerView
    private lateinit var mAdapter:BubbleSuggestAdapter
    private var mIsStructBubble:Boolean = false//是否是结构性气泡
    init {
        orientation = LinearLayout.VERTICAL
        initRecyclerView()
        initTvDesc()
    }

    private fun initTvDesc() {
        val tvDesc = TextView(mContext)
        val params = LayoutParams(LayoutParams.WRAP_CONTENT,LayoutParams.WRAP_CONTENT)
        params.topMargin = ResourceUtils.getDimenSize(R.dimen.dp_10)
        params.leftMargin = ResourceUtils.getDimenSize(R.dimen.dp_15)
        tvDesc.apply {
            layoutParams = params
            textSize = 12f
            setTextColor(ResourceUtils.getColor(R.color.color_999))
            setSingleLine()
            text = "点击后文本中的高亮区域会被气泡内容替换"
        }
        addView(tvDesc)
    }

    private fun initRecyclerView() {
        mRecyclerView = RecyclerView(mContext)
        val dp_15 = ResourceUtils.getDimenSize(R.dimen.dp_15)
        mRecyclerView.setPadding(dp_15,0,dp_15,0)
        mRecyclerView.clipToPadding = false
        mRecyclerView.layoutParams = LayoutParams(LayoutParams.MATCH_PARENT,LayoutParams.WRAP_CONTENT)
        mRecyclerView.layoutManager = StaggeredGridLayoutManager(2,StaggeredGridLayoutManager.HORIZONTAL)
        mRecyclerView.addItemDecoration(BubbleItemDecoration(ResourceUtils.getDimenSize(R.dimen.dp_10)))
        mAdapter = BubbleSuggestAdapter(mContext)
        mRecyclerView.adapter = mAdapter
        addView(mRecyclerView)
    }
    @JvmOverloads
    fun setSuggest(typeKeyValue: TypeKeyValue?,isStructBubble:Boolean = false){
        this.mIsStructBubble = isStructBubble
        if (typeKeyValue == null){
            visibility = View.GONE
            return
        }
        visibility = View.VISIBLE
        val spanCount = if (typeKeyValue.values?.size == 1) 1 else 2
        (mRecyclerView.layoutManager as StaggeredGridLayoutManager).spanCount = spanCount
        mAdapter.setData(typeKeyValue)
    }

    inner class BubbleSuggestAdapter(val context: Context): RecyclerView.Adapter<BubbleSuggestAdapter.BubbleSuggestViewHolder>() {
        private val DP_14 = ResourceUtils.getDimenSize(R.dimen.dp_14)
        private val DP_5 = ResourceUtils.getDimenSize(R.dimen.dp_5)
        private var suggests:TypeKeyValue? = null

        fun setData(typeKeyValue:TypeKeyValue?){
            this.suggests = typeKeyValue
            notifyDataSetChanged()
        }
        inner class BubbleSuggestViewHolder(itemView:View):RecyclerView.ViewHolder(itemView)
        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BubbleSuggestViewHolder {
            val tvSuggest = TextView(context).apply {
                setTextColor(ResourceUtils.getColor(R.color.color_00b38a))
                textSize = 14f
                setPadding(DP_14,DP_5,DP_14,DP_5)
                background = ResourceUtils.getDrawable(R.drawable.shape_e0f5f1_r15)
            }
            return BubbleSuggestViewHolder(tvSuggest)
        }

        override fun onBindViewHolder(holder: BubbleSuggestViewHolder, position: Int) {
           val suggestContent = suggests?.values?.get(position)?.value
            (holder.itemView as? TextView)?.text = suggestContent
            holder.itemView.setOnClickListener {
               if (suggests != null && suggests!!.type.isNullOrEmpty().not() && suggestContent.isNullOrEmpty().not()){
                   onItemClick?.invoke(suggests!!.type!!,suggestContent!!,mIsStructBubble)
                   mRecyclerView.scrollToPosition(0)
               }
            }
        }

        override fun getItemCount() = suggests?.values?.size ?: 0
    }

    class BubbleItemDecoration(private val space:Int): ItemDecoration() {
        override fun getItemOffsets(
            outRect: Rect,
            view: View,
            parent: RecyclerView,
            state: RecyclerView.State
        ) {
            super.getItemOffsets(outRect, view, parent, state)
            val itemCount = parent.childCount
            if (itemCount == 0) {
                return
            }
            val itemPosition = parent.getChildAdapterPosition(view)
            if (itemPosition == RecyclerView.NO_POSITION) {
                return
            }
            val layoutManager = parent.layoutManager as? StaggeredGridLayoutManager ?: return
            val params = view.layoutParams as? StaggeredGridLayoutManager.LayoutParams ?: return

            itemOffsetsHorizontal(outRect, itemPosition, layoutManager, params)
        }

        private fun itemOffsetsHorizontal(
            outRect: Rect,
            itemPosition: Int,
            layoutManager: StaggeredGridLayoutManager,
            params: StaggeredGridLayoutManager.LayoutParams
        ) {
            val spanCount = layoutManager.spanCount
            val spanIndex = params.spanIndex
            val isFullSpan = params.isFullSpan

            when {
                isFullSpan -> {
                    outRect.top = 0
                    outRect.bottom = 0
                }
                spanIndex == 0 -> {
                    //first row
                    outRect.top = 0
                    outRect.bottom = space / 2
                }
                (spanIndex + 1) % spanCount == 0 -> {
                    //last row
                    outRect.top = space / 2
                    outRect.bottom = 0
                }
                else -> {
                    outRect.top = space / 2
                    outRect.bottom = space / 2
                }
            }
            if (itemPosition/spanCount > 0){
                //不是第一列
                outRect.left = space
            }
        }
    }

    fun setOnItemClickListener(onItemClick:((String,String,Boolean) -> Unit)?){
        this.onItemClick = onItemClick
    }
    //第一个参数是类型，第二个参数是suggest内容
    private var onItemClick:((String,String,Boolean) -> Unit)? = null
}