package com.lagou.lagouee.ui.position

import android.graphics.Color
import android.text.Spannable
import android.text.style.BackgroundColorSpan
import android.view.View
import android.widget.EditText
import android.widget.TextView
import com.common.bean.PositionDescAIDictionary
import com.common.bean.TypeKeyValue
import com.common.bean.TypeKeyValueObj
import com.common.manager.JanusManager
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.bean.json.ClickInfo
import com.lagou.commonlibs.engine.ThreadUtil
import com.lagou.lagouee.db.DbManager
import com.lagou.socketchannel.util.GsonUtil

/**
 * 职位描述提示管理类
 */
class PositionDescSuggestManager(
    private val editText: EditText,
    private val bubbleView: PositionDescBubbleView,
    private val tvWarnDescView: TextView
) {
    //输入关键词匹配的字典
    private var inputSuggestMap:LinkedHashMap<String,TypeKeyValue> = LinkedHashMap()
    //点击suggest时匹配的字典
    private var clickSuggestMap:LinkedHashMap<String,TypeKeyValue> = LinkedHashMap()
    private var minIndex = 0
    private var step = INIT_STEP
    private var startIndex = 0
    private var endIndex = 0
    private var backgroundColorSpan: BackgroundColorSpan = BackgroundColorSpan(Color.parseColor("#D0F3EC"))
    private var isInitShowWarn:Boolean = false
    private var maxStep = 0
    private var textWatcherIsOpen:Boolean = true
    private var isAStrategy: Boolean = true
    private var clickBubbleCount = 0
    private var maxKeyLength = 0//关键词最大长度
    companion object{
        const val INIT_STEP = 2
        const val MAX_STEP_20 = 20
        const val CHAR_MARKS = "，。！？、；：“”‘’（）【】《》——……~!@#$%^&*()-_+=\\|{}[];:'\",.<>/?\n"

    }
    init {
        isAStrategy = JanusManager.positionDescJanus.isA
        isInitShowWarn = tvWarnDescView.isShown
        initSuggestMap()
        bubbleView.setOnItemClickListener { type, suggestContent,isStructBubble ->
            AnalyticHelper.getInstance().sendClickInfo("t1u8q",if (isStructBubble) "w1u8t" else "x1u8u")
            textWatcherIsOpen = false
            clickBubbleCount++
            if(startIndex != endIndex){
                editText.editableText.replace(startIndex, endIndex, suggestContent)
            }else{
                editText.editableText.insert(editText.selectionEnd,suggestContent)
            }
            editText.editableText.removeSpan(backgroundColorSpan)
            setSuggestByClickBubble(type)
            textWatcherIsOpen = true
            resetStartEndIndex()
        }
    }
    private fun initSuggestMap() {
        ThreadUtil.io {
            val positionDescDictionaryStr =  DbManager.getInstance().getDataByKey(DbManager.KEY_B_POSITION_DESC_DICTIONARY)
            val positionDescDictionary:PositionDescAIDictionary? = GsonUtil.fromJson(positionDescDictionaryStr, PositionDescAIDictionary::class.java)
            val typeKeyValueObj = GsonUtil.fromJson(positionDescDictionary?.value, TypeKeyValueObj::class.java)
            val typeValue:List<TypeKeyValue>? = typeKeyValueObj?.typeKeyValue
            val typeKeyValue:List<TypeKeyValue>? = typeKeyValueObj?.keyWordKeyValue
            typeKeyValue?.asSequence()?.filter { it.key.isNullOrEmpty().not() }?.forEach {
                inputSuggestMap[it.key!!.toUpperCase()] = it
                maxKeyLength = maxKeyLength.coerceAtLeast(it.key!!.length)
            }
            typeValue?.asSequence()?.filter { it.type.isNullOrEmpty().not() }?.forEach {
                clickSuggestMap[it.type!!] = it
            }
        }
    }

    private fun loopFindSuggest():TypeKeyValue?{
        step = INIT_STEP
        if (isAStrategy.not()){
            calMinIndex()
            maxStep = MAX_STEP_20
            val spanCount = editText.selectionEnd - (if (minIndex > 0) minIndex + 1 else minIndex)
            if (spanCount > maxStep){
                //B策略，长度超过maxStep，就不在匹配
                return null
            }
        }else{
            minIndex = 0
            maxStep = maxKeyLength
        }
        var typeKeyValue:TypeKeyValue? = getSuggest()
        while ((typeKeyValue == null) && step < maxStep && startIndex > minIndex){
            step++
            typeKeyValue = getSuggest()
        }
        if (isAStrategy.not()){
            startIndex = if (minIndex > 0) minIndex + 1 else minIndex
        }
        return typeKeyValue
    }

    private fun getSuggest():TypeKeyValue?{
        endIndex = 0.coerceAtLeast(editText.selectionEnd)
        startIndex = 0
        if (endIndex - step >= 0){
            startIndex = endIndex - step
            val key = editText.editableText.subSequence(startIndex,endIndex).toString()
            val matchedKey = if (isAStrategy) key else inputSuggestMap.keys.find { key.contains(it, ignoreCase = true) }
            return inputSuggestMap[matchedKey?.toUpperCase()]
        }
        return null
    }

    private fun calMinIndex(){
        var beginIndex = editText.selectionEnd - 1
        while (beginIndex >= 0){
            if (editText.editableText[beginIndex] in CHAR_MARKS){
                minIndex = beginIndex
                return
            }
            beginIndex--
        }
        minIndex = 0
    }

    fun afterTextChanged() {
        if (textWatcherIsOpen.not()){
            return
        }
        show()
    }

    /**
     * 通过当前type查找下一个要显示的气泡
     * 如果type为空，则显示第一个
     */
    private fun setSuggestByClickBubble(type:String?){
        if (clickSuggestMap.isEmpty()){
            bubbleView.setSuggest(null,isStructBubble = true)
            return
        }
        if (type.isNullOrEmpty()){
            val firstKey = clickSuggestMap.keys.first()
            bubbleView.setSuggest(clickSuggestMap[firstKey],isStructBubble = true)
            return
        }
        val index = clickSuggestMap.keys.indexOf(type)
        if (index == -1 || (index + 1) == clickSuggestMap.keys.size){
            //未找到或者已经是最后一个
            bubbleView.setSuggest(null,isStructBubble = true)
            return
        }
        val nextKey = clickSuggestMap.keys.elementAt(index+1)
        bubbleView.setSuggest(clickSuggestMap[nextKey],isStructBubble = true)
    }

    /**
     * 该区域包含三部分①AI入口②关键词提示气泡③描述内容冲突提示
     * 展示策略如下
     * 无输入内容 -》请求AI入口  -》展示入口，隐藏②关键词提示气泡与③描述内容冲突；
     *                                   -》不展示入口，展示②关键词提示气泡，隐藏③描述内容冲突
     * 有输入内容 -》隐藏①AI入口-》若匹配到关键词，展示②关键词提示气泡，隐藏描述内容冲突提示
     *                         -》未匹配到关键词，隐藏②关键词提示气泡，如有描述内容冲突，则展示
     *
     */
    fun show(){
        val isInputEmpty = editText.text.isNullOrEmpty()
        if (isInputEmpty){
            resetStartEndIndex()
            tvWarnDescView.visibility = View.GONE
            setSuggestByClickBubble(null)
            return
        }
        bubbleView.visibility = View.VISIBLE
        val typeKeyValue = loopFindSuggest()
        if (typeKeyValue != null){
            editText.editableText.setSpan(backgroundColorSpan,startIndex,endIndex, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
            tvWarnDescView.visibility = View.GONE
        }else{
            tvWarnDescView.visibility = if (isInitShowWarn) View.VISIBLE else View.GONE
            editText.editableText.getSpans<Any>(0,editText.length(),Any::class.java)?.forEach {
                if (it is BackgroundColorSpan){
                    editText.editableText.removeSpan(it)
                }
            }
        }
        bubbleView.setSuggest(typeKeyValue)
    }
    private fun resetStartEndIndex(){
        startIndex = 0
        endIndex = 0
    }

    fun save() {
        val clickInfo = ClickInfo().setPage_id("t1u8q").setAddress_id("y1u8v").addClick_props("bubbleTimes",clickBubbleCount).addClick_props("ABTest",if (isAStrategy) "A" else "B")
        AnalyticHelper.getInstance().sendClickInfo(clickInfo)
    }
}