package com.lagou.lagouee.ui.position

import android.content.Context
import android.util.AttributeSet
import android.view.LayoutInflater
import android.widget.RelativeLayout
import com.alpha.lagouapk.R
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.bean.json.ClickInfo
import com.lagou.baselib.analytic.bean.json.ShowLogInfo
import com.lagou.commonlibs.utils.ListUtil
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.lagouee.bean.jsonbean.business.BusinessPropsList
import com.lagou.lagouee.bean.jsonbean.business.BusinessPropsList.BusinessGoodsBean
import com.lagou.lagouee.bean.jsonbean.position.BPositionDetail
import com.lagou.lagouee.utils.IntentUtils
import kotlinx.android.synthetic.main.layout_position_detail_type.view.*

/**
 * Created by mihao on 10/13/22
 * 职位详情类型卡片
 */
class PositionDetailTypeCard : RelativeLayout {

    constructor(context: Context) : this(context, null)

    constructor(context: Context, attrs: AttributeSet?) : this(context, attrs, 0)

    constructor(context: Context, attrs: AttributeSet?, defStyleAttr: Int) : super(
        context, attrs, defStyleAttr
    ) {
        LayoutInflater.from(context).inflate(R.layout.layout_position_detail_type, this)
    }

    fun setData(positionDetail: BPositionDetail) {
        tv_position_type.text = if(positionDetail.isShowCompetePositionTag) ResourceUtils.getString(R.string.compete_position) else positionDetail.typeTag
        tv_position_rights.text = if(positionDetail.isShowCompetePositionTag) ResourceUtils.getString(R.string.compete_position_expire_tips) else positionDetail.tipsContent
        tv_position_status.text =
            if (positionDetail.isOffline) ResourceUtils.getString(R.string.position_offline)
            else{
               if (positionDetail.isGrayCompeteTag) ResourceUtils.getString(R.string.expire) else  ResourceUtils.getString(R.string.position_online)
            }

        if (positionDetail.isOffline || positionDetail.isGrayCompeteTag) {
            setBackgroundResource(R.drawable.shape_f4f6f8_r4)
            tv_position_type.setTextColor(ResourceUtils.getColor(R.color.color_666666))
            tv_position_rights.setTextColor(ResourceUtils.getColor(R.color.color_666666))
            tv_position_status.setTextColor(ResourceUtils.getColor(R.color.color_999999))
            iv_vip_bg1.visibility = GONE
            iv_vip_bg2.visibility = GONE
            return
        }

        if (positionDetail.isPrivilegePosition || positionDetail.isSchoolPosition
            || positionDetail.isHotPositionPreview || positionDetail.isHotPosition || positionDetail.isRedCompeteTag
        ) {
            setBackgroundResource(R.drawable.tequan_bg1)
            tv_position_type.setTextColor(ResourceUtils.getColor(R.color.color_885630))
            tv_position_rights.setTextColor(ResourceUtils.getColor(R.color.color_905427))
            tv_position_status.setTextColor(ResourceUtils.getColor(R.color.color_99521a))
            iv_vip_bg1.visibility = VISIBLE
            iv_vip_bg2.visibility = VISIBLE
            return
        }

        setBackgroundResource(R.drawable.shape_e0f5f1_r4)
        tv_position_type.setTextColor(ResourceUtils.getColor(R.color.theme_text_color))
        tv_position_rights.setTextColor(ResourceUtils.getColor(R.color.theme_text_color))
        tv_position_status.setTextColor(ResourceUtils.getColor(R.color.theme_text_color))
        iv_vip_bg1.visibility = GONE
        iv_vip_bg2.visibility = GONE
    }

    fun setBusinessPropsListData(
        businessPropsList: BusinessPropsList?,
        positionDetail: BPositionDetail
    ) {
        if (businessPropsList == null || ListUtil.isEmpty(businessPropsList?.businessGoods?.filterNotNull())) {
            lin_ad_bar.visibility = GONE
            return
        }

        lin_ad_bar.visibility = VISIBLE
        val goodProps = businessPropsList?.businessGoods.filterNotNull()[0]
        showAnalytic(goodProps)

        tv_ad_txt.text = goodProps.content
        if (positionDetail.isCommonPosition) {
            //1 广告背景
            lin_ad_bar.background =
                ResourceUtils.getDrawable(R.drawable.shape_gradient_00b38a_5ccdb4_blbr_4)
            tv_ad_txt.setCompoundDrawablesWithIntrinsicBounds(
                ResourceUtils.getDrawable(R.drawable.xxhdpi_jd_purchase_ic),
                null,
                null,
                null
            )
        } else {
            lin_ad_bar.background =
                ResourceUtils.getDrawable(R.drawable.shape_gradient_e9a66d_dfab67_blbr_4)
            tv_ad_txt.setCompoundDrawablesWithIntrinsicBounds(
                ResourceUtils.getDrawable(R.drawable.jd_purchase_ic_yellow),
                null,
                null,
                null
            )
        }
        setOnClickListener {
            clickAnalytic(goodProps)
            IntentUtils.startLagouWebViewActivity(context, goodProps.buttonUrl)
        }
    }


    private fun showAnalytic(bean: BusinessGoodsBean) {
        val showLogInfo = ShowLogInfo()
        showLogInfo.setPage_Id("1qfq")
        showLogInfo.setAddress_Id("x1u84")
        showLogInfo.addShowProp("type", bean.type)
        AnalyticHelper.getInstance().sendShowInfo(showLogInfo)
    }


    private fun clickAnalytic(bean: BusinessGoodsBean) {
        val clickInfo = ClickInfo()
        clickInfo.page_id = "1qfq"
        clickInfo.address_id = "x1u84"
        clickInfo.addClick_props("type", bean.type)
        AnalyticHelper.getInstance().sendClickInfo(clickInfo)
    }

}