package com.lagou.lagouee.ui.position

import android.os.Bundle
import android.text.Html
import butterknife.OnClick
import com.alpha.lagouapk.R
import com.lagou.baselib.BaseKotlinFragment
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.bean.json.ClickInfo
import com.lagou.baselib.analytic.bean.json.ShowLogInfo
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListenerImpl
import com.lagou.commonlibs.event.Event
import com.lagou.commonlibs.event.OkBus
import com.lagou.commonlibs.inter.OnSingleMdTwoPsListener
import com.lagou.commonlibs.utils.ActivityStackHelper
import com.lagou.commonlibs.utils.NumberUtils
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.utils.ToastUtil
import com.lagou.component.ui.itemDecoration.CustomDividerItemDecoration
import com.lagou.lagouee.bean.*
import com.lagou.lagouee.manager.net.BusinessNetManager
import com.lagou.lagouee.observer.BEventTags
import com.lagou.lagouee.ui.HomeActivity
import com.lagou.lagouee.ui.adapter.BatchChatTalentAdapter
import com.lagou.lagouee.utils.IntentUtils
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.android.synthetic.main.fragment_publish_success_batch_chat.*

/**
 * Created by mihao on 10/20/22
 * 发布成功显示一键开聊
 */
class PublishSuccessBatchChatFragment : BaseKotlinFragment() {

    private var mTalentList: ArrayList<TalentCardBean>? = null
    private var batchChatTalentAdapter: BatchChatTalentAdapter? = null
    private var mBatchChatTalentBean: BatchChatTalentBean? = null
    private var mPublishInfo: PublishInfo? = null
    private var selectSet: HashSet<Int>? = null

    /**
     * 购买权益成功
     */
    private val mPaySuccessEvent = Event { getBusinessRights() }

    companion object {
        fun newInstance(
            publishInfo: PublishInfo,
            batchChatTalentBean: BatchChatTalentBean
        ): PublishSuccessBatchChatFragment {
            val args = Bundle()
            args.putParcelable(PublishInfo.KEY, publishInfo)
            args.putSerializable(PublishSuccessActivity.INTENT_KEY_BATCH_CHAT, batchChatTalentBean)
            val fragment = PublishSuccessBatchChatFragment()
            fragment.arguments = args
            return fragment
        }
    }

    override fun getLayoutId() = R.layout.fragment_publish_success_batch_chat

    override fun fillUi() {
        initRecyclerView()
        initData()
        OkBus.getInstance().register(BEventTags.PAY_SUCCESS_COMMUNICATE_USER, mPaySuccessEvent)
        reportShow()
    }

    private fun initRecyclerView() {
        mTalentList = ArrayList()
        batchChatTalentAdapter = BatchChatTalentAdapter(baseActivity, mTalentList)
        batchChatTalentAdapter!!.setOnItemCheckListener(OnSingleMdTwoPsListener { position: Int?, talentCardBean: TalentCardBean? ->
            if (talentCardBean != null) {
                if (talentCardBean.isChecked) {
                    selectSet!!.add(talentCardBean.userId)
                } else {
                    selectSet!!.remove(talentCardBean.userId)
                }
                updateClickText()
                updateRightsText()
            }
        })
        rv_batch_chat.adapter = batchChatTalentAdapter
        val itemDecoration = CustomDividerItemDecoration.newBuilder(baseActivity)
            .setLeftSpace(10).setRightSpacce(10)
            .setColor(ResourceUtils.getColor(R.color.color_e8e9eb))
            .setDividerWidth(0.5f).build()
        rv_batch_chat.addItemDecoration(itemDecoration)
        rv_batch_chat.setInitLoading(false)
    }

    private fun initData() {
        mPublishInfo = arguments?.getParcelable(PublishInfo.KEY) as PublishInfo?
        mBatchChatTalentBean =
            arguments?.getSerializable(PublishSuccessActivity.INTENT_KEY_BATCH_CHAT) as BatchChatTalentBean?
        if (mPublishInfo == null || mBatchChatTalentBean == null) {
            baseActivity?.onBackPressed()
            return
        }
        selectSet = HashSet()
        mTalentList!!.clear()
        mTalentList!!.addAll(mBatchChatTalentBean!!.showTalentList)
        batchChatTalentAdapter!!.setBatchChatData(mBatchChatTalentBean)
        batchChatTalentAdapter!!.notifyDataSetChanged()
        for (talentCardBean in mBatchChatTalentBean!!.showTalentList) {
            if (talentCardBean == null) {
                continue
            }
            if (talentCardBean.isChecked) {
                selectSet!!.add(talentCardBean.userId)
            }
        }
        tv_publish_success_title.text = getPublishSuccessTitle()
        tv_publish_success_desc.text = getPublishSuccessDesc()
        updateClickText()
        updateRightsText()
    }

    private fun getPublishSuccessTitle(): String {
        if (mPublishInfo?.isPrivilegePosition == true) {
            return ResourceUtils.getString(
                R.string.position_publish_success_title,
                ResourceUtils.getString(R.string.special_position)
            )
        }

        if (mPublishInfo?.isSchoolPosition == true) {
            return ResourceUtils.getString(
                R.string.position_publish_success_title,
                ResourceUtils.getString(R.string.school_position)
            )
        }

        if (mPublishInfo?.isHotPosition == true) {
            return ResourceUtils.getString(
                R.string.position_publish_success_title,
                ResourceUtils.getString(R.string.hot_position)
            )
        }
        if (mPublishInfo?.isCompetePosition == true){
            return ResourceUtils.getString(
                R.string.position_publish_success_title,
                ResourceUtils.getString(R.string.compete_position)
            )
        }

        return ResourceUtils.getString(
            R.string.position_publish_success_title,
            ResourceUtils.getString(R.string.common_position)
        )
    }

    private fun getPublishSuccessDesc(): String {
        if (mPublishInfo?.isPrivilegePosition == true || mPublishInfo?.isSchoolPosition == true
            || mPublishInfo?.isHotPosition == true || mPublishInfo?.isCompetePosition == true
        ) {
            return ResourceUtils.getString(R.string.position_publish_success_desc)
        }
        return ResourceUtils.getString(R.string.common_position_publish_success_desc)
    }

    /**
     * 设置底部按钮文案
     */
    private fun updateClickText() {
        val selectSize = selectSet!!.size
        tv_batch_chat.isEnabled = selectSize != 0
        tv_batch_chat.text =
            if (selectSize == 0) ResourceUtils.getString(R.string.talent_batch_chat_no_select)
            else ResourceUtils.getString(R.string.talent_batch_chat_select_num, selectSize)
    }

    private fun updateRightsText() {
        val remainRights = mBatchChatTalentBean!!.remainRights
        if (remainRights < selectSet!!.size) {
            tv_batch_chat_rights.text = Html.fromHtml(
                ResourceUtils.getString(
                    R.string.talent_batch_chat_rights_num_red,
                    NumberUtils.formatZero(remainRights)
                )
            )
        } else {
            tv_batch_chat_rights.text = Html.fromHtml(
                ResourceUtils.getString(
                    R.string.talent_batch_chat_rights_num_green,
                    NumberUtils.formatZero(remainRights)
                )
            )
        }
    }

    /**
     * 获取一键开聊权益信息
     */
    private fun getBusinessRights() {
        BusinessNetManager.getBusinessRights(BusinessNetManager.BaseGoodsId.BATCH_CHAT_ID, object :
            OnAdapterCommonListenerImpl<UserBusinessRightsBean, HttpResponse<UserBusinessRightsBean?>?>(
                this
            ) {
            override fun onSuccess(rightsBean: UserBusinessRightsBean) {
                mBatchChatTalentBean!!.remainRights = rightsBean.lastNum
                updateRightsText()
            }
        })
    }

    @OnClick(R.id.tv_position_home)
    fun goPositionHome() {
        IntentUtils.startPositionMain(baseActivity)
        ActivityStackHelper.clearAndFinishOtherActivity(HomeActivity::class.java)
    }

    @OnClick(R.id.tv_batch_chat)
    fun batchChatClick() {
        reportClick()
        BusinessNetManager.createBatchChat(getSubmitTalent(), mBatchChatTalentBean!!.lgPositionId,
            object :
                OnAdapterCommonListenerImpl<SubmitBatchChatBean, HttpResponse<SubmitBatchChatBean?>?>(
                    this
                ) {
                override fun onSuccess(submitBatchChatBean: SubmitBatchChatBean) {
                    if (submitBatchChatBean.isBuyStatus) {
                        IntentUtils.startLagouWebViewActivity(
                            baseActivity,
                            submitBatchChatBean.buyUrl
                        )
                        return
                    }
                    ToastUtil.show(
                        mContext,
                        ResourceUtils.getString(R.string.batch_chat_success_tips, selectSet!!.size)
                    )
                    baseActivity?.onBackPressed()
                }
            })
    }

    private fun getSubmitTalent(): String {
        val builder = StringBuilder()
        for (talentId in selectSet!!) {
            builder.append(talentId).append(",")
        }
        val index = builder.lastIndexOf(",")
        if (index >= 0) {
            builder.delete(index, builder.length)
        }
        return builder.toString()
    }

    override fun onDestroy() {
        super.onDestroy()
        OkBus.getInstance().unRegister(mPaySuccessEvent)
    }

    private fun reportShow() {
        val showLogInfo = ShowLogInfo()
        showLogInfo.setPage_Id("b1u1p")
        showLogInfo.setAddress_Id("d1u1r")
        showLogInfo.addShowProp("tal_card_num", mBatchChatTalentBean?.showTalentList?.size)
        AnalyticHelper.getInstance().sendShowInfo(showLogInfo)
    }

    private fun reportClick() {
        val clickInfo = ClickInfo()
        clickInfo.page_id = "b1u1p"
        clickInfo.address_id = "e1u1s"
        clickInfo.addClick_props("tal_chat_num", selectSet?.size)
        AnalyticHelper.getInstance().sendClickInfo(clickInfo)
    }
}