package com.lagou.lagouee.ui.position

import android.content.Intent
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.ui.dialog.PositionPosterSelectDialog
import com.common.adapter.BaseRecycleAdapter
import com.common.adapter.BaseViewHolder
import com.common.adapter.listener.OnItemClick
import com.lagou.baselib.BaseActivity
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.lagouee.bean.position.PositionCompeteExperience
import com.lagou.lagouee.utils.DialogUtils
import com.lagou.lagouee.utils.IntentUtils
import kotlinx.android.synthetic.main.activity_publish_success_compete_experience.recyle_view
import kotlinx.android.synthetic.main.activity_publish_success_compete_experience.title_bar
import kotlinx.android.synthetic.main.activity_publish_success_compete_experience.tv_publish_success
import kotlinx.android.synthetic.main.activity_publish_success_compete_experience.tv_rights_show
import kotlinx.android.synthetic.main.activity_publish_success_compete_experience.tv_see_position
import kotlinx.android.synthetic.main.activity_publish_success_compete_experience.tv_special_tips

/**
 * @date： Crate by wangjunwei on 2023/12/15
 * @description：竞招体验职位  发布成功
 **/

class PublishSuccessCompeteExperienceActivity : BaseActivity() {

    private var positionCompeteExperience: PositionCompeteExperience? = null
    private var positionId: Int = 0
    private var competeAdapter: CompeteExperienceAdapter? = null

    companion object {
        const val KEY_DATA_BEAN = "key_data_bean"
        const val KEY_POSITION_ID = "key_position_id"

        @JvmStatic
        fun startCompeteExperienceActivity(activity: BaseActivity, positionId: Int, positionCompeteExperience: PositionCompeteExperience?){
            val intent = Intent(activity, PublishSuccessCompeteExperienceActivity::class.java)
            intent.putExtra(KEY_DATA_BEAN, positionCompeteExperience)
            intent.putExtra(KEY_POSITION_ID, positionId)
            IntentUtils.startActivityNoException(activity, intent)
            activity.overridePendingTransition(R.anim.anim_down_in, R.anim.anim_down_out)
        }
    }

    override fun getContentLayoutId(): Int {
        return R.layout.activity_publish_success_compete_experience
    }

    override fun init() {
        positionCompeteExperience = intent.getSerializableExtra(KEY_DATA_BEAN) as? PositionCompeteExperience
        positionId = intent.getIntExtra(KEY_POSITION_ID, 0)
        val dividerItemDecoration = DividerItemDecoration(this, DividerItemDecoration.VERTICAL)
        dividerItemDecoration.setDrawable(ResourceUtils.getDrawable(R.drawable.shape_tran_size0x15))
        recyle_view.addItemDecoration(dividerItemDecoration)
        competeAdapter = CompeteExperienceAdapter()
        recyle_view.adapter = competeAdapter

        positionCompeteExperience?.apply {
            competeAdapter?.setData(contentList)
        }

        positionCompeteExperience?.apply {
            tv_publish_success.text = title
            tv_rights_show.text = subTitle
            tv_special_tips.text = tips
        }

        initListener()
    }

    private fun initListener() {
        title_bar.setLeftImgListener{
            IntentUtils.startPositionMain(this@PublishSuccessCompeteExperienceActivity)
            overridePendingTransition(R.anim.anim_down_in, R.anim.anim_down_out)
            finish()
        }

        title_bar.setRightImgListener {
            DialogUtils.showPositionPosterSelectDialog(this, PositionPosterSelectDialog.FROM_SCENE_REGULAR_SHARE,
                listOf(positionId), PositionDetailActivity.NO_LAGOU_ID
            )
        }
        tv_see_position.setOnClickListener {
            //查看职位，跳到职位管理页面
            IntentUtils.startPositionMain(this@PublishSuccessCompeteExperienceActivity)
            overridePendingTransition(R.anim.anim_down_in, R.anim.anim_down_out)
            finish()
        }
    }

    inner class CompeteExperienceAdapter :
        BaseRecycleAdapter<PositionCompeteExperience.ContentListBean, CompeteExperienceAdapter.CompeteExperienceHolder>(this) {

        inner class CompeteExperienceHolder(itemView: View) :
            BaseViewHolder<PositionCompeteExperience.ContentListBean>(itemView) {

            private val tvTitle = itemView.findViewById<TextView>(R.id.tv_title)
            private val tvSubTips = itemView.findViewById<TextView>(R.id.tv_sub_tips)

            override fun onBindViewHolder(
                view: View?,
                competeExperience: PositionCompeteExperience.ContentListBean?,
                position: Int,
                onItemClick: OnItemClick<PositionCompeteExperience.ContentListBean>?
            ) {
                competeExperience?.apply {

                    tvTitle.text = title
                    tvSubTips.text = subTitle
                }
            }

            override fun getTitle(): String {
                return "竞招体验期权益介绍"
            }
        }

        override fun onCreateViewHolder(p0: ViewGroup, p1: Int): RecyclerView.ViewHolder {
            val rootView = LayoutInflater.from(mActivity).inflate(R.layout.adapter_compete_experience, p0, false)
            return CompeteExperienceHolder(rootView)
        }
    }
}