package com.lagou.lagouee.ui.position.address

import android.app.Activity
import android.os.Bundle
import android.os.Message
import com.alpha.lagouapk.R
import com.alpha.lagouapk.observer.EventTags
import com.lagou.baselib.BaseKotlinFragment
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.bean.json.ClickInfo
import com.lagou.baselib.analytic.definition.b.BAid
import com.lagou.baselib.analytic.definition.b.BMid
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListenerImpl
import com.lagou.commonlibs.event.Event
import com.lagou.commonlibs.event.OkBus
import com.lagou.commonlibs.inter.OnSingleMethodListener
import com.lagou.commonlibs.utils.LogUtils
import com.lagou.commonlibs.utils.ToastUtil
import com.lagou.component.ui.dialog.listener.LgDialogListener
import com.lagou.lagouee.bean.jsonbean.company.CompanyAddress
import com.lagou.lagouee.manager.net.AddressNetManager
import com.lagou.lagouee.utils.DialogUtils
import com.lagou.lagouee.utils.IntentUtils
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.android.synthetic.main.fragment_address_list.*

/**
 * @date 创建时间：2020/6/19
 * @author 作者：codeman
 * @Description  描述：B端有地址数据样式
 * @version
 */
class AddressListFragment : BaseKotlinFragment() {

    private var defaultAddressId = 0

    private var mAddressAdapter: AddressAdapter? = null

    private var isCompany = false

    /***
     * fragment和Activity通信
     */
    private var singleListener: OnSingleMethodListener<List<CompanyAddress>>? = null
    private val flushAddressEvent = Event { requestAddressList() }

    private val delAddressEvent = Event { delAddress(it.obj as CompanyAddress) }
    private var addressCount = 0
    override fun getLayoutId() = R.layout.fragment_address_list

    companion object {
        const val MAX_ADDRESS_COUNT = 10
        fun getInstance(
            addressType: Int, isFromPositionEdit: Boolean,
            positionId: Int, defaultAddressId: Int
        ): AddressListFragment {
            var bundle = Bundle()
            bundle.apply {
                putInt(AddressListActivity.KEY_ADDRESS_TYPE, addressType)
                putBoolean(AddressListActivity.KEY_FROM_POSITION_EDIT, isFromPositionEdit)
                putInt(AddressListActivity.KEY_POSITION_ID, positionId)
                putInt(AddressListActivity.KEY_ADDRESS_ID, defaultAddressId)
            }
            var addressListFragment = AddressListFragment()
            addressListFragment.arguments = bundle
            return addressListFragment
        }
    }

    fun setActivityCallback(singleListener: OnSingleMethodListener<List<CompanyAddress>>) {
        this.singleListener = singleListener;
    }

    override fun fillUi() {
        var addressType = AddAddressActivity.TYPE_POSITION
        arguments?.apply {
            defaultAddressId = getInt(AddressListActivity.KEY_ADDRESS_ID)
            addressType = getInt(AddressListActivity.KEY_ADDRESS_TYPE)
            isCompany = addressType == AddAddressActivity.TYPE_COMPANY
            mAddressAdapter = AddressAdapter(
                baseActivity, addressType,
                getBoolean(AddressListActivity.KEY_FROM_POSITION_EDIT),
                getInt(AddressListActivity.KEY_POSITION_ID)
            )
        }
        if (mAddressAdapter == null) {
            mAddressAdapter = AddressAdapter(baseActivity)
        }
        mAddressAdapter?.setOnAddressOperateListener { companyAddress, position ->
            DialogUtils.showDelAddressDialog(childFragmentManager, object : LgDialogListener() {
                override fun onRight() {
                    super.onRight()
                    OkBus.getInstance().onEvent(EventTags.DEL_COMPANY_ADDRESS, companyAddress)
                    analytic2(isCompany, position)
                }
            })
        }
        lv_address.adapter = mAddressAdapter

        OkBus.getInstance().register(EventTags.DEL_COMPANY_ADDRESS, delAddressEvent)
            .register(EventTags.ADD_COMPANY_ADDRESS, flushAddressEvent)

        requestAddressList()
        btnAddAddress.setOnClickListener {
            if (mContext is AddressListActivity){
                AnalyticHelper.getInstance().sendClickInfo(pageId, if (isCompany) BAid._1yG0 else BAid._1xq0)
                if (isCompany && addressCount >= MAX_ADDRESS_COUNT) {
                    ToastUtil.show(mContext, resources.getString(R.string.company_address_most))
                    return@setOnClickListener
                }
                IntentUtils.startAddAddressActicity(mContext as Activity, AddAddressActivity.TYPE_ADD, addressType)
            }
        }
    }

    private fun requestAddressList() {
        btnAddAddress.isEnabled = false
        showLoading()
        AddressNetManager.getAddressList(isCompany,
            object :
                OnAdapterCommonListenerImpl<List<CompanyAddress>, HttpResponse<List<CompanyAddress>>>(
                    this
                ) {
                override fun onSuccess(companyAddressList: List<CompanyAddress>?) {
                    hideLoading()
                    btnAddAddress.isEnabled = true
                    addressCount = companyAddressList?.size ?: 0
                    singleListener?.onCallBack(companyAddressList)
                    companyAddressList?.apply {
                        //1 空列表时
                        if (size <= 0) {
                            return
                        }
                        //2 有数据时
                        if (defaultAddressId == -1) {
                            get(0)?.getPd_id()?.let {
                                mAddressAdapter?.setAddressList(
                                    companyAddressList,
                                    it
                                )
                            }
                        } else {
                            mAddressAdapter?.setAddressList(companyAddressList, defaultAddressId)
                        }
                        for (i in 0 until size) {
                            if (get(i)?.getPd_id() == defaultAddressId) {
                                val msg = Message()
                                msg.obj = get(i)
                                OkBus.getInstance().onEvent(EventTags.MOD_COMPANY_ADDRESS, msg)
                                break
                            }
                        }
                    }
                }
            })
    }

    private fun delAddress(companyAddress: CompanyAddress) {
        AddressNetManager.delAddress(isCompany, companyAddress,
            object : OnAdapterCommonListenerImpl<Any?, HttpResponse<Any>?>(this) {
                override fun onSuccess(t: Any?) {
                    mAddressAdapter?.removeAddress(companyAddress)
                    // 2018/4/4 change by lizheng 回掉给发布职位页面，删除选择地址
                    OkBus.getInstance()
                        .onEvent(EventTags.B_CLEAN_ADDRESS_IS_EMPTY, companyAddress.getPd_id())
                    addressCount = mAddressAdapter?.data?.size ?: 0
                    if (mAddressAdapter?.itemCount == null || mAddressAdapter?.itemCount == 0) {
                        singleListener?.onCallBack(mAddressAdapter?.data)
                    }
                }
            })
    }

    override fun onDestroy() {
        super.onDestroy()
        OkBus.getInstance()
            .unRegister(flushAddressEvent)
            .unRegister(delAddressEvent)
    }

    private fun analytic2(isCompany: Boolean, position: Int) {
        val info = ClickInfo()
            .setmId(if (isCompany) BMid._1yM0 else BMid._1xu0)
            .setAddress_id(if (isCompany) BAid._1yD0 else BAid._1xn0)
            .setSeq(position)
        AnalyticHelper.getInstance().sendClickInfo(info)
    }
}