package com.lagou.lagouee.ui.position.dialog

import android.app.Activity
import android.os.Bundle
import android.text.TextUtils
import androidx.recyclerview.widget.DividerItemDecoration
import com.alpha.lagouapk.R
import com.common.adapter.listener.OnItemClick
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListenerImpl
import com.lagou.commonlibs.ui.dialog.BaseBottomSheetDialogFragment
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.lagouee.bean.jsonbean.business.CouponReceiveBean
import com.lagou.lagouee.bean.position.RefuseCompeteFeedbackRes
import com.lagou.lagouee.manager.net.PositionNetManager
import com.lagou.lagouee.ui.adapter.RefuseCompetePayFeedbackAdapter
import com.lagou.lagouee.utils.DialogUtils
import com.lagou.socketchannel.http.HttpResponse
import kotlinx.android.synthetic.main.dialog_refuse_compete_pay_feedback.image_close
import kotlinx.android.synthetic.main.dialog_refuse_compete_pay_feedback.recycle_view
import kotlinx.android.synthetic.main.dialog_refuse_compete_pay_feedback.tv_submit

/**
 * @date： Crate by wangjunwei on 2024/1/11
 * @description：竞招职位拒绝购买反馈弹窗
 **/
class RefuseCompetePayFeedbackDialog : BaseBottomSheetDialogFragment() {

    private var refuseCompeteFeedbackRes: RefuseCompeteFeedbackRes? = null
    private var refuseCompetePayFeedbackAdapter: RefuseCompetePayFeedbackAdapter? = null

    override fun getLayoutId(): Int {
        return R.layout.dialog_refuse_compete_pay_feedback
    }

    companion object {
         const val INTENT_KEY_BEAN = "intent_key_bean"

        @JvmStatic
        fun newInstance(refuseCompeteFeedbackRes: RefuseCompeteFeedbackRes?): RefuseCompetePayFeedbackDialog {
            val dialog = RefuseCompetePayFeedbackDialog()
            val bundle = Bundle()
            bundle.putSerializable(INTENT_KEY_BEAN, refuseCompeteFeedbackRes)
            dialog.arguments = bundle
            return dialog
        }
    }

    override fun onAttach(activity: Activity) {
        super.onAttach(activity)
        val args = arguments
        if (args != null) {
            refuseCompeteFeedbackRes =
                args.getSerializable(INTENT_KEY_BEAN) as? RefuseCompeteFeedbackRes
        }
    }

//    override fun initConfigAtCreateView() {
//        super.initConfigAtCreateView()
//        if (!DialogShowManager.getInstance().checkShowNewDialogInActivity(activity, dialog)) {
//            dismiss()
//            return
//        }
//        dialog?.setCanceledOnTouchOutside(false)
//        isCancelable = false
//    }

    override fun fillUI() {
        super.fillUI()

        val itemDecoration = DividerItemDecoration(activity, DividerItemDecoration.VERTICAL)
        itemDecoration.setDrawable(ResourceUtils.getDrawable(R.drawable.shape_rect_translpant_9))
        recycle_view.addItemDecoration(itemDecoration)
        refuseCompetePayFeedbackAdapter = RefuseCompetePayFeedbackAdapter(activity)
        recycle_view.adapter = refuseCompetePayFeedbackAdapter

        refuseCompetePayFeedbackAdapter?.setData(refuseCompeteFeedbackRes?.content)

        refuseCompetePayFeedbackAdapter?.onItemClick =
            OnItemClick<RefuseCompeteFeedbackRes.ContentBean> { contentBean, position -> // 获取当前选中的反馈，判断提交按钮是否可以点击
                val selectFeedbacks = refuseCompetePayFeedbackAdapter?.getSelectBean()
                tv_submit.isEnabled = !selectFeedbacks?.filter { !TextUtils.isEmpty(it.decription) }.isNullOrEmpty()
                tv_submit.alpha = when (selectFeedbacks?.filter { !TextUtils.isEmpty(it.decription) }.isNullOrEmpty()) {
                    true -> 0.3f
                    false -> 1.0f
                }
            }
    }

    override fun initListener() {
        super.initListener()
        //1 关闭按钮
        image_close.setOnClickListener {
            dismiss()
        }

        //2 提交按钮
        tv_submit.setOnClickListener {
            val selectFeedbacks = refuseCompetePayFeedbackAdapter?.getSelectBean()
            PositionNetManager.submitCompeteRefuseFeedback(selectFeedbacks, object :
                OnAdapterCommonListenerImpl<CouponReceiveBean.CouponVoListBean,
                        HttpResponse<CouponReceiveBean.CouponVoListBean>>(activity as BaseActivity) {
                override fun onSuccess(couponVoListBean: CouponReceiveBean.CouponVoListBean?) {
                    super.onSuccess(couponVoListBean)
                    // 提交反馈成功，展示奖励弹窗
                    DialogUtils.showCompeteFeedbackGiftDialog(activity as BaseActivity, couponVoListBean)
                }
            })

            dismiss()
        }
    }
}