package com.lagou.lagouee.ui.position.release

import android.content.Context
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.widget.FrameLayout
import androidx.core.content.ContextCompat
import com.alpha.lagouapk.R
import com.lagou.lagouee.Constants
import kotlinx.android.synthetic.main.layout_position_custom_tag.view.*

/**
 * 职位关键词
 *
 * @author yph
 * @date 2020/6/16
 */
class TagView(context: Context, attrs: AttributeSet?) : FrameLayout(context, attrs) {
    private val dp15: Int = resources.getDimensionPixelSize(R.dimen.dp_15)
    private val dp4p5: Int = resources.getDimensionPixelSize(R.dimen.dp_4_05)
    private var style: Int = TAG_NONE;

    var text: String? = ""
        set(value) {
            field = value
            if (value == null) {
                tv_tag_custom?.text = ""
            } else {
                val array = value.split(Constants.AT)
                if (array.size == 2) {
                    tv_tag_custom?.text = array[0]
                } else {
                    tv_tag_custom.text = value
                }
            }
        }

    var onCloseListener: OnClickListener? = null

    init {
        LayoutInflater.from(context).inflate(R.layout.layout_position_custom_tag, this)
        iv_tag_custom?.setOnClickListener {
            onCloseListener?.onClick(it)
        }
    }

    override fun setSelected(selected: Boolean) {
        super.setSelected(selected)
        if (selected && (style == TAG_UNSELECTED || style == TAG_NONE)) {
            setStyle(TAG_SELECTED)
            return
        }
        if (!selected && (style == TAG_SELECTED || style == TAG_NONE)) {
            setStyle(TAG_UNSELECTED)
        }
    }

    fun setStyle(style: Int) {
        this.style = style
        when (style) {
            TAG_SELECTED -> {
                tv_tag_custom?.setTextColor(ContextCompat.getColor(context, R.color.color_00b38a))
                iv_tag_custom?.apply {
                    visibility = View.VISIBLE
                    setImageResource(R.drawable.register_ic_label_delete_green)
                }
                ll_tag_custom?.apply {
                    setBackgroundResource(R.drawable.shape_0a00b38a_stroke05_00b38a_r2)
                    isSelected = true
                    setPadding(dp15, dp4p5, 0, dp4p5)
                }
            }
            TAG_ADD -> {
                tv_tag_custom?.setTextColor(ContextCompat.getColor(context, R.color.color_00b38a))
                iv_tag_custom?.visibility = View.GONE
                ll_tag_custom?.apply {
                    setBackgroundResource(R.drawable.selector_filter_label_bg)
                    isSelected = false
                    setPadding(dp15, dp4p5, dp15, dp4p5)
                }
            }
            TAG_UNSELECTED -> {
                tv_tag_custom?.setTextColor(ContextCompat.getColor(context, R.color.color_666666))
                iv_tag_custom?.apply {
                    visibility = View.VISIBLE
                    setImageResource(R.drawable.register_ic_label_delete_gray)
                }
                ll_tag_custom?.apply {
                    setBackgroundResource(R.drawable.shape_f8f9fa_r2)
                    isSelected = false
                    setPadding(dp15, dp4p5, 0, dp4p5)
                }
            }
            TAG_SELECTED_TOP -> {
                tv_tag_custom?.setTextColor(ContextCompat.getColor(context, R.color.color_ffffff))
                iv_tag_custom?.apply {
                    visibility = View.VISIBLE
                    setImageResource(R.drawable.register_ic_label_delete_white)
                }
                ll_tag_custom?.apply {
                    setBackgroundResource(R.drawable.shape_00b38a_r1)
                    isSelected = false
                    setPadding(dp15, dp4p5, 0, dp4p5)
                }
            }
        }
    }


    companion object {
        const val TAG_NONE = 0;

        /**
         * 选中
         */
        const val TAG_SELECTED = 1

        /**
         * 添加关键词
         */
        const val TAG_ADD = 2

        /**
         * 未选中
         */
        const val TAG_UNSELECTED = 3

        /**
         * 选中展示在顶部
         */
        const val TAG_SELECTED_TOP = 4;
    }
}