package com.lagou.lagouee.ui.resume.card

import android.content.Context
import android.text.*
import android.text.method.LinkMovementMethod
import android.text.style.ClickableSpan
import android.text.style.ForegroundColorSpan
import android.text.style.UnderlineSpan
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.TextView
import androidx.core.content.ContextCompat
import com.alpha.lagouapk.R
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.commonlibs.utils.BaseUtils
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.utils.StringUtil
import com.lagou.lagouee.bean.jsonbean.resume.ResumeBaseInfo
import com.lagou.lagouee.bean.jsonbean.resume.ResumeBean
import com.lagou.lagouee.utils.DialogUtils
import kotlinx.android.synthetic.main.layout_b_resume_contact.view.*

/**
 * B端简历联系方式
 *
 * @author yph
 * @date 2019/8/13
 */

class BResumeContactCard(context: Context, attr: AttributeSet? = null) :
    FrameLayout(context, attr) {

    private var filterListener: FilterListener? = null
    private var virtualPhoneListener: VirtualPhoneListener? = null

    fun setFilterListener(listener: FilterListener?) {
        this.filterListener = listener
    }

    fun setVirtualPhoneListener(listener: VirtualPhoneListener?) {
        this.virtualPhoneListener = listener
    }

    init {
        LayoutInflater.from(context).inflate(R.layout.layout_b_resume_contact, this)
    }

    fun setData(bean: ResumeBean?) {
        if (bean == null || bean.baseInfo == null) {
            visibility = GONE
            return
        }

        if (bean.baseInfo.isUseVirtualPhone) {
            visibility = VISIBLE
            ll_resume_contact_unencrypted.visibility = GONE
            ll_resume_contact_encrypted.visibility = GONE
            rl_virtual_phone.visibility = VISIBLE
            setVirtualData(bean.baseInfo)
            return
        }

        if ((TextUtils.isEmpty(bean.baseInfo.contactTel) && TextUtils.isEmpty(bean.baseInfo.contactEmail))) {
            visibility = GONE
            return
        }
        visibility = VISIBLE
        rl_virtual_phone.visibility = GONE

        if (bean.isHideContact) {
            // 加密
            ll_resume_contact_unencrypted.visibility = GONE
            ll_resume_contact_encrypted.visibility = VISIBLE
            setEncryptedData(bean)
        } else {
            // 未加密
            ll_resume_contact_encrypted.visibility = GONE
            ll_resume_contact_unencrypted.visibility = VISIBLE
            setUnencryptedData(bean.baseInfo)
        }
    }

    /**
     * 设置虚拟号
     */
    private fun setVirtualData(baseInfo: ResumeBaseInfo) {
        iv_virtual_help_tips.setOnClickListener {
            DialogUtils.showVirtualPhoneRuleDialog(
                context as BaseActivity?,
                baseInfo.getxTelHelpTips()
            )
        }

        if (baseInfo.isNormalVirtual && !TextUtils.isEmpty(baseInfo.getxTel())) {
            tv_virtual_phone.text = baseInfo.getxTel()
            tv_virtual_phone.setOnClickListener {
                var tel = baseInfo.getxTel()
                tel = if (!tel.contains("-")) {
                    StringUtil.formatPhone(tel)
                } else {
                    StringUtil.formatPhoneBranch(tel)
                }
                BaseUtils.dialNumber(context, tel)
            }
            if (!TextUtils.isEmpty(baseInfo.getxTelBindTips())) {
                rl_virtual_bind_tips.visibility = VISIBLE
                tv_virtual_bind_tips.text = baseInfo.getxTelBindTips()
                iv_virtual_phone_edit.setOnClickListener {
                    virtualPhoneListener?.editPhone()
                }
            }
            return
        }

        if (baseInfo.isObtainVirtual) {
            tv_virtual_phone.text = ResourceUtils.getString(R.string.virtual_phone_obtain)
            tv_virtual_phone.setOnClickListener {
                virtualPhoneListener?.obtain()
            }
            rl_virtual_bind_tips.visibility = GONE
            return
        }

        if (baseInfo.isBuyVirtual) {
            val buyTips = ResourceUtils.getString(R.string.virtual_phone_buy)
            if (TextUtils.isEmpty(baseInfo.resumeDownloadCardGoodsUrl)) {
                tv_virtual_phone.text = buyTips
                tv_virtual_phone.setTextColor(ResourceUtils.getColor(R.color.color_666666))
            } else {
                val spannableString = SpannableString(buyTips)
                spannableString.setSpan(
                    ForegroundColorSpan(ResourceUtils.getColor(R.color.color_666666)),
                    0, spannableString.length - 4,
                    Spanned.SPAN_INCLUSIVE_EXCLUSIVE
                )
                tv_virtual_phone.text = spannableString
                tv_virtual_phone.setOnClickListener {
                    virtualPhoneListener?.buy(baseInfo.resumeDownloadCardGoodsUrl)
                }
            }
            rl_virtual_bind_tips.visibility = GONE
            return
        }

        visibility = GONE
    }

    /**
     * 未认证时，处于初筛状态，但是联系方式是加密的，此时需要隐藏此提示，此显示仍依赖于父控件是否显示
     */
    fun setFilterEncryptedTipVisible(visible: Boolean) {
        v_resume_contact_encrypted_filter?.visibility = if (visible) VISIBLE else GONE
        tv_resume_contact_encrypted_filter?.visibility = if (visible) VISIBLE else GONE
    }

    private fun setEncryptedData(bean: ResumeBean) {
        setText(
            tv_resume_contact_encrypted_phone,
            ll_resume_contact_encrypted_phone,
            bean.baseInfo.contactTel
        )
        setText(
            tv_resume_contact_encrypted_email,
            ll_resume_contact_encrypted_email,
            bean.baseInfo.contactEmail
        )

        if (!bean.isShowFilterTips) {
            setFilterEncryptedTipVisible(false)
            return
        }

        setFilterEncryptedTipVisible(true)

        val builder = SpannableStringBuilder("简历 ")
        val pre = builder.length
        builder.append("通过初筛")
        builder.setSpan(
            ForegroundColorSpan(
                ContextCompat.getColor(
                    context,
                    R.color.theme_text_color
                )
            ), pre, builder.length, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE
        )
        builder.setSpan(object : ClickableSpan() {
            override fun onClick(widget: View) {
                filterListener?.filter()
            }

            override fun updateDrawState(ds: TextPaint) {
                ds.isUnderlineText = false
            }
        }, pre, builder.length, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
        builder.append(" 即可获取完整联系方式")
        tv_resume_contact_encrypted_filter.apply {
            text = builder
            movementMethod = LinkMovementMethod.getInstance()
        }
    }

    private fun setUnencryptedData(baseInfo: ResumeBaseInfo) {
        setText(
            tv_resume_contact_unencrypted_phone,
            ll_resume_contact_unencrypted_phone,
            baseInfo.contactTel
        )
        setText(
            tv_resume_contact_unencrypted_email,
            ll_resume_contact_unencrypted_email,
            baseInfo.contactEmail
        )

        var tel = baseInfo.contactTel
        if (!TextUtils.isEmpty(tel)) {
            tv_resume_contact_unencrypted_phone.apply {
                val builder = SpannableStringBuilder()
                builder.append(tel)
                builder.setSpan(
                    UnderlineSpan(),
                    0,
                    builder.length,
                    Spannable.SPAN_EXCLUSIVE_EXCLUSIVE
                )
                text = builder
                setOnClickListener {
                    AnalyticHelper.getInstance().sendClickInfo("1q2m")
                    if (!tel.contains("-")) {
                        tel = StringUtil.formatPhone(tel)
                    }
                    // V7.29.0体验修改为跳转拨号界面
                    BaseUtils.dialNumber(context, tel)
                }
            }
        }
    }

    private fun setText(textView: TextView, group: ViewGroup, str: String?) {
        if (TextUtils.isEmpty(str)) {
            group.visibility = GONE
        } else {
            group.visibility = VISIBLE
            textView.text = str
        }
    }

    interface FilterListener {
        /**
         * 通过初筛
         */
        fun filter()
    }

    interface VirtualPhoneListener {
        /**
         * 获取虚拟号
         */
        fun obtain()

        /**
         * 重新购买
         */
        fun buy(url: String)

        /**
         * 编辑绑定电话
         */
        fun editPhone()
    }
}