package com.lagou.lagouee.ui.resume.dialog

import android.os.Bundle
import android.widget.TextView
import butterknife.BindView
import com.alpha.lagouapk.R
import com.common.utils.SpUserNotClearUtils
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.commonlibs.ui.dialog.BaseBottomSheetDialogFragment
import com.lagou.lagouee.bean.jsonbean.resume.ResumeInviteGuideRes
import com.lagou.lagouee.utils.IntentUtils

/**
 * @date： Crate by wangjunwei on 2023/5/5
 * @description：简历管理页-新简历列表触发批量邀约购买&消耗弹窗
 *               判断该账号下是否有批量邀约的权益，如没有，则需展示；
 *               判断该账号下是否有正在使用批量邀约，如没在使用，则需展示；
 *               展示规则：每周仅展示1次、仅在“新简历”这个tab下展示（大于等于6个新简历）
 *
 **/
class ResumeInviteGuideDialog : BaseBottomSheetDialogFragment() {

    @BindView(R.id.tb_invite_click)
    lateinit var tvInviteClick: TextView

    private var resumeInviteGuideRes: ResumeInviteGuideRes? = null
    private var mdsPositionId: Int? = null

    companion object {
        private const val KEY_RESUME_INVITE_GUIDE = "key_resume_invite_guide"
        private const val KEY_MDS_POSITION_ID= "key_mds_positionId"

        fun getInstance(resumeInviteGuideRes: ResumeInviteGuideRes?, mdsPositionId: Int?) : ResumeInviteGuideDialog{
            val bundle = Bundle()
            bundle.putSerializable(KEY_RESUME_INVITE_GUIDE, resumeInviteGuideRes)
            if (mdsPositionId != null) {
                bundle.putInt(KEY_MDS_POSITION_ID, mdsPositionId)
            }
            val resumeInviteDialog = ResumeInviteGuideDialog()
            resumeInviteDialog.arguments = bundle

            return resumeInviteDialog
        }
    }

    override fun getLayoutId(): Int {
       return R.layout.dialog_resume_invite
    }

    override fun initConfigAtCreateView() {
        super.initConfigAtCreate()
        //dialog?.setCanceledOnTouchOutside(false)
        //dialog?.setCancelable(false)
    }

    override fun fillUI() {
        super.fillUI()
        AnalyticHelper.getInstance().sendShowInfo("1ave", "b1u7h")

        mdsPositionId = arguments?.getInt(KEY_MDS_POSITION_ID)
        resumeInviteGuideRes = arguments?.getSerializable(KEY_RESUME_INVITE_GUIDE) as? ResumeInviteGuideRes?
        //记录简历管理页当前显示时间，作为下次显示与否判断的基础时间
        SpUserNotClearUtils.getInstance().setResumeManagerInviteTime()
    }

    override fun initListener() {
        super.initListener()
        tvInviteClick?.setOnClickListener {
            dismiss()
            AnalyticHelper.getInstance().sendClickInfo("1ave", "c1u7i")
            resumeInviteGuideRes?.apply {
                //1 跳转到批量邀约页面
                if (isToInvite()) {
                    mdsPositionId?.let {
                        if (it > 0) {
                            IntentUtils.startBatchInvite(context, it)
                            return@setOnClickListener
                        }
                    }
                }

                //2 跳转到发布职位页
                if (isToPublish()) {
                    IntentUtils.startReleaseActivity(context)
                    return@setOnClickListener
                }

                //3 跳转到职位管理页
                IntentUtils.startPositionMain(context)
                return@setOnClickListener
            }

            //3 跳转到职位管理页兜底
            IntentUtils.startPositionMain(context)
        }
    }

    override fun getWidth(): Int {
        return screenWidth
    }
}