package com.lagou.lagouee.ui.resume.dialog

import android.app.Activity
import android.graphics.Rect
import android.text.InputFilter
import android.text.TextUtils
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.common.adapter.BaseRecycleAdapter
import com.common.adapter.BaseViewHolder
import com.common.adapter.listener.OnItemClick
import com.lagou.commonlibs.inter.OnSingleMethodListener
import com.lagou.commonlibs.ui.dialog.BaseDialogFragment
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.utils.StringUtil
import com.lagou.commonlibs.utils.ToastUtil
import com.lagou.commonlibs.view.MaxInputWatcher
import com.lagou.commonlibs.view.filter.EmojiFilter
import com.lagou.lagouee.ui.resume.bean.OutReasonBean
import kotlinx.android.synthetic.main.dialog_resume_rpo_out_reason.btn_submit
import kotlinx.android.synthetic.main.dialog_resume_rpo_out_reason.edit_input_content
import kotlinx.android.synthetic.main.dialog_resume_rpo_out_reason.image_reason_close
import kotlinx.android.synthetic.main.dialog_resume_rpo_out_reason.recycle_view
import kotlinx.android.synthetic.main.dialog_resume_rpo_out_reason.tv_text_length

/**
 * @date： Crate by wangjunwei on 2023/8/18
 * @description：淘汰repo简历，填写原因的弹窗
 **/
class ResumeRpoOutReasonDialog : BaseDialogFragment(), MaxInputWatcher.OnMaxInputListener  {

    /**
     * 选择的原因
     */
    private var selectReason = ""
    private lateinit var outReasonAdapter: OutReasonAdapter

    private val LIMIT_MAX_COUNT = 100
    private val SPAN_COUNT = 3
    private val DP_0 = 0
    private val DP_5 = ResourceUtils.getDimenSize(R.dimen.dp_5)
    private val DP_20 = ResourceUtils.getDimenSize(R.dimen.dp_20)
    private lateinit var callback: OnSingleMethodListener<String>
    private val outReasonSource = listOf(OutReasonBean("基本信息不符"),
        OutReasonBean("教育经历不符"), OutReasonBean("个人履历不符"))

    companion object {

        @JvmStatic
        fun getInstance(callback: OnSingleMethodListener<String>): ResumeRpoOutReasonDialog{
            val resumeRpoOutReasonDialog = ResumeRpoOutReasonDialog()
            resumeRpoOutReasonDialog.callback = callback
            return resumeRpoOutReasonDialog
        }
    }

    override fun getLayoutId(): Int {
        return R.layout.dialog_resume_rpo_out_reason
    }

    override fun initListener() {
        super.initListener()
        image_reason_close.setOnClickListener { dismiss() }
        btn_submit.setOnClickListener {
            //1 空内容检查 未选择原因、也未填写
            if (TextUtils.isEmpty(selectReason) && TextUtils.isEmpty(edit_input_content.text)) {
                ToastUtil.show(activity, "请选择或填写不合适原因")
                return@setOnClickListener
            }

            //2 淘汰原因提交到接口
            callback.onCallBack(StringUtil.jointString("，", selectReason,
                edit_input_content.text.toString()))
            dismiss()
        }
    }

    override fun fillUI() {
        super.fillUI()
        //1 控制输入框，字数限制、限制输入emjo表情
        edit_input_content.addTextChangedListener(MaxInputWatcher(LIMIT_MAX_COUNT)
            .setMaxInputListener(this).setIsCalTextLength(true).setCalNewLine(false))
        edit_input_content.filters = arrayOf<InputFilter>(EmojiFilter())
        showLengthTip(0, "")

        //2 拒绝原因-选择方框列表
        outReasonAdapter = OutReasonAdapter(activity)
        recycle_view.apply {
            layoutManager = GridLayoutManager(activity,SPAN_COUNT)
            addItemDecoration(OutItemDecoration())
            adapter = outReasonAdapter
        }
        outReasonAdapter.onItemClick =
            OnItemClick<OutReasonBean> { outReason, position ->
                selectReason = outReason.selectReasons
            }
        outReasonAdapter.setData(outReasonSource)
    }

   inner class OutReasonAdapter(activity: Activity?) : BaseRecycleAdapter<OutReasonBean, OutReasonAdapter.OutReasonViewHolder>(activity) {

       inner class OutReasonViewHolder(itemView: View?) : BaseViewHolder<OutReasonBean>(itemView) {
            override fun onBindViewHolder(
                view: View?,
                outReasonBean: OutReasonBean,
                position: Int,
                onItemClick: OnItemClick<OutReasonBean>?
            ) {
                val tvView = view as TextView
                tvView.text = outReasonBean.name
                tvView.isSelected = outReasonBean.isSelected

                tvView.setOnClickListener {
                    //2 设置新的选中项
                    outReasonBean.isSelected = !outReasonBean.isSelected
                    notifyItemChanged(position, "single")
                    outReasonBean.selectReasons =  StringUtil.jointListString("，",
                        mList.filter { it.isSelected }?.map { it.name })

                    //3 回调使用方
                    onItemClick?.onClick(outReasonBean, position)
                }
            }

            override fun getTitle(): String {
               return "可选择的拒绝原因"
            }
        }

        override fun getItemViewType(position: Int): Int {
            return R.layout.item_company_filter_default
        }

        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
            val rootView: View = LayoutInflater.from(activity).inflate(viewType, parent, false)
            return OutReasonViewHolder(rootView)
        }
    }

   inner class OutItemDecoration : RecyclerView.ItemDecoration() {
        override fun getItemOffsets(
            outRect: Rect,
            view: View,
            parent: RecyclerView,
            state: RecyclerView.State
        ) {
            super.getItemOffsets(outRect, view, parent, state)
            val currentPosition = parent.getChildAdapterPosition(view)

            val totalCount = state.itemCount
            //1 left和right设置
            if (currentPosition % SPAN_COUNT == 0){
                outRect.left = DP_0
                outRect.right = DP_5
            }else if (currentPosition % 3 == 1){
                //中间的一个
                outRect.left = DP_5
                outRect.right = DP_5
            }else {
                //最有边的
                outRect.left = DP_5
                outRect.right = DP_0
            }

            //2 特殊情况处理-只有一行的情况，处理top-bottom
            if (totalCount <= SPAN_COUNT){
                outRect.top = DP_20
                outRect.bottom = DP_20
                return
            }

            //3 大于一行-top和bottom 设置 - 第一行
            if (currentPosition / SPAN_COUNT == 0){
                outRect.top = DP_20
                outRect.bottom = DP_5
            }else if (currentPosition >= totalCount - SPAN_COUNT) {
                //最后一行
                outRect.top = DP_5
                outRect.bottom = DP_20
            }else{
                //其它行
                outRect.top = DP_5
                outRect.bottom = DP_5
            }
        }
    }

    override fun getGravity(): Int {
        return Gravity.BOTTOM
    }

    override fun equalMaxLength(isEqual: Boolean) {

    }

    override fun onTextChanged(s: String?) {
    }

    override fun showLengthTip(length: Int, text: String?) {
        tv_text_length.text = "$length/$LIMIT_MAX_COUNT"
    }
}