package com.lagou.lagouee.ui.resume.mail

import com.ApiService
import com.common.utils.PageHelper
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.commonlibs.utils.presenter.WeakPresenter
import com.lagou.lagouee.bean.ColleagueList
import com.lagou.lagouee.bean.MailItem
import com.lagou.lagouee.bean.MailRecord
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse

class MailSharePresenter : WeakPresenter<MailShareContract.View>(), MailShareContract.Presenter {

    private var mPage: Int = 0

    override fun getMergeList(loading: Boolean) {
        ifViewAttached(loading) {
            it.showLoading()
        }

        val flag = arrayOf(false, false)
        val result = arrayOf<Any?>(null, null)
        mPage = 1
        ApiService.api.getColleagueList(mPage, PageHelper.COUNT_20).enqueue(object : BaseCallback<ColleagueList?>() {
            override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<ColleagueList?>?) {
                val list = response?.content?.result ?: ArrayList<ColleagueList.ResultBean?>(0)
                val hasNext = response?.content?.isHasNextPage ?: false
                flag[0] = true
                result[0] = Pair(list, hasNext)
                checkMerge(flag, result)
            }

            override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<ColleagueList?>?) {
                super.onFailure(request, response)
                flag[0] = true
                checkMerge(flag, result)
            }

        })

        ApiService.api.mailRecordList.enqueue(object : BaseCallback<List<MailRecord?>?>() {
            override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<List<MailRecord?>?>?) {
                flag[1] = true
                result[1] = response?.content ?: ArrayList<MailRecord?>(0)
                checkMerge(flag, result)
            }

            override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<List<MailRecord?>?>?) {
                super.onFailure(request, response)
                flag[1] = true
                checkMerge(flag, result)
            }
        })

    }

    @Suppress("UNCHECKED_CAST")
    private fun checkMerge(flag: Array<Boolean>, result: Array<Any?>) {
        ifViewAttached(flag[0] && flag[1]) {
            it.hideLoading()
            if (result[0] == null || result[1] == null) {
                // 一旦网络错误即报失败
                it.showError()
            } else {
                val pair = result[0] as Pair<List<ColleagueList.ResultBean?>, Boolean>
                val recent = result[1] as List<MailRecord?>
                val list = arrayListOf<MailItem>()
                // 先添加最近使用
                recent.apply {
                    if (!isEmpty()) {
                        list.add(MailItem(MailItem.SINGLE_TITLE, RECENT))
                        for (bean in this) {
                            if (bean != null) {
                                list.add(MailItem.transform(bean))
                            }
                        }
                    }
                }
                // 其次同事列表
                pair.first.apply {
                    if (!isEmpty()) {
                        list.add(MailItem(MailItem.SINGLE_TITLE, COLLEAGUE))
                        for (bean in this) {
                            if (bean != null) {
                                list.add(MailItem.transform(bean))
                            }
                        }
                    }
                }

                it.showLoadSuccess(true, pair.second, list)

                if (pair.second) {
                    mPage++
                }
            }
        }

    }

    override fun getMoreColleagueList() {
        ApiService.api.getColleagueList(mPage, PageHelper.COUNT_20).enqueue(object : BaseCallback<ColleagueList?>() {
            override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<ColleagueList?>?) {
                val list = response?.content?.result ?: ArrayList<ColleagueList.ResultBean?>(0)
                val hasNext = response?.content?.isHasNextPage ?: false
                val mailList = list.filterNotNull().map { MailItem.transform(it) }
                ifViewAttached {
                    it.showLoadSuccess(false, hasNext, mailList)
                    if (hasNext) {
                        mPage++
                    }
                }
            }

            override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<ColleagueList?>?) {
                super.onFailure(request, response)
                ifViewAttached {
                    it.showLoadMoreFailure()
                }
            }

        })
    }

    override fun refreshRecentList() {
        ApiService.api.mailRecordList.enqueue(object : BaseCallback<List<MailRecord?>?>(false) {
            override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<List<MailRecord?>?>?) {
                ifViewAttached {
                    response?.content?.run {
                        val list: ArrayList<MailItem>
                        if (isEmpty()) {
                            list = ArrayList(0)
                        } else {
                            list = ArrayList(size + 1)
                            list.add(MailItem(MailItem.SINGLE_TITLE, RECENT))
                            for (bean in this) {
                                if (bean != null) {
                                    list.add(MailItem.transform(bean))
                                }
                            }
                        }
                        it.updateRecentList(list)
                    }
                }
            }
        })
    }

    companion object {
        const val RECENT: String = "最近使用"
        const val COLLEAGUE: String = "你的同事"
    }

}
