package com.lagou.lagouee.ui.resume.mail

import android.text.Editable
import android.text.TextUtils
import android.view.View
import com.alpha.lagouapk.R
import com.alpha.lagouapk.tools.BaseUtils
import com.common.widget.CustomTextWatcher
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.util.BarUtil
import com.lagou.commonlibs.event.OkBus
import com.lagou.commonlibs.utils.ToastUtil
import com.lagou.lagouee.bean.MailItem
import com.lagou.lagouee.bean.ResumeShareParams
import com.lagou.lagouee.observer.BEventTags
import kotlinx.android.synthetic.main.activity_mail_share_search.*

/**
 * 邮箱转发搜索
 *
 * @author yph
 * @date 2019/9/30
 */
class MailShareSearchActivity : BaseActivity(), MailSearchContract.View, MailShareDialog.Callback {
    private var mParams: ResumeShareParams? = null
    private val mPresenter by lazy(LazyThreadSafetyMode.NONE) {
        MailSearchPresenter()
    }

    private val mAdapter by lazy(LazyThreadSafetyMode.NONE) {
        MailShareAdapter(this)
    }

    override fun getContentLayoutId(): Int = R.layout.activity_mail_share_search

    override fun init() {
        mParams = intent.getSerializableExtra(ResumeShareParams.TAG) as? ResumeShareParams
        mPresenter.attachView(this)
        initListener()
        rv_mail_search.apply {
            adapter = mAdapter
            setInitLoading(false)
        }
        BarUtil.keyboardEnable(this)
    }

    private fun initListener() {
        rv_mail_search.setOnItemClickListener { _, _, position ->
            val item = mAdapter.getItem(position)
            val params = mParams
            if (item == null || item.type == MailItem.SINGLE_TITLE
                    || params == null) {
                return@setOnItemClickListener
            }
            if (TextUtils.isEmpty(item.email)) {
                ToastUtil.show(this, R.string.no_email_tip)
                return@setOnItemClickListener
            }
            MailShareDialog(item, params).show(supportFragmentManager)
        }
        rv_mail_search.setOnLoadMoreListener {
            val keyword = et_mail_search.text?.toString()?.trim() ?: ""
            if (!TextUtils.isEmpty(keyword)) {
                mPresenter.getSearchList(keyword, false, rv_mail_search.makeLabel())
            }
        }

         et_mail_search.addTextChangedListener(object : CustomTextWatcher() {
            override fun afterTextChanged(s: Editable?) {
                iv_mail_search_del.visibility = if (s?.length ?: 0 > 0) View.VISIBLE else View.GONE
                refresh()
            }
        })
        et_mail_search.postDelayed({
            BaseUtils.showKeyboard(this, et_mail_search)
        }, 100)
        iv_mail_search_del.setOnClickListener { et_mail_search.setText("") }
        tv_mail_search_cancel.setOnClickListener { finish() }
    }

    private fun refresh() {
        val keyword = et_mail_search.text?.toString()?.trim() ?: ""
        val label = rv_mail_search.makeLabel()
        if (TextUtils.isEmpty(keyword)) {
            rv_mail_search.visibility = View.GONE
            tv_mail_search_tip.visibility = View.GONE
        } else {
            mPresenter.getSearchList(keyword, true, label)
        }
    }

    override fun onDestroy() {
        mPresenter.detachView()
        super.onDestroy()
    }

    override fun showSearchList(list: List<MailItem>, keyword: String, first: Boolean, hasNext: Boolean, label: Long) {
        if (!rv_mail_search.checkLabel(label)) {
            return
        }
        rv_mail_search.completeOnSuccess(hasNext, true)
        if (first) {
            rv_mail_search.visibility = View.VISIBLE
            tv_mail_search_tip.visibility = View.GONE
        }
        mAdapter.addList(first, list)
        mAdapter.setKeyword(keyword)
        mAdapter.notifyDataSetChanged()
    }

    override fun showSearchListFailure(first: Boolean, label: Long) {
        if (!rv_mail_search.checkLabel(label)) {
            return
        }
        rv_mail_search.completeOnFailure()
        if (first) {
            rv_mail_search.visibility = View.GONE
            tv_mail_search_tip.visibility = View.GONE
        }
    }

    override fun showSearchTip(keyword: String, label: Long) {
        if (!rv_mail_search.checkLabel(label)) {
            return
        }
        tv_mail_search_tip.text = resources.getString(R.string.search_email_tip, keyword)
        rv_mail_search.visibility = View.GONE
        tv_mail_search_tip.visibility = View.VISIBLE
    }

    override fun shareSuccess() {
        if (isActive) {
            ToastUtil.show(this, R.string.forward_success)
        }
        OkBus.getInstance().onEvent(BEventTags.REFRESH_RECENT_MAIL_LIST)
    }


}
