package com.lagou.lagouee.ui.settled

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.UserRoleManager
import com.alpha.lagouapk.tools.IntentUtils
import com.alpha.lagouapk.tools.UserHeadUtils
import com.alpha.lagouapk.widget.decoration.DividerItemDecoration
import com.common.afterTextChanged
import com.common.mvp.presenter.impl.CompanyUserManagerPresenterImpl
import com.common.mvp.view.CompanyUserManagerView
import com.lagou.baselib.BaseActivity
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.utils.ToastUtil
import com.lagou.commonlibs.view.CircleImageView
import com.lagou.commonlibs.view.recyclerview.LmRecyclerView
import com.lagou.lagouee.bean.CompanyUserCard
import kotlinx.android.synthetic.main.activity_company_member_list.*

/**
 * 公司成员列表
 */
class CompanyUserListActivity:BaseActivity(), CompanyUserManagerView {
    private var mAdapter:CompanyMemberListAdapter? = null
    private var mPresenter: CompanyUserManagerPresenterImpl? = null
    private var mMemberList = mutableListOf<CompanyUserCard>()
    override fun getContentLayoutId(): Int = R.layout.activity_company_member_list

    override fun init() {
        mPresenter = CompanyUserManagerPresenterImpl()
        mPresenter!!.attachView(this)
        mAdapter = CompanyMemberListAdapter(this@CompanyUserListActivity)
        recycler_view.apply {
            layoutManager = LinearLayoutManager(this@CompanyUserListActivity)
            addItemDecoration(DividerItemDecoration(this@CompanyUserListActivity,DividerItemDecoration.HORIZONTAL_LIST,ResourceUtils.getDimenSize(R.dimen.dp_05),ResourceUtils.getColor(R.color.color_e8e9eb)))
            setInitLoading(false)
            setHasFixedSize(true)
            adapter = mAdapter
        }
        requestData()
        this.initListener()
    }

    private fun initListener() {
        title_bar.setLeftImgListener { finish() }
        edit_input.afterTextChanged{
            requestData(keyword = it?.toString(), loading = false)
        }
        edit_input.setOnFocusChangeListener { _, hasFocus ->
            tv_input_hint?.visibility = if (hasFocus) View.GONE else View.VISIBLE
        }
        wv_company_user_list.setReLoadListener {
            requestData()
        }
        recycler_view.setOnLoadMoreListener(LmRecyclerView.OnLoadMoreListener {
            requestData(first = false, loading = false)
        })
        recycler_view.setOnItemClickListener { view, adapter, position ->
           val member =  mMemberList.getOrNull(position)
           member?.let {
               if (member.userId.toLong() == UserRoleManager.getInstance().userId){
                   ToastUtil.show(this,"不可交接给自己")
                   return@setOnItemClickListener
               }
               val intent = Intent()
               intent.putExtra(IntentUtils.INTENT_KEY_DATA,member)
               setResult(Activity.RESULT_OK,intent)
               finish()
           }
        }
    }

    @JvmOverloads
    private fun requestData(first: Boolean = true, loading: Boolean = true, keyword:String? = null) {
        mPresenter?.getCompanyUserList(first, loading, recycler_view.makeLabel(),keyword)
    }

    override fun showCompanyUserListSuccess(users: MutableList<CompanyUserCard>?, hasNext: Boolean, first: Boolean, label: Long) {
        if (!recycler_view.checkLabel(label)) {
            return
        }
        recycler_view.completeOnSuccess(hasNext, true)
        if (first) {
            mMemberList.clear()
            hideViews(wv_company_user_list)
        }
        if (users.isNullOrEmpty().not()){
            mMemberList.addAll(users!!)
        }
        mAdapter?.setMemberList(mMemberList)

    }

    override fun showCompanyUserListFailure(first: Boolean, label: Long) {
        if (!recycler_view.checkLabel(label)) {
            return
        }
        recycler_view.completeOnFailure()
        if (first) {
            showViews(wv_company_user_list)
        }
    }

    override fun onRemoveUserSuccess(userIds: MutableSet<Int>) {
    }

    override fun onCheckAuthority(hasDelAuthority: Boolean) {
    }

    class CompanyMemberListAdapter(val mContext: Context): RecyclerView.Adapter<CompanyMemberListAdapter.CompanyMemberViewHolder>() {
        private var memberList:MutableList<CompanyUserCard>? = null

        fun setMemberList(memberList:MutableList<CompanyUserCard>?){
            this.memberList = memberList
            notifyDataSetChanged()
        }

        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): CompanyMemberViewHolder {
            return CompanyMemberViewHolder(LayoutInflater.from(mContext).inflate(R.layout.item_company_member,parent,false))
        }

        override fun onBindViewHolder(holder: CompanyMemberViewHolder, position: Int) {
            val memberBean = memberList?.get(position)
            UserHeadUtils.setUserHeader(holder.ivMemberAvatar, memberBean?.portrait)
            holder.tvMemberName.text = memberBean?.name
            setMemberLabel(holder.tvMemberLabel,memberBean)
        }

        private fun setMemberLabel(tvMemberLabel:TextView,memberBean:CompanyUserCard?) {
            val userRole: String? = memberBean?.userRole
            if (TextUtils.isEmpty(userRole)) {
                tvMemberLabel.visibility = View.GONE
            } else {
                var colorId = -1
                var backgroundId = -1
                var labelName: String? = null
                when (userRole) {
                    CompanyUserRole.PAY_SUPER -> {
                        colorId = R.color.color_907251
                        backgroundId = R.drawable.shape_f2ece5_e8ded5_r1
                        labelName = CompanyUserRole.SUPER
                    }
                    CompanyUserRole.PAY_SENIOR -> {
                        colorId = R.color.color_907251
                        backgroundId = R.drawable.shape_f2ece5_e8ded5_r1
                        labelName = CompanyUserRole.SENIOR
                    }
                    CompanyUserRole.PAY_PRIVILEGE -> {
                        colorId = R.color.color_907251
                        backgroundId = R.drawable.shape_f2ece5_e8ded5_r1
                        labelName = CompanyUserRole.PRIVILEGE
                    }
                    CompanyUserRole.FREE_SUPER -> {
                        colorId = R.color.theme_text_color
                        backgroundId = R.drawable.shape_e6f8f3_r1
                        labelName = CompanyUserRole.SUPER
                    }
                    else -> {}
                }
                if (TextUtils.isEmpty(labelName)) {
                    tvMemberLabel.visibility = View.GONE
                } else {
                    tvMemberLabel.apply {
                        visibility = View.VISIBLE
                        setBackgroundResource(backgroundId)
                        setTextColor(ContextCompat.getColor(mContext, colorId))
                        text = labelName
                    }
                }
            }
        }

        override fun getItemCount(): Int = memberList?.size ?: 0

        class CompanyMemberViewHolder(itemView: View):RecyclerView.ViewHolder(itemView){
            val ivMemberAvatar: CircleImageView = itemView.findViewById(R.id.iv_company_user_head)
            val tvMemberName: TextView = itemView.findViewById(R.id.tv_company_user_name)
            val tvMemberLabel: TextView = itemView.findViewById(R.id.tv_company_member_label)
        }
    }

    override fun onDestroy() {
        mPresenter!!.detachView()
        super.onDestroy()
    }

    override fun getPageId() = "i1u96"
}