package com.lagou.lagouee.ui.settled

import android.content.Intent
import android.view.View
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProvider
import com.alpha.lagouapk.R
import com.lagou.baselib.BaseActivity
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.lagouee.bean.*
import com.lagou.lagouee.ui.settled.WorkHandOverViewModel.Companion.KEY_IS_SUPER_MANAGER
import com.lagou.lagouee.ui.settled.WorkHandOverViewModel.Companion.REQUEST_CODE_COMPANY_MEMBER
import com.lagou.lagouee.utils.IntentUtils
import kotlinx.android.synthetic.main.activity_quit_company.*
import kotlinx.android.synthetic.main.view_work_handover.view.*

/**
 * 退出公司
 */
class QuitCompanyActivity:BaseActivity() {
    private var isSuperManager:Boolean = false
    private var positionAndResumeBean: PositionAndResumeNumBean? = null
    private lateinit var mWorkHandOverViewModel:WorkHandOverViewModel
    override fun getContentLayoutId(): Int  = R.layout.activity_quit_company
    companion object{
        const val KEY_POSITION_RESUME_BEAN = "key_position_resume_bean"
    }
    override fun init() {
        isSuperManager = intent.extras?.getBoolean(KEY_IS_SUPER_MANAGER) ?: false
        positionAndResumeBean = intent.extras?.getParcelable(KEY_POSITION_RESUME_BEAN)
        setCanClick(false,title_quit_company.tvRight)
        work_handover_quit.setWorkHandOverDesc("退出公司需要将你在目前公司拥有的管理权限和以下数据信息进行交接")
        work_handover_quit.setNumbs(positionAndResumeBean?.onlinePositionCount,positionAndResumeBean?.offlinePositionCount,positionAndResumeBean?.totalResumeCount)
        this.registerObserver()
        this.initListener()
        mWorkHandOverViewModel.checkTransferTask()
    }

    private fun initListener() {
        title_quit_company.setLeftImgListener { finish() }
        work_handover_quit.setCallback {
            //打开新的同事页
            IntentUtils.startCompanyMemberListActivity(this,REQUEST_CODE_COMPANY_MEMBER)
        }
        title_quit_company.setRightTvListener {
            mWorkHandOverViewModel.commitQuitOrChangeCompany()
        }
    }


    private fun registerObserver() {
        mWorkHandOverViewModel = ViewModelProvider(this).get(WorkHandOverViewModel::class.java)
        mWorkHandOverViewModel.isSuperManager = isSuperManager
        mWorkHandOverViewModel.isChangeCompany = false
        mWorkHandOverViewModel.loadingLiveData.observe(this, Observer<Boolean> { show -> if (show) showLoading() else hideLoading() })
        mWorkHandOverViewModel.progressLiveData.observe(this, Observer<Int> { progress ->
            work_handover_progress.visibility = View.VISIBLE
            work_handover_progress.setProgress(progress)
        })
        mWorkHandOverViewModel.workmateLiveData.observe(this, Observer<String?> { memberName -> work_handover_quit.setWorkHandOverWorkmate(memberName) })
        mWorkHandOverViewModel.taskIsProcessLiveData.observe(this,Observer<Boolean>{ taskIsProcess ->
            setCanClick(!taskIsProcess,title_quit_company.tvRight,work_handover_quit.li_work_handover.tvContent)
        })
    }

    private fun setCanClick(isCanClick: Boolean,vararg views:View) {
        views.forEach { view ->
            view.isEnabled = isCanClick
            if (view == title_quit_company.tvRight){
                view.alpha = if (isCanClick) 1.0f else 0.3f
            }else if(view == work_handover_quit.li_work_handover.tvContent){
                work_handover_quit.li_work_handover.tvContent.setTextColor(ResourceUtils.getColor(if (isCanClick) R.color.color_666666 else R.color.color_cccccc))
            }
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (resultCode != RESULT_OK) {
            return
        }
        if (data == null || data.extras == null) {
            return
        }
        val member: CompanyUserCard? = data.extras?.getSerializable(IntentUtils.INTENT_KEY_DATA) as? CompanyUserCard
        member?.let {
            mWorkHandOverViewModel.onSelectNewMember(member)
            setCanClick(member.name.isNullOrEmpty().not(),title_quit_company.tvRight)
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        mWorkHandOverViewModel.onDestroy()
    }
}