package com.lagou.lagouee.ui.settled

import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import com.ApiService
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.baselib.util.mmkv.MKVUtils
import com.lagou.lagouee.bean.*
import com.lagou.lagouee.bean.jsonbean.WorkHandOverCacheBean
import com.lagou.lagouee.bean.jsonbean.settle.ChangeCompanyReq
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse
import com.lagou.socketchannel.util.GsonUtil
import java.util.*

class WorkHandOverViewModel:ViewModel() {
    var isSuperManager:Boolean = false
    var isChangeCompany:Boolean = false
    private var mNewWorkmate:PublisherInfo? = null
    private var mTimer: Timer? = null
    companion object{
        const val TASK_PERIOD:Long = 2 * 1000
        const val REQUEST_CODE_COMPANY_MEMBER = 1
        const val KEY_IS_SUPER_MANAGER = "key_is_super_manager"
        const val WORK_HAND_OVER_ING = "正在交接中"
        const val KEY_CLOSE_RECRUITMENT_SERVICE_FROM_CHANGE_COMPANY = "key_close_recruitment_service_from_change_company"
    }

     var loadingLiveData:MutableLiveData<Boolean> = MutableLiveData()
     var progressLiveData:MutableLiveData<Int> = MutableLiveData()
     var taskIsProcessLiveData:MutableLiveData<Boolean> = MutableLiveData()
     var workmateLiveData:MutableLiveData<String?> = MutableLiveData()
     var companyLiveData:MutableLiveData<String?> = MutableLiveData()
     var emailLiveData:MutableLiveData<String?> = MutableLiveData()
     var toastLiveData:MutableLiveData<String?> = MutableLiveData()
     var changeCompanyLiveData:MutableLiveData<UserInfoBean> = MutableLiveData()
    /**
     * 检测是否有任务正在交接
     */
    fun checkTransferTask(){
        loadingLiveData.value = true
        ApiService.api.searchPositionTransferTask().enqueue(object: BaseCallback<PositionTransferResponse>(){
            override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<PositionTransferResponse>?) {
                loadingLiveData.value = false
                //有提交接任务
                taskInProcess(response?.content?.taskAndRecordVO)
            }

            override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<PositionTransferResponse>?) {
                loadingLiveData.value = false
            }
        })
    }

    /**
     * 有任务正在转移
     */
    private fun taskInProcess(taskAndRecordVO: TaskAndRecordVO?) {
        taskIsProcessLiveData.value = true
        mNewWorkmate = GsonUtil.fromJson(taskAndRecordVO?.taskInfo?.params, PublisherInfo::class.java)
        workmateLiveData.value = mNewWorkmate?.newPublisherName
        val taskId = taskAndRecordVO?.taskInfo?.id ?: 0
        val leftNeedTransferCount = (taskAndRecordVO?.leftNeed2TransferOfflineCount ?: 0) + (taskAndRecordVO?.leftNeed2TransferOnlineCount ?: 0)
        var cacheBean = MKVUtils.getInstance().decodeParcelable(taskId.toString(),WorkHandOverCacheBean::class.java)
        if (cacheBean == null){
            //没有缓存需要缓存的职位总数
            cacheBean = WorkHandOverCacheBean(null,null,mNewWorkmate?.newPublisherName,mNewWorkmate?.need2TransferOnlineCount ?: 0,mNewWorkmate?.need2TransferOfflineCount ?: 0)
            MKVUtils.getInstance().encodeParcelable(taskId.toString(),cacheBean)
        }
        companyLiveData.value = if (cacheBean.companyName.isNullOrEmpty()) WORK_HAND_OVER_ING else cacheBean.companyName
        emailLiveData.value = cacheBean.email
        showProgress(taskId,leftNeedTransferCount)
        loopCheckTransferTask(taskId)
    }

    /**
     * 每隔2秒检查一次职位是否转移完成
     */
    private fun loopCheckTransferTask(taskId:Int){
        if (mTimer == null){
            mTimer = Timer()
        }
        mTimer!!.scheduleAtFixedRate(object: TimerTask(){
            override fun run() {
                ApiService.api.fetchPositionAndResumeNum().enqueue(object:BaseCallback<PositionAndResumeNumResponse>(){
                    override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<PositionAndResumeNumResponse>?) {
                        val positionNumBean = response?.content
                        if (positionNumBean?.positionResumeStatisticVO?.isNeedWorkHandOver() == false) {
                            //职位交接完成，进入下一步
                            cancelTask()
                            showProgress(taskId,0)
                            if (companyLiveData.value == WORK_HAND_OVER_ING){
                                companyLiveData.value = ""
                            }
                            MKVUtils.getInstance().removeKey(taskId.toString())
                            taskIsProcessLiveData.value = false
                            transferManagerAndCloseRecruitmentService(mNewWorkmate?.newPublisherId ?: 0)
                        }else{
                            val leftNeedTransferCount = (positionNumBean?.positionResumeStatisticVO?.onlinePositionCount ?: 0) + (positionNumBean?.positionResumeStatisticVO?.offlinePositionCount ?: 0)
                            showProgress(taskId,leftNeedTransferCount)
                        }
                    }

                })
            }
        }, TASK_PERIOD, TASK_PERIOD)
    }

    /**
     * 转移管理员与解除招聘者身份
     */
    private fun transferManagerAndCloseRecruitmentService(userId:Int){
        if (isSuperManager){
            //超管，需要先转移管理员
            loadingLiveData.value = true
            ApiService.api.transferManager(userId).enqueue(object : BaseCallback<Any>() {
                override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<Any>?) {
                    isSuperManager = false
                    enterCompanyOrCloseRecruitmentService()
                }

                override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<Any>?) {
                    super.onFailure(request, response)
                    loadingLiveData.value = false
                }
            })
        }else{
            //不是超管，解除招聘者服务
            loadingLiveData.value = true
            enterCompanyOrCloseRecruitmentService()
        }
    }

    /**
     * 更换公司
     */
    fun changeCompany(){
        val companyFullName: String? = companyLiveData.value
        if (companyFullName.isNullOrEmpty()){
            toastLiveData.value = "操作中断, 请重新提交"
            return
        }
        ApiService.api.fetchCompanyInfo(companyFullName).enqueue(object:BaseCallback<ChangeCompanyBean>(){
            override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<ChangeCompanyBean>?) {
                loadingLiveData.value = false
               val userInfoBean = response?.content?.parseUserInfo()?.apply { this.setmMineMail(emailLiveData.value) }
                changeCompanyLiveData.value = userInfoBean
            }

            override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<ChangeCompanyBean>?) {
                super.onFailure(request, response)
                loadingLiveData.value = false
            }
        })

    }


    /**
     * 进入公司创建or解除招聘者服务
     */
    private fun enterCompanyOrCloseRecruitmentService(){
        if (isChangeCompany){
            changeCompany()
            return
        }
        ApiService.api.closeRecruitmentService().enqueue(object:BaseCallback<Any>(){
            override fun onSuccess(p0: HttpRequest<*>?, p1: HttpResponse<Any>?) {}
            override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<Any>?) {
                super.onFailure(request, response)
                loadingLiveData.value = false
            }
        })
    }



    /**
     * 展示进度
     */
    private fun showProgress(taskId:Int,leftNeedTransferCount:Int){
        //计算当前进度
        try {
            var cacheBean = MKVUtils.getInstance().decodeParcelable(taskId.toString(),WorkHandOverCacheBean::class.java)
            val totalCount = (cacheBean?.onlinePositionCount ?: 0) + (cacheBean?.offlinePositionCount ?: 0)
            val progress = if (totalCount == 0) 0 else { (100 - (leftNeedTransferCount.toFloat() * 100 / totalCount)).toInt() }
            progressLiveData.value = progress
        } catch (e: Exception) {
        }
    }

    /**
     * 退出公司
     * 如果是超管，需要1.转移职位2.转移管理员3.解除招聘者身份
     * 不是超管，需要1.转移职位2.解除招聘者身份
     */
     fun commitQuitOrChangeCompany() {
        ApiService.api.fetchPositionAndResumeNum().enqueue(object:BaseCallback<PositionAndResumeNumResponse>(){
            override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<PositionAndResumeNumResponse>?) {
                val positionNumBean = response?.content
                if (positionNumBean?.positionResumeStatisticVO == null){
                    return
                }
                if (positionNumBean.positionResumeStatisticVO.isNeedWorkHandOver()) {
                    taskIsProcessLiveData.value = true
                    ApiService.api.transferPositions(mNewWorkmate?.newPublisherId ?: 0).enqueue(object:BaseCallback<TransferPositionTaskBean>(){
                        override fun onSuccess(p0: HttpRequest<*>?, response: HttpResponse<TransferPositionTaskBean>?) {
                            val taskId = response?.content?.taskId ?: 0
                            var cacheBean = MKVUtils.getInstance().decodeParcelable(taskId.toString(),WorkHandOverCacheBean::class.java)
                            if (cacheBean == null){
                                cacheBean =  WorkHandOverCacheBean(companyLiveData.value,emailLiveData.value,mNewWorkmate?.newPublisherName,positionNumBean.positionResumeStatisticVO.onlinePositionCount,positionNumBean.positionResumeStatisticVO.offlinePositionCount)
                            }else{
                                cacheBean.companyName = companyLiveData.value
                                cacheBean.email = emailLiveData.value
                            }
                            MKVUtils.getInstance().encodeParcelable(taskId.toString(),cacheBean)
                            val leftNeedTransferCount = positionNumBean.positionResumeStatisticVO.onlinePositionCount  + positionNumBean.positionResumeStatisticVO.offlinePositionCount
                            showProgress(taskId,leftNeedTransferCount)
                            loopCheckTransferTask(taskId)
                        }

                        override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<TransferPositionTaskBean>?) {
                            super.onFailure(request, response)
                            taskIsProcessLiveData.value = false
                        }
                    })
                }else{
                    transferManagerAndCloseRecruitmentService(mNewWorkmate?.newPublisherId ?: 0)
                }
            }

        })
    }


    fun onSelectNewMember(member: CompanyUserCard) {
        if (mNewWorkmate == null){
            mNewWorkmate = PublisherInfo(member.name,member.userId)
        }else{
            mNewWorkmate!!.newPublisherName = member.name
            mNewWorkmate!!.newPublisherId = member.userId
        }
        workmateLiveData.value = member.name
    }

    fun onDestroy() {
        cancelTask()
    }
    private fun cancelTask(){
        mTimer?.cancel()
        mTimer?.purge()
        mTimer = null
    }
}