package com.lagou.lagouee.ui.settled.perfect

import android.text.InputFilter
import android.text.TextUtils
import android.view.View
import android.widget.RadioGroup
import com.alpha.lagouapk.R
import com.alpha.lagouapk.ui.login.perfect.EditInfoCode
import com.alpha.lagouapk.ui.login.perfect.EditInfoContract.SuggestView
import com.alpha.lagouapk.ui.login.perfect.EditInfoModel
import com.alpha.lagouapk.ui.login.perfect.EditInfoPresenter
import com.common.manager.db.CommonDbManager
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.net.httpnetback.OnAdapterCommonListenerImpl
import com.lagou.baselib.util.BarUtil
import com.lagou.commonlibs.engine.ThreadUtil
import com.lagou.commonlibs.ui.dialog.inter.ADismissListener
import com.lagou.commonlibs.utils.InputMethodUtil
import com.lagou.commonlibs.utils.ListUtil
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.utils.ToastUtil
import com.lagou.commonlibs.view.MaxInputWatcher
import com.lagou.commonlibs.view.MaxInputWatcher.OnMaxInputListener
import com.lagou.commonlibs.view.filter.EmojiFilter
import com.lagou.lagouee.bean.UserInfoBean
import com.lagou.lagouee.bean.jsonbean.AutoCompanyPack
import com.lagou.lagouee.manager.net.CommonNetManager
import com.lagou.lagouee.utils.DialogUtils
import com.lagou.lagouee.utils.IntentUtils
import kotlinx.android.synthetic.main.activity_input_company_name.btn_loading
import kotlinx.android.synthetic.main.activity_input_company_name.et_company_name
import kotlinx.android.synthetic.main.activity_input_company_name.rg_company_type_group
import kotlinx.android.synthetic.main.activity_input_company_name.title_bar
import kotlinx.android.synthetic.main.activity_input_company_name.tv_company_guide
import kotlinx.android.synthetic.main.activity_input_company_name.tv_company_type
import kotlinx.android.synthetic.main.activity_input_company_name.tv_error_tip

/**
 * Created by mihao on 2023/3/15
 * 入驻输入公司全称
 */
class InputCompanyNameActivity : BaseActivity(), SuggestView<MutableList<AutoCompanyPack.Content.AutoCompanyBean>> {

    private var mRequestId = 0

    private var mUserInfo: UserInfoBean? = null

    private var mPresenter: EditInfoPresenter? = null

    private var isAutoQuery = true

    /**
     * 可用于选择的提示词
     */
    private var sourceList: MutableList<AutoCompanyPack.Content.AutoCompanyBean>? = null

    /**
     * 用户选择的公司类型，默认普通职位
     */
    companion object {
        const val REQUEST_ID = "request_id"
        const val GUIDE_TEXT = "guide_text"
        const val USER_INFO = "user_info"

        //选择的公司类型是否是普通公司
        var isCommon: Boolean? = null
    }

    private val textWatcher: OnMaxInputListener = object : OnMaxInputListener {
        override fun equalMaxLength(isEqual: Boolean) {
            if (isEqual) {
                ToastUtil.showCenter(
                    mContext,
                    ResourceUtils.getString(R.string.company_name_input_limit_b)
                )
            }
        }

        override fun onTextChanged(s: String) {
            checkPerfect(false)
            //删除输入内容，显示选择公司类型
            if (TextUtils.isEmpty(s)){
                tv_company_type.visibility = View.VISIBLE
                rg_company_type_group.visibility = View.VISIBLE
            }
            if (TextUtils.isEmpty(et_company_name.textContent)) {
                et_company_name.setAutoData(null)
                et_company_name.showAutoComplete(false)
                return
            }
            ThreadUtil.uiDelay({
                if (!isAutoQuery) {
                    isAutoQuery = true
                    return@uiDelay
                }
                mPresenter?.getDataByKey<String>(
                    EditInfoCode.B_TYPE_COMPANY_FULL,
                    et_company_name.textContent,
                    null
                )
            }, 100)
        }

        override fun showLengthTip(length: Int, text: String) {}
    }

    override fun getContentLayoutId() = R.layout.activity_input_company_name

    override fun isEnableCommonBarSetting(): Boolean {
        return false
    }

    override fun init() {
        initListener()
        initEditText()
        mRequestId = intent.getIntExtra(REQUEST_ID, 0)
        if (intent.getSerializableExtra(USER_INFO) is UserInfoBean) {
            mUserInfo = intent.getSerializableExtra(USER_INFO) as UserInfoBean
            if (!TextUtils.isEmpty(mUserInfo?.getmMineCompanyFull())) {
                et_company_name.setEditText(mUserInfo?.getmMineCompanyFull())
            }
        }
        if (mUserInfo == null) {
            mUserInfo = UserInfoBean.getUserInforBean()
        }
        tv_company_guide.text = intent.getStringExtra(GUIDE_TEXT)
        mPresenter = EditInfoPresenter()
        mPresenter?.setVM(this, EditInfoModel())
        checkPerfect(false)
        //默认选中普通公司类型
        rg_company_type_group.check(R.id.rb_common_company)

        //接口动态控制跳过按钮隐藏与显示
        ThreadUtil.io {
            val entryConfig = CommonDbManager.getInstance().commonConfig ?: return@io
            ThreadUtil.ui {
                title_bar.setRightTxt(
                    if (entryConfig.isBVerifySkipOpen) ResourceUtils.getString(
                        R.string.jump
                    ) else ""
                )
            }
        }
    }

    private fun initListener() {
        BarUtil.headerImageBarInit(this, R.id.title_bar)
        title_bar.setLeftImgListener { finish() }
        title_bar.setRightTvListener { DialogUtils.showBGuestTipsDialog(baseActivity, true) }
        btn_loading.setBigClick { goNext() }

        //公司类型，用户选择公司类型
        rg_company_type_group.setOnCheckedChangeListener { group: RadioGroup?, checkedId: Int ->
            isCommon = when (checkedId) {
                R.id.rb_headhunter_company -> {
                    false
                }
                else -> {
                    true
                }
            }
        }
    }

    private fun initEditText() {
        val maxInputWatcher = MaxInputWatcher(100)
        maxInputWatcher.setIsCalTextLength(true)
        maxInputWatcher.setInputType(MaxInputWatcher.INPUT_TYPE_DOUBLE)
        maxInputWatcher.setMaxInputListener(textWatcher)
        et_company_name.apply {
            showTitleSubTitle(false)
            showAutoComplete(false)
            setGetDataOrNot(false)
            setLineColorFocus(true)
            editTextView.filters = arrayOf<InputFilter>(EmojiFilter())
            editTextView.addTextChangedListener(maxInputWatcher)
        }
        et_company_name.setOnClickItemListener {
            isAutoQuery = false
            //能查出来公司说明能确定类型（普通公司或者猎头），则隐藏公司类型选择
            tv_company_type.visibility = View.GONE
            rg_company_type_group.visibility = View.GONE

            InputMethodUtil.hideKeyboard(this)
            et_company_name.showAutoComplete(false)
            //选中的公司信息
            val selectCompanyName = et_company_name.textContent
            val selectCompanyList =  sourceList?.filter { it.companyFullName == selectCompanyName }
            isCommon = selectCompanyList?.get(0)?.isHunter?.not()
        }
    }

    private fun checkPerfect(isShowRead: Boolean): String? {
        tv_error_tip.visibility = View.GONE
        var tips: String? = null
        if (TextUtils.isEmpty(et_company_name.textContent)) {
            tips = ResourceUtils.getString(R.string.perfect_company_name_tips)
            btn_loading.showLoadingStyle(0.65f)
            if (isShowRead) {
                et_company_name.setLineColorError()
            }
        } else {
            btn_loading.hideLoadingStyle(1.0f)
        }
        return tips
    }

    private fun doNext() {
        mUserInfo?.setmMineCompanyFull(et_company_name.textContent)
        CommonNetManager.saveCompanyName(mRequestId,
            et_company_name.textContent,
            mUserInfo,
            object : OnAdapterCommonListenerImpl<Any, Any>(baseActivity) {
                override fun showLoading() {
                    btn_loading.showLoading()
                    btn_loading.showLoadingStyle(0.65f)
                }

                override fun hideSuccessLoading() {
                    btn_loading.hideLoading()
                    btn_loading.hideLoadingStyle(1.0f)
                }

                override fun onError(errorBean: Any) {
                    if (errorBean is String) {
                        tv_error_tip.visibility =
                            if (errorBean.isNullOrEmpty()) View.GONE else View.VISIBLE
                        tv_error_tip.text = errorBean
                        et_company_name.setLineColorError()
                    }
                }

                override fun onSuccess(infoBean: Any?) {
                    super.onSuccess(infoBean)
                    IntentUtils.startSelectOrGuideCompanyInfoActivity(
                        this@InputCompanyNameActivity,
                        infoBean as? UserInfoBean,
                        RESULT_OK
                    )
                }
            })
    }

    private fun goNext() {
        et_company_name.editTextView.clearFocus()
        val isShowDialog = tv_error_tip.visibility != View.VISIBLE
        val perfectTips = checkPerfect(true)
        if (!TextUtils.isEmpty(perfectTips)) {
            ToastUtil.showCenter(baseActivity, perfectTips)
            return
        }
        if (isShowDialog) {
            DialogUtils.showCompanyNameConfirmDialog(
                baseActivity,
                et_company_name.textContent,
                object : ADismissListener<Any>() {
                    override fun onRight() {
                        doNext()
                    }
                })
        } else {
            doNext()
        }
    }

    override fun showSuggest(
        key: String?,
        suggests: MutableList<String>?,
        sourceList: MutableList<AutoCompanyPack.Content.AutoCompanyBean>?
    ) {
        if (!TextUtils.equals(key, et_company_name.textContent)) {
            return
        }
        //查不到说明不确定类型，所以才要显示出来供用户选择
        if(sourceList.isNullOrEmpty()){
            tv_company_type.visibility = View.VISIBLE
            rg_company_type_group.visibility = View.VISIBLE
        }
        et_company_name.showAutoComplete(!ListUtil.isEmpty(suggests))
        et_company_name.setAutoData(suggests)
    }
}