package com.lagou.lagouee.ui.settled.requirement

import android.app.Activity
import android.content.Intent
import android.graphics.Rect
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import android.widget.Toast
import com.alpha.lagouapk.R
import com.alpha.lagouapk.UserRoleManager
import com.common.utils.ViewUtils
import com.lagou.baselib.BaseActivity
import com.lagou.commonlibs.utils.ActivityStackHelper
import com.lagou.commonlibs.utils.ToastUtil
import com.lagou.commonlibs.utils.VisibilityUtil
import com.lagou.lagouee.ui.HomeActivity
import com.lagou.lagouee.utils.IntentUtils
import kotlinx.android.synthetic.main.activity_recruit_requirement.*

/**
 * 入驻流程-选择招聘需求
 *
 * @author yph
 * @date 2019/10/28
 */
class RecruitRequirementActivity : BaseActivity(), RecruitRequireContract.View, SelectHelper.OnSelectedListener {
    private val mPresenter by lazy(LazyThreadSafetyMode.NONE) {
        RecruitRequirementPresenter()
    }

    private val mAdapter by lazy {
        RequirementAdapter(this, rv_recruit_requirement)
    }

    override fun getContentLayoutId(): Int = R.layout.activity_recruit_requirement

    override fun init() {
        mPresenter.attachView(this)
        initRecyclerView()
        initListener()
        requestData()
    }

    private fun requestData() {
        mPresenter.getRecruitInfo()
    }

    private fun initListener() {
        SelectHelper.getInstance().setOnSelectListener(this)
        ViewUtils.ditherClick(tv_recruit_requirement_submit) {
            SelectHelper.getInstance().list()?.apply {
                // 提交
                mPresenter.submit(this)
            }
        }
        ViewUtils.ditherClick(tv_recruit_requirement_cancel) {
            // 我不需要优质简历
            mPresenter.skip()
        }

        wv_recruit_requirement.setReLoadListener {
            requestData()
        }
    }


    override fun onDestroy() {
        SelectHelper.getInstance().removeOnSelectListener()
        mPresenter.detachView()
        super.onDestroy()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == REQUEST_CODE && resultCode == Activity.RESULT_OK) {
            data?.getStringExtra(KEY)?.let {
                mAdapter.addLabel(it.trim())
            }
        }
    }

    override fun onBackPressed() {
        // 屏蔽返回键
    }

    private fun initRecyclerView() {
        val spanCount = resources.getInteger(R.integer.recruit_requirement_span_count)
        val gridLayoutManager = GridLayoutManager(this, spanCount)
        rv_recruit_requirement.apply {
            setHasFixedSize(true)
            layoutManager = gridLayoutManager
            adapter = mAdapter
            // 网格设置
            gridLayoutManager.spanSizeLookup = object : GridLayoutManager.SpanSizeLookup() {
                override fun getSpanSize(position: Int): Int = when (mAdapter.getItemViewType(position)) {
                    RequirementItem.CELL -> 1
                    else -> spanCount
                }
            }
            // 间距
            val dp5 = resources.getDimensionPixelSize(R.dimen.dp_5)
            val dp10 = resources.getDimensionPixelSize(R.dimen.dp_10)
            addItemDecoration(object : RecyclerView.ItemDecoration() {
                override fun getItemOffsets(outRect: Rect, itemPosition: Int, parent: RecyclerView) {
                    val item = mAdapter.getItem(itemPosition)
                    if (item.itemType == RequirementItem.CELL) {
                        // 均分计算
                        val mod = item.index % spanCount
                        val left = (mod.toFloat() / spanCount * dp10 + 0.5f).toInt()
                        val right = ((spanCount - 1 - mod.toFloat()) / spanCount * dp10 + 0.5f).toInt()
                        outRect.set(left, dp5, right, 0)
                    } else {
                        super.getItemOffsets(outRect, itemPosition, parent)
                    }
                }
            })
        }

    }

    override fun onChanged(labels: List<String>?) {
        tv_recruit_requirement_submit?.apply {
            if (labels == null || labels.isEmpty()) {
                isEnabled = false
                text = getString(R.string.submit)
            } else {
                isEnabled = true
                text = getString(R.string.submit_args, labels.size)
            }
        }
    }

    override fun onOverstep() {
        ToastUtil.show(this, getString(R.string.attention_num_max, SelectHelper.MAX))
    }

    override fun showRecruitInfoSuccess(list: List<RequirementItem>) {
        mAdapter.refreshList(list)
        VisibilityUtil.setVisible(rv_recruit_requirement, ll_recruit_requirement)
        VisibilityUtil.setGone(wv_recruit_requirement)
    }

    override fun showRecruitInfoFailure() {
        VisibilityUtil.setGone(rv_recruit_requirement, ll_recruit_requirement)
        VisibilityUtil.setVisible(wv_recruit_requirement)
    }

    override fun submitSuccess() {
        ToastUtil.show(this, "关注成功，简历将于\n工作日10:30发送到邮箱", Toast.LENGTH_LONG)
        next()
    }

    override fun skipSuccess() {
        next()
    }

    private fun next() {
        UserRoleManager.getInstance().isChanging = false
        IntentUtils.startHomeActivity(this)
        ActivityStackHelper.clearAndFinishOtherActivity(HomeActivity::class.java)
    }

    companion object {
        const val REQUEST_CODE = 0x1000
        const val KEY = "key"
        const val SUBMIT = "0"
        const val SKIP = "2"
    }

}