package com.lagou.lagouee.ui.settled.requirement

import android.text.TextUtils
import com.ApiService
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.commonlibs.utils.ListUtil
import com.lagou.commonlibs.utils.presenter.WeakPresenter
import com.lagou.lagouee.bean.RequireParams
import com.lagou.lagouee.bean.jsonbean.RecruitRequirementInfo
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse

/**
 * 选择招聘需求Presenter
 *
 * @author yph
 * @date 2019/10/29
 */
class RecruitRequirementPresenter : WeakPresenter<RecruitRequireContract.View>(), RecruitRequireContract.Presenter {

    override fun getRecruitInfo() {
        ifViewAttached {
            it.showLoading()
        }
        ApiService.api.recruitRequirementInfo.enqueue(object : BaseCallback<RecruitRequirementInfo?>() {
            override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<RecruitRequirementInfo?>?) {
                ifViewAttached {
                    it.hideLoading()
                    val list = arrayListOf<RequirementItem>().apply {
                        // 添加头部
                        add(RequirementItem(RequirementItem.HEADER))
                    }
                    val labels = response?.content?.hotLables ?: emptyList()
                    labels.forEach { bean ->
                        if (bean != null && !ListUtil.isEmpty(bean.lables)) {
                            // 添加小标题
                            list.add(RequirementItem(RequirementItem.TITLE, bean.name ?: ""))
                            var index = 0
                            bean.lables.forEach { label ->
                                if (label != null && !TextUtils.isEmpty(label.name)) {
                                    // 添加每个item
                                    list.add(RequirementItem(RequirementItem.CELL, label.name, index++))
                                }
                            }
                        }
                    }
                    it.showRecruitInfoSuccess(list)
                }
            }

            override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<RecruitRequirementInfo?>?) {
                super.onFailure(request, response)
                ifViewAttached {
                    it.hideLoading()
                    it.showRecruitInfoFailure()
                }
            }
        })
    }

    override fun submit(list: List<String>) {
        save(list, RecruitRequirementActivity.SUBMIT)
    }

    override fun skip() {
        save(ListUtil.emptyList(), RecruitRequirementActivity.SKIP)
    }

    private fun save(list: List<String>, type: String) {
        ifViewAttached { it.showLoading() }
        ApiService.api.saveRecruitRequirement(RequireParams(list, type)).enqueue(object : BaseCallback<Any?>() {
            override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<Any?>?) {
                ifViewAttached {
                    it.hideLoading()
                    when (type) {
                        RecruitRequirementActivity.SUBMIT -> it.submitSuccess()
                        RecruitRequirementActivity.SKIP -> it.skipSuccess()
                    }
                }
            }

            override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<Any?>?) {
                super.onFailure(request, response)
                ifViewAttached { it.hideLoading() }
            }
        })
    }


}