package com.lagou.lagouee.ui.settled.requirement

import android.text.TextUtils
import android.widget.TextView
import java.util.*


/**
 * 选择招聘需求帮助类
 *
 * @author yph
 * @date 2019/10/29
 */
class SelectHelper private constructor() {

    private var mList: LinkedList<String>? = null
    private var mListener: OnSelectedListener? = null

    interface OnSelectedListener {
        /**
         * 变化回调
         */
        fun onChanged(labels: List<String>?)

        /**
         * 越界回调
         */
        fun onOverstep()
    }

    fun setOnSelectListener(listener: OnSelectedListener?) {
        mListener = listener
        mListener?.onChanged(mList)
    }

    fun removeOnSelectListener() {
        mListener = null
        mList = null
    }

    fun add(label: String) {
        // 若为null，则创建list
        // 然后添加label
        (mList ?: LinkedList<String>().also { mList = it }).apply {
            add(label)
            mListener?.onChanged(this)
        }
    }

    fun remove(label: String) {
        mList?.apply {
            if (remove(label)) {
                mListener?.onChanged(this)
            }
        }
    }

    fun size(): Int = mList?.size ?: 0

    fun list(): List<String>? = mList

    fun isSelected(text: String?): Boolean {
        return mList.let {
            if (it == null || TextUtils.isEmpty(text)) false else it.contains(text)
        }
    }

    /**
     * 手动选中
     */
    fun select(textView: TextView?) {
        val text = textView?.text?.toString()
        if (text == null || text.isEmpty()) {
            return
        }
        val selected = mList?.contains(text) ?: false
        if (!selected) {
            if (size() < MAX) {
                textView.isSelected = true
                add(text)
            } else {
                mListener?.onOverstep()
            }
        }
    }

    /**
     * 手动选中
     */
    fun select(text: String): Boolean {
        return if (isSelected(text)) {
            false
        } else {
            if (size() < MAX) {
                add(text)
                true
            } else {
                mListener?.onOverstep()
                false
            }
        }
    }

    /**
     * 切换选中
     */
    fun click(textView: TextView?) {
        val text = textView?.text?.toString()
        if (text == null || text.isEmpty()) {
            return
        }
        val selected = mList?.contains(text) ?: false
        if (selected) {
            textView.isSelected = false
            remove(text)
        } else {
            if (size() < MAX) {
                textView.isSelected = true
                add(text)
            } else {
                mListener?.onOverstep()
            }
        }
    }

    companion object {
        /**
         * 最多选中5个
         */
        const val MAX = 5

        @Volatile
        private var instance: SelectHelper? = null

        @JvmStatic
        fun getInstance(): SelectHelper = instance ?: synchronized(this) {
            instance ?: SelectHelper().also { instance = it }
        }
    }

}