package com.lagou.lagouee.ui.settled.search

import android.app.Activity
import android.content.Intent
import android.text.Editable
import android.text.TextUtils
import com.alpha.lagouapk.R
import com.alpha.lagouapk.manager.HttpReuqestManager
import com.alpha.lagouapk.tools.BaseUtils
import com.common.utils.ViewUtils
import com.common.widget.CustomTextWatcher
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.util.BarUtil
import com.lagou.commonlibs.utils.ToastUtil
import com.lagou.commonlibs.utils.VisibilityUtil
import com.lagou.lagouee.ui.settled.requirement.RecruitRequirementActivity
import com.lagou.lagouee.ui.settled.requirement.SelectHelper
import kotlinx.android.synthetic.main.activity_position_name_search.*
import org.json.JSONObject
import java.util.regex.Pattern

/**
 * 职位名称搜索
 *
 * @author yph
 * @date 2019/10/30
 */
class PositionNameSearchActivity : BaseActivity() {
    private val mAdapter by lazy(LazyThreadSafetyMode.NONE) {
        PositionNameSearchAdapter(this)
    }

    override fun getContentLayoutId(): Int = R.layout.activity_position_name_search

    override fun init() {
        lv_position_name_search.adapter = mAdapter
        lv_position_name_search.setOnItemClickListener { _, _, position, _ ->
            val size = SelectHelper.getInstance().size()
            if (size >= SelectHelper.MAX) {
                ToastUtil.show(this, getString(R.string.attention_num_max, SelectHelper.MAX))
            } else {
                val positionName = mAdapter.getItem(position) as String
                setResult(Activity.RESULT_OK, Intent().apply {
                    putExtra(RecruitRequirementActivity.KEY, positionName)
                })
                finish()
            }
        }

        et_position_name_search.addTextChangedListener(object : CustomTextWatcher() {
            private val pattern = Pattern.compile("[a-zA-Z0-9\\u4e00-\\u9fa5]")

            override fun afterTextChanged(s: Editable?) {
                val keyword = s?.toString() ?: ""
                keyword.apply {
                    if (isEmpty()) {
                        VisibilityUtil.setInvisible(iv_position_search_clear, lv_position_name_search)
                    } else {
                        VisibilityUtil.setVisible(iv_position_search_clear)
                        if (pattern.matcher(this).find()) {
                            // 含有汉字、数字或字母，输入有效
                            refresh(this.trim())
                        } else {
                            VisibilityUtil.setInvisible(lv_position_name_search)
                            ToastUtil.show(this@PositionNameSearchActivity, "请输入正确职位名称")
                        }
                    }
                }
            }
        })

        ViewUtils.ditherClick(iv_position_search_clear) {
            et_position_name_search.setText("")
        }
        ViewUtils.ditherClick(tv_position_search_cancel) {
            finish()
        }

        BarUtil.keyboardEnable(this)
        et_position_name_search.postDelayed({ BaseUtils.showKeyboard(this, et_position_name_search) }, 100)
    }

    /**
     * 校验搜索关键字
     */
    private fun checkKeyword(keyword: String): Boolean = TextUtils.equals(keyword, et_position_name_search?.text?.toString())

    private fun refresh(keyword: String) {
        HttpReuqestManager.getSearchSuggestion(keyword, CITY_CODE, NUM, object : HttpReuqestManager.Callback<String?, Void?> {
            override fun onSuccess(var1: String?) {
                if (isActivityDestroyed || !checkKeyword(keyword)) {
                    return
                }
                val list = arrayListOf<String>()
                list.add(keyword)
                try {
                    JSONObject(var1).optJSONArray(POSITION)?.let {
                        for (index in 0 until it.length()) {
                            it.optJSONObject(index)?.optString(CONT)?.apply {
                                // 已手动添加keyword，若再与它相同会重复
                                if (isNotEmpty() && !equals(keyword, true)) {
                                    list.add(this)
                                }
                            }
                        }
                    }
                } catch (ignore: Exception) {
                    // empty
                }
                mAdapter.refreshList(list, keyword)
                VisibilityUtil.setVisible(lv_position_name_search)
            }

            override fun onFailure(f: Void?) {
                if (isActivityDestroyed || !checkKeyword(keyword)) {
                    return
                }
                mAdapter.refreshList(listOf(keyword), keyword)
                VisibilityUtil.setVisible(lv_position_name_search)
            }
        })
    }

    companion object {
        const val CITY_CODE = "0"
        const val NUM = "10"
        const val POSITION = "POSITION"
        const val CONT = "cont"
    }


}