package com.lagou.lagouee.widget

import android.content.Context
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.widget.FrameLayout
import android.widget.TextView
import com.alpha.lagouapk.R
import java.text.SimpleDateFormat
import java.util.*

/**
 * 面试日期及日程
 *
 * @author yph
 * @date 2019/12/20
 */
class InterviewTextView(context: Context, attrs: AttributeSet? = null) : FrameLayout(context, attrs) {
    private val tvDayDes: TextView
    private val tvDay: TextView
    private val tvNum: TextView
    private val sdf = SimpleDateFormat()
    private val calendar = Calendar.getInstance()
    private var currMillis = 0L
    private var baseMillis = 0L
    private var num = Int.MIN_VALUE

    init {
        LayoutInflater.from(context).inflate(R.layout.layout_interview_text, this)
        tvDayDes = findViewById(R.id.tv_interview_day_des)
        tvDay = findViewById(R.id.tv_interview_day)
        tvNum = findViewById(R.id.tv_interview_num)
    }

    /**
     * 数据显示规则：
     * 今天/明天/昨天的面试，显示为：今天/明天/昨天（XX.XX）；eg：今天（09.16）
     * 今年内，显示为：XX.XX：eg：09.18
     * 跨自然年，显示为：XXXX年XX月XX日：2020.01.01
     * @param baseMillis 基线时间
     * @param millis 设置时间
     * @param num 面试日程
     */
    fun setData(baseMillis: Long, millis: Long, num: Int) {
        if (this.num != num) {
            this.num = num
            tvNum.text = "共${num}个面试日程"
        }

        var changed = false
        if (this.baseMillis != baseMillis) {
            this.baseMillis = baseMillis
            changed = true
        }
        if (this.currMillis != millis) {
            this.currMillis = millis
            changed = true
        }
        if (!changed) {
            return
        }

        calendar.timeInMillis = baseMillis
        val baseYear = calendar.get(Calendar.YEAR)
        calendar.set(Calendar.HOUR_OF_DAY, 0)
        calendar.set(Calendar.MINUTE, 0)
        calendar.set(Calendar.SECOND, 0)
        calendar.set(Calendar.MILLISECOND, 0)
        val todayStart = calendar.timeInMillis
        val nearbyStr = when (millis) {
            in (todayStart - UNIT) until todayStart -> "昨天"
            in todayStart until (todayStart + UNIT) -> "今天"
            in (todayStart + UNIT) until (todayStart + 2 * UNIT) -> "明天"
            else -> null
        }

        calendar.timeInMillis = millis
        val interviewYear = calendar.get(Calendar.YEAR)
        sdf.applyPattern(if (interviewYear == baseYear) "MM.dd" else "yyyy.MM.dd")

        val date = sdf.format(millis)
        if (nearbyStr == null) {
            tvDayDes.text = date
            tvDay.visibility = View.GONE
        } else {
            tvDayDes.text = nearbyStr
            tvDay.text = "(${date})"
            tvDay.visibility = View.VISIBLE
        }
    }

    companion object {
        const val UNIT = 24 * 60 * 60 * 1000
    }

}