package com.lagou.lagouee.widget

import android.content.Context
import android.util.AttributeSet
import android.view.LayoutInflater
import android.widget.FrameLayout
import com.alpha.lagouapk.R
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.commonlibs.inter.OnSingleMethodNoParameterListener
import com.lagou.commonlibs.utils.DeviceUtils
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.commonlibs.utils.SafeTypeConvert
import com.lagou.commonlibs.utils.TextViewUtils
import com.lagou.lagouee.bean.jsonbean.aigc.AIGCReleaseRes
import com.nineoldandroids.animation.*
import com.nineoldandroids.animation.ValueAnimator.AnimatorUpdateListener
import kotlinx.android.synthetic.main.adapter_aigc_release_change_question.view.*

/**
 * @date： Crate by wangjunwei on 2023/7/17
 * @description：AI更换个问题的自定义View
 **/
class ItemAIGCReleaseChangeView : FrameLayout {

    private lateinit var animatorSet: AnimatorSet

    private lateinit var animatorChangeQuestion: ObjectAnimator

    private lateinit var animatorOpenQuestion: ValueAnimator
    private lateinit var animatorOpenQuestionAlpha: ObjectAnimator

    private lateinit var animatorLeftLine: ValueAnimator

    /**
     * 修改问题动画时长
     */
    private val CHNAGE_TIME_DURATION = 400L

    /**
     * 打开问题动画时长
     */
    private val OPEN_TIME_DURATION = 400L

    /**
     * 线展开动画时长
     */
    private val LINE_TIME_DURATION = 300L

    private val DP_05 = ResourceUtils.getDimenSize(R.dimen.dp_05)
    private val DP_20 = ResourceUtils.getDimenSize(R.dimen.dp_20)

    /**
     * 开启新问题的线的最大宽度
     */
    private var lineMaxWidth = 0

    /***
     * 开启新问题点击事件
     */
    private var openCallback: OnSingleMethodNoParameterListener? = null

    private lateinit var aigcReleaseChange: AIGCReleaseRes

    constructor(context: Context) : this(context, null) {}
    constructor(context: Context, attrs: AttributeSet?) : this(context, attrs, 0) {}
    constructor(context: Context, attrs: AttributeSet?, defStyleAttr: Int) : super(
        context,
        attrs,
        defStyleAttr
    ) {
        LayoutInflater.from(context)?.inflate(R.layout.adapter_aigc_release_change_question, this)
        setPadding(
            0,
            ResourceUtils.getDimenSize(R.dimen.dp_10),
            0,
            ResourceUtils.getDimenSize(R.dimen.dp_15)
        )
        initAnimator()
        initListener()
    }

    private fun initAnimator() {
        animatorChangeQuestion = ObjectAnimator.ofFloat(tv_change_question, "alpha", 1f, 0f)
        animatorChangeQuestion.duration = CHNAGE_TIME_DURATION

        animatorOpenQuestion = ValueAnimator.ofInt(DP_20, 0)
        animatorOpenQuestion.duration = OPEN_TIME_DURATION
        animatorOpenQuestionAlpha = ObjectAnimator.ofFloat(lin_open_show, "alpha", 0f, 1f)
        animatorOpenQuestionAlpha.duration = OPEN_TIME_DURATION

        val lineMargin =
            (ResourceUtils.getDimenSize(R.dimen.dp_15) + ResourceUtils.getDimenSize(R.dimen.dp_4)) * 2
        val textSpaceX = TextViewUtils.getMeasureWidth(
            tv_open_text.text.toString(),
            tv_open_text.textSize.toInt()
        )
        lineMaxWidth = ((DeviceUtils.getScreenWidth() - lineMargin - textSpaceX) / 2)
        animatorLeftLine = ValueAnimator.ofInt(0, lineMaxWidth)
        animatorLeftLine.duration = LINE_TIME_DURATION

        animatorSet = AnimatorSet()
    }

    fun fillUiData(aigcReleaseRes: AIGCReleaseRes){
        aigcReleaseChange = aigcReleaseRes
        //1 默认不做处理
        if (aigcReleaseChange.isShowOpenDefault){
            handleUIStyle(true)
            return
        }

        //2 已经点击过更换问题，直接显示....开启问题....
        handleUIStyle(false)
    }

    private fun handleUIStyle(isShowQuestionDefault: Boolean) {
        tv_change_question.visibility = if (isShowQuestionDefault) VISIBLE  else GONE
        tv_change_question.alpha = 1f
        lin_open_show.visibility = if (isShowQuestionDefault) GONE else VISIBLE
        val openParams: LayoutParams = lin_open_show.layoutParams as LayoutParams
        openParams.topMargin = if (isShowQuestionDefault) DP_20 else 0
        lin_open_show.layoutParams = openParams

        val leftLineParams = vm_open_left.layoutParams
        leftLineParams.width = if (isShowQuestionDefault) 0 else lineMaxWidth
        leftLineParams.height = DP_05
        vm_open_left.layoutParams = leftLineParams
        val rightLineParams = vm_open_right.layoutParams
        rightLineParams.width = if (isShowQuestionDefault) 0 else lineMaxWidth
        rightLineParams.height = DP_05
        vm_open_right.layoutParams = rightLineParams
    }

    private fun initListener() {
        animatorSet.playTogether(
            animatorChangeQuestion,
            animatorOpenQuestion,
            animatorOpenQuestionAlpha
        )
        animatorOpenQuestion.addUpdateListener(object : AnimatorUpdateListener{
            override fun onAnimationUpdate(animation: ValueAnimator?) {
                val openParams: LayoutParams = lin_open_show.layoutParams as LayoutParams
                openParams.topMargin = SafeTypeConvert.toInt(animation?.animatedValue)
                lin_open_show.layoutParams = openParams
            }
        })
        animatorSet.addListener(object : AnimatorListenerAdapter() {
            override fun onAnimationStart(animation: Animator?) {
                super.onAnimationStart(animation)
                lin_open_show.visibility = VISIBLE
            }

            override fun onAnimationEnd(animation: Animator?) {
                super.onAnimationEnd(animation)
                tv_change_question.visibility = GONE
                //更换问题文字动画结束，紧接着出现line的动画
                animatorLeftLine.addUpdateListener { animation ->
                    val leftLineParams = vm_open_left.layoutParams
                    val width = SafeTypeConvert.toInt(animation?.animatedValue)
                    leftLineParams.width = width
                    leftLineParams.height = ResourceUtils.getDimenSize(R.dimen.dp_05)
                    vm_open_left.layoutParams = leftLineParams

                    val rightLineParams = vm_open_right.layoutParams
                    rightLineParams.width = width
                    rightLineParams.height = ResourceUtils.getDimenSize(R.dimen.dp_05)
                    vm_open_right.layoutParams = rightLineParams
                }
                animatorLeftLine.start()
            }
        })

        tv_change_question.setOnClickListener {
            animatorSet.start()
            openCallback?.onCallBack()
            //换个问题点击埋点
            AnalyticHelper.getInstance().sendClickInfo("i1uam", "q1uat")
        }
    }

    fun setOpenCallback(openCallback: OnSingleMethodNoParameterListener?) {
        this.openCallback = openCallback
    }
}