package com.lagou.lagouee.widget.session

import android.content.Context
import android.util.AttributeSet
import android.view.View
import android.widget.ImageView
import android.widget.TextView
import com.alpha.lagouapk.R
import com.alpha.lagouapk.tools.UserHeadUtils
import com.common.chat.SessionListShownUtils
import com.google.gson.reflect.TypeToken
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.analytic.AnalyticHelper
import com.lagou.baselib.analytic.definition.b.BAid
import com.lagou.baselib.analytic.definition.c.PageId
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.chat.chatlibrary.bean.ConversationBean
import com.lagou.chat.chatlibrary.bean.SessionSummaryBean
import com.lagou.chat.chatlibrary.view.EmojiconTextView
import com.lagou.commonlibs.utils.ListUtil
import com.lagou.commonlibs.utils.ResourceUtils
import com.lagou.lagouee.bean.BlockPositionTalentBean
import com.lagou.lagouee.ui.business.helper.BlockPositionHelper
import com.lagou.lagouee.ui.candidate.utils.CandidateUtils
import com.lagou.lagouee.utils.IntentUtils
import com.lagou.lagouee.widget.business.BlockPositionPhotoUtils
import com.lagou.lagouee.widget.business.InterestedTalentPhotoView
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse
import com.lagou.socketchannel.util.GsonUtil

/**
 * B端特权职位
 */
class PrivilegePositionSessionView @JvmOverloads constructor(context: Context, attr: AttributeSet? = null, defStyleResId: Int = 0): BaseSessionView(context,attr,defStyleResId) {
    private var ivAvatar: ImageView = findViewById(R.id.iv_conversation_logo)
    private var ivUnRead: ImageView = findViewById(R.id.iv_unread_count)
    private var tvTitle: TextView = findViewById(R.id.tv_title)
    private var tvMessage: EmojiconTextView = findViewById(R.id.tv_message)
    private var dividerView: View = findViewById(R.id.divider_bottom)
    private var viewRoot: View = findViewById(R.id.session_privilege_position_root)
    private var photoView:InterestedTalentPhotoView = findViewById(R.id.view_head_photo)

    override fun getContentLayoutId(): Int = R.layout.layout_b_session_privilege_position

    override fun bindData(bean: ConversationBean?, isLast: Boolean) {
        if (bean == null) {
            visibility = GONE
            return
        }
        visibility = VISIBLE
        viewRoot.setBackgroundResource(if (bean.isTop) R.color.color_f7fdfc else R.drawable.item_list_click_bg)
        dividerView.visibility = if (isLast) View.GONE else View.VISIBLE
        setUnReadRedDot(ivUnRead,bean.unReadCount)
        UserHeadUtils.setUserHeader(ivAvatar, bean.icon)
        setTvAutoShowByText(tvTitle, bean.titleFirst)
        SessionListShownUtils.setLastMessage(bean, tvMessage)
        blockPositionPhoto(bean)
        mTvTopAction.text = ResourceUtils.getString(if (bean.isTop) R.string.message_list_unpin else R.string.message_list_pin)
    }

    /**
     * 阻断职位发布头像设置
     * 特权职位待发布和热招职位待发布用接口请求的真实头像,接口头像小于3，用本地随机图片
     * 特权职位扩展或者其他情况用本地随机图片
     */
    private fun blockPositionPhoto(conversationBean: ConversationBean) {
        val summaryBean = GsonUtil.from<SessionSummaryBean>(conversationBean.extendData, object : TypeToken<SessionSummaryBean?>() {})
        if (summaryBean == null) {
            showLocalImg()
            return
        }
        if (summaryBean.isPrivilegePositionPost || summaryBean.isHotPositionPost) {
            val talentPhotoList = BlockPositionPhotoUtils.getInstance().talentPhoto
            if (ListUtil.size(talentPhotoList) < BlockPositionPhotoUtils.TALENT_NUM) {
                photoView.visibility = GONE
            } else {
                photoView.visibility = VISIBLE
                photoView.setTalentPhoto(talentPhotoList)
            }
            return
        }
        showLocalImg()
    }

    /**
     * 显示本地图片
     */
    private fun showLocalImg() {
        val localImgList = BlockPositionPhotoUtils.getInstance().localImgList
        if (ListUtil.size(localImgList) < BlockPositionPhotoUtils.TALENT_NUM) {
            photoView.visibility = GONE
        } else {
            localImgList.shuffle() //头像list乱序
            photoView.visibility = VISIBLE
            photoView.setLocalImg(localImgList.subList(0, 3))
        }
    }

    override fun onSessionClick(bean: ConversationBean, position: Int) {
        super.onSessionClick(bean, position)
        AnalyticHelper.getInstance().sendClickInfo(PageId._1qg2, BAid._1rga)
        getBlockPositionTalentInfo(bean)
    }

    /**
     * 获取阻断职位跳转信息
     *
     * @param conversationBean
     */
    private fun getBlockPositionTalentInfo(conversationBean: ConversationBean?) {
        val summaryBean = GsonUtil.from<SessionSummaryBean>(conversationBean?.extendData, object : TypeToken<SessionSummaryBean?>() {})
        if (summaryBean == null) {
            IntentUtils.startTalentRadarSearch(context, conversationBean?.lastMessageContent)
            return
        }
        val activity = context as? BaseActivity ?: return
        if (summaryBean.isPrivilegePositionPost || summaryBean.isHotPositionPost) {
            activity.showLoading()
            BlockPositionHelper.getBlockPositionTalentInfo(
                summaryBean.bussinessId, activity, object : BaseCallback<BlockPositionTalentBean?>() {
                    override fun onSuccess(request: HttpRequest<*>?, response: HttpResponse<BlockPositionTalentBean?>?) {
                        if (activity.isActivityDestroyed) {
                            return
                        }
                        activity.hideLoading()
                        if (response?.content == null) {
                            IntentUtils.startTalentRadarSearch(activity, conversationBean?.lastMessageContent)
                            return
                        }
                        handleBlockPositionTalentInfo(response.content!!)
                    }

                    override fun onFailure(request: HttpRequest<*>?, response: HttpResponse<BlockPositionTalentBean?>) {
                        if (activity.isActivityDestroyed) {
                            return
                        }
                        activity.hideLoading()
                        IntentUtils.startTalentRadarSearch(activity, conversationBean?.lastMessageContent)
                    }
                })
        } else {
            IntentUtils.startTalentRadarSearch(activity, conversationBean?.lastMessageContent)
        }
    }

    private fun handleBlockPositionTalentInfo(talentBean: BlockPositionTalentBean) {
        if (talentBean.isInterestedTalent) {
            if (!ListUtil.isEmpty(talentBean.list) && talentBean.list.size > 2) {
                IntentUtils.startBlockPositionInterestedTalent(context, talentBean)
            } else {
                IntentUtils.startReleaseActivity(context)
            }
            return
        }
        if (talentBean.isTalentDetail) {
            if (!ListUtil.isEmpty(talentBean.list)) {
                IntentUtils.startCandidateInfoActivity(
                    context, null,
                    CandidateUtils.convertToTalentBeans(talentBean.list)
                )
            } else {
                IntentUtils.startReleaseActivity(context)
            }
            return
        }
        IntentUtils.startReleaseActivity(context)
    }
}